%macro dfr_standalone(standalone_name=,id=,
forecast_start=,forecast_num_periods=52,kpis=,
dfr_merch_level=,dfr_loc_level=,dfr_time_level=,
dfr_merch_level_to=0,dfr_loc_level_to=0,dfr_time_level_to=0,
dfr_extract=3,dfr_archive=0,dfr_purge=0,
dfr_filter_start=,dfr_filter_end=,dfr_filter_periods=,
dfr_recon_single=1,dfr_recon_hier=1,dfr_recon_merch=0,dfr_recon_loc=0,
dfr_ind=,dfr_revisit=0);

/*-------------------------------------
      Create default jobparams file
*-------------------------------------*/
	data _null_; call symput('dfr_horizonstart1',"&forecast_start."d); run;
/*	%let dfr_horizonstart1=%trim(&dfr_horizonstart1.);*/
	%let dfr_horizonstart1=%sysfunc(ktrim(&dfr_horizonstart1.));
	%let wktid=&id.;

    %let memname=;
    proc sql noprint;
        select memname into :memname 
			from sashelp.vtable 
			where kupcase(libname)="MMDATA" and kupcase(memname)="JOBPARAM_&wktid.";
    quit;
    %if (&memname eq) %then %do;
		proc sql noprint;
			create table mmdata.jobparam_&wktid. as select * from mmdata.jobparam where type in ('ma','dfr');
		quit;
		%end;

/*-------------------------------------
      Clear parameters
*-------------------------------------*/
proc sql noprint;

	update mmdata.jobparam_&wktid. set srcfield="" where kupcase(name) in ('TIME_FROM_LEVEL','TIME_TO_LEVEL','MERCH_FROM_LEVEL',
		'MERCH_TO_LEVEL','LOC_FROM_LEVEL','LOC_TO_LEVEL','MERCH_FROM_ID','LOC_TEMPLATE','LOC_FROM_ID',
		'MERCH_TEMPLATE','TIME_TEMPLATE','CALENDAR','RECON_SINGLE','RECON_HIER','RECON_LOC','RECON_MERCH','JOBNAME');

/*-------------------------------------
      Set parameters
*-------------------------------------*/
	update mmdata.jobparam_&wktid. set value="&dfr_time_level." where kupcase(name)='TIME_FROM_LEVEL';		/* 5=week */
	update mmdata.jobparam_&wktid. set value="&dfr_merch_level." where kupcase(name)='MERCH_FROM_LEVEL';
	update mmdata.jobparam_&wktid. set value="&dfr_loc_level." where kupcase(name)='LOC_FROM_LEVEL';
	update mmdata.jobparam_&wktid. set value="&dfr_filter_start." where kupcase(name)='DFR_FILTER_START_DATE';
	update mmdata.jobparam_&wktid. set value="&dfr_filter_end." where kupcase(name)='DFR_FILTER_END_DATE';
	update mmdata.jobparam_&wktid. set value="&dfr_filter_periods." where kupcase(name)='DFR_FILTER_PRIOR_HIST';

	%if (&dfr_recon_single.=0) %then %do; /* reconcile parameters for multi-level forecast */
		update mmdata.jobparam_&wktid. set value="&dfr_merch_level_to." where kupcase(name)='MERCH_TO_LEVEL';
		update mmdata.jobparam_&wktid. set value="&dfr_loc_level_to." where kupcase(name)='LOC_TO_LEVEL';
		update mmdata.jobparam_&wktid. set value="&dfr_time_level_to." where kupcase(name)='TIME_TO_LEVEL';
		%end;
	%else %do;
		update mmdata.jobparam_&wktid. set value="&dfr_merch_level." where kupcase(name)='MERCH_TO_LEVEL';
		update mmdata.jobparam_&wktid. set value="&dfr_loc_level." where kupcase(name)='LOC_TO_LEVEL';
		update mmdata.jobparam_&wktid. set value="&dfr_time_level." where kupcase(name)='TIME_TO_LEVEL';
		%end;

	/* Metadata */
	%let num_kpis=1;
	%let item = %sysfunc(kscan(%quote(&kpis.),&num_kpis.));
	%do %while (%quote(&item.) ne);
		%let num_kpis = %eval(&num_kpis.+1);
		%let item = %sysfunc(kscan(%quote(&kpis.),&num_kpis.));
		%end;
	%let num_kpis=%eval(&num_kpis.-1);

	update mmdata.jobparam_&wktid. set value="&num_kpis." where kupcase(name)='DFR_NUMKPIS_1';
	update mmdata.jobparam_&wktid. set value="&kpis." where kupcase(name)='DFR_KPINAMESET_1';
	update mmdata.jobparam_&wktid. set value="&dfr_ind." where kupcase(name)='DFR_IND_VAR1';

	/* Workflow */
	update mmdata.jobparam_&wktid. set value="&dfr_extract." where kupcase(name)='DFR_WF_EXTRACT';		/* Input from inp.fx_1 dataset */
	update mmdata.jobparam_&wktid. set value="&dfr_archive." where kupcase(name)='DFR_WF_ARCHIVE';
	update mmdata.jobparam_&wktid. set value="&dfr_purge." where kupcase(name)='DFR_WF_PURGE';
	update mmdata.jobparam_&wktid. set value="&dfr_revisit." where kupcase(name)='DFR_WF_REVISIT';

	/* Reconciliation - only needed if > 1 merchandise or location level needed */
	update mmdata.jobparam_&wktid. set value="&dfr_recon_single." where kupcase(name)='RECON_SINGLE';
	update mmdata.jobparam_&wktid. set value="&dfr_recon_hier." where kupcase(name)='RECON_HIER';
	update mmdata.jobparam_&wktid. set value="&dfr_recon_merch." where kupcase(name)='RECON_MERCH';
	update mmdata.jobparam_&wktid. set value="&dfr_recon_loc." where kupcase(name)='RECON_LOC';

	/* No changes needed */
	update mmdata.jobparam_&wktid. set value="&standalone_name." where kupcase(name)='JOBNAME';
	update mmdata.jobparam_&wktid. set value="&forecast_num_periods." where kupcase(name)='ENG_LEAD';
	update mmdata.jobparam_&wktid. set value="&dfr_horizonstart1." where kupcase(name)='DFR_HORIZONSTART';
	update mmdata.jobparam_&wktid. set value="1" where kupcase(name)='DFR_WF_OPTIMIZED';
	update mmdata.jobparam_&wktid. set value="0" where kupcase(name)='MERCH_FROM_ID';
	update mmdata.jobparam_&wktid. set value="0" where kupcase(name)='LOC_FROM_ID';
	update mmdata.jobparam_&wktid. set value="0" where kupcase(name)='LOC_TEMPLATE';
	update mmdata.jobparam_&wktid. set value="0" where kupcase(name)='MERCH_TEMPLATE';
	update mmdata.jobparam_&wktid. set value="0" where kupcase(name)='TIME_TEMPLATE';
	update mmdata.jobparam_&wktid. set value="0" where kupcase(name)='CALENDAR';
	update mmdata.jobparam_&wktid. set value="10" where kupcase(name)='DFR_MAXMERCH';
	update mmdata.jobparam_&wktid. set value="4" where kupcase(name)='DFR_MAXLOC';
quit;

/*-------------------------------------
      Run Forecast
*-------------------------------------*/
%mm_forecast(wktid=&wktid.,dsn=&dsn.,dbtype=oracle, paramfile=jobparam_&wktid., standalone=1, set_analytic=1);
%mend;

