/**************************************************************/
/*                                                            */
/*       SAS RETAIL INTELLIGENCE SOLUTIONS					  */
/* 		 Demand Forecast For Retail Planning 4.2       		  */
/*                                                            */
/**************************************************************/
/*                                                            */
/* SAS Retail Group, All rights reserved (c) 2006,2007        */
/*                                                            */
/**************************************************************/
 
/************************************************************/
/*Functionality- To combine the separate merch/loc IDs forecast datasets into corresponding merch and loc levels */
/************************************************************/
 
%macro combine2(mode=OUTFOR,DName=URM,lib=for);
 
	/*Read the forecast metadata table row by row */
	%let nobs=0;
	%noobs(ds=dp.F_&proj._forecast_meta);
	%if &nobs ne 0 %then %do;/* START OF check if forecast control table has any rows*/
	 
		/* Splitting the forecast control table into overridden and non-overridden dataset  */
		data W_&proj.._over_  W_&wktid.._non_over_;
			set dp.F_&proj._forecast_meta;
			if m_id ne 0 or l_id ne 0 or group_key ne 0 then output W_&wktid.._over_;
			else output W_&wktid.._non_over_;
		run;
		%RCSet (&syserr);

		%let nobs=0;
		%noobs(ds=W_&wktid.._non_over_);
		%if &nobs ne 0 %then %do;/* START OF check if non overridden has rows*/

			/*Temporary SAS program To renaming all non overridden dataset*/
			filename tmp temp;
			/*Data Step to combine IDs into corresponding level datasets if there are no ID overrides*/
			data W_&wktid.._rename_( drop=v rc);
			   length ds_name $40;
			   length ds_ref_no 8;
			   length v $2000;
			   file tmp;
			  if _N_ = 1 then do;
			       declare hash d_name(dataset: "DP.F_&proj._forecast_meta");
			       d_name.defineKey('M_LVL','L_LVL','Ref_Merch_ID','Ref_location_ID','group_key','KPI');
				   d_name.defineData("ds_name","ds_ref_no");
			       d_name.defineDone();
				   v= 'proc datasets library='||"&lib." || '  nodetails memtype=data nolist ;';
				   put v;
			   end;
				set W_&wktid.._non_over_ end=L;
				rc = d_name.find();
				v='change ' ||"&mode."||'_F_' || strip(&proj.)|| '_'|| strip(ds_ref_no)|| '=' ||"&DName."||'_F_' || "&proj." || '_'||strip(ds_ref_no) || ';';
				put v;
				if L then do;
					call symputx('new_ref_no',ds_ref_no+1);
					put 'Quit;';
				end;
			run;
			%RCSet (&syserr);
	 
			%include tmp;/*Executing temporary SAS program*/
		%end;/* END OF check if non overridden has rows*/
	 
		%let nobs=0;
		%noobs(ds=W_&wktid.._over_);
		%if &nobs ne 0 %then %do;/* START OF check if  overridden has rows*/
			/*Check for the parent of override dataset*/
			data W_&wktid.._over_parent_ W_&wktid.._over_no_parent_;
			   length ds_name $40;
			   length ds_ref_no  8;
				   if _N_ = 1 then do;
		             declare hash d_name(dataset: "W_&wktid.._non_over_");
		       		 d_name.defineKey('M_LVL','L_LVL','KPI');
			   		 d_name.defineData("ds_ref_no");
		       		 d_name.defineDone();
		   		   end;
				set W_&wktid.._over_ end=L;
				rc = d_name.find();
				if rc = 0 then
					output W_&wktid.._over_parent_;
				else 
					output  W_&wktid.._over_no_parent_;
			run;
			%RCSet (&syserr);
			%let nobs=0;
			%noobs(ds=W_&wktid.._over_parent_);
			%if &nobs ne 0 %then %do;
					/*Temporary SAS program (macro) To merge all overridden dataset*/
					filename tmp2 temp;
					/*Data step to combine IDs with corresponding levels, if IDs have been overridden*/
					data W_&wktid.._non__;
					   length ds_name $40;
					   length ds_ref_no 8;
					   length v $2000;
					   file  tmp2 ;
					  if _N_ = 1 then do;
					      /* load SMALL data set into the hash object */
					       declare hash d_name(dataset: "W_&wktid.._rename_");
					       d_name.defineKey('M_LVL','L_LVL','KPI');
						   d_name.defineData("ds_ref_no");
					       d_name.defineDone();
					   end;
						set W_&wktid.._over_parent_ end=L;
						rc = d_name.find();
						if rc=0 then do;
						v= 'data '|| "&lib." ||'.' || "&DName." || '_F_' || strip(&proj.) || '_'|| strip(ds_ref_no)|| ';';
						put v;
						v= 'set '|| "&lib." ||'.' || "&DName." || '_F_' || strip(&proj.) || '_'|| strip(ds_ref_no)|| ' ' || "&lib." ||'.'|| strip(ds_name)|| '; ';
						put v;
						put 'run;';
						v= 'proc datasets lib='||"&lib." ||' nolist; delete '||  strip(ds_name)||'; quit;';
						put v;
						end;
					run;
					%RCSet (&syserr);
					%include tmp2; /*Executing temporary SAS program*/
				%end;
				%let nobs=0;
				%noobs(ds=W_&wktid.._over_no_parent_);
				%if &nobs ne 0 %then %do;
					proc sort data=W_&wktid.._non_over_;
						by ds_ref_no;
					run;
					data _null_;
						set W_&wktid.._non_over_ end=L;
						if L then call symputx('new_ref_no',ds_ref_no+1);
					run;
					%RCSet (&syserr);
					proc sort data=W_&wktid.._over_no_parent_ out=W_&wktid.._over_no_parent_;
						by M_LVL L_LVL KPI group_key;
					run;
					%RCSet (&syserr);

					data W_&wktid.._over_no_parent_ ;
						set W_&wktid.._over_no_parent_;
						by M_LVL L_LVL KPI group_key;
						old_ds_ref_no=ds_ref_no;
						drop ds_ref_no;
					run;
					%RCSet (&syserr);

					filename tmp3 temp;
					data W_&wktid.._over_no_parent_1 (drop=rc v old_ds_ref_no) ;
						retain ds_ref_no &new_ref_no;
						set W_&wktid.._over_no_parent_ end=L;
						length v $2000;
						file tmp3;
						by M_LVL L_LVL KPI group_key;
						if first.kpi then do;
							if _n_ >1 then ds_ref_no=ds_ref_no+1;
							v= 'proc datasets library='||"&lib." || '  nodetails memtype=data nolist ;';
						    put v;
							v='change ' || strip(ds_name)||  '=' ||"&DName."||'_F_' || "&proj." || '_'||strip(ds_ref_no) || ';';
						    put v;
							put 'Quit;';
							Ref_Merch_ID=0;
							Ref_location_ID=0;
							group_key=0;
							m_id = 0;
							l_id= 0;
							ds_desc="F_&proj._M&merch_level._L&loc_level._0_0_0_&drop_kpi_var";
						end;
						else do;
							v= 'data '|| "&lib." ||'.' || "&DName." || '_F_' || strip(&proj.) || '_'|| strip(ds_ref_no)|| ';';
							put v;
							v= 'set '|| "&lib." ||'.' || "&DName." || '_F_' || strip(&proj.) || '_'|| strip(ds_ref_no)|| ' ' || "&lib." ||'.'|| strip(ds_name)|| '; ';
							put v;
							put 'run;';
							v= 'proc datasets lib='||"&lib." ||' nolist; delete '||  strip(ds_name)||'; quit;';
							put v;
						end;
						
					run;
					%RCSet (&syserr);

					data W_&wktid.._over_no_parent_1 ;
						set W_&wktid.._over_no_parent_1 ;
						by 	M_LVL L_LVL KPI;
						if first.kpi;
					run;
					%RCSet (&syserr);

					data W_&wktid.._non_over_;
						set W_&wktid.._non_over_  W_&wktid.._over_no_parent_1;
					run;
					%RCSet (&syserr);
					%include tmp3;
				%end;
		%end;/* END OF check if overridden has rows*/
	 
		%if &mode.=OUTFOR %then %do;/*Execute following code only if call mode is FOR*/
				data dp.F_&proj._forecast_meta;
					set W_&wktid.._non_over_;
					ds_name='URM_F_' || strip(&proj.) ||'_' ||strip(ds_ref_no);
					if m_id ne 0 or l_id ne 0 or group_key ne 0 then delete;
				run;
		%end;/*END OF Execute following code only if call mode is FOR*/
	%end;/*END OF check if forecast control table has any rows*/
	/*Clear WORK library*/
	proc datasets library=W_&wktid. kill nodetails nolist;
	quit;
	
%mend;
