/**************************************************************/
/*                                                            */
/*       SAS RETAIL INTELLIGENCE SOLUTIONS					  */
/* 		 Demand Forecast For Retail Planning 4.2       		  */
/*                                                            */
/**************************************************************/
/*                                                            */
/* SAS Retail Group, All rights reserved (c) 2006,2007        */
/*                                                            */
/**************************************************************/
 
/************************************************************/
/*Functionality- To combine the separate "Performance Split" forecast datasets into corresponding merch and loc levels */
/************************************************************/
 
%macro combine3(mode=OUTFOR,DName=URM,lib=for);
 
	/*Read the forecast metadata table row by row */
	%let nobs=0;
	%noobs(ds=dp.F_&proj._forecast_meta);
	%if &nobs ne 0 %then %do;/* START OF check if forecast control table has any rows*/
	 
		/* Splitting the forecast control table into overridden and non-overridden dataset  */
		data W_&wktid..p_split;
			set dp.F_&proj._forecast_meta;
			if p_split ne 0 ;
		run;
		%RCSet (&syserr);
		proc sort data=W_&wktid..p_split;
			by M_LVL L_LVL Ref_Merch_ID Ref_location_ID group_key KPI p_split;
		run;

		%let nobs=0;
		%noobs(ds=W_&wktid..p_split);
		%if &nobs ne 0 %then %do;/* START OF check if overridden has rows*/

			/*Temporary SAS program To renaming all non overridden dataset*/
			filename tmp_PS temp;
			/*Data Step to combine IDs into corresponding level datasets if there are no ID overrides*/
				data W_&wktid.._non_;
					   length ds_name $40;
					   length ds_ref_no 8;
					  if _N_ = 1 then do;
					 	 declare hash d_name(dataset: "DP.F_&proj._meta_copy");
		 	 	         d_name.defineKey('M_LVL','L_LVL','Ref_Merch_ID','Ref_location_ID','group_key','KPI');
				         d_name.defineData("ds_ref_no");
			             d_name.defineDone();
					 end;
						set W_&wktid..p_split end=L;
						rc = d_name.find();
				run;
				%RCSet (&syserr);

				proc sort data=W_&wktid.._non_;
					by ds_ref_no p_split;
				run;
				%RCSet (&syserr);
				filename tmp_PS temp;
				data _null_;
					file  tmp_PS ;
					length v $2000;
					set W_&wktid.._non_;
					by ds_ref_no p_split;
					if first.ds_ref_no then do;
						v= "Data &lib.." || "&mode."||'_F_' || strip(&proj.)|| '_'|| strip(ds_ref_no)||'; set ';
						put v;
					end;
					v="&lib.." || strip(ds_name);
					put v;
					if last.ds_ref_no then do;
						v= "; run;";
						put v;
					end;
				 run;
	 			%include tmp_PS;/*Executing temporary SAS program*/
				data data _null_;
				  file  tmp_PS ;
				  length v $2000;
				  set W_&wktid.._non_ end=L;
				  if _n_=1 then put 'Proc sql noprint; drop table ';
				  if L=0 then do;	
					  v= "&lib.." ||strip(ds_name) || ' , ';
					  put v;
				  end;
				  if L then do;
				  	v= "&lib.." ||strip(ds_name) || ' ; quit; ';
					put v;
				  end;
				run;
				%include tmp_PS;/*Executing temporary SAS program*/
				data dp.F_&proj._forecast_meta;
					set W_&wktid.._non_;
					by ds_ref_no p_split;
					p_split=0;
					ds_name='OUTFOR_F_'||strip(&projid) || '_' || strip(ds_ref_no);
					ds_desc=strip(ds_name) || '_m' || strip(m_lvl) ||'_L'||strip(l_lvl) || '_'||strip(M_id)||'_'|| strip(L_id) || '_'|| strip(p_split)||'_'||strip(kpi) ;
					if first.ds_ref_no;
				run;
		%end;/* END OF check if non overridden has rows*/
		%end;
	 /*Clear WORK library*/
	proc datasets library=W_&wktid. kill nodetails nolist;
	quit;
	
%mend;
