/**************************************************************/
/*                                                            */
/*       SAS RETAIL INTELLIGENCE SOLUTIONS				  	  */
/* 		 Demand Forecast For Retail Planning 4.2       	      */
/*                                                            */
/**************************************************************/
/*                                                            */
/* SAS Retail Group, All rights reserved (c) 2006,2007        */
/*                                                            */
/**************************************************************/
/*****************************************************************************************/
/*Functionality: This macro attaches required the independent variables to the sales data*/
/*****************************************************************************************/
 
%macro dfr_data_gen();
 
%let data_name=f_&proj;
filename temp1 temp;
/*Getting time period columns*/
%if &promo_time_mode.=RET %then %do;
	%let a=%sysfunc(KTRIM(%sysfunc(KLEFT(&dfr_interval))));
	%let tp_st_col=ret_&a._start_dt;
	%let tp_end_col=ret_&a._end_dt;
%end;
%else %if &promo_time_mode.=CAL %then %do;
	%let tp_st_col=EFFECTIVE_FROM_DT;
	%let tp_end_col=EFFECTIVE_TO_DT;
%end;
 

proc sort data=inp.&data_name out=W_&wktid..mmx_tmp1(drop=mname: lname:) noequals;
	by mid: lid: &dfr_time_field.;
run;
/*Processing for promotion effect*/
%if &dfr_data_promo_num gt 0 and &promo_effect gt 0 %then %do;
/*Macro call to initialise 'promo_type_cd' variables*/
	%let promo_zero=&dfr_data_promo_num;
	%let promo_nm=;
	%attach_promo();
	%let dfr_data_promo_num=&promo_zero;
	%let dfr_data_promo_types=&promo_nm;
	%RCSet (&syserr);
%end;
 
/*Processing for price effects */
%if &price_effect gt 0 %then %do;

%create_price_data;

	data _null_;
		infile temp1 mod;
		file temp1;
		Put 'proc sql;';
		Put 'create table W_&wktid..mmx_tmp2 as';
		Put 'select A.* ,';
		Put 'B.weighted_price_ratio as price_ratio ';
		PUT 'from W_&wktid..mmx_tmp1 as A left join inp.Price_abt_&wktid. as b ';
		V='on (A.mID_'||KTRIM(KLEFT(&merch_to_level))||' = B.item_rk and B.merch_lvl=' ||
		KTRIM(KLEFT(&merch_to_level));
		PUT V;
		V='and A.LID_'||KTRIM(KLEFT(&loc_to_level)) ||' = B.retail_outlet_rk and B.loc_lvl = '
		||KTRIM(KLEFT(&loc_to_level));
		put V;
		V= ' and A.'||symget('dfr_time_field') || '=b.time_period_rk);';
		put V;
		put 'quit;';
		put '%rcset(&sqlrc);';
	run;
	%RCSet (&syserr);
%end;
%else %do;
	/*If no price effect then set input dataset to next temporary output dataset*/
	data _null_;
		infile temp1 mod;
		file temp1;
		put 'proc datasets lib = W_&wktid. nolist;';
		put 'change mmx_tmp1=mmx_tmp2;';
		put 'quit;';
		put '%rcset(&syserr);';
	run;
%RCSet (&syserr);
%end;
 
/*Processing for halo cannibalization effects */
%if &dfr_data_promo_num gt 0 and &promo_effect gt 0 and &halo_cannib_effect gt 0 and &merch_to_level.-&merch_from_level. >=3 %then %do;
 
	%let keys_add_M=;
	%let keys_add_L=;
	%get_IDs(from_level=&loc_from_level,to_level=&loc_to_level,type=L);
	%get_IDs(from_level=&merch_from_level,to_level=&merch_to_level-1,type=M);
 
	data _null_;
		infile temp1 mod;
		file temp1;
/**************for level mIDx-1*****************************/
		put 'proc sort data=W_&wktid..mmx_tmp2 OUT =W_&wktid..mmx_tmp2_srt noequals;';
		put 'by';
		v=symget('keys_add_L');
		put v;
		a1=symget('keys_add_M');
		put a1;
		m=symget('dfr_time_field') ||';';
		put m;
		put 'run;';
		put '%rcset(&syserr);';
 
		put 'proc summary data=W_&wktid..mmx_tmp2_srt NWAY;';
		put 'class';
		v=symget('keys_add_L');
		put v;
		a1=symget('keys_add_M');
		put a1;
		m=symget('dfr_time_field') ||';';
		put m;
		put 'var sales;';
		put 'output out=W_&wktid..temp1 (drop=_type_ _freq_) SUM=TOTAL_SUB_CLASS;';
		put 'run;';
		put '%rcset(&syserr);';
 
		put 'data W_&wktid..mmx_tmp3 (drop=TOTAL_SUB_CLASS);';
		put 'merge W_&wktid..mmx_tmp2_srt  W_&wktid..temp1;';
		put 'by';
		v=symget('keys_add_L');
		put v;
		a1=symget('keys_add_M');
		put a1;
		m=symget('dfr_time_field') ||';';
		put m;
		put 'TOTAL_SUB_CLASS_WO_AMT=TOTAL_SUB_CLASS-SALES;';
		put 'run;';
		put '%rcset(&syserr);';
	run;
/**********************for level mIDx-2**********************************************/
	%let keys_add_M=;
	%let keys_add_L=;
	%get_IDs(from_level=&loc_from_level,to_level=&loc_to_level,type=L);
	%get_IDs(from_level=&merch_from_level,to_level=&merch_to_level-2,type=M);
 
	data _null_;
		infile temp1 mod;
		file temp1;
		
		put 'proc sort data=W_&wktid..mmx_tmp3 out =W_&wktid..mmx_tmp3_srt noequals;';
		put 'by';
		v=symget('keys_add_L');
		put v;
		a1=symget('keys_add_M');
		put a1;
		m=symget('dfr_time_field') ||';';
		put m;
 
		put 'run;';
		put '%rcset(&syserr);';
 
		put 'proc summary data=W_&wktid..mmx_tmp3_srt NWAY;';
		put 'class';
		v=symget('keys_add_L');
		put v;
		a1=symget('keys_add_M');
		put a1;
		m=symget('dfr_time_field') ||';';
		put m;
		put 'var sales;';
		put 'output out=W_&wktid..temp2 (drop=_type_ _freq_) SUM=TOTAL_CLASS;';
		put 'run;';
		put '%rcset(&syserr);';
 
		put 'data W_&wktid..mmx_tmp4 (drop=TOTAL_CLASS);';
		put 'merge W_&wktid..mmx_tmp3_srt  W_&wktid..temp2;';
		put 'by';
		v=symget('keys_add_L');
		put v;
		a1=symget('keys_add_M');
		put a1;
		m=symget('dfr_time_field') ||';';
		put m;
		put 'TOTAL_CLASS_WO_AMT=TOTAL_CLASS-SALES;';
		put 'run;';
		put '%rcset(&syserr);';
	run;
/**********************for level mIDx-3****************************************/
	%let keys_add_M=;
	%let keys_add_L=;
	%get_IDs(from_level=&loc_from_level,to_level=&loc_to_level,type=L);
	%get_IDs(from_level=&merch_from_level,to_level=&merch_to_level-3,type=M);
 
	data _null_;
		infile temp1 mod;
		file temp1;
		put 'proc sort data=W_&wktid..mmx_tmp4 out =W_&wktid..mmx_tmp4_srt noequals;';
		put 'by';
		v=symget('keys_add_L');
		put v;
		a1=symget('keys_add_M');
		put a1;
		m=symget('dfr_time_field') ||';';
		put m;
		put 'run;';
		put '%rcset(&syserr);';
 
		put 'proc summary data=W_&wktid..mmx_tmp4_srt NWAY;';
		put 'class';
		v=symget('keys_add_L');
		put v;
		a1=symget('keys_add_M');
		put a1;
		m=symget('dfr_time_field') ||';';
		put m;
		put 'var sales;';
		put 'output out=W_&wktid..temp3 (drop=_type_ _freq_) SUM=TOTAL_;';
		put 'run;';
		put '%rcset(&syserr);';
 
		put 'data W_&wktid..mmx_tmp5 (drop=TOTAL_);';
		put 'merge W_&wktid..mmx_tmp4_srt  W_&wktid..temp3;';
		put 'by';
		v=symget('keys_add_L');
		put v;
		a1=symget('keys_add_M');
		put a1;
		m=symget('dfr_time_field') ||';';
		put m;
		put 'TOTAL_WO_AMT=TOTAL_-SALES;';
		put 'run;';
		put '%rcset(&syserr);';
	RUN;
 
	%RCSet (&syserr);
%end;
%else %do;
	/*If no halo cannib  effect then set input dataset to next temporary output dataset*/
	data _null_;
		infile temp1 mod;
		file temp1;
		put 'proc datasets lib = W_&wktid. nolist;';
		put 'change mmx_tmp2=mmx_tmp5;';
		put 'quit;';
		put '%rcset(&syserr);';
	run;
%RCSet (&syserr);
%end;
 
 
/*Set pantry flags*/
%if &dfr_data_promo_num gt 0 and &promo_effect gt 0 and &pantry_effect gt 0  %then %do;
	%set_pantry_flags(in_data=W_&wktid..mmx_tmp5 ,out_data=W_&wktid..mmx_tmp6,promos=&dfr_data_promo_num,pl_prd=3);
%end;
%else %do;
	/*If no pantry effect then set input dataset to next temporary output dataset*/
	data _null_;
		infile temp1 mod;
		file temp1;
		put 'proc datasets lib = W_&wktid. nolist;';
		put 'change mmx_tmp5=mmx_tmp6;';
		put 'quit;';
		put '%rcset(&syserr);';
	run;
%RCSet (&syserr);
%end;
 
/*Set delay flags*/
%if &dfr_data_promo_num gt 0 and &promo_effect gt 0 and &delay_effect gt 0 %then %do;
/* write reverse sorting*/
	data _null_;
		infile temp1 mod;
		file temp1;
		length V $2000;
		put 'proc sort data=W_&wktid..mmx_tmp6 out=W_&wktid..mmx_tmp7 noequals;';
		put 'By';
		V='mID_'||KTRIM(KLEFT(&merch_to_level));
		put V;
		V='lID_'||KTRIM(KLEFT(&loc_to_level));
		put V;
		V= 'DESCENDING ' || symget('dfr_time_field') ||';';
		put V;
		put 'RUN;';
		put '%rcset(&syserr);';
	run;
	%RCSet (&syserr);
%set_cd_flags(in_data=W_&wktid..mmx_tmp7 ,out_data=W_&wktid..mmx_tmp8,promos=&dfr_data_promo_num,cd_prd=1);
%end;
%else %do;
	/*If no delay effect then set input dataset to next temporary output dataset*/
	data _null_;
		infile temp1 mod;
		file temp1;
		put 'proc datasets lib = W_&wktid. nolist;';
		put 'change mmx_tmp6=mmx_tmp8;';
		put 'quit;';
		put '%rcset(&syserr);';
	run;
%RCSet (&syserr);
%end;
 
/*Processing for weather effects */
%let tot_count=0;

%if &weather_effect gt 0 %then %do;

%import_weather_data(csvfile=&path_data&slash.input&slash.&dfr_ext_weather_abt,out=inp.WEATHER_ABT_&wktid);

proc sql noprint;
		 select count (distinct loc_lvl)
		 	into: tot_count 
				from inp.weather_abt_&wktid;
quit;
%end;



%if &weather_effect gt 0 and &tot_count = 1 %then %do;

		/*Check for distinct values of location level field*/
		
		proc sql noprint;
			 select distinct loc_lvl
			 	into: weather_loc_lvl
					from inp.weather_abt_&wktid;
			quit;
		/*Writing the SQL that attaches weather information to sales data*/
			data _null_;
				infile temp1 mod;
				file temp1;
				Put 'proc sql;';
				Put 'create table W_&wktid..mmx_tmp9 as';
				Put 'select A.* ,';
				Put 'B.LOW_TEMPERATURE AS TEMPERATURE_LOW,B.HIGH_TEMPERATURE AS TEMPERATURE_HIGH';
				PUT 'from W_&wktid..mmx_tmp8 as A left join inp.WEATHER_ABT_&wktid as b ';
				V='on (A.LID_'||KTRIM(KLEFT(&weather_loc_lvl)) ||' = B.retail_outlet_rk ';
				put V;
				V= ' and A.' ||symget('dfr_time_field') ||'=b.time_period_rk);';
				put V;
				put 'quit;';
				put '%rcset(&sqlrc);';
			run;
			%RCSet (&syserr);
%end;

%else %do;
	/*If no weather effect then set input dataset to next temporary output dataset*/
	data _null_;
		infile temp1 mod;
		file temp1;
		put 'proc datasets lib = W_&wktid. nolist;';
		put 'change mmx_tmp8=mmx_tmp9;';
		put 'quit;';
		put '%rcset(&syserr);';
	run;
%RCSet (&syserr);

%end;
/*Copying input dataset to stage library and updating input dataset in inp library*/

data _null_;
	infile temp1 mod;
	file temp1;

/*	put 'proc datasets nolist;';*/
/*	put 'copy in=inp out=stage;';*/
/*	put 'select ';*/
/*	v= symget('data_name');*/
/*	put v;*/
/*	Put ';quit;';*/

	put 'proc sql noprint;';
	put 'drop table inp.';
	v= symget('data_name');
	put v;
	put ';quit;';

	put 'proc datasets lib=W_&wktid. nolist;';
	put 'change mmx_tmp9=';
	v= '__' || symget('data_name') || '__';
	put v;
	Put ';quit;';

	put 'proc datasets nolist;';
	Put 'copy in=W_&wktid. out=inp;';
	v= symget('data_name');
	put 'select ';
	v= '__' || symget('data_name') || '__';
	put v;
	put ';quit;';

	put 'proc datasets lib=inp nolist;';
	put 'change ';
	v= '__' || symget('data_name') || '__';
	put v;
	put '=';
	v=symget('data_name');
	put v;
	Put ';quit;';
	put '%rcset(&syserr);';
run;

%RCSet (&syserr);
 
/*Run the file generated */
%include temp1;

/*Clear WORK library*/
proc datasets library=W_&wktid. kill nodetails nolist;
quit;

%mend;
 
