%macro dfr_exceptions();
	/* Output exception file for use in analysis such as SAS Forecast Studio                */
	/* Output dataset is: out.fe_[job name]_[job id]                            */
	/* Parameter: dfr_exception_filter contains filter used such as   MAPE > 50                 */
 
	/* loop through all lowest merch/loc level KPI outstat datasets - each KPI is evaluated separately */
%let num_processed=0;
%do k=1 %to &dfr_numkpis.;
	%let ikpi = %sysfunc(KSCAN(%quote(&dfr_kpinameset.),&k.));
	data  W_&wktid..F_&wktid._forecast_meta;
		set  dp.F_&wktid._forecast_meta;
		length name1 $2200;
		name1='for.Outstat_f_'||"&&wktid"||'_' || ktrim(kleft(ds_ref_no));
		if &except_merch_to_level=m_lvl and &except_loc_to_level=l_lvl and "&ikpi"=kpi ;
	run;
	%RCSet (&syserr);
	%let nobs=0;
	%NOOBS(ds=W_&wktid..F_&wktid._forecast_meta);
	%if &nobs ne 0 %then %do; 
	filename excep temp;
	data  W_&wktid..F_&wktid._forecast_meta;
	set W_&wktid..F_&wktid._forecast_meta end=L;
	file  excep ;
	if _n_=1 then put 'set ';
	put  name1 ;
	if L then put  ';';
	run;
	%RCSet (&syserr);


	/* create or insert exception table */
	data W_&wktid..keys_&k. (keep=merch_id location_id);
		%include excep;
		where &dfr_exception_filter.;
		merch_id=mid_&m_lev.;
		location_id=lid_&l_lev.;
	run;
	%RCSet (&syserr);

	%let nobs=0;
	%NOOBS(ds=W_&wktid..keys_&k.);
	%if &nobs ne 0 %then %do;
		%let num_processed=%eval(&num_processed.+1);
		proc sort data=W_&wktid..keys_&k.;
			by merch_id location_id;
		run;
		%RCSet (&syserr);

	%end;
	%end;
%end; /* KPI Loop */

/* merge KPI exception data sets into a final fe exception file ... this ensures merch/loc series aren't duplicated */
%if &num_processed.>0 %then %do;
		proc sort data=inp.f_&projid.;
			by merch_id location_id;
		run;
		%RCSet (&syserr);


		data out.fe_&projid.;
			merge inp.f_&projid. (in=fe)
			%do k=1 %to &dfr_numkpis.; W_&wktid..keys_&k. (in=k_&k.) %end;
			;
			by merch_id location_id;
			if fe
			%do k=1 %to &dfr_numkpis.; and k_&k. %end;
			then output;
		run;
		%RCSet (&syserr);

%end;

%mend;
