%macro dfr_extract();
 
/***********************************************************************************************/
/************************************** HISTORY EXTRACT ****************************************/
/***********************************************************************************************/
 
/* clear optimized flag if using new products or no priveous extract exists */
		%let extract_optimized=0;
/*        %if (&dfr_wf_optimized.=1) and (&dfr_wf_newitem.=0) and (%sysfunc(exist(inp.f_&projid.,DATA))=1) %then
			%let extract_optimized=1;
*/
/*******************************************/
/*********** CORE DATA EXTRACT *************/
/*******************************************/
 
/* IF to level=cluster then make sure extract will pull stores */
%if &orig_loc_to_level.=1002 %then %do;
	%let loc_from_level=1001;
	%let loc_from_id=&dfr_clstr_st_id.;
%end;

/* comma delimited list of source db fields with table name for extract */
%let hist_fields=&dfr_fcast_kpis.;
%let fcast_fields=;
/* DEM INDEPENDENT VARS 120208 */
%if (&dfr_ind_var_mp. ne) %then %do;
	%let hist_fields=&hist_fields., &dfr_ind_var_mp.;
	%end;
%if (&dfr_wf_revisit=1) %then %do;  /* add previous forecast and std fields to extract */
	%let fcast_fields=&dfr_fcast_import_kpis., &dfr_fcast_import_kpis_std.;
%end;

%mm_extract12(
	out=W_&wktid..extract,
	uid=&projid.,
	dsn=&dsn.,
	dbtype=&dbtype.,
	dbpassword=&dbpassword.,
 
	/* KPIS */
	include_hist1=0,
	hist_fields=%quote(&hist_fields.),
	fcast_fields=%quote(&fcast_fields.),
	fcast_versions=&dfr_import_version.,

	/* MERCHANDISE PARAMETERS */
	merch_template=&merch_template.,
	merch_from_level=&merch_from_level.,
	merch_to_level=&merch_to_level.,
	merch_from_id=&merch_from_id.,
	 
	/* LOCATION PARAMETERS */
	loc_acts_like=&dfr_wf_newloc.,
	loc_template=&loc_template.,
	loc_from_level=&loc_from_level.,
	loc_to_level=&loc_to_level.,
	loc_from_id=&loc_from_id.,
	 
	/* TIME PARAMETERS */
	time_type=Event,
	time_template=0,
	calendar=&calendar.,
	time_from_level=&time_from_level.,
	time_to_level=&time_to_level.,
	from_time_id=&hist_from_time.,
	to_time_id=&hist_to_time.);

proc datasets library=work nolist; 
   copy in=work out=W_&wktid.; 
quit;
 
/* Keep all merch and location ids of worksheet cube */
proc sql noprint;
create table stage.mkeys_&projid. as select m_id as merch_id, m_lev as merch_level from W_&wktid..t_cube_merch;
	        %if &orig_loc_to_level.=1002 %then %do;  /* only add clusters not stores */
				create table stage.lkeys_&projid. (location_id num, location_level num);
				%end;
			%else %do;
	create table stage.lkeys_&projid. as select l_realid as location_id, l_lev as location_level from W_&wktid..t_cube_loc;
				%end;
/* add cluster key if clusters in use */
%if (&dfr_clstr_st_id.>0) %then %do;
	/* add Cluster set id */
	insert into stage.lkeys_&projid. (location_id, location_level) values (&dfr_clstr_st_id.,1001);
	insert into stage.lkeys_&projid. (location_id, location_level)
	select distinct(CLSTR_SPC_ID),1002 from maxdata.clstr_spc where CLSTR_GRP_ID>0 and CLSTR_ST_ID = &dfr_clstr_st_id.;
%end;
quit;
 
/**************************************/
/**************************************/
/******* NEW ITEM SUBSTITUTION ********/
/**************************************/
/**************************************/

/* Test to make sure new items exist */
%let dfr_newprod_level=&merch_to_level.;
%if &dfr_newprod_level.<10 %then %do; %let acttab=lv&dfr_newprod_level.ctree; %end; %else %do; %let acttab=lv10mast; %end;
%let like_count=0;

%if (&dfr_wf_newitem.>0) and (&job_rc.<5) and (&dfr_wf_newitem_external.=0) %then %do;
		proc sql noprint;
			select count(*) into :like_count from maxdata.&acttab.  
			where (acts_like1 is not null and acts_like1>0)
			or (acts_like2 is not null and acts_like2>0)
			or (acts_like3 is not null and acts_like3>0)
			or (act_parent_lev>0 and act_parent_lev < &dfr_newprod_level.);
		quit;
		%if (&like_count.=0) %then %do;
			%let job_rc=250; 
			%let job_rc_msg= %sysfunc(sasmsg(catalog.dfrmsg, DFR_NEWITEM_NO_RECORD, noquote));  
			%put &job_rc_msg.;
		%end;
%end;

%if (&dfr_wf_newitem.>0) and (&job_rc.<5) %then %do;

		/**********************************************/
		/*******CREATE LIKE ITEM STAGING TABLE ********/
		/**********************************************/
		proc sql noprint;
			create table W_&wktid..like_item_staging (
				Like_ITEM_type char(1),
				new_item_level num,
				new_item_id num,
				like_item_level num,
				like_item_id num,
				pct_of_parent num,
				start_date date,
				end_date date);
		quit;
		%RCSet(&sqlrc.);
		 
		/**********************************************/
		/* LOAD FROM AN EXTERNAL FILE */
		/**********************************************/
		%if (&dfr_wf_newitem_external.=1) %then %do;
		 
		/* Load external file into staging data set */
		data W_&wktid..like_item_staging1;
			infile "&path_input./dfr_likeitem.txt" dsd;
			format like_item_type $1. new_item_level_name $50. new_item_name $25. like_item_level_name $50. like_item_name $25. start_date date9. end_date date9.;
			input like_item_type $
			new_item_level_name $
			new_item_name $
			like_item_level_name $
			like_item_name $
			pct_of_parent
			start_date
			end_date
			;
		run;
		 
		/* Translate level names to level ids */
		proc sql noprint;
		Create Table W_&wktid..like_item_level_1 as
			Select a.*, (b.level_id -10) as like_item_level from W_&wktid..like_item_staging1 a,
			maxdata.hier_level b where b.hier_id = 11 and upper(KTRIM(a.like_item_level_name)) = upper(KTRIM(b.level_name));
			 
		Create Table W_&wktid..like_item_staging2 as
			Select a.*, (b.level_id -10) as new_item_level, 0 as like_item_id,  0 as new_item_id from W_&wktid..like_item_level_1 a,
			maxdata.hier_level b where b.hier_id = 11 and upper(KTRIM(a.new_item_level_name)) = upper(KTRIM(b.level_name));
		 
		Select max(item_level), min(item_level) into :max_level, :min_level from
			(  select  distinct like_item_level as item_level from  W_&wktid..like_item_staging2
			union all
			Select    distinct new_item_level as item_level from W_&wktid..like_item_staging2);
			 
		/* Translate userid fields for merchandise members to system ids */
		%do i = &min_level. %to &max_level. ;
		Update W_&wktid..like_item_staging2 a set like_item_id =
			( select lv&i.ctree_id from maxdata.lv&i.ctree b , maxdata.lv&i.cmast c
			where b.lv&i.cmast_id = c.lv&i.cmast_id and upper(KTRIM(c.lv&i.cmast_userid)) = upper(KTRIM(a.like_item_name)))
			where like_item_level = &i.;
		 
		Update W_&wktid..like_item_staging2 a set new_item_id =
			( select lv&i.ctree_id from maxdata.lv&i.ctree b , maxdata.lv&i.cmast c
			where b.lv&i.cmast_id = c.lv&i.cmast_id and upper(KTRIM(c.lv&i.cmast_userid)) = upper(KTRIM(a.new_item_name)))
			where new_item_level = &i.;
		%end;
		 
		/* Fill in all parent like_item_id's */
		%do i = &min_level. %to &max_level. ;
			Update W_&wktid..like_item_staging2 a set like_item_id =
				( select lv&i.ctree_id from maxdata.lv&dfr_newprod_level.ctree where new_item_id=lv&dfr_newprod_level.ctree_id)
				where like_item_id is null and like_item_level = &i.;
		%end;
		 
		/* load staging table from temp table */
		insert into W_&wktid..like_item_staging
			(like_item_type,new_item_level,new_item_id,like_item_level,like_item_id, pct_of_parent,start_date, end_date)
			select like_item_type,new_item_level,new_item_id, like_item_level, like_item_id, pct_of_parent, start_date, end_date
			from W_&wktid..like_item_staging2;
		quit;
		%end;
		 
		/***********************************************************/
		/*******SETUP LIKE ITEM STAGING TABLE FROM DATABASE ********/
		/***********************************************************/
		%else %do;
		 
		/* Add acts like merchandise from siblings and parent if they exist */
		proc sql noprint;
			insert into W_&wktid..like_item_staging select 'M',&dfr_newprod_level.,&acttab._id, case act_parent_lev when . then &dfr_newprod_level. else act_parent_lev end, acts_like1,act_pct_adjust,datepart(act_start_date), datepart(act_end_date)
			from maxdata.&acttab. where acts_like1 is not null and acts_like1>0/* and act_end_date is not null*/;
			insert into W_&wktid..like_item_staging select 'M',&dfr_newprod_level.,&acttab._id, case act_parent_lev when . then &dfr_newprod_level. else act_parent_lev end, acts_like2,act_pct_adjust,datepart(act_start_date), datepart(act_end_date)
			from maxdata.&acttab. where acts_like2 is not null and acts_like2>0/* and act_end_date is not null*/;
			insert into W_&wktid..like_item_staging select 'M',&dfr_newprod_level.,&acttab._id, case act_parent_lev when . then &dfr_newprod_level. else act_parent_lev end, acts_like3,act_pct_adjust,datepart(act_start_date), datepart(act_end_date)
			from maxdata.&acttab. where acts_like3 is not null and acts_like3>0/* and act_end_date is not null*/;
			insert into W_&wktid..like_item_staging select 'M',&dfr_newprod_level.,&acttab._id, case act_parent_lev when . then &dfr_newprod_level. else act_parent_lev end,
				case act_parent_lev when 1 then lv1cmast_id
				%do i = 2 %to &dfr_newprod_level. ;
					%if &i.=10
						%then %do; when &i. then lv&i.mast_id %end;
						%else %do; when &i. then lv&i.ctree_id %end;
				%end;
				else 0 end
				, act_pct_adjust,datepart(act_start_date), datepart(act_end_date)
				from maxdata.&acttab.
				where act_parent_lev is not null and act_parent_lev>0 and act_parent_lev < &dfr_newprod_level.;
		quit;
		 
		%end;
		%RCSet(&sqlrc.);
		 
		/***********************************************************/
		/* Get Merchandise items in request worksheet              */
		/***********************************************************/
		%get_merch_members(merch_from_level=&merch_from_level.,merch_to_level=&merch_to_level.,merch_template=&merch_template.);
		 
		/***********************************************************/
		/* Load merch_extract_items with all members that need like-item history */
		/***********************************************************/
		proc sql noprint;
			Create table W_&wktid..merch_extract_items as
				Select Like_ITEM_type as base_Like_ITEM_type,
					Like_ITEM_type as base_Like_ITEM_type,
					new_item_level as base_new_item_level,
					new_item_id as base_new_item_id,
					like_item_level as base_like_item_level,
					like_item_id as base_like_item_id,
					pct_of_parent as base_pct_of_parent,
					start_date as base_start_date,
					end_date as base_end_date
				from W_&wktid..like_item_staging a, W_&wktid..t_cube_merch b
				where a.like_item_type = 'M' and a.new_item_level = b.m_lev and a.new_item_id = b.m_id;
		quit;
		%RCSet(&sqlrc.);
					
					proc sql noprint;
					select COUNT(*) into :count_new_items from W_&wktid..merch_extract_items;
					%if &count_new_items<1 %then %do;
				        %let job_rc=40;
						%let job_rc_msg=%sysfunc(sasmsg(catalog.dfrmsg, dfr_like_item_error, noquote));  
						%put &job_rc_msg.;
						%end;
					quit;
		 
		/***********************************************************/
		/* MAIN LIKE ITEM ROUTINE: FIND HIERARCHY OF ACTS-LIKE ASSIGNMENTS AND LIKE MERCH/TIME RECORDS */
		/***********************************************************/
		%if &job_rc.<4 %then %do;
            %include "&path_pgm./dfrcode/dfr_likeitem.sas";

            /**************************************/
            /* GET HISTORICAL ACTS-LIKE DATA      */
            /**************************************/

            /* Set up merch and time members for extract */
			proc sql noprint;
                drop table t_cube_merch;
                create table t_cube_merch as select distinct like_item_level as m_lev, like_item_id as m_id
                    from W_&wktid..merch_items_cleansed;

                /* Set up time members for extract */
                drop table time_members;
                create table time_members as select distinct time_id as hist0_id, time_level
                    from W_&wktid..merch_items_cleansed;
                quit;
            %RCSet(&sqlrc.);

            %mm_extract12(
                out=W_&wktid..like_items_history,
                uid=&projid.,
                dsn=&dsn.,
                dbtype=&dbtype.,
				dbpassword=&dbpassword.,

                /* KPIS */
                include_hist1=0,
                hist_fields=%quote(&hist_fields.),
				fcast_versions=&dfr_import_version.,

                /* MERCHANDISE PARAMETERS */
                find_merch_ids=0,
                merch_template=&merch_template.,
                merch_from_level=&merch_from_level.,
                merch_to_level=&merch_to_level.,
                merch_from_id=&merch_from_id.,

                /* LOCATION PARAMETERS */
                loc_acts_like=&dfr_wf_newloc.,
                loc_template=&loc_template.,
                loc_from_level=&loc_from_level.,
                loc_to_level=&loc_to_level.,
                loc_from_id=&loc_from_id.,

                /* TIME PARAMETERS */
                find_time_ids=0,
                time_type=Event,
                time_template=0,
                calendar=&calendar.,
                time_from_level=&time_from_level.,
                time_to_level=&time_to_level.,
                from_time_id=&hist_from_time.,      /* use forecast time frame for extract */
                to_time_id=&hist_to_time.);

            /************************************************************************/
            /* ASSOCIATE LIKE HISTORY TO NEW ITEMS AND CALC ADJUSTMENTS FOR EACH KPI*/
            /************************************************************************/
            proc sql noprint;
            Create table stage.new_&projid. as
            select
                %do k=1 %to &dfr_numkpis.;
                    avg(%sysfunc(KSCAN(%quote(&h1_fcastkpi.),%eval(&k.*2))) * (case  pct_of_parent when . then 100 when 0 then 100 else pct_of_parent end) /100)
                        as %sysfunc(KSCAN(%quote(&h1_fcastkpi.),%eval(&k.*2))),
                    %end;
                a.new_item_level as merch_level, a.new_item_id as merch_id,
                b.location_level, b.location_id, b.cluster,
                a.time_level, a.time_id
            from W_&wktid..like_items_history b, W_&wktid..merch_items_cleansed a
            where b.merch_level = a.like_item_level and b.merch_id = a.like_item_id and b.time_level = a.time_level and b.time_id = a.time_id
            group by a.new_item_level, a.new_item_id, b.location_level, b.location_id, a.time_level, a.time_id;
            quit;

            proc sort data=stage.new_&projid. NODUPRECS;
                by &sixkeys.;
                run;

            %RCSet(&sqlrc.);
		%end;

		/************************************************************/
		/******* MERGE ACTS-LIKE HISTORY AND REGULAR HISTORY ********/
		/************************************************************/
		%if ((&dfr_wf_newitem.=1) or (&dfr_wf_newitem.=3)) and (&count_new_items.>0) and (&job_rc.<5) %then %do;
			proc sort data=W_&wktid..extract;
			by &sixkeys.;
		run;
		 
		data W_&wktid..extract;
			merge W_&wktid..extract stage.new_&projid.;
			by &sixkeys.;
		run;
		%RCSet(&syserr.);
		%end;
		 
		/******************************************************************/
		/******* NEW ITEMS EXPORT OPTION  *********************************/
		/******************************************************************/
		%if ((&dfr_wf_newitem.=3) or ((&dfr_wf_newitem.=4) and (&count_new_items.>0))) and (&job_rc.<5) %then %do;
		 
			%let newtable=stage.new_&projid.;                      /* export only new items */
			%if (&dfr_wf_newitem.=3) %then %let newtable=W_&wktid..extract;  /* export new and historical items */
			 
			/* Aggregate to cluster if cluster set and import method=1 (import at cluster level) */
			%if (&orig_loc_to_level.=1002) and (&import_cluster_method.=1) %then %do;
				%let loc_to_level=1002;
				/* aggregate locations */
				proc sql noprint;
				create table W_&wktid..agg as select
				%do k=1 %to &dfr_numkpis.;
					sum(%sysfunc(KSCAN(%quote(&h1_fcastkpi.),%eval(&k.*2)))) as %sysfunc(KSCAN(%quote(&h1_fcastkpi.),%eval(&k.*2))),
					%if &dfr_wf_revisit=1 %then %do;  /* add previous forecast and std fields to extract */
						sum(f_%sysfunc(KSCAN(%quote(&h1_fcastkpi.),%eval(&k.*2)))) as f_%sysfunc(KSCAN(%quote(&h1_fcastkpi.),%eval(&k.*2))),
						sum(s_%sysfunc(KSCAN(%quote(&h1_fcastkpi.),%eval(&k.*2)))) as s_%sysfunc(KSCAN(%quote(&h1_fcastkpi.),%eval(&k.*2))),
						%end;
					%end;
				merch_id, merch_level, time_id, time_level, cluster, cluster as location_id, 1002 as location_level
				from W_&wktid..&newtable.
				group by merch_id, merch_level, time_id, time_level, cluster
				order by merch_id, merch_level, time_id, time_level, cluster;
				%let newtable = W_&wktid..agg;
				 
				/* remove all of the unassigned cluster data */
				delete from W_&wktid..agg where location_id not in (select distinct CLSTR_SPC_ID from stage.cl_&projid.);
				quit;
				%end;
			 
			%mm_extract12_hier(
				in=&newtable.,
				out=W_&wktid..extract_ids,
				uid=&projid.,
				%if (&import_cluster_method.=2) and (&loc_to_level.=4) %then %do;
					                	do_loc=0,
					%end;
				keys=ids,
				attribute_fields=%quote(),
				merch_from_level=&merch_from_level.,
				merch_to_level=&merch_to_level.,
				loc_from_level=&loc_from_level.,
				loc_to_level=&loc_to_level.,
				calendar=&calendar.,
				time_from_level=&time_from_level.,
				time_to_level=&time_to_level.
				);
			 
			%aggregation(
				in=W_&wktid..extract_ids,
				out=W_&wktid..extract2,
				%if (&import_cluster_method.=2) and (&loc_to_level.=4) %then %do;
						do_l=0,
					%end;
				kpilist=%quote(&h1_fcastkpi.),
				numkpis=&dfr_numkpis.,
				cluster_id=&dfr_clstr_st_id.,
				m_from=&merch_from_level.,
				m_to=&merch_to_level.,
				l_from=&loc_from_level.,
				l_to=&loc_to_level.,
				t_from=&time_from_level.,
				t_to=&time_to_level.);
			 
			data W_&wktid..extract (rename=(
				%do k=1 %to &dfr_numkpis.;
					%sysfunc(KSCAN(%quote(&h1_fcastkpi.),%eval(&k.*2)))=%sysfunc(KSCAN(%quote(&dfr_fcast_import_kpis.),%eval(&k.*2)))
				%end;
				));
				set W_&wktid..extract2;
				%if &dfr_import_version.>0 %then %do;
					FV_VERSION_ID=&dfr_import_version.;
				%end;
			run;

			%let pubtab=%sysfunc(KSCAN(%quote(&dfr_fcast_import_kpis.),1));
			 
			/* send merged history to fact table */
			%import_fact_table(indata=W_&wktid..extract,finalname=out.rm_f_&projid.,outtable=maxdata.&pubtab.,import_version=&dfr_import_version.,use_hist=1);		 
			%RCSet(&syserr.);
			%goto EndMacro;
		%end;
	 
		/************************************************************/
		/******* MERGE HISTORY WITH FORECASTED REGULAR ITEMS ********/
		/************************************************************/
		%if (&dfr_wf_newitem.=2) and (&job_rc.<5) %then %do;
			%let data_name=stage.new_&projid.;
			 
			/* add hierarchy to new items */
			%mm_extract12_hier(
			in=&data_name.,
			out=&data_name.,
			uid=&projid.,
			keys=full,
			 
			attribute_fields=%quote(),
			 
			/* MERCHANDISE PARAMETERS */
			merch_from_level=&merch_from_level.,
			merch_to_level=&merch_to_level.,
			 
			/* LOCATION PARAMETERS */
			loc_from_level=&loc_from_level.,
			loc_to_level=&loc_to_level.,
			 
			/* TIME PARAMETERS */
			calendar=&calendar.,
			time_from_level=&time_from_level.,
			time_to_level=&time_to_level.
			);
		%end;
 
%end; /* end new item code */
 
/******************************************************/
/******************************************************/
/******************************************************/
/******************************************************/
/******************************************************/
/******* ADD HIERARCHY NAMES/LEVELS TO EXTRACT ********/
/******************************************************/
/******************************************************/
/******************************************************/
/******************************************************/
 
	/*****************************************************/
	/* Check for empty extract set                       */
	/*****************************************************/
	proc sql noprint;
	    select COUNT(*) into :fcount from W_&wktid..extract;
		%if (&fcount=0) %then %do; 
			%let job_rc=201; 
			%let job_rc_msg= %sysfunc(sasmsg(catalog.dfrmsg, DFR_EXTRACT_NO_RECORD, noquote));  
			%put &job_rc_msg.;
			%goto EndMacro;
		%end;
		create table inp.e_&projid. as select * from W_&wktid..extract order by merch_id, location_id, time_id;
	quit;
	%RCSet(&sqlrc.);

	/*****************************************************/
    /* Add secondary variables							 */
    /*****************************************************/
	%if (&job_rc.<5) %then %do;
		%let f = 1;
		%let exttabs = %mm_parsegettables2(in=%quote(&dfr_ind_var_external.));
		%let atab = %sysfunc(kscan(%quote(&exttabs.),&f,%str(,)));
		%do %while (&atab ne);
		/* loop for each different external table */
		    %let table_type2 = %sysfunc(kscan(%quote(&atab.),2));
		    %let external_var=%mm_parsefields3(in=%quote(&dfr_ind_var_external.),tab=%sysfunc(kscan(%quote(&atab.),1)));
			%put &external_var.;

		    Data ext_data;
			    %if %sysfunc(kupcase(&table_type2.))=SAS7BDAT %then %do;
					set inp.%sysfunc(kscan(%quote(&atab.),1)) (keep=merch_id location_id time_id &external_var.);
				%end;
			    %else %do;
					%let external_file=%sysfunc(kscan(%quote(&atab.),1)).%sysfunc(kscan(%quote(&atab.),2));
					infile "&path_input/&external_file." dsd delimiter=',' missover;
					input merch_id location_id time_id &external_var.;
				%end;
				merch_level=&merch_to_level.;
				location_level=&loc_to_level.;
				time_level=&real_time_to_level.;
				cluster=0;
		    run;
			proc sort data=ext_data;
				by merch_id location_id time_id;
			run;

			/* append to extract file */
			data W_&wktid..extract;
				merge W_&wktid..extract (in=a) ext_data(in=b);
				by merch_id location_id time_id;
				if a;
			run;

			%let f = %eval(&f+1);
			%let atab = %sysfunc(kscan(%quote(&exttabs.),&f,%str(,)));
		%end;
	%end;

 	/*****************************************************/
    /* Aggregate store to cluster if to level is cluster */
    /*****************************************************/
    %if (&orig_loc_to_level.=1002) and (&job_rc.<5) %then %do;
        /* aggregate locations */
        proc sql noprint;
        create table W_&wktid..extract as select 
            %do k=1 %to &dfr_numkpis.;
                sum(%sysfunc(kscan(%quote(&h1_fcastkpi.),%eval(&k.*2)))) as %sysfunc(kscan(%quote(&h1_fcastkpi.),%eval(&k.*2))),
                %if &dfr_wf_revisit=1 %then %do;  /* add previous forecast and std fields to extract */
                    sum(f_%sysfunc(kscan(%quote(&f1_dfr_fcast_import_kpis.),%eval(&k.*2)))) as f_%sysfunc(kscan(%quote(&f1_dfr_fcast_import_kpis.),%eval(&k.*2))),
                    sum(s_%sysfunc(kscan(%quote(&f1_dfr_fcast_import_kpis_std.),%eval(&k.*2)))) as s_%sysfunc(kscan(%quote(&f1_dfr_fcast_import_kpis_std.),%eval(&k.*2))),
                    %end;
                %end;
            merch_id, merch_level, time_id, time_level, cluster as location_id, 2 as location_level, cluster
            from W_&wktid..extract 
			group by merch_id, merch_level, cluster, time_id, time_level
            order by merch_id, merch_level, cluster, time_id, time_level;

	delete from W_&wktid..extract where location_id not in (select CLSTR_SPC_ID from stage.cl_&projid.);
        quit;
        %end;
   %RCSet(&sqlrc.);

    /***********************************************/
    /*** Get all time periods and SAS Date *********/
    /***********************************************/
	%if (&job_rc.<5) %then %do;
	    proc sql noprint;
		create table W_&wktid..time_tab as select a.hist0_id as time_id, a.time_level,
			%if (&dbtype=db2) %then %do;
				lv&time_to_level.time_start_date as tname
				%end;
			%else %if &time_to_level.=4 %then %do;
		        datepart(lv&time_to_level.time_start_date)+6 as tname
		        %end;
		    %else %do;
		     	datepart(lv&time_to_level.time_start_date) as tname
		        %end;
			from W_&wktid..time_members a, maxapp.lv&time_to_level.time t
			where a.hist0_id=t.lv&time_to_level.time_lkup_id;
		quit;
		%end;

    /***********************************************/
    /*** Fill zeros and format extract     *********/
    /***********************************************/
    %if (&dfr_wf_dataprep.=1) and (&job_rc.<5) %then %do;
		proc sort data=W_&wktid..extract(keep=merch_id location_id merch_level location_level cluster) out=W_&wktid..mlkeys nodupkeys;
			by merch_id location_id;
		run;

        proc sql noprint;
        	create table W_&wktid..sixkeys as select m.merch_id, m.location_id, t.time_id, t.tname, merch_level, location_level, cluster, time_level 
				from W_&wktid..mlkeys m, W_&wktid..time_tab t
				order by merch_id, location_id, time_id;
        quit;
        %RCSet(&sqlrc.);

        /* merge data and keys to get fi dataset */
        data W_&wktid..extract (drop=tname);
            merge W_&wktid..extract W_&wktid..sixkeys;
            by merch_id location_id time_id;
	        rename
				%do k=1 %to &dfr_numkpis.;
		        	%sysfunc(KSCAN(%quote(&h1_fcastkpi.),%eval(&k.*2)))=%sysfunc(KSCAN(%quote(&dfr_kpinameset.),&k.))
		            %if &dfr_wf_revisit=1 %then %do;  /* add previous forecast and std fields to extract */
		                %sysfunc(KSCAN(%quote(&f1_dfr_fcast_import_kpis.),%eval(&k.*2)))=f_%sysfunc(KSCAN(%quote(&dfr_kpinameset.),&k.))
		                %sysfunc(KSCAN(%quote(&f1_dfr_fcast_import_kpis_std.),%eval(&k.*2)))=s_%sysfunc(KSCAN(%quote(&dfr_kpinameset.),&k.))
		                %end;
		        %end;
				/* DEM INDEPENDENT VARS 120208 */
				%if (&dfr_ind_var_mp. ne) %then %do;
					%let iv=1;
					%let indvar = %sysfunc(KSCAN(%quote(&dfr_ind_var_mp.),&iv.,%str(,)));
					%do %while (&indvar ne);	
						%sysfunc(KSCAN(%quote(&h1_dfr_ind_var_mp.),%eval(&iv.*2)))=%sysfunc(KSCAN(%quote(&dfr_ind_var_mp.),%eval(&iv.*2))) 
						%let iv=%eval(&iv.+1);
						%let indvar = %sysfunc(KSCAN(%quote(&dfr_ind_var_mp.),&iv.,%str(,)));
					%end;
				%end;				
				;

	        /* Set date for month, qtr, season, year to 1st of month always, leave week as is */
	        time_&time_to_level.=
	             %if (&time_to_level.=3) %then %do;
	                intnx( 'qtr',intnx( 'qtr', tname, -1),1);
					%end;
	            %if (&time_to_level.=4) %then %do;
	                mdy(month(tname),1,year(tname));
					%end;
	            %if (&time_to_level.=5) %then %do;
	                tname;
					%end;

	        format time_&time_to_level. date9.;
		run;
		%end;

/****************************************************************/
/******************* No Time Period Fill Case *******************/
/****************************************************************/
	%else %if (&job_rc.<5) %then %do;
		proc sql;
			update W_&wktid..extract a set cluster=(select tname from W_&wktid..time_tab t where a.time_id=t.time_id);
		quit;

	    data W_&wktid..extract (drop=cluster);
	        set W_&wktid..extract;
	        rename
				%do k=1 %to &dfr_numkpis.;
		        	%sysfunc(KSCAN(%quote(&h1_fcastkpi.),%eval(&k.*2)))=%sysfunc(KSCAN(%quote(&dfr_kpinameset.),&k.))
		            %if &dfr_wf_revisit=1 %then %do;  /* add previous forecast and std fields to extract */
		                %sysfunc(KSCAN(%quote(&f1_dfr_fcast_import_kpis.),%eval(&k.*2)))=f_%sysfunc(KSCAN(%quote(&dfr_kpinameset.),&k.))
		                %sysfunc(KSCAN(%quote(&f1_dfr_fcast_import_kpis_std.),%eval(&k.*2)))=s_%sysfunc(KSCAN(%quote(&dfr_kpinameset.),&k.))
		                %end;
		        %end;

				/* DEM INDEPENDENT VARS 120208 */
				%if (&dfr_ind_var_mp. ne) %then %do;
					%let iv=1;
					%let indvar = %sysfunc(KSCAN(%quote(&dfr_ind_var_mp.),&iv.,%str(,));
					%do %while (&indvar ne);	
						%sysfunc(KSCAN(%quote(&h1_dfr_ind_var_mp.),%eval(&iv.*2)))=%sysfunc(KSCAN(%quote(&dfr_ind_var_mp.),%eval(&iv.*2))) 
						%let iv=%eval(&iv.+1);
						%let indvar = %sysfunc(KSCAN(%quote(&dfr_ind_var_mp.),&iv.,%str(,)));
					%end;
				%end;				
				;

	        /* Set date for month, qtr, season, year to 1st of month always, leave week as is */
	        time_&time_to_level.=
	            %if (&time_to_level.=3) %then %do;
	                intnx( 'qtr',intnx( 'qtr', cluster, -1),1);
					%end;
	            %if (&time_to_level.=4) %then %do;
	                mdy(month(cluster),1,year(cluster));
					%end;
	            %else %do;
	                cluster;
					%end;

	        format time_&time_to_level. date9.;
	        run;
		%RCSet(&syserr);
		%end;

	%if (&job_rc.<5) /*and (&dfr_wf_dataprep.>1)*/ %then %do;
	    data W_&wktid..extract;
	        set W_&wktid..extract;
			%do k=1 %to &dfr_numkpis.;
            	if %sysfunc(KSCAN(%quote(&dfr_kpinameset.),&k.)) = . then %sysfunc(KSCAN(%quote(&dfr_kpinameset.),&k.)) = &dfr_zero_fill_extract;
	        %end;
			/* DEM INDEPENDENT VARS 120208 */
			%if (&dfr_ind_var_mp. ne) %then %do;
				%let iv=1;
				%let indvar = %sysfunc(KSCAN(%quote(&dfr_ind_var_mp.),&iv.,%str(,)));
				%do %while (&indvar ne);	
	            	if %sysfunc(KSCAN(%quote(&dfr_ind_var_mp.),%eval(&iv.*2))) = . then %sysfunc(KSCAN(%quote(&dfr_ind_var_mp.),%eval(&iv.*2))) = 0;
					%let iv=%eval(&iv.+1);
					%let indvar = %sysfunc(KSCAN(%quote(&dfr_ind_var_mp.),&iv.,%str(,)));
				%end;
			%end;				
		%end;

	/****************************************************************/
	/******************* Get Time Horizon Parameter *****************/
	/****************************************************************/
	%if (&job_rc.<5) %then %do;
	    /* set horizon */
	    proc sql noprint;
		    /* find horizon: how many periods to forecast from end of history to end of worksheet*/
		    select max(time_&time_to_level.) into :dfr_horizonstart from W_&wktid..extract;
			%let dfr_horizonstart=%sysfunc(KTRIM(&dfr_horizonstart.));
			%if &time_to_level.<5 %then %do; %let dfr_horizonstart=%eval(&dfr_horizonstart.+10); %end;
	        %let forecast_start_time=%sysfunc(putn(&dfr_horizonstart.,date9.));
			/* ensure forecast start is END Date of time period found for dfr_horizonstart. */
			select datepart(lv&time_to_level.time_end_date)+1 into :forecast_start from maxapp.lv&time_to_level.time 
				where "&forecast_start_time."d between lv&time_to_level.time_start_date and lv&time_to_level.time_end_date;
	        %let forecast_startdate=%sysfunc(putn(&forecast_start.,date9.));

		    /* note end of history may be before start of worksheet */
		    select count(*) into :eng_lead from maxapp.lv&time_to_level.time 
		        where (lv&time_to_level.time_start_date > "&forecast_startdate."d and lv&time_to_level.time_end_date < "&to_time."d)
		           or (lv&time_to_level.time_start_date = "&forecast_startdate."d) or (lv&time_to_level.time_end_date = "&to_time."d);
	    quit;
	    %let eng_lead=%sysfunc(KTRIM(&eng_lead.));
	    %RCSet(&sqlrc.);
	%end;

	/****************************************************************/
	/******************* ADD HIERARCHY            *******************/
	/****************************************************************/
	%if (&job_rc.<5) %then %do;
        %mm_extract12_hier(
            in=W_&wktid..extract,
			%if (&extract_optimized.=1) %then %do;  /* must append so put data in temp dataset instead of f_ final extract dataset */
				out=W_&wktid..hierdata,
				%let e_lib=W_&wktid.;
				%let e_dataset=hierdata;
				%end;
			%else %do;
				out=inp.f_&projid.,
				%let e_lib=inp;
				%let e_dataset=f_&projid.;
				%end;
			%if (&dfr_wf_optimized.=1) %then %do;  /* if optimized use only ids instead of names */
				keys=ids,
				%let keys=ids;
				%end;
			%else %do;
				keys=full,
				%let keys=full;
				%end;
            uid=&projid.,
            attribute_fields=%quote(),
			do_time=0,

            /* MERCHANDISE PARAMETERS */
            merch_from_level=&merch_from_level.,
            merch_to_level=&merch_to_level.,

            /* LOCATION PARAMETERS */
		    cluster_set_id=&dfr_clstr_st_id.,
            loc_from_level=&loc_from_level.,
            loc_to_level=&orig_loc_to_level.
            );
		%end;

	%if (&job_rc.<5) %then %do;
		/* Adjust for merchandise or location skipped levels */
		proc datasets
			library=&e_lib.;
			modify &e_dataset.;
			%if &m_lev. ne &merch_to_level. %then %do;
				rename mid_&merch_to_level.=mid_&m_lev.
				%if (&dfr_wf_optimized.=0) %then %do;
					mname_&merch_to_level.=mname_&m_lev.
					%end;
				;
				%end;
		    %if (&orig_loc_to_level.<1000) and (&job_rc.<5) %then %do;
				%if &l_lev. ne &loc_to_level. %then %do;
					rename lid_&loc_to_level.=lid_&l_lev.
					%if (&dfr_wf_optimized.=0) %then %do;
						lname_&loc_to_level.=lname_&l_lev.
						%end;
					;
					%end;
				%end;
		run;
	%end;

	/*****************************************************/
	/* if optimized is on append new extract to existing */
	/*****************************************************/
	%if (&extract_optimized.=1) and (&job_rc.<5) %then %do;
		proc append base=inp.f_&projid. data=W_&wktid..hierdata force;
		run;
	%end;
	%RCSet(&syserr);

	/*****************************************************/
	/* For month make sure align=BEGINNING option is set */
	/*****************************************************/
	%if (&time_to_level.=4 or &time_to_level.=3) and (&job_rc.<5) %then %do;
		%let DIAG_ID_OPTION=%subarg(astr=&DIAG_ID_OPTION.,sstr=align=,rstr=beginning);
		%let ENG_ID_OPTION=%subarg(astr=&ENG_ID_OPTION.,sstr=align=,rstr=beginning);
		%put &DIAG_ID_OPTION &ENG_ID_OPTION;
	%end;
	%EndMacro:
%mend dfr_extract;
 
/***************************************************************************************/
/* Append new product history to forecasted standard items before reconciling all data */
/***************************************************************************************/
%macro newprod_merge();
	/* get forecast time ids and related LY, LLY time ids for forecast time frame: from_time to to_time dates */
	%get_time_ids(timetype=Event,fromdate=&from_time.,todate=&to_time.,timetolevel=&time_to_level.,timefromlevel=&time_from_level.,calendar=&calendar.);
	 
	/* Convert new item history to forecast time-frame */
	proc sql noprint;
		/* drop all history not needed to match forecast time-frame */
		delete from stage.new_&projid. where time_id not in (select hist1_id from W_&wktid..time_members);
		 
		/* modify new product history to forecast time-frames */
		update stage.new_&projid. a set time_id= (select hist0_id from W_&wktid..time_members b where a.time_id=b.hist1_id);
		update stage.new_&projid. a set tname_&time_to_level. = (select lv&time_to_level.time_start_date from inp.lv&time_to_level.time_&projid. b where a.time_id=b.lv&time_to_level.time_lkup_id);
	quit;
	 
	/* loop for each KPI as there is a different URM dataset per level and KPI */
	/* Only lowest merch and location level dataset will get new product data for all KPIs*/
	%do k=1 %to &dfr_numkpis.;
		%let kpi=%sysfunc(KSCAN(&dfr_kpinameset.,&k.));
		%let srckpi=%sysfunc(KSCAN(%quote(&h1_fcastkpi.),%eval(&k.*2)));
		 
		/* create records similar to URM file outfor format: one row per KPI in _NAME_ column, mid, lid, actual, predict and time_5 (as SAS Date) fields */
		/*    need to include mname and lname level fields but no values in lower, upper, error, std fields */
		/*    history and forecasted values go into this dataset */
		proc sort data=stage.new_&projid.; by &sixkeys.; run;
		proc transpose data=stage.new_&projid. out=W_&wktid..new_items (drop=_LABEL_ rename=(col1=Predict)); by &sixkeys.; var &srckpi.; copy mname: lname: mid: lid: tname:; run;
		 
		data W_&wktid..new_items (keep=mid: mname: lid: lname: actual predict upper lower std error _NAME_ time_&time_to_level.);
			set W_&wktid..new_items;
			actual=.;
			time_period_rk=datepart(tname_&time_to_level.);
			time_&time_to_level.=time_period_rk;
			format time_&time_to_level. date9.;
			lower=predict;
			upper=predict;
			_NAME_="&kpi.";
			std=0;
			error=0;
		run;
		 
		/* append to pre-reconcile table for given KPI at lowest merch and location levels */
		proc sql noprint;
			select ds_name into :ds_name from dp.f_&projid._forecast_meta where 
				m_lvl=&merch_to_level. and l_lvl=&loc_to_level. and kpi="&kpi.";
		quit;

		proc append base=for.&ds_name. data=W_&wktid..new_items force;
		run;
	%end;
%mend newprod_merge;
