/**************************************************************/
/*                                                            */
/*       SAS RETAIL INTELLIGENCE SOLUTIONS                    */
/*       Demand Forecast For Retail Planning 4.2              */
/*                                                            */
/**************************************************************/
/*                                                            */
/* SAS Retail Group, All rights reserved (c) 2006,2007        */
/*                                                            */
/**************************************************************/
 
/****************************************************************************************************/
/*Functionality: This macro filters time_series at the level and KPI that don't have enough history */
/****************************************************************************************************/
 
%macro dfr_filter();
 
/**************************************************************************/
/*********** FILTER TIME-SERIES                                      ******/
/**************************************************************************/
 
/* Filter eliminates series without dfr_filter_prior_hist / dfr_filter_prior_periods % data
where prior hist is # periods of history within # prior weeks
prior periods is from the last loaded week backward in time
Filter periods is always expressed in # of weeks
 
This eliminates old products with no recent sales, negative sales only, or new products without enough history to forecast
 
Code counts # of >0 periods within dfr_filter_prior_periods total prior periods
if # of periods < dfr_filter_prior_hist then series is eliminated
*/
 
%if (&job_rc<5) and (&dfr_filter_prior_hist.>0) and (&dfr_filter_prior_periods.>0) and (&dfr_filter_start_date. ne) and (&dfr_filter_end_date. ne) %then %do;
	/* Get # of datasets to filter */
	data _null_;
		set Dp.F_&proj._meta end=L;
		if L then
			call symput('row_no',_n_);
	run;
	%RCSet (&syserr);
	data W_&wktid..F_&proj._meta W_&wktid..F_&proj._meta_copy;
		set dp.F_&proj._meta ;
	run;

	/* Following code gets executed for every row in metadata table - each merch/loc/kpi valid combination */
		%do frow=1 %to &row_no; /*Start of each row execution*/
			%getmetabyrow(proj=&proj.,row=&frow);
			data W_&wktid..counts(keep=group_key mid: lid: count);
				set stage.&dataname(where=(time_&time_to_level. between "&dfr_filter_start_date."d and "&dfr_filter_end_date."d and &kpi. > &dfr_zero_fill_filter));
				by  &&UNRD.ID: &&RD.ID:;
				retain count;
				if first.&&RD.ID_&&&FULL_RECON_DIM._level then count=0;
				count=count+1;
				if last.&&RD.ID_&&&FULL_RECON_DIM._level then output;
			run;

			%RCSet (&syserr);

			data stage.filter_&dataname stage.&dataname;
				merge stage.&dataname W_&wktid..counts;
				by &&UNRD.ID: &&RD.ID:;
				if count<=&dfr_filter_prior_hist. then
				output stage.filter_&dataname;
				else
				output stage.&dataname;
				drop count;
			run;
			%RCSet (&syserr);
			%let nobs=0;
			%noobs(ds=stage.&dataname);
			%if &nobs = 0 %then %do;
				data W_&wktid..F_&proj._meta_copy;
					set W_&wktid..F_&proj._meta_copy;
					if ("&dataname." eq ds_name) then delete;
				run;
			%end;

		%end;
    %let nobs=0;
	%noobs(ds=W_&wktid..F_&proj._meta_copy);
	%if &nobs ne 0 %then %do;
		data dp.F_&proj._meta;
			set W_&wktid..F_&proj._meta_copy ;
		run;
	%end;
	%else %do;
		%put %sysfunc(sasmsg(catalog.dfrmsg, dfr_data_no_obs, noquote));
		%let job_rc=10;
	%end;

%end;
/*Code to delete intermediate datasets(WORK)*/
proc datasets library=W_&proj. kill nodetails nolist;
quit;
%RCSet (&syserr);
%mend;
 
