/*-------------------------------------------------------------------------------------------------------
    Demand Forecasting for Retail
    Descriptions    : Primary Aggregation preperation 
    Developer       : Paul Dettorre, X8265
    Created         : 11/2008
    Modification Log:

    Parameters      
        VNEWTABNAME            : 
        TMP_TBL_NAME           : 
        MMX_TBL_NAME           : 
        FACT_TYPE              : 
        FCAST_VERSION          : 
        AGG_TO_TOP_MERCH_LEVEL : 
        AGG_FROM_MERCH_LEVEL   : 
        AGG_TO_TOP_LOC_LEVEL   : 
        AGG_FROM_LOC_LEVEL     : 
        AGG_TO_TOP_TIME_LEVEL  : 
        AGG_FROM_TIME_LEVEL    : 
        AGG_MERCH              : 
        AGG_LOC                : 
        AGG_TIME               : 
        PRESERVE_ML            : 
        PRESERVE_TIME          : 
        MERCH_TAB_NAME         : 
        LOC_TAB_NAME           : 
        CLSTR_ST_ID            : 
        STATUS=                : 

    Process Flow
        1) 
-------------------------------------------------------------------------------------------------------*/
%macro p_agg(vNewTabName=,
             TMP_TBL_NAME=,
             MMX_TBL_NAME=,
             FACT_TYPE=,
             FCAST_VERSION=,
             AGG_TO_TOP_MERCH_LEVEL=,
             AGG_FROM_MERCH_LEVEL=,
             AGG_TO_TOP_LOC_LEVEL=,
             AGG_FROM_LOC_LEVEL=,
             AGG_TO_TOP_TIME_LEVEL=,
             AGG_FROM_TIME_LEVEL=,
             AGG_MERCH=,
             AGG_LOC=,
             AGG_TIME=,
             PRESERVE_ML=,
             PRESERVE_TIME=,
             MERCH_TAB_NAME=,
             LOC_TAB_NAME=,
             CLSTR_ST_ID=,
             STATUS=);

    /*
        Preserve_ML indicates to create a table with all Merch & Loc key from the forecast used to compare to
        the target worksheet.
    */
    %IF (&PRESERVE_ML eq Y) %THEN
        %DO;
            %Let vMLCoord=TMP_ML_&vNewTabName.;
            proc datasets library=MAXDATA nolist;
                delete &vMLCoord.;
                delete &MERCH_TAB_NAME.;
                delete &LOC_TAB_NAME.;
            Quit;

            proc copy in=out out=maxdata memtype=data;
                select &MERCH_TAB_NAME. &LOC_TAB_NAME.;
            run;

            PROC SQL;
                CONNECT TO &dbtype. (USER=MAXDATA ORAPW=&dbpassword. PATH=&DSN.);
                EXECUTE(Create Table &vMLCoord. parallel 10 nologging pctfree 0 as 
                        Select merch_level, merch_id, location_level, location_Id
                          from &MERCH_TAB_NAME. , &LOC_TAB_NAME.
                        %IF (&Agg_from_loc_level. gt 1000 or &Agg_to_top_loc_level. > 1000) %THEN
                            %DO;
                                Where location_level <> 4 
                            %END;
                )BY &dbtype.;
                DISCONNECT FROM &dbtype.;
                %RDBMS_Report(macro_call=DFR_IMPORT_P_AGG_P_AGG,code_area=1,switch=ON);
            QUIT;
        %END;

    /*  Use to store a all the partition or subpartition names */
    %Let vPartTable=PART_&vNewTabName.;
    proc datasets library=MAXDATA nolist;
        delete &vPartTable.;
    Quit;

    PROC SQL;
        CONNECT TO &dbtype. (USER=MAXDATA ORAPW=&dbpassword. PATH=&DSN.);
        EXECUTE(CREATE TABLE &VPARTTABLE. (PART_SUB_NAME VARCHAR2(30)) ) BY &dbtype.;
    QUIT;
    %RDBMS_Report(macro_call=DFR_IMPORT_P_AGG,code_area=2,switch=ON);

    %If (&Fact_Type. eq F) %Then
        %Do;
            PROC SQL;
                CONNECT TO &dbtype. (USER=MAXDATA ORAPW=&dbpassword. PATH=&DSN.);
                EXECUTE(Insert into &vPartTable. 
                        SELECT SUBPARTITION_NAME
                          FROM ALL_TAB_SUBPARTITIONS
                         WHERE TABLE_NAME = %STR(%'&VNEWTABNAME.%')
                           AND SUBSTR(SUBPARTITION_NAME ,  -LENGTH(&FCAST_VERSION.) ) = TO_CHAR(&FCAST_VERSION.)
                      ORDER BY SUBPARTITION_NAME DESC
                       ) BY &dbtype.;
                DISCONNECT FROM &dbtype.;
            QUIT;
            %RDBMS_Report(macro_call=DFR_IMPORT_P_AGG,code_area=3A,switch=ON);

            %Let vPType=SubPartition;
        %End;
    %Else
        %Do;
            PROC SQL;
                CONNECT TO &dbtype. (USER=MAXDATA ORAPW=&dbpassword. PATH=&DSN.);
                EXECUTE(Insert into &vPartTable. 
                        SELECT PARTITION_NAME
                          FROM ALL_TAB_PARTITIONS
                         WHERE TABLE_NAME = %STR(%'&VNEWTABNAME.%')
                      ORDER BY PARTITION_NAME DESC
                       ) BY &dbtype.;
                DISCONNECT FROM &dbtype.;
            QUIT;
            %RDBMS_Report(macro_call=DFR_IMPORT_P_AGG,code_area=3B,switch=ON);

            %Let vPType=Partition;
        %End;

    proc sql;
        create table work.Part_Sub_Table_temp as
        Select Part_Sub_Name
             , 0 as RecordCount 
          From MAXDATA.&vPartTable.
        %If (&vPType. eq SubPartition) %then
            %Do;
                Where Part_Sub_Name in (select SUBPARTITION_NAME from partition_list)
            %End;
        %Else
            %Do;
                Where Part_Sub_Name in (select PARTITION_NAME from partition_list)
            %End;
        ;
    quit;

    data work.Part_Sub_Table;
        set Part_Sub_Table_temp;
    run;

    /*
        Loop through each entry in the partition list table
        For each partition, aggregate data
    */

    %let pst_id=%sysfunc(open(work.Part_Sub_Table_temp,i));
    %let fetch_rec=%sysfunc(fetch(&pst_id.));
    %do %while(&fetch_rec eq 0);
        %let vSubPartName=%sysfunc(getvarc(&pst_id.,%sysfunc(varnum(&pst_id.,Part_Sub_Name)))) ;

        %PUT ============ PROCESSING %sysfunc(kupcase(&vPType.)) &vSubPartName. ============================;

        PROC SQL NOPRINT;
            CONNECT TO &dbtype. (USER=MAXDATA ORAPW=&dbpassword. PATH=&DSN.);
            select iCtr into :iCtr
              from connection to &dbtype.(
                Select count(*) as iCtr 
                  From &vNewTabName. &vPtype. (&vSubPartName.)
                where rownum=1)
            ;
            DISCONNECT FROM &dbtype.;
        QUIT;
        %RDBMS_Report(macro_call=DFR_IMPORT_P_AGG,code_area=4 (do loop),switch=ON);
        %Let iCtr=&iCtr.;


        /*  Keep tract of partitions that contain data */
        proc sql;
            update work.Part_Sub_Table
               set RecordCount = &iCtr.
             where Part_Sub_Name = "&vSubPartName."
            ;
        quit;

        /*  If data exists in a partition then process it.*/
        %If (&iCtr. gt 0) %Then
            %Do;
                %PUT NOTE: &iCtr. Records found in table &vNewTabName. &vPtype. (&vSubPartName.);

                %Let vTabName=TMP_&vSubPartName.;
                proc datasets library=MAXDATA nolist;
                    delete &vTabName.;
                Quit;
                PROC SQL;
                    CONNECT TO &dbtype. (USER=MAXDATA ORAPW=&dbpassword. PATH=&DSN.);
                    EXECUTE(Create Table &vTabName. Parallel 10 nologging as 
                            select * from &vNewTabName &vPType (&vSubPartName)
                           )BY &dbtype.;
                    DISCONNECT FROM &dbtype.;
                QUIT;
                %RDBMS_Report(macro_call=DFR_IMPORT_P_AGG,code_area=5,switch=ON);

                %If (&agg_merch. eq Y) %Then
                    %Do;
                        %p_merch_agg(vmmx_tbl_name=&MMX_TBL_NAME.,
                                     vTabName=&vTabName.,
                                     iAgg_to_top_merch_level=&Agg_to_top_merch_level.,
                                     iAgg_from_merch_level=&Agg_from_merch_level.,
                                     iForecastVersion=&FCAST_VERSION.);
                    %End;

                %If (&agg_loc. eq Y) %Then
                    %Do;
                        %p_loc_agg(vmmx_tbl_name=&MMX_TBL_NAME.,
                                   vTabName=&vTabName.,
                                   iClstrStId=&CLSTR_ST_ID.,
                                   iAgg_to_top_loc_level=&Agg_to_top_loc_level.,
                                   iAgg_from_loc_level=&Agg_from_loc_level.,
                                   iForecastVersion=&FCAST_VERSION.);
                    %End;

                %If (&preserve_ml. eq Y) %Then
                    %Do;
                        %Let vIntMLCoord=TMP_INT_ML_&vNewTabName.;
                        proc datasets library=MAXDATA nolist;
                            delete &vIntMLCoord.;
                        Quit;

                        PROC SQL;
                            CONNECT TO &dbtype.(USER=MAXDATA ORAPW=&dbpassword. PATH=&DSN.);
                            EXECUTE(create table &vIntMLCoord Parallel 10 nologging pctfree 0 as
                                    select * 
                                      from ( Select merch_level, merch_id, location_level, location_id
                                               from &mmx_tbl_name. &vPType (&vSubPartName.)
                                              minus
                                             Select merch_level, merch_id, location_level, location_id
                                               from &VMLCOORD)
                                   )BY &dbtype.;
                            DISCONNECT FROM &dbtype.;
                        QUIT;
                        %RDBMS_Report(macro_call=DFR_IMPORT_P_AGG,code_area=6,switch=ON);

                        PROC SQL;
                            Connect to &dbtype.(User=maxdata Orapw=&dbpassword. path=&DSN.);
                            EXECUTE(insert into &vTabName. 
                                    Select a.*
                                      From &mmx_tbl_name &vPtype (&vSubPartName.) a,
                                           &vIntMLCoord b
                                     Where a.merch_level = b.merch_level 
                                       and a.merch_id = b.merch_id
                                       and a.location_level = b.location_level 
                                       and a.location_id = b.location_id
                                   )BY &dbtype.;
                            DISCONNECT FROM &dbtype.;
                        QUIT; 
                        %RDBMS_Report(macro_call=DFR_IMPORT_P_AGG,code_area=7,switch=ON);

                        proc datasets library=MAXDATA nolist;
                            delete &vIntMLCoord.;
                        Quit;

                    %End;

                %If (&Fact_Type eq F) %Then
                    %Do;
                        %Let Partition_Type=S;
                    %End;
                %Else
                    %Do;
                        %Let Partition_Type=P;
                    %End;
        
                %p_trunc_table(vTabName=&mmx_tbl_name.,vObjectType=&Partition_type.,vObjectName=&vSubPartName.,OraPath=&DSN.,OraUser=MAXDATA,OraPW=&dbpassword.);
                %RDBMS_Report(macro_call=DFR_IMPORT_P_AGG,code_area=8,switch=ON);

                PROC SQL;
                    CONNECT TO &dbtype.(USER=MAXDATA ORAPW=&dbpassword. PATH=&DSN.);
                    EXECUTE(Insert into &mmx_tbl_name. &vPType. (&vSubPartName.)
                            Select * from &vTabName.) BY &dbtype.;
                    DISCONNECT FROM &dbtype.;
                QUIT;
                %RDBMS_Report(macro_call=DFR_IMPORT_P_AGG,code_area=9,switch=ON);

                proc datasets library=MAXDATA nolist;
                    delete &vTabName.;
                Quit;

            %End;%* END (&iCtr gt 0);
        %Else
            %Do;
                %PUT WARNING: No found in table &vNewTabName. &vPtype. (&vSubPartName.);
            %End;
        %let fetch_rec=%sysfunc(fetch(&pst_id.));
    %End;%* End LOOP;
    %Let Close_Rc=%sysfunc(close(&pst_id.));

    /*-------------------------------------------------------------------------------------
        Process Time Aggregation


    --------------------------------------------------------------------------------------*/

    /*   Put Records to aggregate into a temp table */

    %If (&preserve_time. eq N) %Then
        %Do;

            /*  Collect a master list of partitions/Subpartitions */
            %If (&Fact_Type eq F) %Then
                %Do;
                    PROC SQL;
                        CONNECT TO &dbtype.(USER=MAXDATA ORAPW=&dbpassword. PATH=&DSN.);
                        create table work.Master_Partition_Name as
                            Select * from connection to &dbtype.(
                                Select subpartition_name as partition_name
                                  from all_tab_subpartitions 
                                 where table_name = %STR(%'&mmx_tbl_name.%')
                                   and substr(subpartition_name ,  -length(&Fcast_Version.) ) = to_char(&Fcast_Version.) 
                              order by subpartition_name desc
                        );
                        DISCONNECT FROM &dbtype.;
                    QUIT;
                    %RDBMS_Report(macro_call=DFR_IMPORT_P_AGG,code_area=10A,switch=ON);

                %End;
            %Else
                %Do;
                    PROC SQL;
                        CONNECT TO &dbtype.(USER=MAXDATA ORAPW=&dbpassword. PATH=&DSN.);
                        create table work.Master_Partition_Name as
                            Select * from connection to &dbtype.(
                                Select partition_name 
                                  from all_tab_partitions 
                                 where table_name = %STR(%'&mmx_tbl_name.%')
                              order by partition_name desc
                        );
                        DISCONNECT FROM &dbtype.;
                    QUIT;
                    %RDBMS_Report(macro_call=DFR_IMPORT_P_AGG,code_area=10B,switch=ON);

                %End;

            /*
                Here we only want to delete all partitions from the target FACT table where there was no 
                new data inserted from Merch or Loc processing
            */

            proc sql;
                create table filter_time_levels as
                select partition_name
                  from work.Master_Partition_Name
                 where partition_name in (select Part_Sub_Name from work.Part_Sub_Table where recordcount = 0)
                ;
            quit;

            %let c1=%sysfunc(open(work.filter_time_levels,i));
            %let fetch_rec=%sysfunc(fetch(&c1.));
            %Do %while(&fetch_rec eq 0);
                %let vSubPartName=%sysfunc(getvarc(&c1.,%sysfunc(varnum(&c1.,partition_name)))) ;
                %If (&Fact_Type eq F) %Then
                    %Do;
                        %Let Partition_type_2=S;
                    %End;
                %Else
                    %Do;
                        %Let Partition_type_2=P;
                    %End;

                %p_trunc_table(vTabName=&mmx_tbl_name.,vObjectType=&Partition_type_2.,vObjectName=&vSubPartName.,OraPath=&DSN.,OraUser=MAXDATA,OraPW=&dbpassword.);

                %RDBMS_Report(macro_call=DFR_IMPORT_P_AGG,code_area=11,switch=ON);
                %let fetch_rec=%sysfunc(fetch(&c1.));
            %End;%* End Loop;
            %Let Close_Rc=%sysfunc(close(&c1.));
        %End; %* End(&preserve_time. eq N);

    %If &Agg_Time eq Y %Then
        %Do;
            PROC SQL;
                create table work.path_seg as 
                select *
                  from maxdata.path_seg
                 where path_id = 50
                   and lowerlevel_id <= &AGG_TO_TOP_TIME_LEVEL
                   and higherlevel_id >= &Agg_from_time_level
              order by lowerlevel_id desc
                ;
            QUIT;

            %let open_path_seg=%sysfunc(open(work.path_seg,i));
            %let fetch_rec_rc=%sysfunc(fetch(&open_path_seg));
            %do %while(&fetch_rec_rc. eq 0);
                %let higherlevel_id=%sysfunc(getvarn(&open_path_seg,%sysfunc(varnum(&open_path_seg,HIGHERLEVEL_ID))));
                %let lowerlevel_id=%sysfunc(getvarn(&open_path_seg,%sysfunc(varnum(&open_path_seg,LOWERLEVEL_ID))));
            
                %p_trunc_table(vTabName=&vPartTable.,vObjectType=T,vObjectName=,OraPath=&DSN.,OraUser=MAXDATA,OraPW=&dbpassword.);

                PROC SQL;
                    Connect to &dbtype.(User=maxdata Orapw=&dbpassword. path=&DSN.);
                %If &Fact_Type eq F %Then
                    %Do;
                        EXECUTE(Insert into &vPartTable
                                Select subpartition_name 
                                  from all_tab_subpartitions 
                                 where table_name = %str(%'&vNewTabName%')
                                   and substr(subpartition_name ,  -length(&Fcast_Version.) ) = to_char(&Fcast_Version.) 
                                   and substr(subpartition_name,1,length(%str(%'MFVER_&higherlevel_id._%'))) = %str(%'MFVER_&higherlevel_id._%')
                                   and subpartition_name not like %NRSTR('%DEFAULT%')
                              order by subpartition_name desc
                               )BY &dbtype.;
                        EXECUTE(commit)BY &dbtype.;
                    %End;
                %Else
                    %Do;
                        EXECUTE(Insert into &vPartTable
                                Select partition_name 
                                  from all_tab_partitions 
                                 where table_name = %str(%'&vNewTabName%')
                                   and substr(partition_name,1,length(%str(%'MFVER_&higherlevel_id._%'))) = %str(%'MFVER_&higherlevel_id._%')
                                   and partition_name not like %NRSTR('%DEFAULT%')
                              order by partition_name desc
                               )BY &dbtype.;
                    %End;
                    EXECUTE(commit)BY &dbtype.;
                    %RDBMS_Report(macro_call=DFR_IMPORT_P_AGG,code_area=12,switch=ON);

                    DISCONNECT FROM &dbtype.;
                QUIT;
                
                %Let Default_String=%DEFAULT%;

                %* INNER LOOP;
                %let open_vPartTable=%sysfunc(open(maxdata.&vPartTable.,i));
                %let fetch_rec_rc2=%sysfunc(fetch(&open_vPartTable.));
                %do %while(&fetch_rec_rc2. eq 0);
                    %let vSubPartName=%sysfunc(getvarc(&open_vPartTable,%sysfunc(varnum(&open_vPartTable,Part_sub_Name))));

                    %let underscore_loc=%sysfunc(kindex(&vSubPartName,_));
                    %Let vPartString=%sysfunc(ksubstr(&vSubPartName.,1,%Eval(&underscore_loc. - 1)));

                    %If &Fact_Type eq F %Then
                        %Do;
                            %p_trunc_table(vTabName=&MMX_TBL_NAME.,vObjectType=S,vObjectName=&vSubPartName.,OraPath=&DSN.,OraUser=MAXDATA,OraPW=&dbpassword.);
                        %End;
                    %Else
                        %Do;
                            %p_trunc_table(vTabName=&MMX_TBL_NAME.,vObjectType=P,vObjectName=&vSubPartName.,OraPath=&DSN.,OraUser=MAXDATA,OraPW=&dbpassword.);
                        %End;

                    %If &Fact_Type eq F %then
                        %Do;
                            PROC SQL NOPRINT;
                                Connect to &dbtype.(User=maxdata Orapw=&dbpassword. path=&DSN.);
                                SELECT iTimeId into :iTimeId
                                  FROM CONNECTION TO &dbtype.(Select substr(partition_name,(length(%STR(%'&vPartString.%')) + 5),20) as iTimeId
                                          from all_tab_subpartitions
                                         where table_name = %STR(%'&vNewTabName.%')
                                           and subpartition_name = %STR(%'&vSubPartName.%')
                                           and subpartition_name NOT LIKE '%DEFAULT%'
                                       );
                                DISCONNECT FROM &dbtype.;
                            QUIT;
                            %RDBMS_Report(macro_call=DFR_IMPORT_P_AGG,code_area=13,switch=ON);
                            %Let iTimeId=&iTimeId.;
                        %End;
                    %Else
                        %Do;
                            PROC SQL NOPRINT;
                                Connect to &dbtype.(User=maxdata Orapw=&dbpassword. path=&DSN.);
                                SELECT iTimeId into :iTimeId
                                  FROM CONNECTION TO &dbtype.(Select substr(partition_name,(length(%STR(%'&vPartString.%')) + 5),20) as iTimeId
                                          from all_tab_partitions
                                         where table_name = %STR(%'&vNewTabName.%')
                                           and partition_name = %STR(%'&vSubPartName.%')
                                           and partition_name NOT LIKE '%DEFAULT%'
                                       );
                                DISCONNECT FROM &dbtype.;
                            QUIT;
                            %RDBMS_Report(macro_call=DFR_IMPORT_P_AGG,code_area=13,switch=ON);
                            %Let iTimeId=&iTimeId.;
                        %End;

                    %If ((&iTimeId. ne ) and (%sysfunc(KSCAN(&vSubPartName.,3,'_')) NE DEFAULT)) %then
                        %Do;

                            %p_time_agg(vmmx_tbl_name=&MMX_TBL_NAME.,
                                       iHighTimeLevel=&higherlevel_id.,
                                       iHighTimeId=&iTimeId.,
                                       iLowTimeLevel=&lowerlevel_id.,
                                       iForecastVersion=&FCAST_VERSION.,
                                       vSubPartition_name=&vSubPartName.,
                                       vType=&FACT_TYPE.);
                        %End;
                    %let fetch_rec_rc2=%sysfunc(fetch(&open_vPartTable.));
                %End;%* End Inner Loop;
                %Let Close_Rc=%sysfunc(close(&open_vPartTable.));

                %let fetch_rec_rc=%sysfunc(fetch(&open_path_seg));
            %End; %*End Outer Loop;
            %Let Close_Rc=%sysfunc(close(&open_path_seg.));
        %End; %* end Agg_Time eq Y;
%mend;

/*-------------------------------------------------------------------------------------------------------
    Demand Forecasting for Retail
    Descriptions    : Roll up data by Time hierarchy 
    Developer       : Paul Dettorre, X8265
    Created         : 11/2008
    Modification Log:
    Parameters      
        vmmx_tbl_name           : Defines whether to process as a fact table or forecast table
        vTabName                : The target fact/fcast table name. Used as a definition template to build the temp
                                   table for aggregation processing.
        iAgg_to_top_merch_level : Lowest level of the hierarchy
        iAgg_from_merch_level   : Highest level of the hierarchy
        iForecastVersion        : iFvVersionId

    Process Flow
        1) Create Select & Group By Strings
        2) Execute sql based on time partitions/subpartitions
-------------------------------------------------------------------------------------------------------*/
%MACRO p_time_agg(vmmx_tbl_name=,
                  iHighTimeLevel=,
                  iHighTimeId=,
                  iLowTimeLevel=,
                  iForecastVersion=,
                  vSubPartition_name=,
                  vType=);

    %LOCAL iLowTimeLevel_Adj vLowTimeColName vLowTimeTabName vHighTimeColName iFvVersion
           vColString vAggString;

    %Let iLowTimeLevel_Adj=%EVAL(&iLowTimeLevel - 46);
    %Let iHighTimeLevel_adj=%EVAL(&iHighTimeLevel - 46);

    %Let vLowTimeColName  =LV&iLowTimeLevel_adj.TIME_LKUP_ID;
    %Let vLowTimeTabName  =LV&iLowTimeLevel_adj.TIME;
    %Let vHighTimeColName =LV&iHighTimeLevel_adj.TIME_LKUP_ID;

    %let iFvVersion=%EVAL(&iForecastVersion + 0);

    /*  
        Build the Select & Group By Strings
    */
    %If (&iFvVersion eq 0) %then
        %Do;
            %let vColString=%STR(Merch_level, merch_id, location_level, location_id, time_level, time_id);
            %let vAggString=%STR(a.Merch_level, a.Merch_id,  a.location_level, a.location_id,&iHighTimeLevel.,&iHighTimeId.);
        %End;
    %Else
        %Do;
            %let vColString=%STR(fv_version_id, Merch_level, merch_id, location_level, location_id, time_level, time_id);
            %Let vAggString=%STR(&iForecastVersion.,  a.Merch_level, a.Merch_id,  a.location_level, a.location_id, &iHighTimeLevel. , &iHighTimeId);
        %End;

    /*
        Add onto the SELECT string the KPI's and their corresponding aggregations

            MMX_TBL_NAME    MMX_COL_NAME    MERCH_AGG_RULE  LOC_AGG_RULE    TIME_AGG_RULE
        1   MFACT_VERSION   FC_SALES_R      SUM             SUM             SUM
        2   MFACT_VERSION   FC_SALES_U      SUM             SUM             SUM

    */

    data _null_;
        set MAXDATA.dfr_fact_tables(where=(kupcase(mmx_tbl_name) = kupcase("&vmmx_tbl_name."))) end=eof;
        format vColString   $1024.;
        format vAggString   $1024.;
        
        retain vColString vAggString;

        if (_n_ eq 1) then
            do;
                vColString="&vColString.";
                vAggString="&vAggString.";
            end;

        vColString = kcompress(vColString||', '||mmx_col_name);
        vAggString = kcompress(vAggString||', '||time_agg_rule||'(a.'||mmx_col_name||')');
        if (eof eq 1) then
            do;
                call symputx('vColString',vColString,'l');
                call symputx('vAggString',vAggString,'l');
            end;
    run;

    %If (&iForecastVersion = ) %Then %Let iForecastVersion=NULL;

    /*  
        Aggregate the data. Unlike Merch & Loc, need to load aggregated data by time partitions
        or subpartitions

    */
    PROC SQL;
        CONNECT TO &dbtype.(USER=MAXDATA ORAPW=&dbpassword. PATH=&DSN.);
        %If (&vType eq F) %Then
            %Do;
                EXECUTE(INSERT INTO &vmmx_tbl_name. SUBPARTITION (&vSubpartition_name.) (&vColString.)
                SELECT &vAggString. 
                  FROM &vmmx_tbl_name. a, MAXAPP.&vLowTimeTabName. b
                 WHERE a.time_level = &iLowTimeLevel
                   AND a.time_id = b.&vLowTimeColName.
                   AND b.&vHighTimeColName. = &iHighTimeId.
                   AND a.fv_version_id = &iForecastVersion.
              GROUP BY a.merch_level, a.Merch_id,  a.location_level, a.location_id)
              BY &dbtype.;
            %End;
        %Else
            %Do;
                EXECUTE(INSERT INTO &vmmx_tbl_name. PARTITION (&vSubpartition_name.) (&vColString.)
                SELECT &vAggString. 
                  FROM &vmmx_tbl_name. a, MAXAPP.&vLowTimeTabName. b
                 WHERE a.time_level = &iLowTimeLevel
                   AND a.time_id = b.&vLowTimeColName.
                   AND b.&vHighTimeColName. = &iHighTimeId.
              GROUP BY a.merch_level, a.Merch_id,  a.location_level, a.location_id)
              BY &dbtype.;
            %End;
        DISCONNECT FROM &dbtype.;
    QUIT;
%Mend;

/*-------------------------------------------------------------------------------------------------------
    Demand Forecasting for Retail
    Descriptions    : Roll up data by Merchandise hierarchy 
    Developer       : Paul Dettorre, X8265
    Created         : 11/2008
    Modification Log:
    Parameters      
        vmmx_tbl_name           : Defines whether to process as a fact table or forecast table
        vTabName                : The target fact/fcast table name. Used as a definition template to build the temp
                                   table for aggregation processing.
        iAgg_to_top_merch_level : Lowest level of the hierarchy
        iAgg_from_merch_level   : Highest level of the hierarchy
        iForecastVersion        : iFvVersionId

    Process Flow
        1) Grab data from MAXDATA.path_seg to get full path through hierarchy
        2) For each level loop through ad attach MP keys to the final table roll up
           by the keys and INSERT.
-------------------------------------------------------------------------------------------------------*/

%macro p_merch_agg(vmmx_tbl_name=,
                   vTabName=,
                   iAgg_to_top_merch_level=,
                   iAgg_from_merch_level=,
                   iForecastVersion=);

    /*  If iForecastVersion is null, set it to 0 */
    %Let iFvVersion=%EVAL(&iForecastVersion. + 0);

    /*
        Build a list of hierarchy ids
    */
    proc sql;
        create table work.path_seg as
        select *
          from MAXDATA.path_seg
         where path_id = 11
           and higherLevel_id >= (10 + &iAgg_from_merch_level.)
           and lowerlevel_id <= (10 + &iAgg_to_top_merch_level.)
      order by lowerlevel_id descending
        ;
    quit;

    /*
        Loop through the list of hierarchy levels and roll up data
    */
    %let c1=%sysfunc(open(work.PATH_SEG,i));
    %let fetch_rec=%sysfunc(fetch(&c1.));
    %do %while(&fetch_rec eq 0);
        %let lowerlevel_id=%sysfunc(getvarn(&c1.,%sysfunc(varnum(&c1.,LOWERLEVEL_ID)))) ;
        %let lowerlevel_id_adj=%EVAL(&lowerlevel_id. - 10);

        %let higherlevel_id=%sysfunc(getvarn(&c1.,%sysfunc(varnum(&c1.,HIGHERLEVEL_ID))));
        %Let higherlevel_id_adj=%Eval(&higherlevel_id. - 10);

        %let vLowMerchColName=LV&lowerlevel_id_adj.CTREE_ID;
        %let vLowMerchTabName=LV&lowerlevel_id_adj.CTREE;

        /*  Use the primary MAST table for the first level of merchandise. Because there is no
            CTREE table for this level
        */
        %If &higherlevel_id eq 11 %then
            %Do;
                %Let vHighMerchColName=LV1CMAST_ID;
            %End;
        %Else
            %Do;

                %Let vHighMerchColName=LV&higherlevel_id_adj.CTREE_ID;
            %End;

        /*  
            Build the Select & Group By Strings
        */
        %If (&iFvVersion eq 0) %then
            %Do;
                %Let vColString=Merch_level, merch_id, location_level, location_id, time_level, time_id;
                %Let vAggString=&higherlevel_id_adj , &vHighMerchColName. , a.location_level, a.location_id, a.time_level, a.time_id;
            %End;
        %Else
            %Do;
                %Let vColString=fv_version_id, Merch_level, merch_id, location_level, location_id, time_level, time_id;
                %Let vAggString=&iForecastVersion , &higherlevel_id_adj , &vHighMerchColName. , a.location_level, a.location_id, a.time_level, a.time_id;
            %End;

        /*
            Add onto the SELECT string the KPI's and their corresponding aggregations

                MMX_TBL_NAME    MMX_COL_NAME    MERCH_AGG_RULE  LOC_AGG_RULE    TIME_AGG_RULE
            1   MFACT_VERSION   FC_SALES_R      SUM             SUM             SUM
            2   MFACT_VERSION   FC_SALES_U      SUM             SUM             SUM

        */
        data _null_;
            set MAXDATA.dfr_fact_tables(where=(kupcase(mmx_tbl_name) = kupcase("&vmmx_tbl_name."))) end=eof;
            format vColString   $1024.;
            format vAggString   $1024.;
            retain vColString vAggString;

            if (_n_ eq 1) then
                do;
                    vColString="&vColString.";
                    vAggString="&vAggString.";
                end;
            vColString = kcompress(vColString||', '||mmx_col_name);
            vAggString = kcompress(vAggString||', '||merch_agg_rule||'(a.'||mmx_col_name||')');

            if (eof eq 1) then
                do;
                    call symputx('vColString_Str',compbl(vColString));
                    call symputx('vAggString_Str',compbl(vAggString));
                end;
        run;

        /*  Aggregate the data */
        PROC SQL;
            CONNECT TO &dbtype.(USER=MAXDATA ORAPW=&dbpassword. PATH=&DSN.);
            EXECUTE(INSERT INTO &vTabName.(&vColString_Str.)
                    SELECT &vAggString_Str. 
                      FROM &vTabName. a, &vLowMerchTabName. b
                     WHERE a.merch_level = &lowerlevel_id_adj
                       AND a.merch_id = b.&vLowMerchColName.
                  GROUP BY B.&vHighMerchColName., a.location_level, a.location_id, a.time_level, a.time_id)
            BY &dbtype.;
            DISCONNECT FROM &dbtype.;
            %RDBMS_Report(macro_call=DFR_IMPORT_P_MERCH_AGG,code_area=1,switch=ON);
        QUIT;
        %let fetch_rec=%sysfunc(fetch(&c1.));
    %end;
    %Let Close_Rc=%sysfunc(close(&c1.));
%mend;

    
/*-------------------------------------------------------------------------------------------------------
    Demand Forecasting for Retail
    Descriptions    : Roll up data by Location hierarchy 
    Developer       : Paul Dettorre, X8265
    Created         : 11/2008
    Modification Log:
    Parameters      
        vmmx_tbl_name           : Defines whether to process as a fact table or forecast table
        vTabName                : The target fact/fcast table name. Used as a definition template to build the temp
                                   table for aggregation processing.
        iClstrStId              : If Locations is cluster based use the cluster or cluster set ID
        iAgg_to_top_loc_level   : Lowest level of the hierarchy
        iAgg_from_loc_level     : Highest level of the hierarchy
        iForecastVersion        : iFvVersionId

    Process Flow
        1) Grab data from MAXDATA.path_seg to get full path through hierarchy
        2) For each level loop through ad attach MP keys to the final table roll up
           by the keys and INSERT.
-------------------------------------------------------------------------------------------------------*/
%macro p_loc_agg(vmmx_tbl_name=,
                 vTabName=,
                 iClstrStId=,
                 iAgg_to_top_loc_level=,
                 iAgg_from_loc_level=,
                 iForecastVersion=);

    %If (&iAgg_to_top_loc_level gt 1000) Or (&iAgg_from_loc_level gt 1000) %then
        %Do;
            %Let iPathId=999;
        %End;
    %Else
        %Do;
            %Let iPathId=1;
        %End;

    %let iFvVersion=%EVAL(&iForecastVersion + 0);
    /*
        Build a list of hierarchy ids
    */
	%let path_id=1;
	%if (&iAgg_from_loc_level.>1000) %then %do; %let path_id=999; %end;
    proc sql;
        create table work.path_seg as
        select *
          from MAXDATA.path_seg
         where path_id = &path_id. 
           and higherLevel_id >= (&iAgg_from_loc_level.)
           and lowerlevel_id <= (&iAgg_to_top_loc_level.)
      order by lowerlevel_id descending
        ;
    quit;
    /*
        Loop through the list of hierarchy levels and roll up data
    */
    %let c1=%sysfunc(open(work.PATH_SEG,i));
    %let fetch_rec=%sysfunc(fetch(&c1.));
    %do %while(&fetch_rec eq 0);
        %let lowerlevel_id=%sysfunc(getvarn(&c1.,%sysfunc(varnum(&c1.,LOWERLEVEL_ID)))) ;
        %let higherlevel_id=%sysfunc(getvarn(&c1.,%sysfunc(varnum(&c1.,HIGHERLEVEL_ID))));

        %If (&iPathId eq 1) %then
            %Do;
                %Let vLowLocColName =LV&lowerlevel_id.LOC_ID;
                %Let vLowLocTabName =LV&lowerlevel_id.LOC;
                %Let vHighLocColName=LV&higherlevel_id.LOC_ID;
            %End;

        %If (&iPathId eq 999) %then
            %Do;
                %If (&lowerlevel_id eq 4) %then
                    %Do;
                        %let vLowLocColName  =LVNLOC_ID;
                        %let vLowLocTabName  =CLSTR_STR;
                        %let vHighLocColName =FNL_CLSTR_SPC_ID;
                    %End;
                %Else
                    %Do;
                        %let vLowLocColName  =CLSTR_SPC_ID;
                        %let vLowLocTabName  =CLSTR_SPC;
                        %let vHighLocColName =CLSTR_ST_ID;
                    %End;
            %End;
        /*
            Add onto the SELECT string the KPI's and their corresponding aggregations

                MMX_TBL_NAME    MMX_COL_NAME    MERCH_AGG_RULE  LOC_AGG_RULE    TIME_AGG_RULE
            1   MFACT_VERSION   FC_SALES_R      SUM             SUM             SUM
            2   MFACT_VERSION   FC_SALES_U      SUM             SUM             SUM

        */
        %If (&iFvVersion eq 0) %then
            %Do;    
                %Let vColString=%STR(Merch_level, merch_id, location_level, location_id, time_level, time_id);
                %Let vAggString=%STR(a.Merch_level, a.Merch_id,&higherLevel_id.,&vHighlocColName.,  a.time_level, a.time_id);
            %End;
        %Else
            %Do;
                %Let vColString=%STR(fv_version_id, Merch_level, merch_id, location_level, location_id, time_level, time_id);
                %Let vAggString=%STR(&iForecastVersion., a.Merch_level, a.Merch_id,&higherLevel_id.,&vHighlocColName., a.time_level, a.time_id);
            %End;

        data _null_;
            set MAXDATA.dfr_fact_tables(where=(kupcase(mmx_tbl_name) = kupcase("&vmmx_tbl_name."))) end=eof;
            format vColString   $1024.;
            format vAggString   $1024.;
            
            retain vColString vAggString;

            if (_n_ eq 1) then
                do;
                    vColString="&vColString.";
                    vAggString="&vAggString.";
                end;

        vColString = kcompress(vColString||', '||mmx_col_name);
        vAggString = kcompress(vAggString||', '||loc_agg_rule||'(a.'||mmx_col_name||')');
            if (eof eq 1) then
                do;
                    call symputx('vColString',vColString,'l');
                    call symputx('vAggString',vAggString,'l');
                end;
        run;

        PROC SQL;
            CONNECT TO &dbtype.(USER=MAXDATA ORAPW=&dbpassword. PATH=&DSN.);
            EXECUTE(INSERT INTO &vTabName.(&vColString.)
                    SELECT &vAggString. 
                      FROM &vTabName. a, &vLowLocTabName. b
                     WHERE a.location_level = &lowerlevel_id
                       AND a.location_id = b.&vLowLocColName.
                  GROUP BY B.&vHighLocColName., A.Merch_level, A.merch_id, a.time_level, a.time_id)
            BY &dbtype.;
            %RDBMS_Report(macro_call=P_LOC_AGGcode_area=1,switch=ON);
            DISCONNECT FROM &dbtype.;
        QUIT;
        %let fetch_rec=%sysfunc(fetch(&c1.));

    %end;%* End Outer Loop;
    %Let Close_Rc=%sysfunc(close(&c1.));

%mend;
    
