/*****************************************************************************************/
/*  Like item routine traces through 'tree' of like-items over time
/*  Result: merch_items_cleansed dataset with all new/like-items and start/end dates for history
/*****************************************************************************************/

/* create tree table to hold trace of like-items */
data W_&wktid..hier_tree;
    format reg_new_item_id 8.;
    format new_level     8.;
    format new_id        8.;
    format like_level    8.;
    format like_id       8.;
    format hier_level    8.;
    format Status        8.;
    format AddOrder      8.;
    stop;
run;

/* main routine to traverse tree of like-items and find all historical like-assignments */
data _null_;

    set W_&wktid..merch_extract_items end=eof;
    retain base_level master_record_number records_to_process;

    if (_n_ eq 1) then
    do;
        format Like_ITEM_type   $1.;
        format new_item_level   8.;
        format new_item_id      8.;
        format like_item_level  8.;
        format like_item_id     8.;
        format pct_of_parent    8.;
        format start_date       date9.;
        format end_date         date9.;
        /*            format Adjustment_1     $10.;
        format Adjustment_2     $10.;
        format Adjustment_3     $10.;*/

        declare hash H_LikeItems(dataset:"W_&wktid..like_item_staging", hashexp:8, ordered: 'no');
        H_LikeItems.defineKey("new_item_level","new_item_id","like_item_level","like_item_id");
        H_LikeItems.defineData("Like_ITEM_type","new_item_level","new_item_id","like_item_level","like_item_id","pct_of_parent","start_date","end_date" /*,"Adjustment_1","Adjustment_2","Adjustment_3" */);
        H_LikeItems.defineDone();
        declare hiter H_LikeItems_Iter;
        H_LikeItems_Iter = _new_ hiter('H_LikeItems');

        format reg_new_item_id 8.;
        format new_level     8.;
        format new_id        8.;
        format like_level    8.;
        format like_id       8.;
        format hier_level    8.;
        format Status        8.;
        format AddOrder      8.;

        declare hash H_hier_tree(dataset:"W_&wktid..hier_tree", hashexp:8, ordered: 'no');
        H_hier_tree.defineKey("new_level","new_id","like_level","like_id","hier_level");
        H_hier_tree.defineData("reg_new_item_id","new_level","new_id","like_level","like_id","hier_level","Status","AddOrder");
        H_hier_tree.defineDone();
        declare hiter H_hier_tree_Iter;
        H_hier_tree_Iter = _new_ hiter('H_hier_tree');
        master_record_number = 1;
    end;

    base_level=0;

    /*
    Add the "base" record to the hierarchy hash table. This record will be
    the level zero base for which we will build the sub node relationships
    */
    new_level  = Base_new_item_level;
    new_id     = Base_new_item_id;
    like_level = Base_like_item_level;
    like_id    = Base_like_item_id;
    hier_level = base_level;
    Status     = 0;
    AddOrder = master_record_number;
    AddBase_RC = H_hier_tree.add();
    if (AddBase_RC ne 0) then do;
/*      put 'ERROR: Record not added From Base';*/
        %let job_rc_msg= %sysfunc(sasmsg(catalog.dfrmsg, dfr_like_item_NoRecord_Wrng, noquote));  
        %put &job_rc_msg.;
    end;
    else do;
        master_record_number = master_record_number + 1;
        put '###########################################################';
        put 'Processing merch_extract_items Record #' _n_;
        put 'Added MASTER HIER RECORD to H_hier_tree';
        put ' new_level: ' new_level;
        put '    new_id: ' new_id;
        put 'like_level: ' like_level;
        put '   like_id: ' like_id;
        put 'hier_level: ' base_level;
    end;

    /*
    Loop through the current Hierarchy hash table to process
    any nodes with a status of zero
    */
    put 'Entering Loop do while (hier_tree_iter_rc = 0)';
    records_to_process = 1;
    hier_tree_iter_rc = H_hier_tree_Iter.first();
    do while (hier_tree_iter_rc = 0);
        put 'Loop Record #' records_to_process;
        put '====== Record read from  H_hier_tree_Iter======';
        put ' new_level: ' new_level;
        put '    new_id: ' new_id;
        put 'like_level: ' like_level;
        put '   like_id: ' like_id;

        /*
        Read a record from the hierarchy hash table and put the values into
        temporary values which will be used to find/update this records status
        when sub-processing is finished
        */
        temp_new_level  = new_level;
        temp_new_id     = new_id;
        temp_like_level = like_level;
        temp_like_id    = like_id;
        temp_hier_level = hier_level;
        temp_add_order  = AddOrder;

        /*
        Only process records with status codes of zero
        */
        if (Status eq 0) then
        do;
        /*Keep a place holder for Original hier recrod */
            put 'Status=0';
            base_level = temp_hier_level + 1;
            Current_Id = temp_like_id;
            Current_Level = temp_like_level;

            /* Open the Like Items Hash Table */
            Like_Items_Found = 0;
            rc = H_LikeItems_Iter.first();
            put 'read H_LikeItems_Iter.first() and enter loop';
            do while (rc = 0);
            /*
            Loop through like items until the current hierarchy tree records
            like item id is equal to the like items  "new item key"
            
            put ' Condition Check do while (rc = 0) ';
            put '   Current ID: ' Current_Id ;
            put 'Current Level: ' Current_Level ;
            put '       new ID: ' new_item_id ;
            put '    new Level: ' new_item_level ;
            put ' DUMP LIKE ITEM RECORD do while (rc = 0) ';
            put ' new_item_level: ' new_item_level;
            put '    new_item_id: ' new_item_id;
            put 'like_item_level: ' like_item_level;
            put '   like_item_id: ' like_item_id;
            */
            if ((new_item_id eq Current_Id) and (new_item_level eq Current_level)) then
            do;
                put 'TRUE:((new_item_id eq Current_Id) and (new_item_level eq Current_level))';
            /*
                If found pop the current like item record into the
                hierarchy hash table
                padett: 17apr2009 - changed new_item_level to Base_new_item_level
                                    changed new_item_id to Base_new_item_id
                */
                Like_Items_Found = 1;
                reg_new_item_id = new_item_id;
                new_level  = base_new_item_level;
                new_id     = base_new_item_id;
                like_level = like_item_level;
                like_id    = like_item_id;
                hier_level = base_level;
                Status     = 0;
                AddOrder = master_record_number;
                AddBase_RC = H_hier_tree.add();
                put '==>Adding record to H_hier_tree';
                put '==>Like_Items_Found = 1;                    ';                    
                put '==>new_level        = new_item_level;       ' new_item_level;       
                put '==>new_id           = new_item_id;          ' new_item_id;          
                put '==>like_level       = like_item_level;      ' like_item_level;      
                put '==>like_id          = like_item_id;         ' like_item_id;         
                put '==>hier_level       = base_level;           ' base_level;           
                put '==>Status           = 0;                    ' ;                    
                put '==>AddOrder         = master_record_number; ' master_record_number; 

                if (AddBase_RC ne 0) then
                do;
/*                  put 'ERROR: Record not added From Loop 2';*/
                    %let job_rc_msg= %sysfunc(sasmsg(catalog.dfrmsg, dfr_like_item_NoRecord_Add_Wrng, noquote));  
                    %put &job_rc_msg.;
                end;
                else
                do;
                    master_record_number = master_record_number + 1;
                end;

                output;
                end;
                rc = H_LikeItems_Iter.next();
            end;
            /*
            Once all matchs have been found in the like items hash table mark
            the current hierarchy tree record with a status of 1 so it will not get
            processed again. This process is necessary because after we add records
            to the Hierarchy tree hash table it gets reset to the first record.
            */

            new_level  = temp_new_level;
            new_id     = temp_new_id;
            like_level = temp_like_level;
            like_id    = temp_like_id;
            hier_level = temp_hier_level;
            Status     = 1;
            AddOrder = temp_add_order;
            ReplaceBase_RC = H_hier_tree.replace();

            put '==>Replacing record to H_hier_tree';
            put '==>new_level  = temp_new_level;  ' temp_new_level;                    
            put '==>new_id     = temp_new_id;     ' temp_new_id;       
            put '==>like_level = temp_like_level; ' temp_like_level;          
            put '==>like_id    = temp_like_id;    ' temp_like_id;      
            put '==>hier_level = temp_hier_level; ' temp_hier_level;         
            put '==>Status     = 1;               ' ;           
            put '==>AddOrder = temp_add_order;    ' temp_add_order;                    

            put 'ReplaceBase_RC: ' ReplaceBase_RC;
            if (ReplaceBase_RC ne 0) then
            do;
/*              put 'ERROR: Record not replaced From Loop 1';*/
                %let job_rc_msg= %sysfunc(sasmsg(catalog.dfrmsg, dfr_like_item_NoRecord_Replace_Wrng, noquote));  
                %put &job_rc_msg.;
            end;
            /*
                padett: 21APR2009 - Fixed. hier_tree_iter_rc = was not in front of H_hier_tree_Iter.next()
                                    causing a key violation
            */
            hier_tree_iter_rc = H_hier_tree_Iter.next();
        end;
        else do;
            hier_tree_iter_rc = H_hier_tree_Iter.next();
        end;
    end;

    /*
    Mark all remaining records with a status of 0 so they will not be processed again.
    */
    hier_tree_iter_rc = H_hier_tree_Iter.first();
    do while (hier_tree_iter_rc = 0);
        
        Status     = 1;
        ReplaceBase_RC = H_hier_tree.replace();
        if (ReplaceBase_RC ne 0) then do;
/*          put 'ERROR: Record not replaced From Loop 1';*/
            %let job_rc_msg= %sysfunc(sasmsg(catalog.dfrmsg, dfr_like_item_NoRecord_Replace_Wrng, noquote));  
            %put &job_rc_msg.;
        end;

        put '==>Mark all remaining records with a status of 0';
        put '==>new_level  = temp_new_level;  ' temp_new_level;                    
        put '==>new_id     = temp_new_id;     ' temp_new_id;       
        put '==>like_level = temp_like_level; ' temp_like_level;          
        put '==>like_id    = temp_like_id;    ' temp_like_id;      
        put '==>hier_level = temp_hier_level; ' temp_hier_level;         
        put '==>Status     = 1;               ' ;           
        put '==>AddOrder = temp_add_order;    ' temp_add_order;                    

        hier_tree_iter_rc = H_hier_tree_Iter.next();
    end;

    if (eof eq 1) then
    do;
        /* Write the final hash table out */
        rc = H_hier_tree.output(dataset: "W_&wktid..H_hier_tree");
    end;

run;

proc sort data=W_&wktid..H_hier_tree out=out.H_hier_tree_sorted_test;
    by AddOrder;
run;

proc sort data=W_&wktid..H_hier_tree out=W_&wktid..H_hier_tree_sorted;
    by AddOrder;
run;

data W_&wktid..merch_items_working;
    set W_&wktid..H_hier_tree_sorted;
    retain Tree_Branch_ID;
    If (_n_ eq 1) then Tree_Branch_ID = 0;
    Main_New_Item_ID = new_id;

    if (hier_level eq 0) then do;
        Tree_Branch_ID = Tree_Branch_ID + 1;
    end;
    else do;
        new_id=reg_new_item_id;
    end;
run;

proc sql noprint;
    Select datepart(min(lv&time_to_level.time_start_date)) format=date9.
        into :vStartDate from maxapp.lv&time_to_level.time;
quit;

%put &vStartDate.;

proc sql noprint;
    create table W_&wktid..merch_items_working_dateproc as
    Select *
    from (select A.Main_New_Item_ID,A.new_id, A.like_id, A.hier_level,
    B.Like_ITEM_type, B.new_item_level, B.new_item_id,
    B.like_item_level, B.like_item_id, B.pct_of_parent,
    B.start_date format=date9., COALESCE(B.end_date,today()) format=date9. as end_date,
    /*B.Adjustment_1, B.Adjustment_2, B.Adjustment_3, */
                           A.Tree_Branch_ID
    from W_&wktid..merch_items_working A left join W_&wktid..like_item_staging B 
        on A.new_id  = B.new_item_id
    and A.new_level = B.new_item_level
    and A.like_id = B.like_item_id
    and A.like_level = B.like_item_level) C
    order by Tree_Branch_ID, end_date
    ;
quit;

data W_&wktid..merch_items_working_clean;
    set W_&wktid..merch_items_working_dateproc;
    by Tree_Branch_ID;

    format vStartDate date9.;
    format vEndDate date9.;

    format start_date  date9.;
    format end_date  date9.;

    retain vStartDate vEndDate;

    if (first.Tree_Branch_ID) then
    do;
        vStartDate = "&vStartDate."d;
        vEndDate = end_date;
    end;

    if (start_date eq .) then start_date = vStartDate;

    output;
    vStartDate = intnx('day', vEndDate, 7);
    vEndDate = end_date;
run;

proc sql noprint;
create table W_&wktid..merch_items_cleansed as
        Select a.new_item_level, a.main_new_item_id as new_item_id, a.like_item_level, a.like_item_id,
                a.pct_of_parent,
               &real_time_to_level. as time_level, b.lv&time_to_level.time_lkup_id as time_id
from W_&wktid..merch_items_working_clean a, maxapp.lv&time_to_level.time b
         where datepart(b.lv&time_to_level.time_end_date) >= start_date and
               datepart(b.lv&time_to_level.time_start_date) <= end_date and
               b.lv&time_to_level.time_start_date >= "&hist_from_time."d and
               b.lv&time_to_level.time_end_date <= "&hist_to_time."d;
quit;
