%macro dfr_setup();
	options Linesize=max;
	/*
	Parameters in (Setup for every workflow step)
	KPI Field metadata          dfr_numkpis, dfr_kpinameset dfr_fcast_kpis dfr_fcast_import_kpis dfr_ind_var
	dfr_maxmerch                Maximum merchandise hierarchy level above 10
	locfrom
	L_lev
	m_lev
	dfr_clstr_st_id                     Cluster Set id
	*/
	%include "&path_pgm.&slash.dfrcode&slash.dfr_abt_creation.sas";
	%include "&path_pgm.&slash.dfrcode&slash.dfr_extract.sas";
	%include "&path_pgm.&slash.dfrcode&slash.dfr_data_gen.sas";
	%include "&path_pgm.&slash.dfrcode&slash.dfr_purge.sas";
	%include "&path_pgm.&slash.dfrcode&slash.aggregation.sas"; 
		/* include all forecast execution code */
	%include "&path_pgm.&slash.dfrcode&slash.dfr_Splitter.sas";
	%include "&path_pgm.&slash.dfrcode&slash.splitter1.sas";
	%include "&path_pgm.&slash.dfrcode&slash.dfr_filter.sas";
	%include "&path_pgm.&slash.dfrcode&slash.splitter2.sas";
	%include "&path_pgm.&slash.dfrcode&slash.splitter3.sas";
	%include "&path_pgm.&slash.dfrcode&slash.intel_workflow.sas";
	%include "&path_pgm.&slash.dfrcode&slash.run_intel_workflow.sas";

	%include "&path_pgm.&slash.dfrcode&slash.hpf_call.sas";
	%include "&path_pgm.&slash.dfrcode&slash.hpf_diagnose.sas";
	%include "&path_pgm.&slash.dfrcode&slash.hpf_engine.sas";
	%include "&path_pgm.&slash.dfrcode&slash.revisit.sas";
	
	%include "&path_pgm.&slash.dfrcode&slash.combine3.sas";
	%include "&path_pgm.&slash.dfrcode&slash.dfr_reconcile_merge.sas";
	%include "&path_pgm.&slash.dfrcode&slash.combine2.sas";
	%include "&path_pgm.&slash.dfrcode&slash.hpf_reconcile.sas";
	%include "&path_pgm.&slash.dfrcode&slash.combine1.sas";

/*	%include "&path_pgm.&slash.dfrcode&slash.dfr_exceptions_import.sas";*/
	%include "&path_pgm.&slash.dfrcode&slash.dfr_exceptions.sas";
    %Include "&path_pgm.&slash.dfrcode&slash.dfr_import_agg.sas";
    %Include "&path_pgm.&slash.dfrcode&slash.dfr_import.sas";	

	
	%global
	/* forecast metadata fields variables */
	dfr_fcast_kpis h1_fcastkpi dfr_fcast_import_kpis f1_dfr_fcast_import_kpis dfr_ind_var dfr_fcast_import_kpis_std f1_dfr_fcast_import_kpis_std
	h1_dfr_ind_var_mp dfr_ind_var_external dfr_ind_var_mp 

	/* dimension level variables */
	m_lev l_lev locfrom dfr_maxmerch dfr_maxloc merch_level_keys loc_level_keys dfr_clstr_st_id

	/* reconciliation variables */
	proj recon_dim unrecon_dim full_recon_dim full_unrecon_dim rd unrd RIS_weekday_1;
	 
	%let proj=&projid;
	 
	/**************************************************************************/
	/*Setup kpis to forecast and their extract and re-import field names      */
	/**************************************************************************/
	%let dfr_numkpis=%quote(&&dfr_numkpis_&analytic_id.);
	%let dfr_kpinameset=%quote(&&dfr_kpinameset_&analytic_id.);         /* English names for forecast columns */
	/* DEM INDEPENDENT VARS 120208 */
	%let dfr_fcast_kpis=%quote(&&dfr_kpiset_&analytic_id.);             /* Source DB name for forecast KPIs   */
	%let h1_fcastkpi = %sysfunc(tranwrd(%quote(&dfr_fcast_kpis.),%str(.),%str(.h1_)));  /* Add h1_ to all source fields: this is field names after initial extract */
	%let dfr_fcast_import_kpis=%quote(&&dfr_fcastset_&analytic_id.);        /* Import KPI DB names                */
	%let f1_dfr_fcast_import_kpis = %sysfunc(tranwrd(%quote(&dfr_fcast_import_kpis.),%str(.),%str(.f1_)));  /* Add h1_ to all source fields: this is field names after initial extract */
	%let dfr_fcast_import_kpis_std=%quote(&&dfr_fcastset_std_&analytic_id.);        /* Import KPI DB names                */
	%let dfr_ind_var=%quote(&&dfr_ind_var&analytic_id.);
	%let f1_dfr_fcast_import_kpis_std = %sysfunc(tranwrd(%quote(&dfr_fcast_import_kpis_std.),%str(.),%str(.f1_)));  /* Add h1_ to all source fields: this is field names after initial extract */
	%put KPIS: &analytic_id &dfr_numkpis &dfr_fcast_kpis &dfr_fcast_import_kpis;

	/**************************************************************************/
	/* Set up independent variables											  */
	/**************************************************************************/
	%let f = 1;
	%let dfr_ind_var_external=;
	%let dfr_ind_var_mp=;
	%let afield = %sysfunc(kscan(%quote(&dfr_ind_var.),&f,%str(,)));
	%do %while (&afield ne);
	
		%let table_type = %sysfunc(kscan(%quote(&afield.),1));
		
		%if %sysfunc(kupcase(&table_type.))=MAXDATA %then %do;
			%let var=%sysfunc(kscan(%quote(&afield.),2)).%sysfunc(kscan(%quote(&afield.),3));
			%if (&dfr_ind_var_mp. eq) 
				%then %let dfr_ind_var_mp=&var.;
				%else %let dfr_ind_var_mp=&dfr_ind_var_mp.,&var.;
			%end;
			
		%else %do;
			%if (&dfr_ind_var_external. eq) 
				%then %let dfr_ind_var_external=&afield.;
				%else %let dfr_ind_var_external=&dfr_ind_var_external.,&afield.;
			%end;
		
		%let f = %eval(&f+1);
		%let afield = %sysfunc(kscan(%quote(&dfr_ind_var.),&f,%str(,)));
		%end;
	%let h1_dfr_ind_var_mp = %sysfunc(tranwrd(%quote(&dfr_ind_var.),%str(.),%str(.h1_)));  /* Add h1_ to all source fields: this is field names after initial extract */
	%put &dfr_ind_var_external;
	%put &dfr_ind_var_mp;

 	/**************************************************************************/
	/* Error checks for parameters */
	/**************************************************************************/
	%if &DFR_FILTER_PRIOR_HIST.>&DFR_FILTER_PRIOR_PERIODS. %then %do; /*%do is added by kalyan*/
		%let job_rc=100;
		%let job_rc_msg=Parameter dfr_filter_prior_hist must be <= dfr_filter_prior_periods;
	%end;
	%if &job_rc > 4 %then %goto end_setup;

	/**************************************************************************/
	/* Set up Level parameters */
	/**************************************************************************/
	%if (&standalone.=0) or (&standalone.=2) %then %do;  /* if connected get database parameters */
		proc sql noprint;
			select max(level_id)-10 into :dfr_maxmerch from maxdata.hier_level where hier_id = 11 and level_id < 20;
		/* DEM sept 2008 - need dfr_maxloc as jobparam for standalone execution? */
        select max(level_id) into :dfr_maxloc from maxdata.hier_level where hier_id = 1 and level_id <4;
		quit;
		%RCSet(&sqlrc);
	%end;

	%let dfr_time_field=time_&time_to_level.;
	 
	/* m_lev and l_lev are adjusted to levels based on clustering and merch skip level features*/
	%let m_lev =&merch_to_level.;
	%let l_lev=&loc_to_level.;
	%let locfrom=&loc_from_level.;
	%if &merch_to_level>&dfr_maxmerch %then %do;
		%let m_lev =%eval(&merch_to_level.-(9-&dfr_maxmerch.));
	%end;

	/* set up cluster level parameters */
	%let dfr_clstr_st_id=0;
	%if &loc_from_level.>1000 %then %do;
		%if &loc_from_level.=1001 %then %do;
			%let dfr_clstr_st_id=&loc_from_id.;
			%end;
		%else %do;   /* loc from is a single cluster, get cluster set from loc_path_id */
		%if (&standalone.=0) or (&standalone.=2) %then %do;  /* if connected get database parameters */
			proc sql noprint;
			select loc_path_id-1000 into :dfr_clstr_st_id from maxdata.planworksheet where planworksheet_id=&projid.;
			%end;
		%end;
	%end;

	%if &dfr_clstr_st_id. > 0 %then %do;
		%if &orig_loc_to_level.=1002 %then %do;  /* lowest level is cluster */
			%let loc_to_level=4;    /* extract level is store */
			%let locfrom=1;         /* ensure from level is 1 */
			%let l_lev=2;           /* loc to level for clusters - 2=cluster level */
			%end;
		%else %do;
			%let l_lev=3;   /* cluster or cluster set to store */
			%end;
        	%if (&orig_loc_from_level.=1001) and (&standalone.=0) or (&standalone.=2) %then %do;  /* if connected get database parameters */
			proc sql noprint;
				create table stage.cl_&projid as
					select * from maxdata.clstr_spc where clstr_st_id=&dfr_clstr_st_id. and clstr_grp_id>-1;
					select FR_LOC_LVL,FR_LOC_MMBR into :loc_from_level, :loc_from_id from maxdata.clstr_st where CLSTR_ST_ID = &dfr_clstr_st_id.;
				%let loc_from_level = %sysfunc(KTRIM(&loc_from_level));
				%RCSet(&sqlrc);
			quit;
			%end;
		%end;
	%put MERCH: &merch_to_level., &m_lev  LOCATION: &loc_to_level., &l_lev  CLUSTER: &dfr_clstr_st_id.;
	 
	/* get merchandise and location level keys */
	/* set up list of mid and mname keys for use in dataprep */
	%let merch_level_keys=mid_1, mname_1,;
	%if &merch_to_level.>1 %then %do;
		%do i=2 %to &merch_to_level.;
		%if &i<=&dfr_maxmerch or &i=&merch_to_level. %then %do;
			%let merch_level_keys=&merch_level_keys. mid_&i., mname_&i.,;
			%end;
		%end;
	%end;
	%let merch_level_keys=%sysfunc(KSUBSTR(%quote(&merch_level_keys.),1, %eval(%sysfunc(KLENGTH(%quote(&merch_level_keys.)))-1)));
	 
	/* set up list of lid and lname keys for use in dataprep */
	%let loc_level_keys=lid_1, lname_1,;
	%if &l_lev. > 1 %then %do;
		%do i=2 %to &l_lev.;
			/* DEM sept 2008 - Added if statement */
      		%if &i<=&dfr_maxloc or &i=&l_lev. %then %do; 
				%let loc_level_keys=&loc_level_keys. lid_&i., lname_&i., ; 
				%end; 
			%end;
        %end;
    	%let loc_level_keys=%sysfunc(KSUBSTR(%quote(&loc_level_keys.),1,%eval(%sysfunc(klength(%quote(&loc_level_keys.)))-1)));

	/* DEM sept 2008 - set up l_lev for skipped level */
    	%if (&loc_to_level.>&dfr_maxloc.) and (&orig_loc_from_level.<1000) %then %do;
        	%let l_lev =%eval(&loc_to_level.-(3-&dfr_maxloc.)); 
        %end; 	 
	%let RIS_weekday_1=%eval(&dfr_weekday.-1);
	 	 
	/*********************************************/
	/***** HISTORICAL TIME BOUNDARY SELECTION ****/
	/*********************************************/
	/*  Parameters
	*from_time to_time              FORECAST TIME BOUNDARIES
	*hist_from_time hist_to_time    HISTORY TIME BOUNDARIES
	/* set time interval name: month, week, etc. */
	%if &time_to_level.=1 %then
		%let dfr_interval=Year;
	%else %if &time_to_level.=2 %then
		%let dfr_interval=Season;
	%else %if &time_to_level.=3 %then
		%let dfr_interval=Quarter;
	%else %if &time_to_level.=4 %then
		%let dfr_interval=Month;
	%else
		%let dfr_interval=Week;
	 
	%if (&standalone.=0) or (&standalone.=2) %then %do;
		%if (&time_template. eq .) %then %let time_template=0;
		%if &time_from_level. < 1 %then %let time_from_level=1;
	 
		/* History is set to all loaded data but can't go past end of worksheet */
		/* Start of worksheet doesn't matter as forecast always starts after last loaded week of history */
		%put &time_from_level &time_to_level.;
		proc sql noprint;
			select min(lv&time_to_level.time_start_date),max(lv&time_to_level.time_end_date)
				into :hist_from_time,:hist_to_time from maxapp.lv&time_to_level.time where loaded_flag=1;
		quit;
	 
		/* Find worksheet time boundaries and set in &from_time and &to_time dates */
		%if %quote(&time_template.) > 0 %then %do;
			/* set_type=9 = dynamic NUM_PERIODS DIR_PERIOD_FLG(1 next,2 prev)INC_CURR_PERIOD_FLG=1 NOT ACCOUNTED FOR - dynamic events not allowed */
			proc sql noprint;
				select set_type,path_id,start_date,end_date,num_periods,dir_period_flg,inc_curr_period_flg into :set_type, :calendar, :from_time, :to_time, :num_periods,:dir_period_flg,:inc_curr_period_flg from &template. where template_id = &time_template.;
				%let num_periods1=%sysfunc(putn(&num_periods.,4.));
			quit;
			%if &set_type.=9 %then %do; /* dynamic time event */
				proc sql noprint;
					select avg(days_in_period) into :avg_days from maxapp.lv&time_to_level.time;
					%let avg_days1=%sysfunc(putn(&avg_days.,10.));
				quit;
				data _null_;
					fromtime=intnx('week',"&hist_to_time"d,1);
					call symputx('from_time',put(fromtime,date9.),l);
					%let num_periods2=%eval(&num_periods1.*&avg_days1.);
					totime=intnx('day',fromtime,&num_periods2.);
					call symputx('to_time',put(totime,date9.),l);
				run;
				%end;
			%else %do; /*static time event */
				proc sql noprint;
					select min(level_number)-46 into :time_from_level from &template_level. where template_id=&time_template. and level_number<>45;
				quit;
				%end;
			%end;
		%else %do; /*  static time hierarchy */
			proc sql noprint;
				select min(lv&time_to_level.time_start_date),max(lv&time_to_level.time_end_date) into :from_time, :to_time from maxapp.lv&time_to_level.time where lv&time_from_level.time_lkup_id = &from_time_id.;
			quit;
			%end;
		%RCSet(&sqlrc.);
		 
        /* ensure hist to boundary not past end of worksheet */
        data _null_; wktend="&to_time"d; loaded_end="&hist_to_time"d; if wktend < loaded_end then call symput('hist_to_time',wktend); else call symput('hist_to_time',loaded_end); run;
        %let hist_to_time=%sysfunc(putn(&hist_to_time,date9.));
		/* in-season case: set forecast start to just after loaded_end */
        data _null_; wktstart="&from_time"d; loaded_end="&hist_to_time"d; if wktstart < loaded_end then call symput('from_time',put(loaded_end,date9.)); run;
        %RCSet(&syserr.);
 
	%end;

	%put FORECAST TIME BOUNDARIES: *&eng_lead* &from_time &to_time;
	%put HISTORY TIME BOUNDARIES: &hist_from_time &hist_to_time;
	 
	/**************************************************************************/
	/* Get Filter Dates and time reference tables                                             */
	/**************************************************************************/
	%if (&standalone.=0) or (&standalone.=2) %then %do;  /* if connected get database parameters */
		%let dfr_filter_end_date=&hist_to_time.;
		proc sql noprint;

			/* load time table into work table */
			create table inp.lv&time_to_level.time_&projid. as select * from maxapp.lv&time_to_level.time;

			/* find time to id */
			select (cycle_id * 100 + lv&time_to_level.time_id), lv&time_to_level.time_lkup_id into :to_cycletime, :to_time_id
			from inp.lv&time_to_level.time_&projid.
			where "&hist_to_time."d between
			%if &dbtype=db2 %then %do;
				lv&time_to_level.time_start_date and lv&time_to_level.time_end_date
			%end;
			%else %do;
				datepart(lv&time_to_level.time_start_date) and datepart(lv&time_to_level.time_end_date)
			%end;
			;
			 
			/* get from time id */
			Select lv&time_to_level.time_start_date into :dfr_filter_start_date from inp.lv&time_to_level.time_&projid. A where &dfr_filter_prior_periods. =
			( select COUNT(*) from inp.lv&time_to_level.time_&projid.  B where
			(A.cycle_id * 100 + A.lv&time_to_level.time_id) <=(B.cycle_id * 100 + B.lv&time_to_level.time_id) and (B.cycle_id * 100 + B.lv&time_to_level.time_id) <= &to_cycletime. );
		quit;
		 
		/* If not optimized then extract all hierarchy tables for reference and place into input folder */
		%if (&dfr_wf_optimized=0) %then %do;
			/* output location tables */
			/*            %if &loc_to_level.>10 %then %do; %let locto=10; %end; %else %do; %let locto=&loc_to_level.; %end;
			%do i=1 %to &locto.;
			create table inp.lv&i.loc as select * from maxdata.lv&i.loc;
			%end;
			*/
			/* output time tables */
		proc sql noprint;
			%do i=1 %to %eval(&time_to_level.-1);
				create table inp.lv&i.time_&projid. as select * from maxapp.lv&i.time;
			%end;
			/* output merch tables */
			/*            %do i=1 %to &merch_to_level.;
			%if &i<10 %then %do;
				create table inp.lv&i.cmast as select * from maxdata.lv&i.cmast;
			%end;
			%else %do;
				create table inp.lv&i.mast as select * from maxdata.lv&i.mast;
			%end;
			%if &i>1 %then %do;
				create table inp.lv&i.ctree as select * from maxdata.lv&i.ctree;
			%end;
			%end;
			*/
		%end;
	quit;
	%RCSet(&sqlrc);
	%end;

	/**************************************************************************/
	/* Setup Reconciliation parameters                                        */
	/**************************************************************************/
	 
	/* No reconciliation case */
	%if ((&recon_merch.=0) and (&recon_loc.=0)) or (&dfr_wf_recon.=0) or (&recon_single.=1) %then %do;
		%let dfr_wf_recon=0;
		%end;
	%else %do;
		%let dfr_wf_recon=1;
		%end;

	/* Reconcile merch or loc */
	%let recon_stdmethod=AGG;
	/*%if (&recon_hier.=0) or (&recon_hier.=.) %then %do; %let recon_hier=1; %end;*/

	%if ((&recon_single.=1) or (&recon_hier.=1 ))  %then %do;     /* Merchandise reconcile */
		%let recon_dim=MERCH;
		%let unrecon_dim=LOC;
		%let full_recon_dim=MERCH;
		%let full_unrecon_dim=LOCATION;
		%let rd=M;
		%let unrd=L;
		%if (&dfr_wf_recon.=1) %then %do;
			%if &recon_merch>=&m_lev. %then %do;     /* BU */
				%let recon_merch=&m_lev.;
				%end;
			%if &recon_merch<=&merch_from_level. %then %do;   /* TD */
				%let recon_merch=&merch_from_level.;
				%let recon_stdmethod=DISAGG;
				%end;
			%end;
		%end;

	%if (&recon_hier=2)  %then %do;      /* Location Reconcile */
		%let recon_dim=LOC;
		%let unrecon_dim=MERCH;
		%let full_recon_dim=LOCATION;
		%let full_unrecon_dim=MERCH;
		%let rd=L;
		%let unrd=M;
		%if (&dfr_wf_recon.=1) %then %do;
			%if &recon_loc>=&l_lev. %then %do;     /* BU */
				%let recon_loc=&l_lev.;
				%end;
			%if &recon_loc<=&locfrom. %then %do;   /* TD */
				%let recon_loc=&locfrom.;
				%let recon_stdmethod=DISAGG;
				%end;
		 %end;
	%end;

	/* If &Dfr_remove_est_cat=
	0: est and catalog will remain in the system
	1: all the datasets including est and catalog get deleted before forecasting*/
	%if &Dfr_remove_est_cat=1 or &dfr_ivar_chng=1 %then %All_purgedatasets(projid=&proj.);

	/*************New WORK  Libname*********************************/
	%create_work_dir(path=&path_stage&slash.work,name_dir=W_&wktid.);
	%global work_path;
	%let work_path=&path_stage&slash.work&slash.W_&wktid.;
	%put &work_path;
	libname W_&wktid "&path_stage&slash.work&slash.W_&wktid.";
	proc datasets library=W_&wktid. kill nodetails nolist;
	quit;


%end_setup:
%mend;
