/**************************************************************/
/*                                                            */
/*       SAS RETAIL INTELLIGENCE SOLUTIONS                    */
/*       Demand Forecast For Retail Planning 4.2              */
/*                                                            */
/**************************************************************/
/*                                                            */
/* SAS Retail Group, All rights reserved (c) 2006,2007        */
/*                                                            */
/**************************************************************/

/**************************************************************/
/*		Functionality- FORECAST EXECUTION UTILITIES           */
/**************************************************************/
 
/******************************************/
/* Functionality- FORMAT CREATION UTILITY */
/******************************************/

%macro create_format(projid=&proj.);
%if &dfr_wf_optimized = 0 %then 
%do;
	%let colname='';
	%let dash=_;	
	%let i=1;
	proc sql noprint;
		select name into: colname separated by ',' from dictionary.columns
		where libname='INP' AND memname="F_&projid." and (name like 'mid_%' or name like 'lid_%');
	quit;
	%let variable=%sysfunc(kscan(%quote(&colname.),&i,%str(,)));
	%do %while (%quote(&variable.) ne); /*For Each variable*/
		%let id_no=%sysfunc(ksubstr(&variable.,5,1));
		%let dim_typ=%sysfunc(ksubstr(&variable.,1,1));
		%let lnm=%sysfunc(cats(&dim_typ.,&projid.,&id_no.,&dash.));
		proc sql noprint;
			create table W_&wktid..&variable. as
				select distinct &variable. as start, "&lnm" as fmtname, &dim_typ.name_&id_no. as label from inp.f_&projid.; 
			alter table W_&wktid..&variable.
			add label varchar(80);
		quit;
		data _null_;
			set W_&wktid..&variable.;
			if _n_=1 then call symputx('name_fmt',fmtname);
		run;
		%if %sysfunc(cexist(dp.formats.&name_fmt..format)) %then %do;
		proc catalog cat=dp.formats;
  			 delete &name_fmt..format;
		quit;
		%end;
		proc format cntlin = W_&wktid..&variable. library=stage noprint;
			select fmtname;
		run;
		%let i=%eval(&i.+1);
		%let variable=%sysfunc(kscan(%quote(&colname.),&i,%str(,)));
	%end; /*END For Each variable*/
%end;
%mend create_format;

/************************************************/
/*	Functionality- FORMAT APPLICATION UTILITY 	*/
/************************************************/
 
%macro Apply_format(proj=&proj.,data=,M_to_lev=,L_to_lev=,M_from_lev=,L_from_lev=);
	%if &dfr_wf_optimized.=0 %then %do;
		%let App_for_i=0;
		data &data.;
			set &data.;
			%do App_for_i=&M_from_lev. %to &m_to_lev.;
				mname_&App_for_i=put(mid_&App_for_i, m&proj.&App_for_i._.);
			%end;
			%do App_for_i=&L_from_lev. %to &L_to_lev.;
				lname_&App_for_i=put(lid_&App_for_i, l&proj.&App_for_i._.);
			%end;
		run;
	%end;
%mend;
/************************************************/
/*	Functionality- REMOVE FORMAT */
/************************************************/
 
%macro remove_format(lib=,data=);
proc datasets lib=&lib nolist;     
	modify &data;      
	format _all_;          *Remove formats;   
quit;    run;
%mend;
/************************************************************/
/* Importing the promotion data from external .csv file     */
/************************************************************/

%macro import_promo_data(csvfile=,out=);
data &out;
 INFILE "&csvfile"
		DELIMITER=','
		MISSOVER
		DSD
		FIRSTOBS=2
		;
 LENGTH
		MERCH_LVL 8
		ITEM_CATEGORY_RK 8
		LOC_LVL 8
		LOCATION_RK 8
		EFFECTIVE_FROM_DT 8
		EFFECTIVE_TO_DT 8
		RET_MONTH_END_DT 8
		RET_MONTH_START_DT 8
		RET_QUARTER_END_DT 8
		RET_QUARTER_START_DT 8
		RET_WEEK_END_DT 8
		RET_WEEK_START_DT 8
		PROMO_TYPE_CODE $10
	;
	INFORMAT
		EFFECTIVE_FROM_DT YYMMDD8.0
		EFFECTIVE_TO_DT YYMMDD8.0
		RET_MONTH_END_DT YYMMDD8.0
		RET_MONTH_START_DT YYMMDD8.0
		RET_QUARTER_END_DT YYMMDD8.0
		RET_QUARTER_START_DT YYMMDD8.0
		RET_WEEK_END_DT YYMMDD8.0
		RET_WEEK_START_DT YYMMDD8.0
	;
	FORMAT
		EFFECTIVE_FROM_DT DATE9.
		EFFECTIVE_TO_DT DATE9.
		RET_MONTH_END_DT DATE9.
		RET_MONTH_START_DT DATE9.
		RET_QUARTER_END_DT DATE9.
		RET_QUARTER_START_DT DATE9.
		RET_WEEK_END_DT DATE9.
		RET_WEEK_START_DT DATE9.
	;
	INPUT
		MERCH_LVL
		ITEM_CATEGORY_RK 
		LOC_LVL 
		LOCATION_RK 
		EFFECTIVE_FROM_DT 
		EFFECTIVE_TO_DT
		RET_MONTH_END_DT
		RET_MONTH_START_DT
		RET_QUARTER_END_DT
		RET_QUARTER_START_DT
		RET_WEEK_END_DT
		RET_WEEK_START_DT
		PROMO_TYPE_CODE
	;
	
RUN;
%mend import_promo_data;
/******************************************************************/
/*Functionality:Attach promo indicator variables to the sales data*/
/******************************************************************/

%macro attach_promo();

%import_promo_data(csvfile=&path_data&slash.input&slash.&dfr_ext_promo_file,out=inp.promo_abt_&wktid);

	%do i=1 %to &dfr_data_promo_num.;
		%let sum_promo =0;
		%let promo_typ=%sysfunc(KSCAN(&dfr_data_promo_types.,&i));
		proc sql noprint;
			create table W_&wktid..&data_name._promo
			as select a.*,
			case when (kupcase(b.promo_type_code)=kupcase("&promo_typ.")) then 
				1 else 0 end as promo_&promo_typ._dummy format 1.
			from W_&wktid..mmx_tmp1 as a left join inp.promo_abt_&wktid as b
			on (a.mid_&merch_to_level.=B.ITEM_CATEGORY_RK
			and B.merch_lvl=&merch_to_level.
			and A.lid_&loc_to_level.=B.LOCATION_RK
			and B.loc_lvl=&loc_to_level.
			and &dfr_time_field. >=&tp_st_col. and &dfr_time_field.<=&tp_end_col.
			and kupcase(b.promo_type_code)=kupcase("&promo_typ."));

			select sum(promo_&promo_typ._dummy) into :sum_promo from W_&wktid..&data_name._promo;
		quit;
		%if &sum_promo.>0 %then %do;
			proc sort data=W_&wktid..&data_name._promo ;
				by mid: lid: &dfr_time_field.;
			run;
			data W_&wktid..mmx_tmp1;
				merge W_&wktid..mmx_tmp1 W_&wktid..&data_name._promo;
				by mid: lid: &dfr_time_field.;
			run;
			%let promo_nm=&promo_nm &promo_typ.;
		%end;
		%else %do;
			%let promo_zero=%eval(&promo_zero-1);
			data W_&wktid..&data_name._promo;
			 set W_&wktid..&data_name._promo;
			 	drop promo_&promo_typ._dummy;
			run;
		%end;
	%end;
%mend attach_promo;
/*****************************************************/
/* Importing the price data from external .csv file  */
/*****************************************************/
%macro import_price_data(csvfile= , out=);

data &out;
 INFILE "&csvfile"
		DELIMITER=','
		MISSOVER
		DSD
		FIRSTOBS=2
		;
 LENGTH
		ITEM_RK 8
		RETAIL_OUTLET_RK 8
		TIME_PERIOD_RK 8
		ACTUAL_PRICE 8
		LIST_PRICE 8
		WEIGHTS 8
		loc_lvl 8
		merch_lvl 8
	;
	INFORMAT
		TIME_PERIOD_RK YYMMDD8.0
	;
	FORMAT
		TIME_PERIOD_RK 10.
	;
	INPUT
		ITEM_RK
		RETAIL_OUTLET_RK
		TIME_PERIOD_RK
		ACTUAL_PRICE
		LIST_PRICE
		WEIGHTS
		loc_lvl
		merch_lvl
		
	;
	
RUN;
%mend import_price_data;
/**************************************************/
/* Create price data from the imported .csv file  */
/**************************************************/

%macro create_price_data;

%import_price_data(csvfile=&path_data&slash.input&slash.&dfr_ext_price_abt,out=W_&wktid..price_abt);

data inp.Price_abt_&wktid.;
 set W_&wktid..price_abt;
  if weights < 0 or weights > 1 then delete;
  WEIGHTED_PRICE_RATIO = (WEIGHTS * ACTUAL_PRICE)/LIST_PRICE;
run;

%mend create_price_data;
/*****************************************************/
/* Importing the weather data from external .csv file  */
/*****************************************************/
%macro import_weather_data(csvfile=,out=);

data &out;
 INFILE "&csvfile"
		DELIMITER=','
		MISSOVER
		DSD
		FIRSTOBS=2
		;
 LENGTH
		RETAIL_OUTLET_RK 8
		TIME_PERIOD_RK 8
		LOW_TEMPERATURE 8
		HIGH_TEMPERATURE 8
		LOC_LVL 8
		;
INFORMAT
		TIME_PERIOD_RK YYMMDD8.0
		;
INPUT
		RETAIL_OUTLET_RK
		TIME_PERIOD_RK
		LOW_TEMPERATURE
		HIGH_TEMPERATURE
		LOC_LVL
		;
	
RUN;
%mend import_weather_data;
/********************************************/
/*Functionality- GET DATASET REFERENCE NO   */
/********************************************/

 %macro get_ds_ref_no(data=dp.F_&proj._meta,desc=);
	%global get_ds_ref_no last_ds_ref_no;
	data _null_;
		set &data. end=L;
		if kupcase("&desc")=kupcase(ds_desc) then call symputx('get_ds_ref_no',ds_ref_no);
		if L then call symputx('last_ds_ref_no',ds_ref_no);
	run;
%mend;

/********************************************/
/*Functionality- Check Remote Running SAS section */
/*and do the sign off if needed              */
/********************************************/
%macro check_RRSS(task1=,sleep_time=&dfr_hpf_wait_sec.);
	%check_again:
	%let j=1;
	%let signoff_var=;
	%let signon_var=;
	%let flag=0;
	%global task2;
	%let variable=%sysfunc(kscan(%quote(&task1.),&j));
	%do %while (%quote(&variable.) ne); /*For Each variable*/
		%if &&&variable.=0 %then %do;
			%let signoff_var=&signoff_var &variable.;
			%let flag=%eval(&flag.+1);
			%let R_rsubmit_cnt=%eval(&R_rsubmit_cnt-1);
			signoff &variable.;
		%end;
		%else %do;
			%let signon_var=&signon_var &variable.;
		%end;
		%let j=%eval(&j.+1);
		%let variable=%sysfunc(kscan(%quote(&task1.),&j));
	%end; /*END For Each variable*/
	%if &flag=0 %then %do;
		data _null_;
			a=sleep(&sleep_time,1);
		run;
		%goto check_again;
	%end;
	%else %do;
		%let task2=&signon_var;
	%end;
%mend;

/********************************************/
/*Functionality- CONVERT ID TO NAME UTILITY */
/********************************************/

%macro ID_2_Name(html_file=html);
%if &dfr_wf_optimized.=5 %then %do;
	proc sql;
		create table W_&wktid.._a_ as
		select memname,KTRIM(KLEFT(KSUBSTR(name,1,1)||
			   "&proj."||KSUBSTR(name,KINDEX(name,'_')+1)))||'_' as name, 
				type,length,'Format_' || strip(label) as label 
		from sashelp.vcolumn
		where libname=KUPCASE('STAGE')and memname=KUPCASE("F_&proj.")and 
			  (kupcase(name) like 'MID_% %' or kupcase(name) like 'LID_% %');
	quit;
	data _null_;
		set W_&wktid.._a_;
		call symputx(label,name,'g');
	run;
	data W_&wktid.._s_ ;
		length ww $ 32000;
		infile "&html_file." LRECL=32000 length=lenvar;
		input @1 ww $varying. lenvar;
		row_no=_n_;
	run;
	data W_&wktid.._s1_  W_&wktid.._s2_ ;
		set W_&wktid.._s_;
		retain flag ;
		if _n_=1 then a=KINDEX(KUPCASE(ww), '<DIV CLASS="C BYLINE">');
		if flag=0 then a=KINDEX(KUPCASE(ww), '<DIV CLASS="C BYLINE">');
		b=KINDEX(KUPCASE(ww), '</DIV>');
		if a=0 and b=0 then do;
			output W_&wktid.._s1_;
			flag=0;
		end;
		if a=0 and b ne 0 then do;
			output W_&wktid.._s1_;
			flag=0;
		end;
		if a ne 0 and b ne 0 then do;
			output W_&wktid.._s2_;
			flag=0;
		end;
		if a ne 0 and b=0 then do;
			flag=1;
			output W_&wktid.._s2_;
		end;
	run;

	%let nobs=0;
	%noobs(ds=W_&wktid.._s2_);
	%if &nobs ne 0 %then %do; 
		data W_&wktid.._s2_;
			set W_&wktid.._s2_;
			retain e 0;
			if a ne . then e=e+1;
		run;
		data W_&wktid.._s2_(keep=ww ww1 row_no );
			set W_&wktid.._s2_;
			length ww1 $ 16000;
			retain ww1;
			by  e ;
			if first.e=1 then ww1=ww;
			if first.e=0 then ww1= strip(ww1) ||' '||strip(ww);
			if last.e then output;
		run;
		data W_&wktid.._s2_(keep=ww  row_no );
			set W_&wktid.._s2_;
			ww=ww1;
		run;
		data W_&wktid.._s2_(drop =i);
			length ww $ 3200;
			set W_&wktid.._s2_;
			array temp(21) $ 320;
			array temp_val(21)  8;
			temp(1)=KSUBSTR(ww,KLENGTH('"<div class="c Byline">"')-1);
			temp(1)=KSUBSTR(temp(1),1,KLENGTH(temp(1))-6);
			do i=1 to 20;
			temp(i+1)=KSUBSTR(temp(1),1,KINDEX(temp(1),' '));
			if KINDEX(temp(i+1),'=') then do;
				temp_val(i+1)=KSUBSTR(temp(i+1),KINDEX(temp(i+1),'=')+1);
			end;
			temp(1)=KSUBSTR(temp(1),KLENGTH(temp(i+1))+2);
			end;
		run;
		data W_&wktid.._s2_;
			set W_&wktid.._s2_;
			array temp(21) $ 320;
			do i=1 to 20;
			if KINDEX(temp(i+1),'=') then do;
				temp(i+1)=KSUBSTR(temp(i+1),1,KINDEX(temp(i+1),'=')-1);
			end;
			end;
			drop temp1 temp_val1 i;
		run;
		data W_&wktid.._s2_;
			set W_&wktid.._s2_;
			array temp(21) $ 320;
			array temp_val(21)  8;
			array new_name(21) $ 320;
			do i=2 to 20;
			if temp(i) ne '' then new_name(i)=symget(strip('Format_'||strip(temp(i))));
			end;
		run;
		data _null_;
			set W_&wktid.._s2_;
			array new_name(21) $ 320;
			if _n_=1 then do;
				do i=2 to 20;
					if new_name(i) ne '' then call symput('FORMAT'||strip(i), new_name(i));
				end;
			end;
		run;
		 
		%let FORMAT=FORMAT;
		filename tmp temp;
		 
		data _null_;
			set W_&wktid.._s2_ ;
			array temp(21) $ 320;
			retain flag 0;
			do i=2 to 20;
				if temp(i) = '' and flag=0 then do;
					call symput('end_loop',i-1);
					flag=1;
				end;
			end;
			run;
		data _null_;
			length w $320;
			file tmp;
			%do App_for_i=2 %to &end_loop;
				w='if temp'|| strip("&App_for_i.") || ' ne '''' then temp'||strip("&App_for_i.")
				||'=strip(temp'||strip("&App_for_i.")||') ||'' = '' || put(temp_val'||
				strip("&App_for_i.") ||' , '|| symget("format&App_for_i");
				w=strip(w) || '.);';
				put w;
				output;
			%end;
		run;
		data W_&wktid.._s2_;
			set W_&wktid.._s2_;
			%include tmp;
		run;
		data W_&wktid.._s2_;
			set W_&wktid.._s2_;
			array temp(21) $ 320;
			length ww1 $ 3200;
			*retain ww1 ' ';
			do i=2 to &end_loop;
				ww1=strip(ww1) || '  '|| strip(temp(i)) ;
			end;
		run;
		data W_&wktid.._s2_(keep=ww row_no );;
			set W_&wktid.._s2_;
			ww='<div class="c Byline">' || strip(ww1) ||'</DIV>';
		run;
		data W_&wktid..a (keep=ww );
			merge W_&wktid.._s1_ W_&wktid.._s2_;
			by row_no;
		run;
		data _NULL_ ;
			set W_&wktid..a;
			file "&html_file.";
			put ww ;
		run;
	%end;
%end;
%mend ID_2_Name;
 
/***********************************************/
/*Functionality- Catlog entry deletion UTILITY */
/***********************************************/
 
%macro del_cat_entry(cat_name=,entry_name=);
	proc catalog catalog=&cat_name.;
		DELETE &entry_name..xml ;
	quit;
%mend del_cat_entry;

/****************************************/
/*	Functionality- WORD COUNT UTILITY 	*/
/****************************************/
 
%macro wordcnt(string=);
	/* Define and initialise local variables */
	%local word i count strngcpy;
	%let count = 0;
	/* Quote the string - this stops errors where 'string' contains a hyphen
	which would make the macro parser try to interpret it as a subtraction. */
	%let string = %quote(&string);
	%if &string = %then
		%goto EndMac;
	/* If it contains a hyphen, it denotes word(s) which are to be expanded with varexp. */
	%if %eval(%sysfunc(KINDEX(&string, -))) %then %do;
		%let i = 1;
		%let strngcpy = &string;            /* Unexploded copy of string. */
		%do %until(&word =);
			%let word = %qKSCAN(&strngcpy, &i, %str( ));
		%if &word ne %then %do;
			%if %sysfunc(KINDEX(&word, -)) %then
				%let string = %sysfunc(tranwrd(&string, &word,%varexp(vars=&word)));
				%let i = %eval(&i + 1);
			%end;
		%end;
	%end;
	/* Because the hyphens have been removed, the string can be unquoted. */
	%let string = %unquote(&string);
	%let count = 1;
	/* Skip along the string, counting distinct 'words'. */
	%do %until(&word =);
		%let word = %sysfunc(KSCAN(&string, &count, %str( )));
	%if &word ne %then
		%let count = %eval(&count + 1);
	%end;
	%let count = %eval(&count - 1);
	%EndMac:
	/* Return the count to the calling environment.*/
	&count
%mend wordcnt;
 
%macro varexp(vars=) / des='Explode var1-var3 -> var1 var2 var3';
	/* Define local and global variables. Set external error value to 1. */
	%local i word1 word2 wordlen delim pref1 pref2 suff1 suff2 pref0 zformat;
	%global __error;
	%let __error = 1;
	/* Scan vars, splitting into two words. For each word, split into prefix
	and suffix, the latter being the array value. */
	%do i = 1 %to 2;
		%let word&i = %sysfunc(KSCAN(&vars, &i, %str(- )));
		%let wordlen = %sysfunc(KLENGTH(&&word&i));

		/* Reverse the variable (e.g. var1) and find the first non-numeric
		character (r). Now the position of the last character in the prefix 
		is known (3), it's easy to split the variable into the prefix and the suffix. */

		%let delim=%eval(&wordlen-%sysfunc(KVERIFY(%sysfunc(KREVERSE(&&word&i)),0123456789))+1);
		%let pref&i=%sysfunc(KSUBSTR(&&word&i,1,&delim));
		%let suff&i=%sysfunc(KSUBSTR(&&word&i,%eval(&delim+1)),%eval(&wordlen-&delim));
	%end;
	/* The prefixes must match */
	%if &pref1 ne &pref2 %then %do;
		%put ERROR: Mismatched variable prefixes: &pref1 / &pref2..;
		%goto EndMac;
	%end;
	/* If the second suffix is less than the first, set the second to the first. */
	%if %eval(&suff2 < &suff1) %then %do;
		%put WARNING: Second suffix less than first - setting second to first.;
		%let suff2 = &suff1;
	%end;
	/* Allow for two-digit suffixes even when less than 10. */
	%if %sysfunc(KSUBSTR(&suff1, 1, 1)) = 0 %then %do;
		%let pref0 = YES;
		%let zformat = %sysfunc(KLENGTH(&suff2));
	%end;
	%else %let pref0 = NO;
	/* Set up function return value outvar. */
	%let outvar =;
	%do i = &suff1 %to &suff2;
		%if &pref0 = YES
			%then %let outvar = &outvar &pref1.%sysfunc(putn(&i, z&zformat..));
		%else 
			%let outvar = &outvar &pref1.&i;
	%end;
	%let __error = 0;
	/* Return value. */
	&outvar
	%EndMac:
%mend varexp;
 
/********************************************************************************/
/*	Functionality- Adding future records for attaching independent variables 	*/
/********************************************************************************/

%macro add_future_records(ip=inp.&data_name.);
	data &ip. (drop=i);
		set &ip.;
		%if &dfr_wf_optimized = 1 %then drop mname: lname:; ;
			%if &time_to_level.<5 %then %do; 
				if &dfr_time_field.=%eval(&dfr_horizonstart.-10) then do;
			 %end;
			 %else %do;
			 	if &dfr_time_field.=&dfr_horizonstart. then do;
			 %end;
			output;
			do i= 1 to &eng_lead. ;
				&dfr_time_field.=intnx("&dfr_interval", &dfr_time_field., 1);
				%do j=1 %to &dfr_numkpis;
					%sysfunc(kscan(&dfr_kpinameset,&j))=.;
				%end;
				output;
			end;
		end;
		else 
			output;
	run;
%mend add_future_records;

/****************************************/
/*	Functionality- To get mid and lid 	*/
/****************************************/

%macro get_IDs(from_level=,to_level=,type=);
	data _null_;
		if "&type."="M" then do;
			do i= &from_level. to &to_level.;
				call symput('keys_add_M',KTRIM(KLEFT(SYMGET("keys_add_M")))||"  " || 
						    KTRIM(KLEFT(symget("type")))||'ID_'|| KTRIM(KLEFT(i)));
			end;
		end;
		if "&type."="L" and "&keys_add_L" eq "" then do;
			do i= &from_level. to &to_level.;
				call symput('keys_add_L',KTRIM(KLEFT(SYMGET("keys_add_L")))||"  " || 
							 KTRIM(KLEFT(symget("type")))||'ID_'|| KTRIM(KLEFT(i)));
			end;
		end;
	run;
%mend get_IDs;

/********************************************************************/
/*	Functionality- Macro for setting up pantry loading parameters 	*/
/********************************************************************/

%macro set_pantry_flags(in_data=,out_data=,promos=,pl_prd=);
	data _null_;
		infile temp1 mod ;
		file temp1;
		put "data &out_data. ;";
		put "set &in_data. ;";
		length s $500;
		length t $500;
		do i=1 to &promos.;
			promo_typ=KSCAN("&dfr_data_promo_types.",i);
			call symput('promo_typ',promo_typ);
			s= 'if lag(promo_'||KTRIM(KLEFT(promo_typ))||'_dummy)=1 and promo_'
				||KTRIM(KLEFT(promo_typ))||'_dummy=0 then pantry_'
				||KTRIM(KLEFT(promo_typ))||'_dummy1=1; else pantry_'
				||KTRIM(KLEFT(promo_typ))||'_dummy1=0;';
			put s;
		end;
		do i=1 to &promos.;
			promo_typ=KSCAN("&dfr_data_promo_types.",i);
			do j=1 to &pl_prd.;
				s= 'if lag(pantry_'||KTRIM(KLEFT(promo_typ))||'_dummy' 
					|| KTRIM(KLEFT(j))|| ')=1 then pantry_'||KTRIM(KLEFT(promo_typ))
					||'_dummy' ||KTRIM(KLEFT(j+1))  ||'=1; else pantry_'
					||KTRIM(KLEFT(promo_typ))||'_dummy' || KTRIM(KLEFT(j+1))||'=0;';
				put s;
			end ;
		end;
		put 'run;';
	run;
	%RCSet (&syserr);
%mend set_pantry_flags;

/****************************************************************************/
/*	Functionality- Macro for setting up customer delay effect variables 	*/
/****************************************************************************/

%macro set_cd_flags(in_data=,out_data=,promos=,cd_prd=);
	data _null_;
		infile temp1 mod ;
		file temp1;
		put "data &out_data. ;";
		put "set &in_data. ;";
		length t $500;
		do i=1 to &promos.;
			promo_typ=KSCAN("&dfr_data_promo_types.",i);
			t= 'if lag(promo_'|| KTRIM(KLEFT(promo_typ)) 
				||'_dummy)=1 and promo_'||KTRIM(KLEFT(promo_typ)) 
				||'_dummy=0 then delay_'||KTRIM(KLEFT(promo_typ))
				||'_dummy1=1; else delay_'||KTRIM(KLEFT(promo_typ))||'_dummy1=0;';
			put t;
		end;
		do i=1 to &promos.;
			promo_typ=KSCAN("&dfr_data_promo_types.",i);
			do j=1 to &cd_prd.;
				t= 'if lag(delay_'||KTRIM(KLEFT(promo_typ))
					||'_dummy' || KTRIM(KLEFT(j))|| ')=1 then delay_'
					||KTRIM(KLEFT(promo_typ))||'_dummy' ||KTRIM(KLEFT(j+1))  
					||'=1; else delay_'||KTRIM(KLEFT(promo_typ))
					||'_dummy'|| KTRIM(KLEFT(j+1))||'=0;';
				put t;
			end ;
		end;
		put 'run;';
	run;
%RCSet (&syserr);
%mend set_cd_flags;
 
/***************************************************************************************/
/*Functionality- Read the META table and initialize MERCH/LOC levels and IDs and  KPI  */
/***************************************************************************************/
 
%macro getmetabyrow(proj=&proj.,row=,In_data=dp.F_&proj._meta);
/* load current row parameters from Meta */
	data _null_;
		set  &In_data;
		if _n_=&row. then do;
			call symputx("dataname",ds_name,'g');
			call symputx("datalibrary",ds_lib,'g');
			call symputx("datadesc",ds_desc,'g');
			call symputx("Merch_level",M_LVL,'g');
			call symputx("Location_level",L_lvl,'g');
			call symputx("Merch_ID",M_id,'g');
			call symputx("Location_ID",L_id,'g');
			call symputx("Ref_Merch_ID",Ref_Merch_ID,'g');
			call symputx("Ref_Location_ID",Ref_Location_ID,'g');
			call symputx("Group_Key",group_key,'g');
			call symputx("p_split",p_split,'g');
			call symputx("kpi",kpi,'g');
		end;
	run;
%mend getmetabyrow;
/***************************************************************************************/
/*Functionality- Read the META table and initialize MERCH/LOC levels and IDs and  KPI  */
/***************************************************************************************/
 
%macro getmetabyrow_B(proj=&proj.,row=,In_data=dp.F_&proj._meta);
/* load current row parameters from Meta */
	data _null_;
		set  &In_data;
		if _n_=&row. then do;
			call symputx("dataname",ds_name,'g');
			call symputx("datalibrary",ds_lib,'g');
			call symputx("datadesc",ds_desc,'g');
			call symputx("Merch_level",M_LVL,'g');
			call symputx("Location_level",L_lvl,'g');
			call symputx("Merch_ID",M_id,'g');
			call symputx("Location_ID",L_id,'g');
			call symputx("Ref_Merch_ID",Ref_Merch_ID,'g');
			call symputx("Ref_Location_ID",Ref_Location_ID,'g');
			call symputx("Group_Key",group_key,'g');
			call symputx("p_split",p_split,'g');
			call symputx("kpi",kpi,'g');
			call symputx("split_cnt",split_cnt,'g');
			call symputx("number_split",number_split,'g');
		end;
	run;
%mend getmetabyrow_b;

/*******************************************************************************************/
/*Functionality- Read the jobsparam table and initialize MERCH/LOC levels and IDs and  KPI */
/*******************************************************************************************/

%macro getmetabyrow_split(proj=&proj.,row=);
/* load current row parameters from jobsparam and 
intialize merch/loc levels and IDs and KPIs */
	data _null_;
		set W_&wktid..Jobparam_&proj.;
		if _n_=&row. then do;
			call symputx("MERCH_LEVEL",MERCH_LEVEL,'g');
			call symputx("MERCH_ID",MERCH_ID,'g');
			call symputx("LOCATION_LEVEL",LOCATION_LEVEL,'g');
			call symputx("LOCATION_ID",LOCATION_ID,'g');
			call symputx("FIELDKEY",FIELDKEY,'g');
			call symputx("Ref_Merch_ID",Ref_Merch_ID,'g');
			call symputx("Ref_Location_ID",Ref_Location_ID,'g');
			call symputx("Group_Key",group_key,'g');
		end;
	run;
%mend getmetabyrow_split;
/*******************************************************************************************/
/*Functionality- Read the jobsparam table and initialize dfr_ind_var_space				 */
/*******************************************************************************************/
%macro ind_var_name;
	%let i=1;
	%global ind_var_name_space ind_var_name_comma ind_var_name_mean;
	%let ind_var_name_space=;
	%let ind_var_name_comma=;
	%let ind_var_name_mean=;
	%let variable=%sysfunc(kscan(%quote(&dfr_ind_var),&i,%str(,)));
	%do %while (%quote(&variable.) ne); 
		%let variable2=%sysfunc(kscan(%quote(&variable),3,%str(.)));
		%let variable1=%sysfunc(kscan(%quote(&variable),1,%str(.)));
		%if %sysfunc(kupcase(&variable1))=MAXDATA %then %let variable2=H1_&variable2.;
		%let ind_var_name_space=&ind_var_name_space &variable2;
		%let ind_var_name_comma=&ind_var_name_comma, &variable2;
		%let ind_var_name_mean=&ind_var_name_mean mean(&variable2)=;
		%let i=%eval(&i.+1);
		%let variable=%sysfunc(kscan(%quote(&dfr_ind_var),&i,%str(,)));
	%end;
%mend ind_var_name;
/*****************************************************************************/
/*Functionality- 	Read the FORECAST META table and initialize method, 	 */
/*					and setting method,base and top parameters 				 */
/*****************************************************************************/

%macro getmetabyrow_for(proj=&proj.,row_no=);
/* For reading the forecast metadata table row by row*/
	data _null_;
		set W_&wktid..F_&proj._Forecast_meta ;
		if _n_=&row_no. then do;
			call symputx("method",method,'g');
			if method='BU' then do;
				call symputx("out_ds",KSUBSTR(ds_name2,2),'g');
				ds_name='recon.'||ds_name;
				ds_name2='for.'||ds_name2;
				call symputx("base",ds_name,'g');
				call symputx("top",ds_name2,'g');
			end;
			else do;
				call symputx("out_ds",KSUBSTR(ds_name,2),'g');
				ds_name='for.'||ds_name;
				ds_name2='recon.'||ds_name2;
				call symputx("base",ds_name,'g');
				call symputx("top",ds_name2,'g');
			end;
		end;
	run;
%mend getmetabyrow_for;

/************************************************************************/
/*	Functionality- To check if a given dataset has any rows in it 		*/
/************************************************************************/

%macro noobs(ds=);
	%if %sysfunc(exist(&ds)) %then  %do;
		data _null_;
			set &ds. end=L;
			if L then call symput('nobs',_n_);
		run;
	%end;
%mend noobs;
 
/********************************************************************/
/*	Functionality- To get a string of comma separated KPI names 	*/
/********************************************************************/

%macro drop_var(drop1=1);
	%let withoutdrop1=;
	%let drop_var1=;
	data _null_;
		do i=1 to &dfr_numkpis;
			if &drop1 ne i
			then call symput('withoutdrop1',KTRIM(KLEFT(SYMGET("withoutdrop1"))) 
							||"  " ||KSCAN("&dfr_kpinameset", i) );
			else call symput('drop_var1',KSCAN("&dfr_kpinameset", i));
		end;
	run;
	data _null_;
		Call symputx("withoutdrop","&withoutdrop1",'g');
		call symputx("drop_kpi_var","&drop_var1",'g');
	run;
%mend drop_var;

/********************************************************************************************/
/*Functionality- Add new records and target and updating existing records based on a key	*/
/********************************************************************************************/
 
/* 	If update_flag=Y, then a column called '_error1' will be available in target table.
	If _error1=2, then the key exists in the source and target tables
	If _error1=1, then the key exist in the target table but not present in source and hence untouched
	If _error1=0, then the key does not exist in the target table but present in source and hence added
	In short 2=update and 0=insert */

%macro Addnew_and_KUPDATE(source=,source_lib=W_&wktid.,
						  target=,target_lib=W_&wktid.,
						  keys=,Update_flag=N,keynum=);
	data _temp_source_ ;
		set &source_lib..&source. ;
		_error1=0;
	run;
	data _temp_target_ ;
		set &target_lib..&target. ;
		_error1=1;
	run;
	data _target_;
		set _temp_target_ _temp_source_ ;
	run;
	proc sort data=_target_ ;
		by &keys. _error1;
	run;

	%let lastkey=%sysfunc(KSCAN(&keys.,&keynum.));
	data &target_lib..&target. ;
		set _target_;
		by &keys;
		if first.&lastkey. and last.&lastkey.= 0 then do;_error1=2;end;
		if first.&lastkey. then output;
	run;
	 
	%if &Update_flag.=N %then %do;
		data &target_lib..&target.(drop=_error1);
			set &target_lib..&target.;
		run;
	%end;
	proc sql;
		drop table _temp_target_,_temp_source_,_target_;
	quit;
%mend Addnew_and_KUPDATE;

/********************************************************************************/
/*	Functionality- Add new records and target and updating existing records 	*/
/********************************************************************************/

/* 	If update_flag=Y, then a column called '_error1' will be available in target table.
	If _error1=2, then key exists in the source and target tables
	If _error1=1, then key exist in the target table but not present in source & hence untouched
	If _error1=0, then key does not exist in the target table but present in source and hence added
	In short 2=update and 0=insert 	*/

%macro Addnew_and_Update_nokey(source=,source_lib=W_&wktid.,target=,target_lib=W_&wktid.,keys=);
	data _temp_source_ ;
		set &source_lib..&source. ;
		_error2=0;
	run;
	%if %sysfunc(exist(&target_lib..&target.))=0 %then %do;
		data _temp_target_ ;
			set &source_lib..&source. ;
			_error3=1;
			stop;
		run;
	%end;
	%else %do;
		data _temp_target_ ;
			set &target_lib..&target. ;
			_error3=1;
		run;
	%end;
	data _target_;
		set _temp_target_ _temp_source_ ;
	run;
	proc sort data=_temp_target_;
		by &keys.;
	run;
	proc sort data=_temp_source_;
		by &keys.;
	run;
	data _source_ ;
		merge _temp_target_(in=_a)  _temp_source_(in=_b)  ;
		by &keys.;
		if  _a and  _b then delete;
	run ;
	data &target_lib..&target. ;
		merge   _temp_source_ _source_ ;
		by &keys.;
		drop  _error2 _error3;
	run;
	proc sql;
		drop table _temp_target_,_temp_source_, _target_, _source_;
	quit;
%mend Addnew_and_Update_nokey;

/************************************************************************/
/*	Functionality- To initialize variable like promo_type_code or KPIs	*/
/************************************************************************/
 
%macro get_name_KINDEX(var=,varno=,name=);
	data _null_;
		do j=1 to &varno;
			a="&name"||strip(j) ;
			call symputx(a,kscan("&var",j),'g');
			output;
		end;
	run;
%mend;

/************************************************************/
/*	Functionality- Get comma separated list of variables	*/
/************************************************************/
 
%macro words(string=,cnt=);
	%let count=1;
	%let replace=",";
	%let word=%qKSCAN(&string,&count,%str( ));
	%let newst=&word;
	%do i=1 %to &cnt;
		%let count=%eval(&count+1);
		%let word=%qKSCAN(&string,&count,%str( ));
		%let newst=&newst.&replace.&word.;
	%end;
	%let newst="&newst";
%mend words;

/************************************************/
/*	Functionality- create work library folder	*/
/************************************************/

%macro create_work_dir(path=,name_dir=);
	data _null_;
	    rc=filename("mydir","&path.&slash.&name_dir");
	    did=dopen("mydir");
		put did;
	    if (did = 0) then   do;
			a=ktrim("&name_dir");
	        NewDirectory=dcreate(a,"&path");
	    end;
	run;
%mend create_work_dir;
/************************************************/
/*	Functionality- To check Lock is availabe on */
/*					the dataset if not lock it	*/
/************************************************/
%macro LockData(member=,timeout=10,retry=5);
	%local starttime;
	%let starttime = %sysfunc(datetime());
	%let lockit=0;
	%let try_no=1;
	%do %until( &lockit=1 or &try_no >100);
		data _null_;
			dsid = 0;
			dsid = open("&member");
			format s datetime.;
			s=datetime();
			put s;
			put dsid;
			if (dsid = 0) then do;
				call symputx('lockit',0);
				rc = sleep(&retry);
			end;
			if (dsid > 0) then do;
				rc = close(dsid);
				call symputx('lockit',1);
			end;
		run;
		%let try_no=%eval(&try_no+1);
		%put &try_no;
		%if &lockit=1 %then lock &member;
	%end;
%mend LockData;

/*************************************************/
/*	Functionality- main_report        */
/* creat Report file                  */
/*************************************************/

%macro main_report();
%create_work_dir(path=&path_plots,name_dir=&projid.);
filename _webout "&path_plots.&slash&projid.&slash.contents.html";
Data _null_;
file _webout;
set dp.F_&projid._meta end=L;
length ww $2200;
if _n_=1 then do;
	Put '<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">';
	Put '<html>';
	Put '<head>';
	Put '<meta name="Generator" content="SAS Software, see www.sas.com" sasversion="9.1">';
	Put '<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">';
	Put '<title>SAS Output</title>';
	Put '<style type="text/css">';
	Put '<!--';
	Put '.ContentTitle';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: italic;';
	Put 'color: #002288;';
	Put '}';
	Put '.Output';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put 'border-width: 1px;';
	Put '}';
	Put '.HeaderEmphasis';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: italic;';
	Put 'color: #0033AA;';
	Put '}';
	Put '.NoteContent';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put '}';
	Put '.Graph';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put 'border-width: 1px;';
	Put 'border-color: #000000;';
	Put '}';
	Put '.WarnContentFixed';
	Put '{';
	Put 'font-family: Courier;';
	Put 'font-size: x-small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put '}';
	Put '.NoteBanner';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put '}';
	Put '.FatalContent';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put '}';
	Put '.DataStrong';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: medium;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #000000;';
	Put '}';
	Put '.Document';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put '}';
	Put 'a:link {';
	Put 'color:#0066AA';
	Put '}';
	Put 'a:visited {';
	Put 'color:#004488';
	Put '}';
	Put 'a:active {';
	Put 'color:#004488';
	Put '}';
	Put '.BeforeCaption';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: medium;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #000000;';
	Put '}';
	Put '.ContentsDate';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put 'width: 100%;';
	Put '}';
	Put '.Pages';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put ' ';
	Put 'margin-left: 8px;';
	Put 'margin-right: 8px;';
	Put 'list-style-type: decimal;';
	Put '}';
	Put '.TitlesAndFooters';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: medium;';
	Put 'font-weight: bold;';
	Put 'font-style: italic;';
	Put 'color: #002288;';
	Put '}';
	Put '.IndexProcName';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put 'list-style-type: decimal;';
	Put '}';
	Put '.ProcTitle';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: medium;';
	Put 'font-weight: bold;';
	Put 'font-style: italic;';
	Put 'color: #002288;';
	Put '}';
	Put '.IndexAction';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put 'margin-left: 6pt;';
	Put 'list-style-type: NONE;';
	Put '}';
	Put '.Data';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #000000;';
	Put '}';
	Put '.Table';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put 'border-width: 1px;';
	Put 'border-color: #000000;';
	Put '}';
	Put '.top_stacked_value';
	Put '{';
	Put 'padding-bottom: 1px;';
	Put '}';
	Put '.middle_stacked_value';
	Put '{';
	Put 'padding-top: 1px;';
	Put 'padding-bottom: 1px;';
	Put '}';
	Put '.bottom_stacked_value';
	Put '{';
	Put 'padding-top: 1px;';
	Put '}';
	Put '.FooterEmpty';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: medium;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put '}';
	Put '.SysTitleAndFooterContainer';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put 'border-width: 0;';
	Put 'width: 100%;';
	Put '}';
	Put '.RowFooterEmpty';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: medium;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put '}';
	Put '.ContentFolder';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put 'list-style-type: NONE;';
	Put '}';
	Put '.RowFooterEmphasis';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: italic;';
	Put 'color: #0033AA;';
	Put '}';
	Put '.RowFooterStrongFixed';
	Put '{';
	Put 'font-family: Courier New, Courier, monospace;';
	Put 'font-size: x-small;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put '}';
	Put '.RowFooterFixed';
	Put '{';
	Put 'font-family: Courier;';
	Put 'font-size: x-small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put '}';
	Put '.Index';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put '}';
	Put '.FooterFixed';
	Put '{';
	Put 'font-family: Courier;';
	Put 'font-size: x-small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put '}';
	Put '.ExtendedPage';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: italic;';
	Put 'color: #002288;';
	Put 'border-width: 1pt;';
	Put 'text-align: center;';
	Put '}';
	Put '.Container';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put '}';
	Put '.Date';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put 'width: 100%;';
	Put '}';
	Put '.Caption';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: medium;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #000000;';
	Put '}';
	Put '.WarnBanner';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put '}';
	Put '.Frame';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put '}';
	Put '.HeaderStrongFixed';
	Put '{';
	Put 'font-family: Courier New, Courier, monospace;';
	Put 'font-size: x-small;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put '}';
	Put '.IndexTitle';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: italic;';
	Put 'color: #002288;';
	Put '}';
	Put '.NoteContentFixed';
	Put '{';
	Put 'font-family: Courier;';
	Put 'font-size: x-small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put '}';
	Put '.DataEmphasisFixed';
	Put '{';
	Put 'font-family: Courier New, Courier, monospace;';
	Put 'font-size: x-small;';
	Put 'font-weight: normal;';
	Put 'font-style: italic;';
	Put 'color: #000000;';
	Put '}';
	Put '.Note';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put 'background-color: #E0E0E0;';
	Put '}';
	Put '.Byline';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: medium;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put '}';
	Put '.FatalBanner';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put '}';
	Put '.ProcTitleFixed';
	Put '{';
	Put 'font-family: Courier New, Courier, monospace;';
	Put 'font-size: x-small;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put '}';
	Put '.ByContentFolder';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put 'list-style-type: NONE;';
	Put '}';
	Put '.PagesProcLabel';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put 'list-style-type: decimal;';
	Put '}';
	Put '.RowHeaderFixed';
	Put '{';
	Put 'font-family: Courier;';
	Put 'font-size: x-small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put ' ';
	Put '}';
	Put '.RowFooterEmphasisFixed';
	Put '{';
	Put 'font-family: Courier New, Courier, monospace;';
	Put 'font-size: x-small;';
	Put 'font-weight: normal;';
	Put 'font-style: italic;';
	Put 'color: #0033AA;';
	Put ' ';
	Put '}';
	Put '.WarnContent';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put '}';
	Put '.DataEmpty';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #000000;';
	Put '}';
	Put '.Cell';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put 'background-color: #E0E0E0;';
	Put '}';
	Put '.Header';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: medium;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put ' ';
	Put '}';
	Put '.PageNo';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: medium;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put 'background-color: #E0E0E0;';
	Put 'text-align: right;';
	Put 'vertical-align: top;';
	Put '}';
	Put '.ContentProcLabel';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put 'list-style-type: decimal;';
	Put '}';
	Put '.HeaderFixed';
	Put '{';
	Put 'font-family: Courier;';
	Put 'font-size: x-small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put ' ';
	Put '}';
	Put '.PagesTitle';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: italic;';
	Put 'color: #002288;';
	Put '}';
	Put '.RowHeaderEmpty';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: medium;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put ' ';
	Put '}';
	Put '.PagesProcName';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put 'list-style-type: decimal;';
	Put '}';
	Put '.Batch';
	Put '{';
	Put 'font-family: SAS Monospace, Courier New, Courier, monospace;';
	Put 'font-size: x-small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #000000;';
	Put 'background-color: #D3D3D3;';
	Put 'border-width: 1px;';
	Put 'border-color: #000000;';
	Put '}';
	Put '.ContentItem';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put 'list-style-type: NONE;';
	Put '}';
	Put '.Body';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put 'background-color: #E0E0E0;';
	Put 'margin-left: 8px;';
	Put 'margin-right: 8px;';
	Put '}';
	Put '.PagesDate';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put ' ';
	Put 'width: 100%;';
	Put '}';
	Put '.HeaderEmpty';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: medium;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put ' ';
	Put '}';
	Put '.FooterStrong';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: medium;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put ' ';
	Put '}';
	Put '.FooterEmphasis';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: italic;';
	Put 'color: #0033AA;';
	Put ' ';
	Put '}';
	Put '.ErrorContent';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put 'background-color: #E0E0E0;';
	Put '}';
	Put '.DataFixed';
	Put '{';
	Put 'font-family: Courier;';
	Put 'font-size: x-small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #000000;';
	Put 'background-color: #D3D3D3;';
	Put '}';
	Put '.HeaderStrong';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: medium;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put ' ';
	Put '}';
	Put '.DataEmphasis';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: italic;';
	Put 'color: #000000;';
	Put 'background-color: #D3D3D3;';
	Put '}';
	Put '.TitleAndNoteContainer';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put 'background-color: #E0E0E0;';
	Put 'border-width: 0;';
	Put 'width: 100%;';
	Put '}';
	Put '.RowFooter';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: medium;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put ' ';
	Put '}';
	Put '.IndexItem';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put 'margin-left: 6pt;';
	Put 'list-style-type: NONE;';
	Put '}';
	Put '.BylineContainer';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put 'background-color: #E0E0E0;';
	Put 'border-width: 0;';
	Put 'width: 100%;';
	Put '}';
	Put '.FatalContentFixed';
	Put '{';
	Put 'font-family: Courier;';
	Put 'font-size: x-small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put 'background-color: #E0E0E0;';
	Put '}';
	Put '.BodyDate';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put 'background-color: #E0E0E0;';
	Put 'width: 100%;';
	Put 'text-align: right;';
	Put 'vertical-align: top;';
	Put '}';
	Put '.Contents';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put ' ';
	Put 'margin-left: 8px;';
	Put 'margin-right: 8px;';
	Put 'list-style-type: decimal;';
	Put '}';
	Put '.RowFooterStrong';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: medium;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put ' ';
	Put '}';
	Put '.UserText';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put 'background-color: #E0E0E0;';
	Put '}';
	Put '.HeadersAndFooters';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: medium;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put ' ';
	Put '}';
	Put '.RowHeaderEmphasisFixed';
	Put '{';
	Put 'font-family: Courier New, Courier, monospace;';
	Put 'font-size: x-small;';
	Put 'font-weight: normal;';
	Put 'font-style: italic;';
	Put 'color: #0033AA;';
	Put ' ';
	Put '}';
	Put '.ErrorBanner';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put 'background-color: #E0E0E0;';
	Put '}';
	Put '.ContentProcName';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put 'list-style-type: decimal;';
	Put '}';
	Put '.RowHeaderStrong';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: medium;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put 'background-color: #FFFFFF;';
	Put '}';
	Put '.PagesItem';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put 'list-style-type: NONE;';
	Put '}';
	Put '.RowHeader';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: medium;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put ' ';
	Put '}';
	Put '.FooterStrongFixed';
	Put '{';
	Put 'font-family: Courier New, Courier, monospace;';
	Put 'font-size: x-small;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put ' ';
	Put '}';
	Put '.AfterCaption';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: medium;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #000000;';
	Put 'background-color: #E0E0E0;';
	Put '}';
	Put '.RowHeaderStrongFixed';
	Put '{';
	Put 'font-family: Courier New, Courier, monospace;';
	Put 'font-size: x-small;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put ' ';
	Put '}';
	Put '.RowHeaderEmphasis';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: italic;';
	Put 'color: #0033AA;';
	Put ' ';
	Put '}';
	Put '.DataStrongFixed';
	Put '{';
	Put 'font-family: Courier New, Courier, monospace;';
	Put 'font-size: x-small;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #000000;';
	Put 'background-color: #D3D3D3;';
	Put '}';
	Put '.Footer';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: medium;';
	Put 'font-weight: bold;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put ' ';
	Put '}';
	Put '.FolderAction';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #0033AA;';
	Put 'margin-left: 6pt;';
	Put 'list-style-type: NONE;';
	Put '}';
	Put '.HeaderEmphasisFixed';
	Put '{';
	Put 'font-family: Courier New, Courier, monospace;';
	Put 'font-size: x-small;';
	Put 'font-weight: normal;';
	Put 'font-style: italic;';
	Put 'color: #0033AA;';
	Put ' ';
	Put '}';
	Put '.SystemTitle';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: large;';
	Put 'font-weight: bold;';
	Put 'font-style: italic;';
	Put 'color: #002288;';
	Put 'background-color: #E0E0E0;';
	Put '}';
	Put '.ErrorContentFixed';
	Put '{';
	Put 'font-family: Courier;';
	Put 'font-size: x-small;';
	Put 'font-weight: normal;';
	Put 'font-style: normal;';
	Put 'color: #002288;';
	Put 'background-color: #E0E0E0;';
	Put '}';
	Put '.SystemFooter';
	Put '{';
	Put 'font-family: Arial, Helvetica, sans-serif;';
	Put 'font-size: large;';
	Put 'font-weight: bold;';
	Put 'font-style: italic;';
	Put 'color: #002288;';
	Put 'background-color: #E0E0E0;';
	Put '}';
	Put '.FooterEmphasisFixed';
	Put '{';
	Put 'font-family: Courier New, Courier, monospace;';
	Put 'font-size: x-small;';
	Put 'font-weight: normal;';
	Put 'font-style: italic;';
	Put 'color: #0033AA;';
	Put ' ';
	Put '}';
	Put '.l {text-align: left }';
	Put '.c {text-align: center }';
	Put '.r {text-align: right }';
	Put '.d {text-align: "." }';
	Put '.t {vertical-align: top }';
	Put '.m {vertical-align: middle }';
	Put '.b {vertical-align: bottom }';
	Put 'TD, TH {vertical-align: top }';
	Put '.stacked_cell{padding: 0 }';
	Put '-->';
	Put '</style>';
	Put '<style type="text/css">';
	Put '<!--';
	Put '/* Outline Style Sheet */';
	Put ' ';
	Put 'UL { cursor: hand;';
	Put 'list-style-type: decimal;}';
	Put ' ';
	Put 'DL { cursor: hand;';
	Put 'list-style-type: none;}';
	Put ' ';
	Put '// so Netscape wont indent so far';
	Put 'DL {marginLeft: -12pt}';
	Put ' ';
	Put 'SPAN {cursor: hand}';
	Put 'body {';
	Put "	background-color: %sysfunc(sasmsg(catalog.dfrmsg, DFR_REPORT_BG_COLOR, noquote));";
	Put '}';
	Put '-->';
	Put '</style>';
	Put ' ';
	Put '<script language="javascript" type="text/javascript">';
	Put '<!--';
	Put '// This script controls the expanding';
	Put '// and collapsing of lists in MSIE 4.0.';
	Put '//';
	Put '// Check for MSIE 4.0.';
	Put 'var msie4 = 0;';
	Put 'var ver = parseInt(navigator.appVersion);';
	Put 'if (navigator.userAgent.indexOf("MSIE") > -1 & ver >= 4){';
	Put 'msie4 = 1;';
	Put '// Add style sheet info that screws up Netscape 4';
	Put 'document.write(''<style type="text/css"> UL DL ';
	Put '{ display:none; cursor: hand; list-style-type: none; margin-left: 10pt}  </style>'');';
	Put ' ';
	Put '// Set document events';
	Put 'document.onclick = outline;';
	Put 'document.onmouseover = doAction;';
	Put 'document.onmouseout = undoAction;';
	Put ' ';
	Put '// Initialize expander';
	Put 'var display_mode = ''none''';
	Put ' ';
	Put '// Set images to use for folders';
	Put 'var openImage = '''';';
	Put 'var closeImage = '''';';
	Put ' ';
	Put '// Preload images so they''re in the cache';
	Put 'image1 = new Image ();';
	Put 'image1.src = openImage;';
	Put 'image2 = new Image ();';
	Put 'image2.src = closeImage;';
	Put '}';
	Put ' ';
	Put '// Change folder to open or close as appropriate';
	Put 'function changeFolder(actionTag, tagPos) {';
	Put '// Check all tags encapsulated by the clicked tag';
	Put 'while (actionTag.contains(document.all[tagPos])){';
	Put '// If a UL or DL is hit, bail out';
	Put 'if (document.all[tagPos].tagName == ''DL'' || document.all[tagPos].tagName == ''UL'') return;';
	Put ' ';
	Put '// If an image is hit, check for a folder and change as needed';
	Put 'if (document.all[tagPos].tagName == ''IMG''){';
	Put '// Check for closed folder';
	Put 'if (document.all[tagPos].src.indexOf(closeImage) > -1){';
	Put 'document.all[tagPos].src = openImage;';
	Put 'break;';
	Put '}';
	Put '// Check for open folder';
	Put 'else if (document.all[tagPos].src.indexOf(openImage) > -1){';
	Put 'document.all[tagPos].src = closeImage;';
	Put 'break;';
	Put '}';
	Put '}';
	Put 'tagPos++;';
	Put '// If you reach the end of the document, bail out.';
	Put 'if (tagPos >= document.all.length-1) return;';
	Put '}';
	Put '}';
	Put ' ';
	Put '// Source Code for Outlining';
	Put 'function checkParent(src) {';
	Put '// Search for a specific parent of the current element';
	Put 'while (src != null) {';
	Put '// If an LI or DT is clicked, return src.  If not,';
	Put '//  look for at parent elements until you find';
	Put '//  one, and return it.';
	Put 'if (src.tagName == ''LI'' || src.tagName == ''DT'') return src;';
	Put 'src = src.parentElement;';
	Put '}';
	Put '// If there are no LIs or DTs in the current element or parent';
	Put '//  elements, return null.';
	Put 'return null;';
	Put '}';
	Put ' ';
	Put 'function outline() {';
	Put '// Make sure clicked inside an LI or DT.';
	Put '// This test allows rich HTML inside lists.';
	Put 'var clickedElement = checkParent(event.srcElement);';
	Put ' ';
	Put 'if (clickedElement != null) {';
	Put '// Get the position of the clicked HTML element';
	Put 'var pos = clickedElement.sourceIndex+1;';
	Put ' ';
	Put '// Change folder image as necessary';
	Put 'if (closeImage != '''' & openImage != '''') changeFolder(clickedElement, pos);';
	Put ' ';
	Put '// Starting at the first HTML element index after the';
	Put '//  clicked element, look at every HTML element until';
	Put '//  another UL is found and break.';
	Put 'while (clickedElement.contains(document.all[pos])){';
	Put 'if (document.all[pos].tagName == ''DL'' || document.all[pos].tagName == ''UL''){';
	Put '// If you get this far, toggle the display element of the tag';
	Put 'toggleView(pos);';
	Put 'break;';
	Put '}';
	Put 'pos++;';
	Put 'if (pos >= document.all.length-1) return;';
	Put '}';
	Put '}else{';
	Put 'return;';
	Put '}';
	Put ' ';
	Put 'event.cancelBubble = true;';
	Put '}';
	Put ' ';
	Put 'function toggleView(pos){';
	Put '// Now that we''ve found the next UL, decide whether it';
	Put '//  should be visible or hidden.';
	Put 'if (document.all[pos].style.display == '''' ||';
	Put 'document.all[pos].style.display == ''none'') {';
	Put 'document.all[pos].style.display = ''block'';';
	Put '}else{';
	Put 'document.all[pos].style.display = ''none'';';
	Put '}';
	Put '}';
	Put ' ';
	Put '// Display action class when mouse is over SPAN';
	Put 'function doAction(){';
	Put 'if (event.srcElement != null & event.srcElement.tagName == ''SPAN''){';
	Put 'event.srcElement.className = ''action'';';
	Put '}';
	Put '}';
	Put ' ';
	Put '// Display no class when mouse moves off of SPAN';
	Put 'function undoAction(){';
	Put 'if (event.srcElement != null & event.srcElement.tagName == ''SPAN''){';
	Put 'event.srcElement.className = '''';';
	Put '}';
	Put '}';
	Put ' ';
	Put '// Expand all lists and display all folders';
	Put 'function expandAll(level){';
	Put 'display_mode = (display_mode == ''none'') ? ''block'' : ''none''';
	Put 'setDisplay(''DL'', level)';
	Put 'setDisplay(''UL'', level)';
	Put '}';
	Put '// Set the display to the proper mode.';
	Put '// list_type is the tagname of the list to look at.  level';
	Put '// is the maximum nest level to display/hide.';
	Put 'function setDisplay(list_type, level) {';
	Put '// If level is < 0 or undefined, expand/contract all lists.';
	Put 'if (!level || level < 0) level = document.all.length';
	Put ' ';
	Put 'for(i=0; i < document.all.tags(list_type).length; i++){';
	Put 'level_count = 0';
	Put 'object = document.all.tags(list_type)[i]';
	Put 'pos = document.all.tags(list_type)[i].sourceIndex';
	Put ' ';
	Put '// Start at the current list object and work your way';
	Put '// out until we hit a body tag.  Keep track of all of the';
	Put '// lists we hit on the way out, so we know how deep we are.';
	Put 'while (object.tagName != ''BODY'') {';
	Put 'if (object.tagName == ''DL'' || object.tagName == ''UL'')';
	Put 'level_count++';
	Put '// If we''re already nested too deep, break out.';
	Put 'if (level_count > level) break';
	Put 'object = object.parentElement';
	Put '}';
	Put ' ';
	Put '// If we are in legal range, change the display.';
	Put 'if (level_count <= level & level_count > 0) {';
	Put 'document.all[pos].style.display = display_mode';
	Put ' ';
	Put 'if (level_count != level & closeImage != '''' & openImage != '''') {';
	Put 'object = document.all[++pos]';
	Put ' ';
	Put '// Check for open/close folders to change.';
	Put 'while (pos < document.all.length & object.tagName != ''IMG'') {';
	Put 'if (object.tagName == ''UL'' || object.tagName == ''DL'')';
	Put 'break';
	Put 'object = document.all[++pos]';
	Put '}';
	Put ' ';
	Put '// Change the image.';
	Put 'if (pos < document.all.length & object.tagName == ''IMG'' &';
	Put '(object.src.indexOf(closeImage) > -1 ||';
	Put 'object.src.indexOf(openImage)  > -1)) {';
	Put 'if (!display_mode || display_mode == ''none'')';
	Put 'object.src = closeImage';
	Put 'else';
	Put 'object.src = openImage';
	Put '}';
	Put '}';
	Put '}';
	Put '}';
	Put '}';
	Put ' ';
	Put '//-->';
	Put '</script>';
	Put '<noscript></noscript>';
	Put ' ';
	Put '<style type="text/javascript">';
	Put '<!--';
	Put 'contextual(tags.UL, tags.DL).display = ''block'';';
	Put '//-->';
	Put '</style>';
	Put '</head>';
	Put '<body class="Contents"  onload="if (msie4 == 1)expandAll()">';
	Put '<span class="ContentTitle"><span onClick="if(msie4==1)expandAll()">';
	Put 'Table of Contents';
	Put '</span><hr size="3">';
	Put '</span><ol>';
end;

Put '<li class="ContentProcName">';
Put '<span> Dataset Description';
ww="%sysfunc(sasmsg(catalog.dfrmsg, dfr_M_level_label, noquote))"|| ktrim(kleft(m_lvl))|| " , %sysfunc(sasmsg(catalog.dfrmsg, dfr_L_level_label, noquote))="|| ktrim(kleft(L_lvl))|| " 
		, %sysfunc(sasmsg(catalog.dfrmsg, dfr_M_id_label, noquote))="|| ktrim(kleft(m_id))|| " , %sysfunc(sasmsg(catalog.dfrmsg, dfr_L_id_label, noquote))="|| ktrim(kleft(l_id))|| " , 
		%sysfunc(sasmsg(catalog.dfrmsg, dfr_Group_Key_label, noquote))="|| ktrim(kleft(group_key))|| " , %sysfunc(sasmsg(catalog.dfrmsg, dfr_P_Split_label, noquote))="|| ktrim(kleft(P_Split))|| " " ;

put ww;
Put '</span><ul class="ContentProcName">';
Put '<li class="ContentItem">';
Put '<span><b>';

Put '</b><a href=';
ww=".&slash."|| ktrim(kleft(_N_)) || "&slash.fr_&projid._"|| ktrim(kleft(_N_)) || ".html";
put ww;
Put ' target="body">';
put ds_desc;
put '</a>';
Put '</span></li>';
Put '</ul>';
Put '</li>';
if L then do;
	Put '</ol>';
	Put '</body>';
	Put '</html>';
end;
run;

Data _null_;
file "&path_report.&slash.fr_&jobname._&projid..html";
	Put '<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Frameset//EN">';
	Put '<html>';
	Put '<head>';
	Put '<meta name="Generator" content="SAS Software, see www.sas.com" sasversion="9.1">';
	Put '<meta http-equiv="Content-Type" content="text/html; charset=windows-1252">';
	Put "<title>&jobname</title>";
	Put '</head>';
	Put '<frameset frameborder="yes" framespacing="1" cols="23%,*">';
	Put "<frame marginwidth='4' marginheight='0' src=.&slash.plots&slash.&projid.&slash.contents.html 
			name='contents' scrolling='auto' title='The Table of Contents'>";
	Put "<frame marginwidth='9' marginheight='0' src=.&slash.plots&slash.&projid.&slash.1&slash.fr_&projid._1.html
			name='body' scrolling='auto' title='The SAS Output'>";
	Put '<noframes>';
	Put '<body>';
	Put '<ul>';
	Put "<li><a href=.&slash.plots&slash.&projid.&slash.frame.html>The Table of Contents</a></li>";
	Put '<li><a href="fr_3001_1.html">The Contents</a></li>';
	Put '</ul>';
	Put '</body>';
	Put '</noframes>';
	Put '</frameset>';
	Put '</html>';
run;
%mend;

/*************************************************/
/*	Functionality- initialsetup for grid         */
/*Copy the catalog and libname as macro variable and creat Report file */
/*************************************************/
%macro grid_initialsetup();
	%global libnames;
	proc sql noprint;
		select libname into: libnames separated by ' ' from sashelp.vslib where libname ne 'WORK' and libname ne "W_&proj." and libname ne 'SASHELP' and  libname ne 'SASUSER' and libname ne 'MAXAPP' and  libname ne 'MAXDATA' and  libname ne 'DP';
	quit;
	 data Stage.vars_&proj.;
	            set sashelp.vmacro;
	            if scope='GLOBAL';
  	run;
	
	

%mend;
/*************************************************/
/*	Functionality- Remove all datasets include EST*/
/*                                    and catalog */
/*************************************************/
%macro All_purgedatasets(projid=&proj.);
		proc datasets library=work kill nodetails nolist;
		quit;
		proc datasets lib=dia nolist memtype=data;
			delete d_est_f_&projid._: ;
			run;
		quit;
		proc datasets lib=dia nolist memtype=cat;
			delete cat_f_&projid._: ;
			run;
		quit;
		proc datasets lib=dp nolist memtype=data;
			delete f_&projid._: ;
			run;
		quit;
		proc datasets lib=for nolist memtype=data;
			delete f_est_f_&projid._: ;
			delete outstat_f_&projid._: ;
			delete outsum_f_&projid._: ;
			delete urm_f_&projid._: ;
			run;
		quit;
		proc datasets lib=out nolist memtype=data;
			delete fe_&projid. ;
			delete rm_f_&projid._: ;
			run;
		quit;
		proc datasets lib=recon nolist memtype=data;
			delete urm_f_&projid._: ;
			delete rm_f_&projid._: ;
			run;
		quit;
		proc datasets lib=stage nolist memtype=data;
			delete filter_f_&projid._: ;
			delete f_&projid. ;
			delete f_&projid._: ;
			delete vars_&projid. ;
			run;
		quit;
%mend All_purgedatasets;
/*************************************************/
/*	Functionality- Delete all file from a folder */
/*************************************************/
%macro delete_all_files(folder=); 
   filename filelist "&folder"; 
   data _null_; 
      dir_id = dopen('filelist'); 
      total_members = dnum(dir_id); 
      do i = 1 to total_members;   
         member_name = dread(dir_id,i); 
         file_id = mopen(dir_id,member_name,'i',0); 
         if file_id > 0 then do; 
            freadrc = fread(file_id); 
            rc = fclose(file_id); 
            rc = filename('delete',member_name,,,'filelist'); 
            rc = fdelete('delete'); 
         end; 
         rc = fclose(file_id); 
      end; 
      rc = dclose(dir_id); 
   run; 
%mend; 

/*************************************************/
/*	Functionality- Start report                  */
/*************************************************/
%macro start_report(row=&row);
	  %create_work_dir(path=&path_plots.&slash.&projid.,name_dir=&row);
	  %delete_all_files(folder=&path_plots.&slash.&projid.&slash.&row.); 
	  %let rptout = %quote(&path_plots.&slash.&projid.&slash.&row.&slash.fr_&projid._&row..html);
	    ods listing close;
	        ods html body= "&rptout."
	        gpath="&path_plots.&slash.&projid.&slash.&row." (url=".&slash.");
	        footnote "%sysfunc(sasmsg(catalog.dfrmsg, DFR_REPORT_FOOTNOTE, noquote))";
	        title "&jobname. %sysfunc(sasmsg(catalog.dfrmsg, DFR_REPORT_TITLE, noquote))";
	        ods graphics on;
	        goptions device=gif;
	        ods trace off;
%mend;
/*************************************************/
/*	Functionality- END report                  */
/*************************************************/
%macro end_report(row=&row);
        ods listing close;
        ods graphics off;
        ods html close;
        ods listing;
		%let rptout = %quote(&path_plots.&slash.&projid.&slash.&row.&slash.fr_&projid._&row..html);
/*		%ID_2_Name(html_file=&rptout);*/
%mend;
/**********************************************************/
/* Functionality- To get the list of location levels or   */
/* merchandise levels in a delim separated macro variable */
/**********************************************************/
%macro get_full_dim_levels(delim=,dim=);

PROC CONTENTS data=inp.&data_name out=W_&wktid..name_label (keep=LABEL name) noprint;
run;

%global full_list;

proc sql noprint;
 select distinct NAME 
 	into: full_list separated by &delim
 		from W_&wktid..name_label
			where NAME like "&dim%";
quit;
%mend get_full_dim_levels;
/***************************************************************/
/* Functionality- To create sequential BY and DROP macro       */
/* variables for merchandise or location levels reconciliation */
/***************************************************************/
%macro create_drop_by(delim= );

PROC CONTENTS data=inp.&data_name out=W_&wktid..name_label (keep=LABEL name) noprint;
run;

proc sql noprint;
 create table W_&wktid..util_by as
 	select distinct NAME, 1 as ID
		from W_&wktid..name_label
			where NAME like 'lid%' or NAME like 'mid%';
			update util_by 
			 set ID = 2 
			 	where NAME like 'mid%';
quit;

%if &recon_dim = %sysfunc(kupcase(LOC)) %then %do;
	proc sort data=W_&wktid..util_by out=W_&wktid..for_drop ; by  ID descending name ;run;
	proc sort data=W_&wktid..util_by out=W_&wktid..for_by ; by descending ID name;run;
%end;

%else %do;
	proc sort data=W_&wktid..util_by out=W_&wktid..for_drop ; by descending ID descending name;run;
	proc sort data=W_&wktid..util_by out=W_&wktid..for_by ; by ID name;run;
%end;

data W_&wktid..for_by;
	set W_&wktid..for_by;
	length ww $ 2600;
	retain ww;
	ww=ktrim(ww) || "&delim"|| ktrim(name);
	n=_n_;
run;
proc sort data=W_&wktid..for_by ; by descending n;run;

data W_&wktid..for_by;
	set W_&wktid..for_by;
	length name1 $ 100;
	retain ww;
	name1=ktrim("by_") ||ktrim(kleft((_n_)));
	call symputx(name1,ww,'G');
run;

%global drop_1;
%let drop_1=;
data W_&wktid..for_drop;
	set W_&wktid..for_drop;
	length ww $ 2600;
	length name1 $ 100;
	retain ww;
	ww=ktrim(ww) || "&delim"|| ktrim(name);
	name1=ktrim("drop_") ||ktrim(kleft((_n_+1)));
	call symputx(name1,ww,'G');
run;

%mend create_drop_by;
/************************************************************************/
/*	Functionality- Importing the .csv file provided by the user into	*/
/*	SAS data set for event specification								*/
/************************************************************************/
%MACRO ImportEventData(csvfile=,out=);

DATA &out;

	INFILE "&csvfile"
		DELIMITER=','
		MISSOVER
		DSD
		FIRSTOBS=2
	;
	LENGTH
		lid_1 8
		lid_2 8
		lid_3 8
		lid_4 8
		KPI $16.
		EVENT_START_DATE 8
		EVENT_END_DATE 8
		EVENT_CODE $10.
		EVENT_DESC $32.
		EVENT_KEYNAME $14.
		EVENT_TYPE_CODE $15.
		EVENT_NATURE_CODE $17.
		DURATION_AFTER 8
		DURATION_BEFORE 8
				
	;
	INFORMAT
		EVENT_START_DATE YYMMDD8.0
		EVENT_END_DATE YYMMDD8.0
	;
	FORMAT
		EVENT_START_DATE WEEKDATE29.0
		EVENT_END_DATE WEEKDATE29.0
	;
	INPUT
		lid_1 
		lid_2 
		lid_3 
		lid_4 
		KPI 
		EVENT_START_DATE
		EVENT_END_DATE
		EVENT_CODE 
		EVENT_DESC 
		EVENT_KEYNAME 
		EVENT_TYPE_CODE 
		EVENT_NATURE_CODE 
		DURATION_AFTER
		DURATION_BEFORE

	;
	
RUN;
%MEND;
/*************************************************************************/
/* Functionality- Creating the EVENT and EVENTBY datasets for HPFDIAGNOSE*/
/* procedure															 */
/*************************************************************************/
%macro create_event;
/*Expanding the jobsparam dataset for KPI*/

data W_&wktid..expand_all;
			set inp.event_ext_&projid.;
			retain key;
			IF kupcase(KPI)='ALL' THEN DO ;
				CNT_kpi_START=1;
				CNT_kpi_end=&dfr_numkpis;
				END; ELSE DO ;
				CNT_kpi_START=1;
				CNT_kpi_end=1;
			END;
			IF kupcase(KPI)='ALL' then
				key=1;
			else
				key=0;
			&FULL_RECON_DIM._level=&&&RECON_DIM._to_level;
			&FULL_UNRECON_DIM._level=&&&UNRECON_DIM._to_level;
			do j=CNT_kpi_START to CNT_kpi_end;
				if key=1 then KPI=kscan("&dfr_kpinameset",j);
				KPI=kupcase(KPI);
				output;
			end;
			
run;

proc sort data= W_&wktid..expand_all;by EVENT_CODE;run;

data W_&wktid..eventsdata;
	length _NAME_ $20;
	set W_&wktid..expand_all;
		_NAME_ = event_code ;
		_TYPE_ = event_type_code; 
		if kupcase(event_nature_code) = 'H' then do;
	   		_KEYNAME_=event_keyname;
   		end;
		if kupcase(event_nature_code)= 'N' then do;
	   		_STARTDATE_=event_start_date;
			_DATEINTRVL_= 'DAY';
	   		if event_end_date= . then do;
				_DUR_AFTER_='ALL';
			end;
	   		else do;
	   			_ENDDATE_=event_end_date;
	   		end;
		end;
		if kupcase(event_nature_code)= 'R' then do; 
	  		_STARTDATE_= event_start_date;
	  		_ENDDATE_= event_end_date;
			_DATEINTRVL_ = 'DAY';
		end;
run; 

*Creates final HPF events definition datasets*;
proc hpfevents;
   eventdata in=W_&wktid..eventsdata;
   eventdata out= inp.event_&projid.;
run;

proc sort data=inp.event_&projid. noduprecs;
	by _ALL_;
run;

data W_&wktid..event_genrl W_&wktid..event_byspec;
 set W_&wktid..eventsdata;
	_REQUIRED_ = "&diag_event_required";
	keep lid: _NAME_ KPI _REQUIRED_;
	if lid_&loc_to_level = 0 then output W_&wktid..event_genrl;
	else output W_&wktid..event_byspec;
	
run;

proc sort data=W_&wktid..event_genrl out=W_&wktid..event_genrl(keep=_NAME_ _REQUIRED_ KPI); by lid:; 
proc sort data=W_&wktid..event_byspec(keep=lid_1 - lid_&loc_to_level _NAME_ _REQUIRED_ KPI); by lid:;
proc sort data=inp.F_&projid. out=W_&wktid..F_&projid. nodupkey; by lid: mid:;
run;

proc sql noprint;
	create table W_&wktid..merge1 as
		select * 
			from W_&wktid..F_&projid. , W_&wktid..event_genrl;
quit;

proc sql noprint;
	create table W_&wktid..merge2 as
		select *
			from W_&wktid..F_&projid. as p, W_&wktid..Event_byspec as q
				where p.lid_&loc_to_level = q.lid_&loc_to_level;
quit;

proc sort data= W_&wktid..merge1 out=W_&wktid..srt_m1 nodupkey;by lid: mid: _NAME_ KPI;
run;
proc sort data= W_&wktid..merge2 out=W_&wktid..srt_m2 nodupkey;by lid: mid: _NAME_ KPI;
run;

data inp.eventby_&wktid.;
 set W_&wktid..srt_m1 W_&wktid..srt_m2;
  by lid: mid: ;
  keep lid: mid: _NAME_ KPI _REQUIRED_ ;
  rename _NAME_ = _EVENT_
		 KPI = _NAME_;
run;

proc sort data=inp.eventby_&wktid.;
	by &&UNRD.ID: &&RD.ID:;
run; 

%mend create_event;

/********************************************************************************************/
/*	Functionality: 	This macro calculates the dataset split count for each dataset			*/
/*					which are created after splitter2 in stage library						*/
/*		Input dataset: 	W_&wktid.F_&proj._meta_new 											*/
/*		Output dataset: W_&wktid._b_ 														*/
/*		Calculated columns in output dataset												*/
/*						split_cnt: Gives time series count per dataset.						*/
/*						number_split: Dataset Split count									*/
/********************************************************************************************/

%macro calculate_split_number(indata=,outdata=);
%let number_of_data=&dfr_no_ds.;
data W_&wktid..&outdata;
	set W_&wktid..&indata.;
	use=1;
run;
proc sort data=W_&wktid..&outdata; 
	by descending no_ts; 
run;

%LOOP:
proc sql noprint;
	select min(no_ts) into :min_no_ts from W_&wktid..&outdata where use=1;
	select count(no_ts) into :no_rows from W_&wktid..&outdata where use=1;
	select sum(no_ts) into :tot_ts from W_&wktid..&outdata where use=1;
quit;
%let threshold=%sysevalf(&number_of_data/&tot_ts);
%let threshold_reciprocal =%sysevalf(&tot_ts/&number_of_data);
%let flag=0;
data W_&wktid..&outdata;
	retain s 0;
	set W_&wktid..&outdata end=L;
	if use=1 and no_ts < &threshold_reciprocal then do;
		number_split=1;
		call symput('flag',1);
		use=0;
		s=s+1;
	end;
	if L then call symput('number_of_data',&number_of_data-s);
	drop s;
run;
%if &flag=0 %then %do;
	data  W_&wktid..&outdata;
		retain s &number_of_data;
		set W_&wktid..&outdata;
		count = _N_;
		if use=1 and count>1 then do;
			number_split=floor(no_ts*&threshold+0.5);
			use=0;
		end;
	run;
	proc sql noprint;
		select sum(number_split) into: tot_split_exp_first from W_&wktid..&outdata;
	quit;
	data W_&wktid..&outdata;
		set W_&wktid..&outdata;
		if count = 1 then do;
		number_split = %eval(&dfr_no_ds - &tot_split_exp_first);
		end;
		drop count;
	run;
%end; 
%else %do;
	%Goto LOOP; 
%end;
data W_&wktid..&outdata;
	set W_&wktid..&outdata;
	if number_split > no_ts then number_split=no_ts;
	split_cnt=ceil(no_ts/number_split);
	drop use s;
run;
%mend calculate_split_number;

/********************************************************************************************/
/*	Functionality: Delete the temporary datasets in OUT library which gets created after 	*/
/*				   execution of dfr_import (datasets ends with _TRANS, _TEMP001 & _TRANS_1  */
/*											_PREDICT, _PREDICT_1, _STD, _STD_1)				*/ 
/*	deltempds_outlib = Delete Temporary Datasets from OUT Library.							*/
/********************************************************************************************/
%macro deltempds_outlib();
	%let dellist1=;
	%let dellist2=;
	%let list1=%NRBQUOTE(RM_F_&projid.%_TRANS%str(%%));
	%let list2=%NRBQUOTE(RM_F_&projid.%_TEMP001);
	%let list3=%NRBQUOTE(%T_DFR_&projid.%);
	%let list4=%NRBQUOTE(RM_F_&projid.%_PREDICT%str(%%));
	%let list5=%NRBQUOTE(RM_F_&projid.%_STD%str(%%));
	proc sql noprint;
		create table W_&wktid..delete_list1 as
			select distinct memname from dictionary.columns
			where libname='OUT' and memname like "&list1."
			union 
			select distinct memname from dictionary.columns
			where libname='OUT' and memname like "&list2."
			union 
			select distinct memname from dictionary.columns
			where libname='OUT' and memname like "&list3.";
		select memname into: dellist1 separated by " " from W_&wktid..delete_list1;
		create table W_&wktid..delete_list2 as
			select distinct memname from dictionary.columns
			where libname='OUT' and memname like "&list4."
			union 
			select distinct memname from dictionary.columns
			where libname='OUT' and memname like "&list5.";

		select memname into: dellist2 separated by " " from W_&wktid..delete_list2;
	quit;
	proc datasets lib=out nolist nodetails memtype=data;
		delete &dellist1.;
		%if &dfr_wf_revisit. eq 1 %then %do;
			delete &dellist2.;
		%end;
	quit;
%mend deltempds_outlib;
