/**************************************************************/
/*                                                            */
/*       SAS RETAIL INTELLIGENCE SOLUTIONS                    */
/*       Demand Forecast For Retail Planning 4.2              */
/*                                                            */
/**************************************************************/
/*                                                            */
/* SAS Retail Group, All rights reserved (c) 2006,2007,2008   */
/*                                                            */
/**************************************************************/

/****************************************************************/
/*  FUNCTIONALITY :- Calls to hpfdignose,hpfengine Task-FIT,	*/
/*	hpfengine Task=SELECT and hpfengine Task=FORECAST macro 	*/
/****************************************************************/

%macro hpf_call();
	proc datasets lib=for nolist memtype=data ;
		delete URM_f_&proj.:;
		delete OUTFor_f_&proj.:;
		delete outsum_f_&proj.:;
		delete OUTstat_f_&proj.:;
	run;
 	quit;
	%RCSet (&syserr);
 
	proc datasets lib=dp nolist memtype=data ;
		delete f_&proj._forecast_meta:;
	run;
	quit;
	%RCSet (&syserr);
	proc datasets lib=out nolist memtype=data ;
		delete RM_F_&proj.:;
	run;
	quit;
	%RCSet (&syserr);
	
	data dp.F_&proj._Forecast_meta
		(label='Table to store forecast metadata of project');
			attrib ds_ref_no FORMAT=8. informat=8. label='Dataset Reference Number';
			attrib ds_name FORMAT=$32. informat=$32. label='Dataset Name';
			attrib ds_lib format=$100. informat=$100. label='Name of library';
			attrib M_LVL  FORMAT=12. informat=12. label='Merchandise Level';
			attrib L_lvl  FORMAT=12. informat=12. label='Location Level';
			attrib M_id  FORMAT=12. informat=12. label='Merchandise ID';
			attrib L_id  FORMAT=12. informat=12. label='Location ID';
			attrib group_key FORMAT=12. informat=12. label='Group Key';
			attrib p_split FORMAT=12. informat=12. label='Performance Split';
			attrib kpi format=$40. informat=$40. label='Name of KPI';
			attrib ds_desc FORMAT=$255. informat=$255. label='Dataset Description';
			attrib Ref_Merch_ID  FORMAT=12. informat=12. label='Ref_Merch_ID';
			attrib Ref_location_ID FORMAT=12. informat=12. label='Ref_location_ID';
		stop;
	run;
	%RCSet (&syserr);

	/* Get number for forecasts to fit */
	data _null_;
		set Dp.F_&proj._meta end=L;
		if L then
			call symputx('row_no',_n_);
	run;
	%RCSet (&syserr);
	%do i=1 %to &row_no.;
		%create_work_dir(path=&work_path.,name_dir=&i.);
	%end;

	%main_report();
	/*Following code gets executed for every row in metadata table*/
	%grid_initialsetup();

	option sascmd="!sascmd";
	%global R_rsubmit_cnt;
	%let Waitlist=;
	%Let i=1;
	%Let R_rsubmit_cnt=1;
	%do %while (&i<=&row_no);
	%let Waitlist=&WaitList. hpf&i.;
		%If &dfr_wf_grid=1 %then %let hpf_call&i._RC=%sysfunc(grdsvc_enable("hpf&i.", "resource=&BIMetaServerName"));
		signon hpf&i.;
		%syslput proj=&proj./REMOTE=hpf&i.;
		%syslput wktid=&proj./REMOTE=hpf&i.;
		%syslput I=&i./REMOTE=hpf&i.;
		%syslput dbtype_remote=&dbtype./REMOTE=hpf&i.;
		%syslput R_HOSTNAME=&BIMetaDataHost./REMOTE=hpf&i.;
        %syslput dsn_remote=&dsn./REMOTE=hpf&i.;
        %syslput dbpassword_remote=%quote(&dbpassword.)/REMOTE=hpf&i.;
        %syslput share_name=&share_server_name./REMOTE=hpf&i.;


		rsubmit process=hpf&i. WAIT=no  inheritlib=(&libnames) cmacvar=hpf&i.;

			options mstored sasmstore=catalog;
			data _null_;
				rc = sleep(1);
			run;
			option mprint mlogic symbolgen;
			options FMTSEARCH=(stage WORK);
			 
			data _null_;
				set Stage.vars_&proj.;
				if name ne 'I' then call symputx(name,value,'G');
			run;
			%let hpf&i._nodename=%sysfunc(grdsvc_getname("hpf&i.")); 
			%put "HPF&i. node name is &&hpf&i._nodename.";
			
			libname dp "&path_interim&slash.dp" SERVER=&share_name HOSTNAME="&ShareServerHost" user="&Share_Server_User" passwd="&Share_Server_Password";
		
		    libname W_&wktid. "&work_path.&slash.&i.";

			proc datasets library=W_&wktid. kill nodetails nolist;
			quit;

			%start_report(row=&i);
			%hpf_diagnose(proj=&proj.,row=&i.);
			%HPF_ENGINE(proj=&proj.,row=&i.,task=select);
			%HPF_ENGINE(proj=&proj.,row=&i.,task=fit);
			%HPF_ENGINE(proj=&proj.,row=&i.,task=forecast);
			%end_report(row=&i);
			
		endrsubmit;
	
		%let i=%eval(&i+1);
		%Let R_rsubmit_cnt=%eval(&R_rsubmit_cnt+1-&dfr_wf_grid);
		%if &R_rsubmit_cnt >&ma_max_rsubmit.  %then %do;
			%check_RRSS(task1=&Waitlist., sleep_time=&dfr_hpf_wait_sec.);
			%let Waitlist=&task2;
		%end;
	%end;
	%if "&Waitlist" ne ""  %then %do;
	 	waitfor _all_ &Waitlist.;
	 	signoff _all_;
	%end;
	%timecheck(step=%quote(hpf_call));
	libname W_&wktid. "&path_stage&slash.work&slash.W_&wktid.";
	proc datasets library=W_&wktid. kill nodetails nolist;
	quit;

	/****************************************************************************************/
	/****************************************************************************************/
	/* EXCEPTION FILE: Create forecast studio load file from execption file fs_&jobname.    */
	/****************************************************************************************/
	/****************************************************************************************/
	 
	%if (&dfr_exception_filter ne) and (&job_rc.<5) %then %do;
		%dfr_exceptions;
		%mm_step_finish(proj=&projid.,step=exception,standalone=&standalone.);
		%timecheck(step=%quote(EXCEPTION));
	%end;

	%if &job_rc > 4 %then %goto EndMacro;
	/*Functionality- To combine the separate "Performance Split" forecast datasets */
		%combine3;
		%timecheck(step=%quote(combine3));
	
	%if &job_rc > 4 %then %goto EndMacro;
	/*Functionality- Decide the quality of forecasting models */
		%revisit;
		%timecheck(step=%quote(revisit));
	%EndMacro:
%mend hpf_call;

