/**************************************************************/
/*                                                            */
/*       SAS RETAIL INTELLIGENCE SOLUTIONS                    */
/*       Demand Forecast For Retail Planning 4.2              */
/*                                                            */
/**************************************************************/
/*                                                            */
/* SAS Retail Group, All rights reserved (c) 2006,2007        */
/*                                                            */
/**************************************************************/
/*Functionality-PROC HPFDIGNOSE*/
 
%macro hpf_diagnose(proj=,row=);
 
	/*To load parameter specify in the jobparams */
	%getmetabyrow(proj=&proj.,row=&row);
	%get_ds_ref_no(data=dp.F_&proj._meta,desc=F_&proj._m&MERCH_LEVEL._l&LOCATION_LEVEL._&merch_id._&location_id._&Group_Key._&p_split._&kpi.);
	 
	/***************************************************************/
	/*  LOAD RECORDS TO SELECT                                     */
	/***************************************************************/
	%let cnt=0;
	%let cnt1=0;
	data W_&wktid..ctrl_tab;
		set dp.F_&proj._ctrl_tab
		(where=(kupcase(kpi)=kupcase("&kpi.") and m_lvl=&Merch_level. and l_lvl=&Location_level. 
		and umd_dia=1 and proj_id=&proj.)) end=L;
		if L then call symputx('cnt1',_n_);
	run;

	%if &cnt1>0 %then %do;
	/*SQL which will take only diagnose from ana_table*/
	proc sql noprint;
		select COUNT(*) into :cnt
		from &datalibrary..&dataname. as a , W_&wktid..ctrl_tab as b
		where a.mID_&Merch_level. =b.m_id
		and a.lID_&Location_level. =b.l_id;
	quit;
	%rcset(&sqlrc);
	%end;
	 
	%if &cnt>0 %then %do; /* START OF if input dataset has any rows*/
		%getparams(proj=&proj,merch_level=&Merch_level.,location_level=&Location_level.,
			fieldkey=&kpi.,merch_id=&merch_id.,location_id=&location_id., group_key=&Group_Key.);

		%let indata=W_&wktid..&dataname.;
			proc sql noprint;
				create table &indata. as
				select A.*
				from &datalibrary..&dataname. as a
				where a.mID_&Merch_level. in (select b.m_id from W_&wktid..ctrl_tab as b)
				and a.lID_&Location_level. in (select b.l_id from W_&wktid..ctrl_tab as b);
			quit;
	 	%let nobs=0;
		%noobs(ds=dia.d_est_F_&proj._&get_ds_ref_no.);
		%if &nobs=0 %then %noobs(ds=for.f_est_F_&proj._&get_ds_ref_no.);

		%if %sysfunc(cexist(dia.Cat_F_&proj._&get_ds_ref_no.))=1 and (&nobs > 0) %then %do;

			proc sort data=&indata. out=W_&wktid..del_for_cat nodupkey;
				by &&UNRD.ID: &&RD.ID: ;
			run;
			%RCSet (&syserr);

			proc sql;
				create table W_&wktid..a as
				select _model_ from for.f_est_F_&proj._&get_ds_ref_no. as a,
					   W_&wktid..del_for_cat as b
				where a.mID_&Merch_level.=b.mID_&Merch_level. 
					and a.LID_&Location_level.=b.LID_&Location_level.
					and KUPCASE(a._name_) = KUPCASE("&kpi")
					and KUPCASE(_model_) ne 'SMSEAS'
					and KUPCASE(_select_) ne 'BEST'
				union
				select _select_ from dia.d_est_F_&proj._&get_ds_ref_no. as a,
					   W_&wktid..del_for_cat as b
				where a.mID_&Merch_level. =b.mID_&Merch_level.
					and a.LID_&Location_level. =b.LID_&Location_level.
					and  KUPCASE(a._name_)=KUPCASE("&kpi");
			quit;
			%rcset(&sqlrc);

			proc catalog catalog=dia.Cat_F_&proj._&get_ds_ref_no.;
				CONTENTS out=W_&wktid..cat_in(keep =name);
			quit;
			run;
			%RCSet (&syserr);

			proc sql;
				create table W_&wktid..c as
				select a._model_ 
				from W_&wktid..a as a, W_&wktid..cat_in as b
				where a._model_=b.name;
			quit;
			%rcset(&sqlrc);

			filename temp1 temp;
			data _null_;
				set W_&wktid..c;
				file temp1;
				length w $ 256;
				w=strip(_model_ ) ||'.xml';
				put w;
			run;
			%RCSet (&syserr);

			proc catalog catalog=dia.Cat_F_&proj._&get_ds_ref_no.;
						DELETE 
				%include temp1;
				;
					quit;
			run;
			%RCSet (&syserr);

	%end;
	/***********************************************/
	/***********************************************/
	/*      HPFDIAGNOSE                            */
	/***********************************************/
	/***********************************************/
	 
	/* load up custom models if they exist */
		%if "&diag_custom_model_file" ne "" %then %do; 
			%include "&path_interim&slash.dia&slash.&diag_custom_model_file";
		%end;

		%let temp_inselect=dia.&diag_inselectname;

	/*Executing proc hpfdiagnose*/

		proc hpfdiagnose
			ALPHA=&diag_alpha.
			BACK=&diag_back.
			BASENAME=&diag_basename.
			CRITERION=&diag_criterion.
			DATA=&indata.
			DELAYEVENT=&diag_delayevent.

			%if &diag_input_lag_user = 1 %then %do;
				DELAYINPUT=&diag_delayinput.
			%end;

			ENTRYPCT=&diag_entrypct.
			HOLDOUTPCT=&diag_holdoutpct.

			%if "&diag_custom_model_file." ne "" %then %do;
				INSELECTNAME=&diag_inselectname.

				%if &dfr_forced_custom_model = 1 %then %do;
					NODIAGNOSE
				%end;

			%end;

			ERRORCONTROL=&diag_error_control.
			MINOBS=&diag_minobs.
		/*Creating a temporary table from ESTs in WORK library.*/
			OUTEST=W_&wktid..est_F_&proj._&get_ds_ref_no.
			OUTPROCINFO=&diag_outprocinfo_data. 
			PREFILTER=&diag_prefilter.
			PRINT=&diag_print.
			REPOSITORY=dia.Cat_F_&proj._&get_ds_ref_no.
			SEASONALITY=&dfr_seasonality.
			SELECTINPUT=&diag_selectinput.
			SELECTEVENT=&diag_selectevent.
			

			%if &loc_to_level = &Location_level and "&dfr_ext_event_file." ne "" %then %do;
				INEVENT=inp.event_&proj
				EVENTBY=inp.eventby_&proj
			%end;

			SIGLEVEL=&diag_siglevel.
			SELECTBASE=&diag_selectbase.
			SPECBASE=&diag_specbase.
			TESTINPUT=&diag_testinput.
		;

		by &&UNRD.ID: &&RD.ID: ;

		%if %sysfunc(KUPCASE("&dfr_interval."))="WEEK" %then %do;
			id &dfr_time_field. interval=&dfr_interval..&dfr_weekday. &diag_id_option.;
		%end;

		%else %do;
			id &dfr_time_field. interval=&dfr_interval. &diag_id_option.;
		%end;
		 
		forecast &kpi. / &diag_forecast_option.;
		;

		%if &&&RECON_DIM._to_level-&&&full_recon_dim._level.< &effects_level %then %do;
			&input_price ;
			&input_Promo ;
			&input_Pantry ;
			&input_DELAY ;
			&input_Halo ;
			&input_Weather ;
		%end;
		%if &&&RECON_DIM._to_level=&&&full_recon_dim._level. %then %do;
			input &ind_var_name_space;
		%end;
		;
		TRANSFORM &diag_transform.;
		;
		TREND &diag_trend.;
		;
		%if &diag_include_arimax. =1 %then %do; ARIMAX &diag_arimax.;; %end
		;
		%if &diag_include_esm. =1 %then %do; ESM &diag_esm.;; %end
		;
		%if &diag_include_idm. =1 %then %do; IDM &diag_idm.;; %end
		;
		%if &diag_include_ucm. =1 %then %do; UCM &diag_ucm.;; %end
		;
		%if &dfr_wf_optimized.=0 %then %do;
			%do for_i=&loc_from_level. %to &Location_level.;
				format lid_&for_i l&proj.&for_i._.;
			%end;
			%do for_i=&merch_from_level. %to &merch_level.;
				format mid_&for_i m&proj.&for_i._.;
			%end;
		%end;
		run;

		%RCSet (&syserr);
	 %if &dfr_wf_optimized.=0 %then %do;
			%remove_format(lib=W_&wktid.,data=est_F_&proj._&get_ds_ref_no);
		%end;
	 
	/*Code to update CTRL_TAB*/
	 
	/* ESTs will be used to update the CTRL_TAB*/
	 
		proc sort data=W_&wktid..est_F_&proj._&get_ds_ref_no.
			out=W_&wktid..upd_ctrl_tab;
				by lid_&location_level.  mid_&merch_level. ;
		run;

		%RCSet (&syserr);
	 
	 
	/* Making metadata of the source and target table same. Also distinct records are selected
	from the source. Source table added with a column hist_dia=1 */
		data W_&wktid..upd_ctrl_tab;
			attrib M_LVL  FORMAT=12. informat=12. label='Merchandise Level';
			attrib L_lvl  FORMAT=12. informat=12. label='Location Level';
			attrib M_id  FORMAT=12. informat=12. label='Merchandise ID';
			attrib L_id  FORMAT=12. informat=12. label='Location ID';
			attrib kpi  FORMAT= $40. informat=$40. label='Name of KPI';
			attrib group_key FORMAT=12. informat=12. label='Group Key';
			attrib hist_mod_dttm length = 8 format = DATETIME19. informat = DATETIME19. label = 'Last run modification datetime stamp';
			set W_&wktid..upd_ctrl_tab;
				by lid_&location_level.  mid_&merch_level. ;
					proj_id=&proj;
					l_id=lid_&location_level.;
					m_id=mid_&merch_level.;
					m_lvl="&merch_level";
					l_lvl="&location_level";
					kpi="&kpi";
					hist_dia=1;
					umd_dia=0;
					GROUP_KEY=&GROUP_KEY;
					hist_mod_dttm=datetime();
					keep proj_id l_id m_id m_lvl l_lvl kpi GROUP_KEY umd_dia hist_dia hist_mod_dttm;
					if first.mid_&merch_level. ;
		run;

		%RCSet (&syserr);
	
		proc sql UNDO_POLICY=NONE noprint ;
			create table W_&wktid..upd_ctrl_tab_1 as
			select a.sr_no
			from dp.f_&proj._ctrl_tab a, W_&wktid..upd_ctrl_tab b 
			where a.M_lvl=b.m_lvl and a.L_lvl=b.L_lvl and a.m_id=b.m_id and
				  a.l_id=b.l_id and a.kpi=b.kpi and a.group_key=b.group_key;
		quit;

%DoSql:
		%let tot_no=0;
		proc sql UNDO_POLICY=NONE noprint ;
			update dp.f_&proj._ctrl_tab a
			set hist_dia=1, umd_dia=0, hist_mod_dttm=datetime()
			where a.sr_no in (select b.sr_no from W_&wktid..upd_ctrl_tab_1 b ); 
			
			create table W_&wktid..upd_ctrl_tab_2 as
			select a.sr_no
			from dp.f_&proj._ctrl_tab a, W_&wktid..upd_ctrl_tab b 
			where a.M_lvl=b.m_lvl and a.L_lvl=b.L_lvl and a.m_id=b.m_id and a.l_id=b.l_id and a.kpi=b.kpi 
				  and a.group_key=b.group_key and a.umd_dia=1;
			
		 	select count(a.sr_no) into: tot_no
			from W_&wktid..upd_ctrl_tab_2 a;
	  	quit;

  		%if &tot_no > 0 %then %do;
			data W_&wktid..upd_ctrl_tab_1;
			set W_&wktid..upd_ctrl_tab_2;
			run;
			%goto DoSql;
		%end;

/*	%RCSet (&syserr);*/
	 
	/*Code for appending EST from WORK to DIA*/
		%let keys_add=;
		data _null_;
			do i= &&&UNRECON_DIM._from_level to &&&FULL_UNRECON_DIM._level.;
			call symput('keys_add',KTRIM(KLEFT(SYMGET("keys_add"))) ||"  " ||KTRIM(KLEFT(SYMGET("UNRD"))) ||'ID_'|| KTRIM(KLEFT(i)));
			end;
			do i=&&&RECON_DIM._from_level to &&&FULL_RECON_DIM._level.;
			call symput('keys_add',KTRIM(KLEFT(SYMGET("keys_add"))) ||"  " ||KTRIM(KLEFT(SYMGET("RD"))) ||'ID_'|| KTRIM(KLEFT(i)));
			end;
		run;
		%RCSet (&syserr);

		%let keynum_add=%eval(&location_level.-&loc_from_level.+&merch_level.-&merch_from_level.+2) ;
		%Addnew_and_Update_nokey(source=est_F_&proj._&get_ds_ref_no.
			,source_lib=W_&wktid.
			,target=d_est_F_&proj._&get_ds_ref_no.
			,target_lib=dia
			,keys=&keys_add. _name_	);

		proc sort data=dia.d_est_F_&proj._&get_ds_ref_no.;
			by &&UNRD.ID: &&RD.ID:  ;
		run;

		%Apply_format(proj=&proj.,
			data=dia.d_est_F_&proj._&get_ds_ref_no.,
			M_to_lev=&merch_level.,
			L_to_lev=&Location_level.,
			M_from_lev=&merch_from_level.,
			L_from_lev=&loc_from_level.);
	 
	%end; /*END OF if input dataset has any rows*/

	%RCSet (&syserr);

	/*Code to delete intermediate datasets(WORK)*/

	proc datasets library=W_&wktid. kill nodetails nolist;
	quit;

	%RCSet (&syserr);

%mend hpf_diagnose;
