/**************************************************************/
/*                                                            */
/*       SAS RETAIL INTELLIGENCE SOLUTIONS                	  */
/*      Demand Forecast For Retail Planning 4.2               */
/*                                                            */
/**************************************************************/
/*                                                            */
/* SAS Retail Group, All rights reserved (c) 2006,2007        */
/*                                                            */
/**************************************************************/
 
%macro HPF_ENGINE(proj=,row=,task=);
 
	/*To load parameter specify in the jobparams */

	%getmetabyrow(proj=&proj.,row=&row);
	%get_ds_ref_no(data=dp.F_&proj._meta,desc=F_&proj._m&MERCH_LEVEL._l&LOCATION_LEVEL._&merch_id._&location_id._&Group_Key._&p_split._&kpi.);
	 
	/*****************************************************************/
	/*               LOAD RECORDS TO FORECAST                        */
	/*****************************************************************/
	%let cnt=0; 
	%let cnt1=0;
	proc sql noprint;
		create table W_&wktid..F_&proj._ctrl_tab_new as
		select * from dp.F_&proj._ctrl_tab as b
		where KUPCASE(kpi)=KUPCASE("&kpi") and m_lvl=&Merch_level. and
			  l_lvl=&Location_level. and proj_id=&proj.
		%if &task.=select %then and b.umd_sel=1 and b.hist_dia=1 ;
		%if &task.=fit  %then and b.umd_fit=1 and b.hist_sel=1 ;
		%if &task.=forecast %then and b.umd_for=1 and b.hist_fit=1 ;
		;
	quit;
	%rcset(&sqlrc);

	data _null_;
		set W_&wktid..F_&proj._ctrl_tab_new end=L;
		if L then call symputx('cnt1',_n_);
	run;

	%if &cnt1>0 %then %do;
		proc  sql noprint;
		 	select COUNT(*) into :cnt
			from &datalibrary..&dataname. as a, 
				 W_&wktid..F_&proj._ctrl_tab_new as b
			where a.mID_&Merch_level. =b.m_id
			and a.lID_&Location_level. =b.l_id
			;
		quit;
		%rcset(&sqlrc); 
	%end;
	
	%if &cnt>0 %then %do;/* START OF if input dataset has any rows*/
		%getparams(proj=&proj,merch_level=&Merch_level.,location_level=&Location_level.,
				fieldkey=&kpi.,merch_id=&merch_id.,location_id=&location_id., group_key=&Group_Key.);
		proc sql noprint;
			create table W_&wktid..&dataname. as
			select A.*
			from &datalibrary..&dataname. as a 
			where a.mID_&Merch_level. in (select b.m_id from  W_&wktid..F_&proj._ctrl_tab_new as b )
			and a.lID_&Location_level. in (select b.l_id from  W_&wktid..F_&proj._ctrl_tab_new as b)
			;
		quit;
		%rcset(&sqlrc);	 
	 
	/***********************************************/
	/*      HPFENGINE FORECAST                     */
	/***********************************************/
	 
		PROC HPFENGINE
			BACK=&eng_back.
			DATA=W_&wktid..&dataname.

			%if "&diag_custom_model_file." ne "" %then %do;
				GLOBALSELECTION=&eng_globalselection.
			%end;

			%if &task.=select %then %do; 
				INEST=dia.d_est_F_&proj._&get_ds_ref_no. 
			%end;

			%else %do; 
				INEST=for.f_est_F_&proj._&get_ds_ref_no.
			%end;

			%if &loc_to_level = &Location_level and "&dfr_ext_event_file." ne "" %then %do;
				INEVENT=inp.event_&proj.
			%end;

			LEAD=&eng_lead.
			OUT=_NULL_
			OUTEST=W_&wktid..est_F_&proj._&get_ds_ref_no.
			OUTFOR=for.outfor_F_&proj._&get_ds_ref_no. (label ="&datadesc")
			OUTSTAT=for.outstat_F_&proj._&get_ds_ref_no. (label ="&datadesc")

			%if "&eng_plot" ne "" %then %do; 
				PLOT=( &eng_plot ) 
			%end;

			%if "&eng_print" ne "" %then %do; 
				PRINT=( &eng_print ) 
			%end;

			OUTSTATSELECT=&eng_outstatselect.
			OUTMODELINFO=&eng_outmodelinfo.
			OUTPROCINFO=&eng_outprocinfo_data.
			OUTSUM=for.outsum_F_&proj._&get_ds_ref_no. (label ="&datadesc")
			REPOSITORY=dia.Cat_F_&proj._&get_ds_ref_no.
			SEASONALITY=&dfr_seasonality.
			SORTNAMES
			TASK=&task.
			ERRORCONTROL=&eng_error_control.;

			%if "&eng_adjust_varlist" ne "" and &&&RECON_DIM._to_level=&&&full_recon_dim._level. %then %do;
					ADJUST &kpi. = (&eng_adjust_varlist) / OPERATION=(&eng_pre_adjust.,&eng_post_adjust.)
			%end;
			;

			by &&UNRD.ID: &&RD.ID:
			;
			forecast &kpi. / &eng_forecast_option.
			;
			%if %sysfunc(KUPCASE("&dfr_interval."))="WEEK" %then %do;
				id &dfr_time_field. interval=&dfr_interval..&dfr_weekday. &eng_id_option.
				;
			%end;

			%else  %do;
				id &dfr_time_field. interval=&dfr_interval. &eng_id_option.
				;
			%end;
			;
			;
			&input_eve ;
			%if (&recon_hier=1) or (&recon_hier=0) %then %do;
				%if &&&RECON_DIM._to_level-&&&RECON_DIM._level.< &effects_level %then %do;
					&input_price ;
					&input_Promo ;
					&input_Pantry ;
					&input_DELAY ;
					&input_Halo ;
					&input_Weather ;
				%end;
			%end;

			%if (&recon_hier=2) %then %do;
				%if &&&RECON_DIM._to_level-&&&full_recon_dim._level.< &effects_level %then %do;
					&input_price ;
					&input_Promo ;
					&input_Pantry ;
					&input_DELAY ;
					&input_Halo ;
					&input_Weather ;
				%end;
			%end;
			;
			%if &&&RECON_DIM._to_level=&&&full_recon_dim._level. %then %do;
					input &ind_var_name_space;
			%end;
			/*code for format*/
			%if &dfr_wf_optimized.=0 %then %do;
			%do for_i=&loc_from_level. %to &Location_level.;
				format lid_&for_i l&proj.&for_i._.;
			%end;
			%do for_i=&merch_from_level. %to &merch_level.;
				format mid_&for_i m&proj.&for_i._.;
			%end;
		%end;
		RUN;            /*  Code to update CTRL_TAB */
	  %if &dfr_wf_optimized.=0 %then %do;
			%remove_format(lib=W_&wktid.,data=est_F_&proj._&get_ds_ref_no);
			%remove_format(lib=FOR,data=OUTFOR_F_&proj._&get_ds_ref_no);
			%remove_format(lib=FOR,data=OUTSTAT_F_&proj._&get_ds_ref_no);
			%remove_format(lib=FOR,data=OUTSUM_F_&proj._&get_ds_ref_no);

	   %end;
	/* Remove negative forecasts */
		%if (&dfr_non_negative_forecast.=1) %then %do;
			data for.outfor_F_&proj._&get_ds_ref_no.;
				set for.outfor_F_&proj._&get_ds_ref_no.;
			 	if ^missing(predict) and predict < 0 then do;
				  	lower = lower - predict;
				  	upper = upper - predict;
				  	predict = 0;
			 	end;
			run;
		%end;
	 
	 
	/* Creating a temporary table from ESTs in WORK library.This temporary table
	will be used to update the CTRL_TAB*/

		proc sort data=for.outfor_F_&proj._&get_ds_ref_no.
			out=W_&wktid..upd_ctrl_tab;
				by lid_&location_level.  mid_&merch_level. ;
		run;
	/* Making metadata of the source and target table same. Also distinct records are selected
		from the source. Source table added with a column hist_for=1 */
		data W_&wktid..upd_ctrl_tab;
			attrib M_LVL FORMAT=12. informat=12. label='Merchandise Level';
			attrib L_lvl FORMAT=12. informat=12. label='Location Level';
			attrib M_id FORMAT=12. informat=12. label='Merchandise ID';
			attrib L_id FORMAT=12. informat=12. label='Location ID';
			attrib group_key length = 8 format = 12. informat = 12. label = 'Group key'; 
			attrib kpi FORMAT= $40. informat=$40. label='Name of KPI';
			attrib hist_mod_dttm length = 8 format = DATETIME19. informat = DATETIME19. label = 'Last run modification datetime stamp';
			set W_&wktid..upd_ctrl_tab;
				by lid_&location_level.  mid_&merch_level. ;
				proj_id=&proj;
				l_id=lid_&location_level.;
				m_id=mid_&merch_level.;
				m_lvl="&merch_level";
				l_lvl="&location_level";
				Group_Key="&Group_Key.";
				kpi="&kpi";
				hist_mod_dttm=datetime();
			%if "&task"="select" %then %do;
				hist_sel=1;
				hist_fit=1;
				hist_for=1;
				umd_sel=0;
				umd_fit=0;
				umd_for=0;
				keep proj_id l_id m_id m_lvl l_lvl Group_Key kpi hist_sel hist_fit hist_for umd_sel 
					umd_fit umd_for hist_mod_dttm;
			%end;
			%if "&task"="fit" %then %do;
				hist_fit=1;
				hist_for=1;
				umd_fit=0;
				umd_for=0;
				keep proj_id l_id m_id m_lvl l_lvl Group_Key kpi hist_fit hist_for umd_fit umd_for hist_mod_dttm;
			%end;
			%if "&task"="forecast" %then %do;
				hist_for=1;
				umd_for=0;
				keep proj_id l_id m_id m_lvl l_lvl Group_Key kpi hist_for umd_for hist_mod_dttm;
			%end;
			if first.mid_&merch_level. ;
		run;
		%RCSet (&syserr);
	 
		proc sql UNDO_POLICY=NONE ;
			create table W_&wktid..upd_ctrl_tab_1 as
			select a.sr_no
			from dp.f_&proj._ctrl_tab a, W_&wktid..upd_ctrl_tab b 
			where a.M_lvl=b.m_lvl and a.L_lvl=b.L_lvl and a.m_id=b.m_id and 
				  a.l_id=b.l_id and a.kpi=b.kpi and a.group_key=b.group_key;
		quit;
%DoSql:
		%let tot_no=0;		
		proc sql UNDO_POLICY=NONE noprint;
			update dp.f_&proj._ctrl_tab a
			set %if "&task"="select" %then %do;
				hist_sel=1,
				hist_fit=1,
				hist_for=1,
				umd_sel=0,
				umd_fit=0,
				umd_for=0,
				hist_mod_dttm=datetime()
			%end;
			%if "&task"="fit" %then %do;
				hist_fit=1,
				hist_for=1,
				umd_fit=0,
				umd_for=0,
				hist_mod_dttm=datetime()
			%end;
			%if "&task"="forecast" %then %do;
				hist_for=1,
				umd_for=0,
				hist_mod_dttm=datetime()
			%end;
			where a.sr_no in (select b.sr_no from W_&wktid..upd_ctrl_tab_1 b ); 

			create table W_&wktid..upd_ctrl_tab_2 as
			select a.sr_no from dp.f_&proj._ctrl_tab a, W_&wktid..upd_ctrl_tab b 
			where a.M_lvl=b.m_lvl and a.L_lvl=b.L_lvl and a.m_id=b.m_id and a.l_id=b.l_id 
				  and a.kpi=b.kpi and a.group_key=b.group_key 
			      and %if "&task"="select" %then %do; a.umd_sel=1 %end;
				      %if "&task"="fit" %then %do; a.umd_fit=1 %end;
				   	  %if "&task"="forecast" %then %do;	a.umd_for=1 %end;
			;

			select count(a.sr_no) into: tot_no
			from W_&wktid..upd_ctrl_tab_2 a;
		quit;

		%if &tot_no > 0 %then %do;
			data W_&wktid..upd_ctrl_tab_1;
				set W_&wktid..upd_ctrl_tab_2;
			run;
			%goto DoSql;
		%end;

		/*Code for appending est from work to dia*/
		%let keys_add=;
		data _null_;
			do i= &&&UNRECON_DIM._from_level to &&&FULL_UNRECON_DIM._level.;
			call symput('keys_add',KTRIM(KLEFT(SYMGET("keys_add"))) ||"  "
			||KTRIM(KLEFT(SYMGET("UNRD"))) ||'ID_'|| KTRIM(KLEFT(i)));
			end;

			do i=&&&RECON_DIM._from_level to &&&FULL_RECON_DIM._level.;
			call symput('keys_add',KTRIM(KLEFT(SYMGET("keys_add"))) ||"  "
			||KTRIM(KLEFT(SYMGET("RD"))) ||'ID_'|| KTRIM(KLEFT(i)));
			end;
		run;
	 
		%let keynum_add=%eval(&location_level.-&loc_from_level.+&merch_level.-&merch_from_level.+2) ;
	 
		%Addnew_and_Update_nokey
		(source=est_F_&proj._&get_ds_ref_no.
		,source_lib=W_&wktid.
		,target=f_est_F_&proj._&get_ds_ref_no.
		,target_lib=for
		,keys=&keys_add. _name_
		);

		proc sort data=for.f_est_F_&proj._&get_ds_ref_no.;
			by &&UNRD.ID: &&RD.ID:  ;
		run;

		%Apply_format
		(proj=&proj.,
		data=for.f_est_F_&proj._&get_ds_ref_no.,
		M_to_lev=&merch_level.,
		L_to_lev=&LOCATION_level.,
		M_from_lev=&merch_from_level.,
		L_from_lev=&loc_from_level.
		);

		%Apply_format
		(proj=&proj.,
		data=for.outfor_F_&proj._&get_ds_ref_no.,
		M_to_lev=&merch_level.,
		L_to_lev=&LOCATION_level.,
		M_from_lev=&merch_from_level.,
		L_from_lev=&loc_from_level.
		);

		%Apply_format
		(proj=&proj.,
		data=for.outstat_F_&proj._&get_ds_ref_no.,
		M_to_lev=&merch_level.,
		L_to_lev=&LOCATION_level.,
		M_from_lev=&merch_from_level.,
		L_from_lev=&loc_from_level.
		);

		%Apply_format
		(proj=&proj.,
		data=for.outsum_F_&proj._&get_ds_ref_no.,
		M_to_lev=&merch_level.,
		L_to_lev=&LOCATION_level.,
		M_from_lev=&merch_from_level.,
		L_from_lev=&loc_from_level.
		);

	%end; /*END OF if input dataset has any rows*/
	 
	%if %sysfunc(exist(for.outfor_F_&proj._&get_ds_ref_no.))=1 and &task=forecast %then %do;
		/* Append observations to forecast Metadata Table*/
		data W_&wktid..F_&proj._Forecast_meta
			(label='Temp Table to store forecast Metadata of project'    );
			attrib ds_ref_no FORMAT=8. informat=8. label='Dataset Reference Number';
			attrib ds_name FORMAT=$32. informat=$32. label='Dataset Name';
			attrib ds_lib format=$100. informat=$100. label='Name of library';
			attrib M_LVL  FORMAT=12. informat=12. label='Merchandise Level';
			attrib L_lvl  FORMAT=12. informat=12. label='Location Level';
			attrib M_id  FORMAT=12. informat=12. label='Merchandise ID';
			attrib L_id  FORMAT=12. informat=12. label='Location ID';
			attrib group_key FORMAT=12. informat=12. label='Group Key';
			attrib p_split FORMAT=12. informat=12. label='Performance Split';
			attrib kpi format=$40. informat=$40. label='Name of KPI';
			attrib ds_desc FORMAT=$255. informat=$255. label='Dataset Description';
			attrib Ref_Merch_ID  FORMAT=12. informat=12. label='Ref_Merch_ID';
			attrib Ref_location_ID FORMAT=12. informat=12. label='Ref_location_ID';
			ds_ref_no=&get_ds_ref_no.;
			ds_name="OUTFOR_F_&proj._&get_ds_ref_no.";
			ds_desc="OUTFOR_F_&proj._m&MERCH_LEVEL._l&LOCATION_LEVEL._&Ref_Merch_ID._&Ref_location_ID._&Group_key._&kpi";
			ds_lib='for';
			M_LVL="&merch_level.";
			L_lvl="&location_level.";
			M_id="&merch_id";
			L_id="&location_id";
			group_key="&Group_Key.";
			kpi="&kpi";
			p_split="&p_split";
			Ref_Merch_ID="&Ref_Merch_ID";
			Ref_location_ID="&Ref_location_ID";
		run;
		/*Check for a lock if there is lock wait for it*/
		proc sql noprint UNDO_POLICY=NONE;
			insert into dp.F_&proj._Forecast_meta(ds_ref_no, ds_name, ds_lib, M_LVL, L_lvl, M_id, L_id, group_key, p_split, kpi, ds_desc, Ref_Merch_ID, Ref_location_ID)
			select ds_ref_no, ds_name, ds_lib, M_LVL, L_lvl, M_id, L_id, group_key, p_split, kpi, ds_desc, Ref_Merch_ID, Ref_location_ID from W_&wktid..F_&proj._Forecast_meta;
		quit;

	%end;

	/*Code to delete intermediate datasets(WORK)*/
	proc datasets library=W_&wktid. kill nodetails nolist;
	quit;
%mend HPF_ENGINE;
