/**************************************************************/
/*                                                            */
/*       SAS RETAIL INTELLIGENCE SOLUTIONS					  */
/* 		 Demand Forecast For Retail Planning 4.2       		  */
/*                                                            */
/**************************************************************/
/*                                                            */
/* SAS Retail Group, All rights reserved (c) 2006,2007        */
/*                                                            */
/**************************************************************/
 
/**********************************************************************************/
/* Reconcile all forecast merch/loc level files top-down, bottom-up or middle out */
/* Either Merchandise or Location levels can be reconciled                        */
/* Uses HPFRECONCILE															  */
/**********************************************************************************/
 
%macro Reconcile();
 
	%if &dfr_wf_recon ne 0 and (&&&RECON_DIM._to_level ne &&&RECON_DIM._from_level) %then %do;/* START of if reconcile is needed*/
	 
		/*Sorting of forecast metadata */
		proc sort data=dp.F_&proj._forecast_meta out=W_&wktid..F_&proj._forecast_meta nodupkey;
			by kpi &RD._lvl   ;
		run;
		
	 
		/* determine which dimension to reconcile */
		data _null_;
			if &recon_hier = 1 then rec='recon_merch';else rec='recon_loc';
			call symputx("start_lvl1",rec,'g');
		run;
		
		%let start_lvl=&&&start_lvl1;
	 
		/***************************************************************************************/
		/* Set up method of reconcile (TD or BU) for each level in merch or loc dimension      */
		/***************************************************************************************/
	 
		/*unchanged level*/
		data W_&wktid..unchg;
			set W_&wktid..F_&proj._forecast_meta;
			keep ds_name new_name;
			new_name=KSUBSTR(ds_name,2);
			if &RD._LVL =&start_lvl. ;
		run;

		filename temp1 temp; /* Temporary SAS program*/
		/*Will rename the dataset for UNchanged level*/
		data W_&wktid..unchg;
			set W_&wktid..unchg;
			file temp1;
			length v $256;
			v= 'data recon.'|| strip(new_name) || ' ; ';
			put v;
			v= 'set  for.'|| strip(ds_name) || ';run;';
			put v;
		run;
		
	 
		%include temp1;/*Executing temporary SAS program*/
	 
		
		data W_&wktid..F_&proj._forecast_meta;
			set W_&wktid..F_&proj._forecast_meta;
			by kpi &RD._lvl   ;
			ds_name2=lag(ds_name);
			if first.kpi then delete;
		run;
		
	 
		data W_&wktid..F_&proj._forecast_meta;
			set W_&wktid..F_&proj._forecast_meta;
			length method $8;
			if &RD._LVL  < &start_lvl. then method='BU';
			/*Method is bottom-up for all levels less than start level*/
			if &RD._LVL > &start_lvl. then method='TD';
			/*Method is Top-down for all levels less than start level*/
			if &RD._LVL =&start_lvl. then
			do;
				output;
				output;
			end;
			else output;
		run;
		
	 
		data W_&wktid..F_&proj._forecast_meta;
			set W_&wktid..F_&proj._forecast_meta;
			by kpi &RD._lvl   ;
			if method='' and ds_name = lag(ds_name) then delete;
			if method='' then method='BU';
			*keep ds_name ds_name2 method;
		run;
		data W_&wktid..F_&proj._forecast_meta;
			set W_&wktid..F_&proj._forecast_meta;
			sort_var=abs(&RD._lvl - &start_lvl);
		run;
		proc sort data=W_&wktid..F_&proj._forecast_meta ;
			by kpi   sort_var ;
		run;
		data W_&wktid..F_&proj._forecast_meta;
			set W_&wktid..F_&proj._forecast_meta;
			if method='BU' then ds_name=KSUBSTR(ds_name,2); else ds_name2=KSUBSTR(ds_name2,2);
			keep ds_name ds_name2 method;
		run;
	 
	 	%let row_no=0;
		data _null_;
			set W_&wktid..F_&proj._forecast_meta end=L;
			if L then call symput('row_no',_n_);
		run;
	 
		/***************************************************************************************/
		/* MAIN RECONCILE LOOP ... one call for each level reconciled 						   */
		/* Each row will reconcile either BU or TD based on method variable set in logic above */
		/***************************************************************************************/
	 
		%do i=1 %to &row_no ; /*Do loop for each row*/
			/* load metadata for row i */
			%getmetabyrow_for(proj=&proj.,row_no=&i);
			/* load reconcile parameters for each level */
			%getparams(proj=&proj,merch_level=&Merch_level.,location_level=&Location_level.,fieldkey=&kpi.);
			
			/* Sorting the input datasets */
			proc sort data=&base. ;
				by &&UNRD.ID: &&RD.ID: ;
			run;
			proc sort data=&top.;
				by &&UNRD.ID: &&RD.ID: ;
			run;
	 

			/*hpfreconcile code*/
			proc hpfreconcile
				disaggdata=&base.
				aggdata=&top.
				direction=&method.
				outfor=recon.&out_ds.
				outprocinfo=&recon_outprocinfo_data.
				CLMETHOD=&recon_clmethod.
				DISAGGREGATION=&recon_disaggregation.
				STDMETHOD=&recon_stdmethod.
				STDDIFBD=&recon_stddifbd.
				%if &recon_diff_required = 1 %then %do;
					RECDIFF
				%end;
				ERRORTRACE = &recon_errtrace.
				alpha=&recon_siglevel.
				PREDICTONLY
				%if (&dfr_non_negative_forecast.=1) %then %do;
					SIGN=NONNEGATIVE
				%end;
				%if &recon_weight_required = 1 %then %do;
					WEIGHTED	
				%end;
				;
				id &dfr_time_field. interval=&dfr_interval..&RIS_weekday_1. start=&eng_end_1.;
				by &&UNRD.ID: &&RD.ID: ;
			run;
		 
		%end; /*END OF do loop for each row*/
	 
		
	 
	%end;/*END OF if reconcile is needed*/

	%else %do;
			filename temp2 temp; /* Temporary SAS program*/
			data _null_;
				set dp.F_&proj._forecast_meta;
				file temp2;
			    length v $256;
				v='proc datasets ;';
				put v;
				v='copy in=for out=recon ;';
				put v;
				v= 'select '||strip(ds_name) || ';';
				put v;
				v= 'quit;';
				put v;
			run;
			%include temp2;
	%end;

%mend Reconcile;
