/**************************************************************/
/*                                                            */
/*       SAS RETAIL INTELLIGENCE SOLUTIONS                */
/*       Demand Forecast For Retail Planning 4.2          */
/*                                                            */
/**************************************************************/
/*                                                            */
/* SAS Retail Group, All rights reserved (c) 2006,2007        */
/*                                                            */
/**************************************************************/
/************************************************************************/
/*Functionality-This program makes updates to the CTRL TABLE to store the
previous run details for forecasting request                      */
/************************************************************************/
 
%macro intel_workflow(proj=,row=,row_number=);
	%let chng_ivar=&dfr_ivar_chng;
 	%getmetabyrow(proj=&proj.,row=&row);
 
	/*First call to getparams to initialize default parameters*/
	%getparams(proj=&proj,merch_level=&Merch_level.,location_level=&Location_level.,fieldkey=&kpi.,merch_id=&merch_id.,location_id=&location_id., group_key=&Group_Key.);

	/*Create CTRL table if not available*/;
	%if %sysfunc(exist(dp.F_&proj._ctrl_tab))=0 %then
		%do; 
			%include "&path_pgm.&slash.dfrcode&slash.ctrl_tab_script.sas";
		%end;
		proc sort data=&datalibrary..&dataname out=W_&wktid..&dataname(keep=group_key mid_&Merch_level lid_&Location_level rename=(mid_&Merch_level=m_id lid_&Location_level=l_id )) nodupkey;
			by group_key mid_&Merch_level lid_&Location_level ;
		run;
	%RCSet (&syserr);
 
	/*Make metadata same*/
	data W_&wktid..&dataname;
		set W_&wktid..&dataname;
		attrib kpi  FORMAT= $40. informat=$40. label='Name of KPI';
		m_lvl=&Merch_level;
		l_lvl=&Location_level;
		kpi="&kpi";
		proj_id=&proj;
	run;
	%let gr_cnt=0;
	proc sql noprint;
		select count(*) into: gr_cnt
		from dp.F_&wktid._ctrl_tab  a , W_&wktid..&dataname b
		where (a.M_lvl=b.m_lvl and a.L_lvl=b.L_lvl and a.m_id=b.m_id and a.l_id=b.l_id and a.kpi=b.kpi) and (a.group_key ne b.group_key);
	quit;
	
	%if &gr_cnt >=1 %then %do;
		/*Stop the process if gr_key is changed */
		%if &Dfr_running_mode=0 %then %do;
			%put %sysfunc(sasmsg(catalog.dfrmsg, DFR_GR_KEY_CHANGED_ERR, noquote));
			%let job_rc=100;
			%goto EndMacro;
		 %end;
		 /*Remove the observations if gr_key is changed */
		 %else %do;
		 	%put %sysfunc(sasmsg(catalog.dfrmsg, DFR_GR_KEY_CHANGED_NOTE, noquote));
		 	proc sql noprint;
				delete from dp.F_&wktid._ctrl_tab as a
				where exists (  select * 
				                 from W_&wktid..&dataname as b
				                 where (a.M_lvl=b.m_lvl and a.L_lvl=b.L_lvl and a.m_id=b.m_id and a.l_id=b.l_id and a.kpi=b.kpi) and (a.group_key ne b.group_key));
			quit;
		 %end;  
	%end;

	/*SQL to get the historical run details and revisit recommendations from the CTRL TABLE*/
	proc sql noprint;
		create table W_&wktid..&dataname. as
		select A.*,
		b.hist_dia,
		b.hist_sel,
		b.hist_fit,
		b.hist_for,
		b.rev_dia,
		b.rev_sel,
		b.rev_fit,
		b.rev_for
		from W_&wktid..&dataname. as a left join dp.F_&proj._ctrl_tab as b
		on( a.m_ID =b.m_id
		and a.l_ID =b.l_id
		and kupcase(a.kpi)=kupcase("&kpi")
		and a.m_lvl=b.m_lvl
		and a.l_lvl=b.l_lvl)
		;
	quit;
 
	/* If request was not run previously , then history values taken as 0 and revisit recommendations taken as 1*/
	data W_&wktid..&dataname.;
		set W_&wktid..&dataname.;
		if hist_dia= . then hist_dia= 0;
		if hist_sel= . then hist_sel= 0;
		if hist_fit= . then hist_for= 0;
		if hist_for= . then hist_for= 0;
		if rev_dia= . then rev_dia=1;
		if rev_sel= . then rev_sel=1;
		if rev_fit= . then rev_fit=1;
		if rev_for= . then rev_for=1;
	run;
	%RCSet (&syserr);
 
	/*Get the control table in WORK to process*/
	data W_&wktid..ctrl_tab;
		set dp.F_&proj._ctrl_tab;
		if (m_lvl="&Merch_level" and l_lvl="&Location_level" and kpi="&kpi" and proj_id="&proj") then output ;
	run;
	%RCSet (&syserr);
 
	/*Updating the CTRL TAB in work with the input table which is history for a merch/loc level*/
	%Addnew_and_KUPDATE(source=&dataname,target=ctrl_tab,keys=proj_id m_id M_LVL l_id L_LVL kpi,Update_flag=Y,keynum=6);
 
	%let base=roff;
	%if &dfr_wf_revisit=1 and &row_number. > 0 %then %let base=ron;
	/*Main data step where user's workflow parameters are modified as per history and revisit recommendation*/
		data W_&wktid..ctrl_tab_1(drop=_error1);
			set W_&wktid..ctrl_tab;
			if (_error1=0) or (_error1=2 and &chng_ivar=1) then do;
				/* If the record is inserted OR */
				/* If the record exists and independent var changed*/
				hist_dia=0;
				hist_sel=0;
				hist_fit=0;
				hist_for=0;
				rev_dia=1;
				rev_sel=1;
				rev_fit=1;
				rev_for=1;
				hist_mod_dttm=datetime();
				rev_mod_dttm=datetime();
				process=0;
			end;
			if &dfr_wf_forecast_intel_exe=0 then do; /*If intelligence option is OFF, then take uer's workflow parameters as they are*/
				umd_dia=&DFR_WF_DIAGNOSE;
				umd_sel=&dfr_wf_select;
				umd_fit=&dfr_wf_fit;
				umd_for=&dfr_wf_forecast;
			end;
			else do;/*If intelligence option is ON, then use historical values or revisit recommendation values depending on whether revisit is ON or OFF */
				if "&base"='roff' then do;
					umd_dia=symget('roff_'||KTRIM(KLEFT(hist_dia))||KTRIM(KLEFT(hist_sel))||KTRIM(KLEFT( hist_fit)) ||KTRIM(KLEFT(hist_for))||'_1_'||KTRIM(KLEFT(&DFR_WF_DIAGNOSE))||KTRIM(KLEFT(&dfr_wf_select))||KTRIM(KLEFT(&dfr_wf_fit))||KTRIM(KLEFT(&dfr_wf_forecast)));
					umd_sel=symget('roff_'||KTRIM(KLEFT(hist_dia))||KTRIM(KLEFT(hist_sel))||KTRIM(KLEFT( hist_fit)) ||KTRIM(KLEFT(hist_for))||'_2_'||KTRIM(KLEFT(&DFR_WF_DIAGNOSE))||KTRIM(KLEFT(&dfr_wf_select))||KTRIM(KLEFT(&dfr_wf_fit))||KTRIM(KLEFT(&dfr_wf_forecast)));
					umd_fit=symget('roff_'||KTRIM(KLEFT(hist_dia))||KTRIM(KLEFT(hist_sel))||KTRIM(KLEFT( hist_fit)) ||KTRIM(KLEFT(hist_for))||'_3_'||KTRIM(KLEFT(&DFR_WF_DIAGNOSE))||KTRIM(KLEFT(&dfr_wf_select))||KTRIM(KLEFT(&dfr_wf_fit))||KTRIM(KLEFT(&dfr_wf_forecast)));
					umd_for=symget('roff_'||KTRIM(KLEFT(hist_dia))||KTRIM(KLEFT(hist_sel))||KTRIM(KLEFT( hist_fit)) ||KTRIM(KLEFT(hist_for))||'_4_'||KTRIM(KLEFT(&DFR_WF_DIAGNOSE))||KTRIM(KLEFT(&dfr_wf_select))||KTRIM(KLEFT(&dfr_wf_fit))||KTRIM(KLEFT(&dfr_wf_forecast)));
				end; 
				else do;
					umd_dia=symget('ron_'||KTRIM(KLEFT(rev_dia))||KTRIM(KLEFT(rev_sel))||KTRIM(KLEFT( rev_fit)) ||KTRIM(KLEFT(rev_for))||'_1_'||KTRIM(KLEFT(&DFR_WF_DIAGNOSE))||KTRIM(KLEFT(&dfr_wf_select))||KTRIM(KLEFT(&dfr_wf_fit))||KTRIM(KLEFT(&dfr_wf_forecast)));
					umd_sel=symget('ron_'||KTRIM(KLEFT(rev_dia))||KTRIM(KLEFT(rev_sel))||KTRIM(KLEFT( rev_fit)) ||KTRIM(KLEFT(rev_for))||'_2_'||KTRIM(KLEFT(&DFR_WF_DIAGNOSE))||KTRIM(KLEFT(&dfr_wf_select))||KTRIM(KLEFT(&dfr_wf_fit))||KTRIM(KLEFT(&dfr_wf_forecast)));
					umd_fit=symget('ron_'||KTRIM(KLEFT(rev_dia))||KTRIM(KLEFT(rev_sel))||KTRIM(KLEFT( rev_fit)) ||KTRIM(KLEFT(rev_for))||'_3_'||KTRIM(KLEFT(&DFR_WF_DIAGNOSE))||KTRIM(KLEFT(&dfr_wf_select))||KTRIM(KLEFT(&dfr_wf_fit))||KTRIM(KLEFT(&dfr_wf_forecast)));
					umd_for=symget('ron_'||KTRIM(KLEFT(rev_dia))||KTRIM(KLEFT(rev_sel))||KTRIM(KLEFT( rev_fit)) ||KTRIM(KLEFT(rev_for))||'_4_'||KTRIM(KLEFT(&DFR_WF_DIAGNOSE))||KTRIM(KLEFT(&dfr_wf_select))||KTRIM(KLEFT(&dfr_wf_fit))||KTRIM(KLEFT(&dfr_wf_forecast)));
				end;
			end;
			umd_mod_dttm=datetime();
		run;
	%RCSet (&syserr);
	/*Updating the control table in dp library*/
	%Addnew_and_KUPDATE(source=ctrl_tab_1,target=F_&proj._ctrl_tab,target_lib=dp,keys=proj_id m_id M_LVL l_id L_LVL kpi,Update_flag=N,keynum=6);
	data dp.F_&proj._ctrl_tab;
		set dp.F_&proj._ctrl_tab;
		sr_no=_n_;
	run;
	%EndMacro:
%mend;
 
 
 
