/**************************************************************/
/*                                                            */
/*       SAS RETAIL INTELLIGENCE SOLUTIONS					  */
/* 		 Demand Forecast For Retail Planning 4.2       		  */
/*                                                            */
/**************************************************************/
/*                                                            */
/* SAS Retail Group, All rights reserved (c) 2006,2007        */
/*                                                            */
/**************************************************************/
 
/****************************************************************/
/*	Functionality- Decide the quality of forecasting models 	*/
/****************************************************************/
 
%macro revisit();
%if &dfr_wf_revisit =1 %then %do; /*START OF if revisit ON*/
	%do i=1 %to &dfr_numkpis;/*For each KPI*/
		
		data W_&wktid..baselimit&i.;
			set inp.F_&proj. ;
			if put(&dfr_time_field.,8.)<(&dfr_horizonstart.-(&revisit_obs.*&dfr_interval_no.)) or  put(&dfr_time_field.,8.)>(&dfr_horizonstart.)
				then delete;
			if s_&&kpi&i. =.  or f_&&kpi&i. =. or &&kpi&i. =. then delete;
		run;
		%RCSet (&syserr);
		proc sort data=W_&wktid..baselimit&i. ;
			by &UNRD.ID_&&&UNRECON_DIM._to_level. &RD.ID_&&&RECON_DIM._to_level. ;
		run;
		%RCSet (&syserr);
		data W_&wktid..limits&i.;
			set W_&wktid..baselimit&i.;
			retain cnt;	
			by &UNRD.ID_&&&UNRECON_DIM._to_level. &RD.ID_&&&RECON_DIM._to_level. ;
			keep &UNRD.ID_&&&UNRECON_DIM._to_level. &RD.ID_&&&RECON_DIM._to_level. s_&&kpi&i.;
			if first.&RD.ID_&&&RECON_DIM._to_level. then cnt=0 ;
			cnt=cnt+1;
			if last.&RD.ID_&&&RECON_DIM._to_level. and cnt>=&revisit_obs.;
		run;
		%RCSet (&syserr);
		%let row_no=0;
		data _null_;
			set W_&wktid..limits&i. end=L;
			if L then call symputx('row_no',_n_);
		run;
		%RCSet (&syserr);

		%if &row_no > 0 %then %do; /*START OF Do revisit if predicted before*/
				/*	creating a outer limit dataset	*/
				data W_&wktid..limits_outer&i.;
					set W_&wktid..limits&i.;
					rename s_&&kpi&i.= _STDDEV_;
					_SIGMAS_=-probit((1-&revisit_outer.)/2);;
					_MEAN_=0;
					_LIMITN_=1;
					_TYPE_='ESTIMATE';
					_SUBGRP_="&dfr_time_field.";
					_VAR_='residual';
					label _SIGMAS_=Multiple of Sigma;
					label _LIMITN_=Sample Size for Limits;
					label _TYPE_=Type of Control Limit Parameters;
					label _SUBGRP_=Rational Subgroup;
					label _VAR_=Process Variable;
					label _MEAN_=Process Mean;
				run;
				%RCSet (&syserr);

				/*creating a innner limit dataset */
				data W_&wktid..limits_inner&i.;
					set W_&wktid..limits&i.;
					rename s_&&kpi&i.= _STDDEV_;
					_SIGMAS_=-probit((1-&revisit_inner.)/2);;
					_MEAN_=0;
					_LIMITN_=1;
					_TYPE_='ESTIMATE';
					_SUBGRP_="&dfr_time_field.";
					_VAR_='residual';
					label _SIGMAS_=Multiple of Sigma;
					label _LIMITN_=Sample Size for Limits;
					label _TYPE_=Type of Control Limit Parameters;
					label _SUBGRP_=Rational Subgroup;
					label _VAR_=Process Variable;
					label _MEAN_=Process Mean;
				run;
				%RCSet (&syserr);

 			proc sql noprint;
                  create table W_&wktid..F_&proj._ctrl_tab_new as
                  select * from dp.F_&proj._ctrl_tab as b
                  where KUPCASE(kpi)=KUPCASE("&&kpi&i.") and m_lvl=&MERCH_TO_LEVEL. and
                    l_lvl=&LOC_TO_LEVEL. and proj_id=&proj.;
            quit;
			%rcset(&sqlrc);
			
		    proc sql;
				create table W_&wktid..baselimit&i._1 as
				select a.* 
				from W_&wktid..baselimit&i. a
				where exists 
					( 	select b.mid_&merch_to_level. 
						from W_&wktid..limits&i. b
					 	where a.lid_&loc_to_level. = b.lid_&loc_to_level. 
						and a.mid_&merch_to_level. = b.mid_&merch_to_level. );
				drop table W_&wktid..baselimit&i.;
			quit;
			%rcset(&sqlrc);

			proc datasets lib=W_&wktid. nodetails nolist;
				change baselimit&i._1=baselimit&i.;
			quit;
			%RCSet (&syserr);

			proc sql noprint;
                   create table W_&wktid..revisit&i. as
                   select a.*,&&kpi&i.-F_&&kpi&i. AS residual
                   from W_&wktid..Baselimit&i. as a
                   where 
                        a.MID_&merch_to_level. in (SELECT b.m_id from W_&wktid..F_&proj._ctrl_tab_new as b)
                        and a.lID_&loc_to_level. in (SELECT b.l_id from W_&wktid..F_&proj._ctrl_tab_new as b);
		                select COUNT(*) into :rec_cnt from W_&wktid..revisit&i.;
              quit;
			  %rcset(&sqlrc);
			
				/*	QC proc shewhart	*/
				proc shewhart data=W_&wktid..revisit&i. limits=W_&wktid..limits_inner&i.;
					xchart residual* &dfr_time_field. / outtable=W_&wktid..summary1_&i.  nochart;
					by &UNRD.ID_&&&UNRECON_DIM._to_level. &RD.ID_&&&RECON_DIM._to_level. ;
				run;
				%RCSet (&syserr);

				/*	Classifying into problematic and non-problematic time series	*/
				data W_&wktid..summary1_&i. W_&wktid..nonprob1_&i.;
					set W_&wktid..summary1_&i.;
					if _EXLIM_ ne '' then 
						output W_&wktid..summary1_&i.; 
					else 
						output W_&wktid..nonprob1_&i.;
				run;
				%RCSet (&syserr);

				data W_&wktid..nonprob1_&i.;
					set W_&wktid..nonprob1_&i.;
					attrib proj_id length=8 format=12. informat=12. label='Project ID';
					attrib M_LVL FORMAT=12. informat=12. label='Merchandise Level';
					attrib L_lvl FORMAT=12. informat=12. label='Location Level';
					attrib M_id FORMAT=12. informat=12. label='Merchandise ID';
					attrib L_id FORMAT=12. informat=12. label='Location ID';
					attrib kpi FORMAT=$40. informat=$40. label='Name of KPI';
					attrib rev_dia length=8 format=12. informat=12.	label='Revisit recommended Diagnose value';
					attrib rev_sel length=8 format=12. informat=12.	label='Revisit recommended Select value';
					attrib rev_fit length=8 format=12. informat=12.	label='Revisit recommended Fit value';;
					attrib rev_for length=8 format=12. informat=12.	label='Revisit recommended Forecast value';
					by &UNRD.ID_&&&UNRECON_DIM._to_level. &RD.ID_&&&RECON_DIM._to_level. ;
					l_id=lid_&loc_to_level.;
					m_id=mid_&merch_to_level.;
					m_lvl=&merch_to_level.;
					l_lvl=&loc_to_level.;
					proj_id=&proj.;
					KPI=KUPCASE("&&kpi&i.");
					rev_dia=0;
					rev_sel=0;
					rev_fit=0;
					rev_for=1;
					if first.&RD.ID_&&&RECON_DIM._to_level.;
						drop LID: MID:;
				run;

				/*	Updateing to control table for non-problematic	*/
				data dp.F_&proj._Ctrl_tab;
					modify dp.F_&proj._Ctrl_tab W_&wktid..nonprob1_&i.;
					by proj_id l_id m_id m_lvl l_lvl kpi ;
					if %sysrc(_DSENMR) eq _iorc_ then output;
					else if %sysrc(_SOK) eq _iorc_ then replace;
					_iorc_ = 0; _error_ = 0;
				run;
				%RCSet (&syserr);

				data W_&wktid..summary1_&i.;
					set W_&wktid..summary1_&i.;
					attrib proj_id length=8 format=12. informat=12. label='Project ID';
					attrib M_LVL FORMAT=12. informat=12. label='Merchandise Level';
					attrib L_lvl FORMAT=12. informat=12. label='Location Level';
					attrib M_id FORMAT=12. informat=12. label='Merchandise ID';
					attrib L_id FORMAT=12. informat=12. label='Location ID';
					attrib kpi FORMAT=$40. informat=$40. label='Name of KPI';
					attrib rev_dia length=8 format=12. informat=12.	label='Revisit recommended Diagnose value';
					attrib rev_sel length=8 format=12. informat=12.	label='Revisit recommended Select value';
					attrib rev_fit length=8 format=12. informat=12.	label='Revisit recommended Fit value';;
					attrib rev_for length=8 format=12. informat=12.	label='Revisit recommended Forecast value';
					by &UNRD.ID_&&&UNRECON_DIM._to_level. &RD.ID_&&&RECON_DIM._to_level. ;
					l_id=lid_&loc_to_level.;
					m_id=mid_&merch_to_level.;
					m_lvl=&merch_to_level.;
					l_lvl=&loc_to_level.;
					proj_id=&proj.;
					KPI=KUPCASE("&&kpi&i.");
					rev_dia=0;
					rev_sel=0;
					rev_fit=1;
					rev_for=1;
					if first.&RD.ID_&&&RECON_DIM._to_level.;
						drop LID: MID:;
				run;

				/*	Updateing to control table for problematic	*/
				data dp.F_&proj._Ctrl_tab;
					modify dp.F_&proj._Ctrl_tab W_&wktid..summary1_&i.;
					by proj_id l_id m_id m_lvl l_lvl kpi ;
					if %sysrc(_DSENMR) eq _iorc_ then output;
					else if %sysrc(_SOK) eq _iorc_ then replace;
					_iorc_ = 0; _error_ = 0;
				run;
				%RCSet (&syserr);

				proc shewhart data=W_&wktid..revisit&i. limits=W_&wktid..limits_outer&i.;
					xchart residual* &dfr_time_field. / outtable=W_&wktid..summary2_&i. (where =(_EXLIM_ ne '')) nochart;
					by &UNRD.ID_&&&UNRECON_DIM._to_level. &RD.ID_&&&RECON_DIM._to_level. ;
				run;
				%RCSet (&syserr);

				data W_&wktid..summary2_&i.;
					set W_&wktid..summary2_&i.;
					attrib proj_id  length=8 format=12. informat=12. label='Project ID';
					attrib M_LVL FORMAT=12. informat=12. label='Merchandise Level';
					attrib L_lvl FORMAT=12. informat=12. label='Location Level';
					attrib M_id FORMAT=12. informat=12. label='Merchandise ID';
					attrib L_id FORMAT=12. informat=12. label='Location ID';
					attrib kpi FORMAT= $40. informat=$40. label='Name of KPI';
					attrib rev_dia length=8 format=12. informat=12.	label='Revisit recommended Diagnose value';
					attrib rev_sel length=8 format=12. informat=12.	label='Revisit recommended Select value';
					attrib rev_fit length=8 format=12. informat=12.	label='Revisit recommended Fit value';;
					attrib rev_for length=8 format=12. informat=12.	label='Revisit recommended Forecast value';
					by &UNRD.ID_&&&UNRECON_DIM._to_level. &RD.ID_&&&RECON_DIM._to_level. ;
					l_id=lid_&loc_to_level.;
					m_id=mid_&merch_to_level.;
					m_lvl=&merch_to_level.;
					l_lvl=&loc_to_level.;
					proj_id=&proj.;
					KPI=KUPCASE("&&kpi&i.");
					rev_dia=1;
					rev_sel=1;
					rev_fit=1;
					rev_for=1;
					if first.&RD.ID_&&&RECON_DIM._to_level.;
						drop LID: MID:;
				run;
				%RCSet (&syserr);

				/*	Updateing to control table for problematic	*/
				data dp.F_&proj._Ctrl_tab;
					modify dp.F_&proj._Ctrl_tab W_&wktid..summary2_&i.;
					by proj_id l_id m_id m_lvl l_lvl kpi ;
					if %sysrc(_DSENMR) eq _iorc_ then output;
					else if %sysrc(_SOK) eq _iorc_ then replace;
					_iorc_ = 0; _error_ = 0;
				run;
				%RCSet (&syserr);
			
		%end;/*END Do revisit if predicted before*/
	%end;/*END OF FOR each KPI*/
%end;/*END OF if revisit ON*/
%mend revisit;
