/**************************************************************/
/*                                                            */
/*       SAS RETAIL INTELLIGENCE SOLUTIONS					  */
/* 		 Demand Forecast For Retail Planning 4.2       		  */
/*                                                            */
/**************************************************************/
/*                                                            */
/* SAS Retail Group, All rights reserved (c) 2006,2007        */
/*                                                            */
/**************************************************************/
 
/*****************************************************************************************************************/
/*Functionality- Setting various effects variables */
/*****************************************************************************************************************/
/* setting the variables for effects. They are initialized to following */
 
%global var_price var_promo var_pantry var_delay var_eve var_weather var_halo dfr_data_promo_num
	input_price input_promo input_pantry input_delay input_eve input_weather input_halo
	control_price control_promo control_pantry control_delay control_Halo control_Weather effects_level 
	tp_st_col tp_end_col dfr_user_define_gk promo_time_mode price_effect promo_effect pantry_effect delay_effect
	events_effect halo_cannib_effect weather_effect effects_level RIS_weekday_2 diag_outprocinfo_data eng_outprocinfo_data
	recon_outprocinfo_data
	;

options FMTSEARCH=(stage WORK);
/*%let dfr_user_define_gk=1; 1 - User defined group key enabled. 0 - User defined group key disabled. */
%let dfr_data_promo_num=%wordcnt(string=&dfr_data_promo_types.);
%let var_price=%BQUOTE (;) ;
%let VAR_Promo=%BQUOTE(;);
%let VAR_Pantry=%BQUOTE(;);
%let var_DELAY=%BQUOTE(;);
%LET var_EVE=%BQUOTE(;);
%let var_Halo=%BQUOTE (;) ;
%let var_Weather=%BQUOTE (;) ;
%let input_price=%BQUOTE(;);
%let input_Promo=%BQUOTE(;) ;
%let input_Pantry =%BQUOTE(;);
%let input_DELAY =%BQUOTE(;);
%let input_eve =%BQUOTE(;);
%let input_Halo=%BQUOTE(;);
%let input_Weather=%BQUOTE(;);
 
%let control_price=%BQUOTE (;) ;
%let control_Promo=%BQUOTE (;) ;
%let control_Pantry=%BQUOTE (;) ;
%let control_DELAY=%BQUOTE (;) ;
%let control_Halo=%BQUOTE (;) ;
%let control_Weather=%BQUOTE (;) ;
 
%get_name_KINDEX(var=&dfr_kpinameset,varno=&dfr_numkpis,name=kpi);
 
%let price_effect=0;
%let promo_effect=0;
%let pantry_effect=0;
%let delay_effect=0;
%let events_effect=0;
%let halo_cannib_effect=0;
%let weather_effect=0;
%let effects_level=1;
%let promo_time_mode=CAL;
/*Setting variables from the jobsparam dataset*/
option mprint mlogic symbolgen;
%macro effects();
      %if "&dfr_data_effect_list" ne "" %then %do;
	  		%let dfr_data_effect_list=%sysfunc(KUPCASE(%BQUOTE(&dfr_data_effect_list)));
            %if "&dfr_ext_price_abt" ne "" %then %let price_effect=%sysfunc(KINDEX("&dfr_data_effect_list",PRICE));
            %if "&dfr_ext_promo_file" ne "" %then %do;
				%let promo_effect=%sysfunc(KINDEX("&dfr_data_effect_list",PROMO));
				%if &promo_effect ne 0 %then %do;
		            %let pantry_effect=%sysfunc(KINDEX("&dfr_data_effect_list",PANTRY));
		            %let delay_effect=%sysfunc(KINDEX("&dfr_data_effect_list",DELAY));
		            %let halo_cannib_effect=%sysfunc(KINDEX("&dfr_data_effect_list",HALO));
				%end;
			%end;
            %if "&dfr_ext_weather_abt" ne "" %then %let weather_effect=%sysfunc(KINDEX("&dfr_data_effect_list",WEATHER));
      %end;
%mend;
%effects;
%ind_var_name;
 
/* Populating individual variables from the jobsparam dataset*/
data _null_;
 
/* Set the variables as required as the effect list in the jobsparam dataset*/
if &price_effect gt 0 then
	do;
	call symput('input_price','input price_ratio');
	call symput('var_price','var price_ratio');
	End;
 
if &promo_effect gt 0  then
	do;
	call symput('input_Promo','input promo:');
	call symput('var_Promo','var promo:');
	End;
 
if &pantry_effect gt 0  then
	do;
	call symput('input_Pantry','input Pantry:');
	call symput('var_Pantry','var Pantry:');
	End;
 
if &delay_effect gt 0  then
	do;
	call symput('input_DELAY','input DELAY:');
	call symput('var_DELAY','var DELAY:');
	End;
 
if &halo_cannib_effect gt 0  and &merch_to_level.-&merch_from_level. >=3 then do;
	call symput('input_Halo','input TOTAL_:');
	call symput('var_Halo','var TOTAL_:');
	call symput('control_Halo','CONTROL TOTAL_:');
	End;
 
if &weather_effect gt 0   then do;
	call symput('input_Weather','input TEMPERATURE:');
	call symput('var_Weather','var TEMPERATURE:');
	call symput('control_Weather','CONTROL TEMPERATURE:');
	end;
run;
%RCSet (&syserr);
 
/* format horizonstart into eng_end parameter  */
data _null_;
	length a $20;
	retain a;
	length b $20;
	retain b;
	a=''''||put(&dfr_horizonstart.,date9.)||'''d';
	call symputx("eng_end",a,'g');
	if KUPCASE("&dfr_interval")='WEEK' then b=''''||put(intnx('week.7',&dfr_horizonstart,1),date9.)||'''d';
	if KUPCASE("&dfr_interval")='MONTH' then b=''''||put(intnx('Month',&dfr_horizonstart,1),date9.)||'''d';
	if KUPCASE("&dfr_interval")='QUARTER' then b=''''||put(intnx('Quarter',&dfr_horizonstart,1),date9.)||'''d';
	call symputx("eng_end_1",b,'g');
run;
 
%RCSet (&syserr);
 
%let RIS_weekday_2=%sysfunc(mod(&dfr_horizonstart,7));

/*setting the dataset for procedure info*/

%let diag_outprocinfo_data=diag_outprocinfo;
%let eng_outprocinfo_data=eng_outprocinfo;
%let recon_outprocinfo_data=recon_outprocinfo;

/************************************************************************/
/*	Functionality- Initializing variable that are required to 			*/
/*	modify user's workflow parameters when intelligence option is ON	*/
/************************************************************************/

%global Roff_0000_1_0001;
%global Roff_0000_1_0010;
%global Roff_0000_1_0011;
%global Roff_0000_1_0100;
%global Roff_0000_1_0101;
%global Roff_0000_1_0110;
%global Roff_0000_1_0111;
%global Roff_0000_1_1000;
%global Roff_0000_1_1001;
%global Roff_0000_1_1010;
%global Roff_0000_1_1011;
%global Roff_0000_1_1100;
%global Roff_0000_1_1101;
%global Roff_0000_1_1110;
%global Roff_0000_1_1111;
%global Roff_0000_2_0001;
%global Roff_0000_2_0010;
%global Roff_0000_2_0011;
%global Roff_0000_2_0100;
%global Roff_0000_2_0101;
%global Roff_0000_2_0110;
%global Roff_0000_2_0111;
%global Roff_0000_2_1000;
%global Roff_0000_2_1001;
%global Roff_0000_2_1010;
%global Roff_0000_2_1011;
%global Roff_0000_2_1100;
%global Roff_0000_2_1101;
%global Roff_0000_2_1110;
%global Roff_0000_2_1111;
%global Roff_0000_3_0001;
%global Roff_0000_3_0010;
%global Roff_0000_3_0011;
%global Roff_0000_3_0100;
%global Roff_0000_3_0101;
%global Roff_0000_3_0110;
%global Roff_0000_3_0111;
%global Roff_0000_3_1000;
%global Roff_0000_3_1001;
%global Roff_0000_3_1010;
%global Roff_0000_3_1011;
%global Roff_0000_3_1100;
%global Roff_0000_3_1101;
%global Roff_0000_3_1110;
%global Roff_0000_3_1111;
%global Roff_0000_4_0001;
%global Roff_0000_4_0010;
%global Roff_0000_4_0011;
%global Roff_0000_4_0100;
%global Roff_0000_4_0101;
%global Roff_0000_4_0110;
%global Roff_0000_4_0111;
%global Roff_0000_4_1000;
%global Roff_0000_4_1001;
%global Roff_0000_4_1010;
%global Roff_0000_4_1011;
%global Roff_0000_4_1100;
%global Roff_0000_4_1101;
%global Roff_0000_4_1110;
%global Roff_0000_4_1111;
%global Roff_1000_1_0001;
%global Roff_1000_1_0010;
%global Roff_1000_1_0011;
%global Roff_1000_1_0100;
%global Roff_1000_1_0101;
%global Roff_1000_1_0110;
%global Roff_1000_1_0111;
%global Roff_1000_1_1000;
%global Roff_1000_1_1001;
%global Roff_1000_1_1010;
%global Roff_1000_1_1011;
%global Roff_1000_1_1100;
%global Roff_1000_1_1101;
%global Roff_1000_1_1110;
%global Roff_1000_1_1111;
%global Roff_1000_2_0001;
%global Roff_1000_2_0010;
%global Roff_1000_2_0011;
%global Roff_1000_2_0100;
%global Roff_1000_2_0101;
%global Roff_1000_2_0110;
%global Roff_1000_2_0111;
%global Roff_1000_2_1000;
%global Roff_1000_2_1001;
%global Roff_1000_2_1010;
%global Roff_1000_2_1011;
%global Roff_1000_2_1100;
%global Roff_1000_2_1101;
%global Roff_1000_2_1110;
%global Roff_1000_2_1111;
%global Roff_1000_3_0001;
%global Roff_1000_3_0010;
%global Roff_1000_3_0011;
%global Roff_1000_3_0100;
%global Roff_1000_3_0101;
%global Roff_1000_3_0110;
%global Roff_1000_3_0111;
%global Roff_1000_3_1000;
%global Roff_1000_3_1001;
%global Roff_1000_3_1010;
%global Roff_1000_3_1011;
%global Roff_1000_3_1100;
%global Roff_1000_3_1101;
%global Roff_1000_3_1110;
%global Roff_1000_3_1111;
%global Roff_1000_4_0001;
%global Roff_1000_4_0010;
%global Roff_1000_4_0011;
%global Roff_1000_4_0100;
%global Roff_1000_4_0101;
%global Roff_1000_4_0110;
%global Roff_1000_4_0111;
%global Roff_1000_4_1000;
%global Roff_1000_4_1001;
%global Roff_1000_4_1010;
%global Roff_1000_4_1011;
%global Roff_1000_4_1100;
%global Roff_1000_4_1101;
%global Roff_1000_4_1110;
%global Roff_1000_4_1111;
%global Roff_1100_1_0001;
%global Roff_1100_1_0010;
%global Roff_1100_1_0011;
%global Roff_1100_1_0100;
%global Roff_1100_1_0101;
%global Roff_1100_1_0110;
%global Roff_1100_1_0111;
%global Roff_1100_1_1000;
%global Roff_1100_1_1001;
%global Roff_1100_1_1010;
%global Roff_1100_1_1011;
%global Roff_1100_1_1100;
%global Roff_1100_1_1101;
%global Roff_1100_1_1110;
%global Roff_1100_1_1111;
%global Roff_1100_2_0001;
%global Roff_1100_2_0010;
%global Roff_1100_2_0011;
%global Roff_1100_2_0100;
%global Roff_1100_2_0101;
%global Roff_1100_2_0110;
%global Roff_1100_2_0111;
%global Roff_1100_2_1000;
%global Roff_1100_2_1001;
%global Roff_1100_2_1010;
%global Roff_1100_2_1011;
%global Roff_1100_2_1100;
%global Roff_1100_2_1101;
%global Roff_1100_2_1110;
%global Roff_1100_2_1111;
%global Roff_1100_3_0001;
%global Roff_1100_3_0010;
%global Roff_1100_3_0011;
%global Roff_1100_3_0100;
%global Roff_1100_3_0101;
%global Roff_1100_3_0110;
%global Roff_1100_3_0111;
%global Roff_1100_3_1000;
%global Roff_1100_3_1001;
%global Roff_1100_3_1010;
%global Roff_1100_3_1011;
%global Roff_1100_3_1100;
%global Roff_1100_3_1101;
%global Roff_1100_3_1110;
%global Roff_1100_3_1111;
%global Roff_1100_4_0001;
%global Roff_1100_4_0010;
%global Roff_1100_4_0011;
%global Roff_1100_4_0100;
%global Roff_1100_4_0101;
%global Roff_1100_4_0110;
%global Roff_1100_4_0111;
%global Roff_1100_4_1000;
%global Roff_1100_4_1001;
%global Roff_1100_4_1010;
%global Roff_1100_4_1011;
%global Roff_1100_4_1100;
%global Roff_1100_4_1101;
%global Roff_1100_4_1110;
%global Roff_1100_4_1111;
%global Roff_1110_1_0001;
%global Roff_1110_1_0010;
%global Roff_1110_1_0011;
%global Roff_1110_1_0100;
%global Roff_1110_1_0101;
%global Roff_1110_1_0110;
%global Roff_1110_1_0111;
%global Roff_1110_1_1000;
%global Roff_1110_1_1001;
%global Roff_1110_1_1010;
%global Roff_1110_1_1011;
%global Roff_1110_1_1100;
%global Roff_1110_1_1101;
%global Roff_1110_1_1110;
%global Roff_1110_1_1111;
%global Roff_1110_2_0001;
%global Roff_1110_2_0010;
%global Roff_1110_2_0011;
%global Roff_1110_2_0100;
%global Roff_1110_2_0101;
%global Roff_1110_2_0110;
%global Roff_1110_2_0111;
%global Roff_1110_2_1000;
%global Roff_1110_2_1001;
%global Roff_1110_2_1010;
%global Roff_1110_2_1011;
%global Roff_1110_2_1100;
%global Roff_1110_2_1101;
%global Roff_1110_2_1110;
%global Roff_1110_2_1111;
%global Roff_1110_3_0001;
%global Roff_1110_3_0010;
%global Roff_1110_3_0011;
%global Roff_1110_3_0100;
%global Roff_1110_3_0101;
%global Roff_1110_3_0110;
%global Roff_1110_3_0111;
%global Roff_1110_3_1000;
%global Roff_1110_3_1001;
%global Roff_1110_3_1010;
%global Roff_1110_3_1011;
%global Roff_1110_3_1100;
%global Roff_1110_3_1101;
%global Roff_1110_3_1110;
%global Roff_1110_3_1111;
%global Roff_1110_4_0001;
%global Roff_1110_4_0010;
%global Roff_1110_4_0011;
%global Roff_1110_4_0100;
%global Roff_1110_4_0101;
%global Roff_1110_4_0110;
%global Roff_1110_4_0111;
%global Roff_1110_4_1000;
%global Roff_1110_4_1001;
%global Roff_1110_4_1010;
%global Roff_1110_4_1011;
%global Roff_1110_4_1100;
%global Roff_1110_4_1101;
%global Roff_1110_4_1110;
%global Roff_1110_4_1111;
%global Roff_1111_1_0001;
%global Roff_1111_1_0010;
%global Roff_1111_1_0011;
%global Roff_1111_1_0100;
%global Roff_1111_1_0101;
%global Roff_1111_1_0110;
%global Roff_1111_1_0111;
%global Roff_1111_1_1000;
%global Roff_1111_1_1001;
%global Roff_1111_1_1010;
%global Roff_1111_1_1011;
%global Roff_1111_1_1100;
%global Roff_1111_1_1101;
%global Roff_1111_1_1110;
%global Roff_1111_1_1111;
%global Roff_1111_2_0001;
%global Roff_1111_2_0010;
%global Roff_1111_2_0011;
%global Roff_1111_2_0100;
%global Roff_1111_2_0101;
%global Roff_1111_2_0110;
%global Roff_1111_2_0111;
%global Roff_1111_2_1000;
%global Roff_1111_2_1001;
%global Roff_1111_2_1010;
%global Roff_1111_2_1011;
%global Roff_1111_2_1100;
%global Roff_1111_2_1101;
%global Roff_1111_2_1110;
%global Roff_1111_2_1111;
%global Roff_1111_3_0001;
%global Roff_1111_3_0010;
%global Roff_1111_3_0011;
%global Roff_1111_3_0100;
%global Roff_1111_3_0101;
%global Roff_1111_3_0110;
%global Roff_1111_3_0111;
%global Roff_1111_3_1000;
%global Roff_1111_3_1001;
%global Roff_1111_3_1010;
%global Roff_1111_3_1011;
%global Roff_1111_3_1100;
%global Roff_1111_3_1101;
%global Roff_1111_3_1110;
%global Roff_1111_3_1111;
%global Roff_1111_4_0001;
%global Roff_1111_4_0010;
%global Roff_1111_4_0011;
%global Roff_1111_4_0100;
%global Roff_1111_4_0101;
%global Roff_1111_4_0110;
%global Roff_1111_4_0111;
%global Roff_1111_4_1000;
%global Roff_1111_4_1001;
%global Roff_1111_4_1010;
%global Roff_1111_4_1011;
%global Roff_1111_4_1100;
%global Roff_1111_4_1101;
%global Roff_1111_4_1110;
%global Roff_1111_4_1111;
%global Ron_1111_1_0001;
%global Ron_1111_1_0010;
%global Ron_1111_1_0011;
%global Ron_1111_1_0100;
%global Ron_1111_1_0101;
%global Ron_1111_1_0110;
%global Ron_1111_1_0111;
%global Ron_1111_1_1000;
%global Ron_1111_1_1001;
%global Ron_1111_1_1010;
%global Ron_1111_1_1011;
%global Ron_1111_1_1100;
%global Ron_1111_1_1101;
%global Ron_1111_1_1110;
%global Ron_1111_1_1111;
%global Ron_1111_2_0001;
%global Ron_1111_2_0010;
%global Ron_1111_2_0011;
%global Ron_1111_2_0100;
%global Ron_1111_2_0101;
%global Ron_1111_2_0110;
%global Ron_1111_2_0111;
%global Ron_1111_2_1000;
%global Ron_1111_2_1001;
%global Ron_1111_2_1010;
%global Ron_1111_2_1011;
%global Ron_1111_2_1100;
%global Ron_1111_2_1101;
%global Ron_1111_2_1110;
%global Ron_1111_2_1111;
%global Ron_1111_3_0001;
%global Ron_1111_3_0010;
%global Ron_1111_3_0011;
%global Ron_1111_3_0100;
%global Ron_1111_3_0101;
%global Ron_1111_3_0110;
%global Ron_1111_3_0111;
%global Ron_1111_3_1000;
%global Ron_1111_3_1001;
%global Ron_1111_3_1010;
%global Ron_1111_3_1011;
%global Ron_1111_3_1100;
%global Ron_1111_3_1101;
%global Ron_1111_3_1110;
%global Ron_1111_3_1111;
%global Ron_1111_4_0001;
%global Ron_1111_4_0010;
%global Ron_1111_4_0011;
%global Ron_1111_4_0100;
%global Ron_1111_4_0101;
%global Ron_1111_4_0110;
%global Ron_1111_4_0111;
%global Ron_1111_4_1000;
%global Ron_1111_4_1001;
%global Ron_1111_4_1010;
%global Ron_1111_4_1011;
%global Ron_1111_4_1100;
%global Ron_1111_4_1101;
%global Ron_1111_4_1110;
%global Ron_1111_4_1111;
%global Ron_0111_1_0001;
%global Ron_0111_1_0010;
%global Ron_0111_1_0011;
%global Ron_0111_1_0100;
%global Ron_0111_1_0101;
%global Ron_0111_1_0110;
%global Ron_0111_1_0111;
%global Ron_0111_1_1000;
%global Ron_0111_1_1001;
%global Ron_0111_1_1010;
%global Ron_0111_1_1011;
%global Ron_0111_1_1100;
%global Ron_0111_1_1101;
%global Ron_0111_1_1110;
%global Ron_0111_1_1111;
%global Ron_0111_2_0001;
%global Ron_0111_2_0010;
%global Ron_0111_2_0011;
%global Ron_0111_2_0100;
%global Ron_0111_2_0101;
%global Ron_0111_2_0110;
%global Ron_0111_2_0111;
%global Ron_0111_2_1000;
%global Ron_0111_2_1001;
%global Ron_0111_2_1010;
%global Ron_0111_2_1011;
%global Ron_0111_2_1100;
%global Ron_0111_2_1101;
%global Ron_0111_2_1110;
%global Ron_0111_2_1111;
%global Ron_0111_3_0001;
%global Ron_0111_3_0010;
%global Ron_0111_3_0011;
%global Ron_0111_3_0100;
%global Ron_0111_3_0101;
%global Ron_0111_3_0110;
%global Ron_0111_3_0111;
%global Ron_0111_3_1000;
%global Ron_0111_3_1001;
%global Ron_0111_3_1010;
%global Ron_0111_3_1011;
%global Ron_0111_3_1100;
%global Ron_0111_3_1101;
%global Ron_0111_3_1110;
%global Ron_0111_3_1111;
%global Ron_0111_4_0001;
%global Ron_0111_4_0010;
%global Ron_0111_4_0011;
%global Ron_0111_4_0100;
%global Ron_0111_4_0101;
%global Ron_0111_4_0110;
%global Ron_0111_4_0111;
%global Ron_0111_4_1000;
%global Ron_0111_4_1001;
%global Ron_0111_4_1010;
%global Ron_0111_4_1011;
%global Ron_0111_4_1100;
%global Ron_0111_4_1101;
%global Ron_0111_4_1110;
%global Ron_0111_4_1111;
%global Ron_0011_1_0001;
%global Ron_0011_1_0010;
%global Ron_0011_1_0011;
%global Ron_0011_1_0100;
%global Ron_0011_1_0101;
%global Ron_0011_1_0110;
%global Ron_0011_1_0111;
%global Ron_0011_1_1000;
%global Ron_0011_1_1001;
%global Ron_0011_1_1010;
%global Ron_0011_1_1011;
%global Ron_0011_1_1100;
%global Ron_0011_1_1101;
%global Ron_0011_1_1110;
%global Ron_0011_1_1111;
%global Ron_0011_2_0001;
%global Ron_0011_2_0010;
%global Ron_0011_2_0011;
%global Ron_0011_2_0100;
%global Ron_0011_2_0101;
%global Ron_0011_2_0110;
%global Ron_0011_2_0111;
%global Ron_0011_2_1000;
%global Ron_0011_2_1001;
%global Ron_0011_2_1010;
%global Ron_0011_2_1011;
%global Ron_0011_2_1100;
%global Ron_0011_2_1101;
%global Ron_0011_2_1110;
%global Ron_0011_2_1111;
%global Ron_0011_3_0001;
%global Ron_0011_3_0010;
%global Ron_0011_3_0011;
%global Ron_0011_3_0100;
%global Ron_0011_3_0101;
%global Ron_0011_3_0110;
%global Ron_0011_3_0111;
%global Ron_0011_3_1000;
%global Ron_0011_3_1001;
%global Ron_0011_3_1010;
%global Ron_0011_3_1011;
%global Ron_0011_3_1100;
%global Ron_0011_3_1101;
%global Ron_0011_3_1110;
%global Ron_0011_3_1111;
%global Ron_0011_4_0001;
%global Ron_0011_4_0010;
%global Ron_0011_4_0011;
%global Ron_0011_4_0100;
%global Ron_0011_4_0101;
%global Ron_0011_4_0110;
%global Ron_0011_4_0111;
%global Ron_0011_4_1000;
%global Ron_0011_4_1001;
%global Ron_0011_4_1010;
%global Ron_0011_4_1011;
%global Ron_0011_4_1100;
%global Ron_0011_4_1101;
%global Ron_0011_4_1110;
%global Ron_0011_4_1111;
%global Ron_0001_1_0001;
%global Ron_0001_1_0010;
%global Ron_0001_1_0011;
%global Ron_0001_1_0100;
%global Ron_0001_1_0101;
%global Ron_0001_1_0110;
%global Ron_0001_1_0111;
%global Ron_0001_1_1000;
%global Ron_0001_1_1001;
%global Ron_0001_1_1010;
%global Ron_0001_1_1011;
%global Ron_0001_1_1100;
%global Ron_0001_1_1101;
%global Ron_0001_1_1110;
%global Ron_0001_1_1111;
%global Ron_0001_2_0001;
%global Ron_0001_2_0010;
%global Ron_0001_2_0011;
%global Ron_0001_2_0100;
%global Ron_0001_2_0101;
%global Ron_0001_2_0110;
%global Ron_0001_2_0111;
%global Ron_0001_2_1000;
%global Ron_0001_2_1001;
%global Ron_0001_2_1010;
%global Ron_0001_2_1011;
%global Ron_0001_2_1100;
%global Ron_0001_2_1101;
%global Ron_0001_2_1110;
%global Ron_0001_2_1111;
%global Ron_0001_3_0001;
%global Ron_0001_3_0010;
%global Ron_0001_3_0011;
%global Ron_0001_3_0100;
%global Ron_0001_3_0101;
%global Ron_0001_3_0110;
%global Ron_0001_3_0111;
%global Ron_0001_3_1000;
%global Ron_0001_3_1001;
%global Ron_0001_3_1010;
%global Ron_0001_3_1011;
%global Ron_0001_3_1100;
%global Ron_0001_3_1101;
%global Ron_0001_3_1110;
%global Ron_0001_3_1111;
%global Ron_0001_4_0001;
%global Ron_0001_4_0010;
%global Ron_0001_4_0011;
%global Ron_0001_4_0100;
%global Ron_0001_4_0101;
%global Ron_0001_4_0110;
%global Ron_0001_4_0111;
%global Ron_0001_4_1000;
%global Ron_0001_4_1001;
%global Ron_0001_4_1010;
%global Ron_0001_4_1011;
%global Ron_0001_4_1100;
%global Ron_0001_4_1101;
%global Ron_0001_4_1110;
%global Ron_0001_4_1111;
 
%Let  Roff_0000_1_0001=1;
%Let  Roff_0000_1_0010=1;
%Let  Roff_0000_1_0011=1;
%Let  Roff_0000_1_0100=1;
%Let  Roff_0000_1_0101=1;
%Let  Roff_0000_1_0110=1;
%Let  Roff_0000_1_0111=1;
%Let  Roff_0000_1_1000=1;
%Let  Roff_0000_1_1001=1;
%Let  Roff_0000_1_1010=1;
%Let  Roff_0000_1_1011=1;
%Let  Roff_0000_1_1100=1;
%Let  Roff_0000_1_1101=1;
%Let  Roff_0000_1_1110=1;
%Let  Roff_0000_1_1111=1;
%Let  Roff_0000_2_0001=1;
%Let  Roff_0000_2_0010=1;
%Let  Roff_0000_2_0011=1;
%Let  Roff_0000_2_0100=1;
%Let  Roff_0000_2_0101=1;
%Let  Roff_0000_2_0110=1;
%Let  Roff_0000_2_0111=1;
%Let  Roff_0000_2_1000=0;
%Let  Roff_0000_2_1001=1;
%Let  Roff_0000_2_1010=1;
%Let  Roff_0000_2_1011=1;
%Let  Roff_0000_2_1100=1;
%Let  Roff_0000_2_1101=1;
%Let  Roff_0000_2_1110=1;
%Let  Roff_0000_2_1111=1;
%Let  Roff_0000_3_0001=1;
%Let  Roff_0000_3_0010=1;
%Let  Roff_0000_3_0011=1;
%Let  Roff_0000_3_0100=0;
%Let  Roff_0000_3_0101=1;
%Let  Roff_0000_3_0110=1;
%Let  Roff_0000_3_0111=1;
%Let  Roff_0000_3_1000=0;
%Let  Roff_0000_3_1001=1;
%Let  Roff_0000_3_1010=1;
%Let  Roff_0000_3_1011=1;
%Let  Roff_0000_3_1100=0;
%Let  Roff_0000_3_1101=1;
%Let  Roff_0000_3_1110=1;
%Let  Roff_0000_3_1111=1;
%Let  Roff_0000_4_0001=1;
%Let  Roff_0000_4_0010=0;
%Let  Roff_0000_4_0011=1;
%Let  Roff_0000_4_0100=0;
%Let  Roff_0000_4_0101=1;
%Let  Roff_0000_4_0110=0;
%Let  Roff_0000_4_0111=1;
%Let  Roff_0000_4_1000=0;
%Let  Roff_0000_4_1001=1;
%Let  Roff_0000_4_1010=0;
%Let  Roff_0000_4_1011=1;
%Let  Roff_0000_4_1100=0;
%Let  Roff_0000_4_1101=1;
%Let  Roff_0000_4_1110=0;
%Let  Roff_0000_4_1111=1;
%Let  Roff_1000_1_0001=0;
%Let  Roff_1000_1_0010=0;
%Let  Roff_1000_1_0011=0;
%Let  Roff_1000_1_0100=0;
%Let  Roff_1000_1_0101=0;
%Let  Roff_1000_1_0110=0;
%Let  Roff_1000_1_0111=0;
%Let  Roff_1000_1_1000=1;
%Let  Roff_1000_1_1001=1;
%Let  Roff_1000_1_1010=1;
%Let  Roff_1000_1_1011=1;
%Let  Roff_1000_1_1100=1;
%Let  Roff_1000_1_1101=1;
%Let  Roff_1000_1_1110=1;
%Let  Roff_1000_1_1111=1;
%Let  Roff_1000_2_0001=1;
%Let  Roff_1000_2_0010=1;
%Let  Roff_1000_2_0011=1;
%Let  Roff_1000_2_0100=1;
%Let  Roff_1000_2_0101=1;
%Let  Roff_1000_2_0110=1;
%Let  Roff_1000_2_0111=1;
%Let  Roff_1000_2_1000=0;
%Let  Roff_1000_2_1001=1;
%Let  Roff_1000_2_1010=1;
%Let  Roff_1000_2_1011=1;
%Let  Roff_1000_2_1100=1;
%Let  Roff_1000_2_1101=1;
%Let  Roff_1000_2_1110=1;
%Let  Roff_1000_2_1111=1;
%Let  Roff_1000_3_0001=1;
%Let  Roff_1000_3_0010=1;
%Let  Roff_1000_3_0011=1;
%Let  Roff_1000_3_0100=0;
%Let  Roff_1000_3_0101=1;
%Let  Roff_1000_3_0110=1;
%Let  Roff_1000_3_0111=1;
%Let  Roff_1000_3_1000=0;
%Let  Roff_1000_3_1001=1;
%Let  Roff_1000_3_1010=1;
%Let  Roff_1000_3_1011=1;
%Let  Roff_1000_3_1100=0;
%Let  Roff_1000_3_1101=1;
%Let  Roff_1000_3_1110=1;
%Let  Roff_1000_3_1111=1;
%Let  Roff_1000_4_0001=1;
%Let  Roff_1000_4_0010=0;
%Let  Roff_1000_4_0011=1;
%Let  Roff_1000_4_0100=0;
%Let  Roff_1000_4_0101=1;
%Let  Roff_1000_4_0110=0;
%Let  Roff_1000_4_0111=1;
%Let  Roff_1000_4_1000=0;
%Let  Roff_1000_4_1001=1;
%Let  Roff_1000_4_1010=0;
%Let  Roff_1000_4_1011=1;
%Let  Roff_1000_4_1100=0;
%Let  Roff_1000_4_1101=1;
%Let  Roff_1000_4_1110=0;
%Let  Roff_1000_4_1111=1;
%Let  Roff_1100_1_0001=0;
%Let  Roff_1100_1_0010=0;
%Let  Roff_1100_1_0011=0;
%Let  Roff_1100_1_0100=0;
%Let  Roff_1100_1_0101=0;
%Let  Roff_1100_1_0110=0;
%Let  Roff_1100_1_0111=0;
%Let  Roff_1100_1_1000=1;
%Let  Roff_1100_1_1001=1;
%Let  Roff_1100_1_1010=1;
%Let  Roff_1100_1_1011=1;
%Let  Roff_1100_1_1100=1;
%Let  Roff_1100_1_1101=1;
%Let  Roff_1100_1_1110=1;
%Let  Roff_1100_1_1111=1;
%Let  Roff_1100_2_0001=0;
%Let  Roff_1100_2_0010=0;
%Let  Roff_1100_2_0011=0;
%Let  Roff_1100_2_0100=1;
%Let  Roff_1100_2_0101=1;
%Let  Roff_1100_2_0110=1;
%Let  Roff_1100_2_0111=1;
%Let  Roff_1100_2_1000=0;
%Let  Roff_1100_2_1001=1;
%Let  Roff_1100_2_1010=1;
%Let  Roff_1100_2_1011=1;
%Let  Roff_1100_2_1100=1;
%Let  Roff_1100_2_1101=1;
%Let  Roff_1100_2_1110=1;
%Let  Roff_1100_2_1111=1;
%Let  Roff_1100_3_0001=1;
%Let  Roff_1100_3_0010=1;
%Let  Roff_1100_3_0011=1;
%Let  Roff_1100_3_0100=0;
%Let  Roff_1100_3_0101=1;
%Let  Roff_1100_3_0110=1;
%Let  Roff_1100_3_0111=1;
%Let  Roff_1100_3_1000=0;
%Let  Roff_1100_3_1001=1;
%Let  Roff_1100_3_1010=1;
%Let  Roff_1100_3_1011=1;
%Let  Roff_1100_3_1100=0;
%Let  Roff_1100_3_1101=1;
%Let  Roff_1100_3_1110=1;
%Let  Roff_1100_3_1111=1;
%Let  Roff_1100_4_0001=1;
%Let  Roff_1100_4_0010=0;
%Let  Roff_1100_4_0011=1;
%Let  Roff_1100_4_0100=0;
%Let  Roff_1100_4_0101=1;
%Let  Roff_1100_4_0110=0;
%Let  Roff_1100_4_0111=1;
%Let  Roff_1100_4_1000=0;
%Let  Roff_1100_4_1001=1;
%Let  Roff_1100_4_1010=0;
%Let  Roff_1100_4_1011=1;
%Let  Roff_1100_4_1100=0;
%Let  Roff_1100_4_1101=1;
%Let  Roff_1100_4_1110=0;
%Let  Roff_1100_4_1111=1;
%Let  Roff_1110_1_0001=0;
%Let  Roff_1110_1_0010=0;
%Let  Roff_1110_1_0011=0;
%Let  Roff_1110_1_0100=0;
%Let  Roff_1110_1_0101=0;
%Let  Roff_1110_1_0110=0;
%Let  Roff_1110_1_0111=0;
%Let  Roff_1110_1_1000=1;
%Let  Roff_1110_1_1001=1;
%Let  Roff_1110_1_1010=1;
%Let  Roff_1110_1_1011=1;
%Let  Roff_1110_1_1100=1;
%Let  Roff_1110_1_1101=1;
%Let  Roff_1110_1_1110=1;
%Let  Roff_1110_1_1111=1;
%Let  Roff_1110_2_0001=0;
%Let  Roff_1110_2_0010=0;
%Let  Roff_1110_2_0011=0;
%Let  Roff_1110_2_0100=1;
%Let  Roff_1110_2_0101=1;
%Let  Roff_1110_2_0110=1;
%Let  Roff_1110_2_0111=1;
%Let  Roff_1110_2_1000=0;
%Let  Roff_1110_2_1001=1;
%Let  Roff_1110_2_1010=1;
%Let  Roff_1110_2_1011=1;
%Let  Roff_1110_2_1100=1;
%Let  Roff_1110_2_1101=1;
%Let  Roff_1110_2_1110=1;
%Let  Roff_1110_2_1111=1;
%Let  Roff_1110_3_0001=0;
%Let  Roff_1110_3_0010=1;
%Let  Roff_1110_3_0011=1;
%Let  Roff_1110_3_0100=0;
%Let  Roff_1110_3_0101=1;
%Let  Roff_1110_3_0110=1;
%Let  Roff_1110_3_0111=1;
%Let  Roff_1110_3_1000=0;
%Let  Roff_1110_3_1001=1;
%Let  Roff_1110_3_1010=1;
%Let  Roff_1110_3_1011=1;
%Let  Roff_1110_3_1100=0;
%Let  Roff_1110_3_1101=1;
%Let  Roff_1110_3_1110=1;
%Let  Roff_1110_3_1111=1;
%Let  Roff_1110_4_0001=1;
%Let  Roff_1110_4_0010=0;
%Let  Roff_1110_4_0011=1;
%Let  Roff_1110_4_0100=0;
%Let  Roff_1110_4_0101=1;
%Let  Roff_1110_4_0110=0;
%Let  Roff_1110_4_0111=1;
%Let  Roff_1110_4_1000=0;
%Let  Roff_1110_4_1001=1;
%Let  Roff_1110_4_1010=0;
%Let  Roff_1110_4_1011=1;
%Let  Roff_1110_4_1100=0;
%Let  Roff_1110_4_1101=1;
%Let  Roff_1110_4_1110=0;
%Let  Roff_1110_4_1111=1;
%Let  Roff_1111_1_0001=0;
%Let  Roff_1111_1_0010=0;
%Let  Roff_1111_1_0011=0;
%Let  Roff_1111_1_0100=0;
%Let  Roff_1111_1_0101=0;
%Let  Roff_1111_1_0110=0;
%Let  Roff_1111_1_0111=0;
%Let  Roff_1111_1_1000=1;
%Let  Roff_1111_1_1001=1;
%Let  Roff_1111_1_1010=1;
%Let  Roff_1111_1_1011=1;
%Let  Roff_1111_1_1100=1;
%Let  Roff_1111_1_1101=1;
%Let  Roff_1111_1_1110=1;
%Let  Roff_1111_1_1111=1;
%Let  Roff_1111_2_0001=0;
%Let  Roff_1111_2_0010=0;
%Let  Roff_1111_2_0011=0;
%Let  Roff_1111_2_0100=1;
%Let  Roff_1111_2_0101=1;
%Let  Roff_1111_2_0110=1;
%Let  Roff_1111_2_0111=1;
%Let  Roff_1111_2_1000=0;
%Let  Roff_1111_2_1001=1;
%Let  Roff_1111_2_1010=1;
%Let  Roff_1111_2_1011=1;
%Let  Roff_1111_2_1100=1;
%Let  Roff_1111_2_1101=1;
%Let  Roff_1111_2_1110=1;
%Let  Roff_1111_2_1111=1;
%Let  Roff_1111_3_0001=0;
%Let  Roff_1111_3_0010=1;
%Let  Roff_1111_3_0011=1;
%Let  Roff_1111_3_0100=0;
%Let  Roff_1111_3_0101=1;
%Let  Roff_1111_3_0110=1;
%Let  Roff_1111_3_0111=1;
%Let  Roff_1111_3_1000=0;
%Let  Roff_1111_3_1001=1;
%Let  Roff_1111_3_1010=1;
%Let  Roff_1111_3_1011=1;
%Let  Roff_1111_3_1100=0;
%Let  Roff_1111_3_1101=1;
%Let  Roff_1111_3_1110=1;
%Let  Roff_1111_3_1111=1;
%Let  Roff_1111_4_0001=1;
%Let  Roff_1111_4_0010=0;
%Let  Roff_1111_4_0011=1;
%Let  Roff_1111_4_0100=0;
%Let  Roff_1111_4_0101=1;
%Let  Roff_1111_4_0110=0;
%Let  Roff_1111_4_0111=1;
%Let  Roff_1111_4_1000=0;
%Let  Roff_1111_4_1001=1;
%Let  Roff_1111_4_1010=0;
%Let  Roff_1111_4_1011=1;
%Let  Roff_1111_4_1100=0;
%Let  Roff_1111_4_1101=1;
%Let  Roff_1111_4_1110=0;
%Let  Roff_1111_4_1111=1;
%Let  Ron_1111_1_0001=1;
%Let  Ron_1111_1_0010=1;
%Let  Ron_1111_1_0011=1;
%Let  Ron_1111_1_0100=1;
%Let  Ron_1111_1_0101=1;
%Let  Ron_1111_1_0110=1;
%Let  Ron_1111_1_0111=1;
%Let  Ron_1111_1_1000=1;
%Let  Ron_1111_1_1001=1;
%Let  Ron_1111_1_1010=1;
%Let  Ron_1111_1_1011=1;
%Let  Ron_1111_1_1100=1;
%Let  Ron_1111_1_1101=1;
%Let  Ron_1111_1_1110=1;
%Let  Ron_1111_1_1111=1;
%Let  Ron_1111_2_0001=1;
%Let  Ron_1111_2_0010=1;
%Let  Ron_1111_2_0011=1;
%Let  Ron_1111_2_0100=1;
%Let  Ron_1111_2_0101=1;
%Let  Ron_1111_2_0110=1;
%Let  Ron_1111_2_0111=1;
%Let  Ron_1111_2_1000=0;
%Let  Ron_1111_2_1001=1;
%Let  Ron_1111_2_1010=1;
%Let  Ron_1111_2_1011=1;
%Let  Ron_1111_2_1100=1;
%Let  Ron_1111_2_1101=1;
%Let  Ron_1111_2_1110=1;
%Let  Ron_1111_2_1111=1;
%Let  Ron_1111_3_0001=1;
%Let  Ron_1111_3_0010=1;
%Let  Ron_1111_3_0011=1;
%Let  Ron_1111_3_0100=0;
%Let  Ron_1111_3_0101=1;
%Let  Ron_1111_3_0110=1;
%Let  Ron_1111_3_0111=1;
%Let  Ron_1111_3_1000=0;
%Let  Ron_1111_3_1001=1;
%Let  Ron_1111_3_1010=1;
%Let  Ron_1111_3_1011=1;
%Let  Ron_1111_3_1100=0;
%Let  Ron_1111_3_1101=1;
%Let  Ron_1111_3_1110=1;
%Let  Ron_1111_3_1111=1;
%Let  Ron_1111_4_0001=1;
%Let  Ron_1111_4_0010=0;
%Let  Ron_1111_4_0011=1;
%Let  Ron_1111_4_0100=0;
%Let  Ron_1111_4_0101=1;
%Let  Ron_1111_4_0110=0;
%Let  Ron_1111_4_0111=1;
%Let  Ron_1111_4_1000=0;
%Let  Ron_1111_4_1001=1;
%Let  Ron_1111_4_1010=0;
%Let  Ron_1111_4_1011=1;
%Let  Ron_1111_4_1100=0;
%Let  Ron_1111_4_1101=1;
%Let  Ron_1111_4_1110=0;
%Let  Ron_1111_4_1111=1;
%Let  Ron_0111_1_0001=0;
%Let  Ron_0111_1_0010=0;
%Let  Ron_0111_1_0011=0;
%Let  Ron_0111_1_0100=0;
%Let  Ron_0111_1_0101=0;
%Let  Ron_0111_1_0110=0;
%Let  Ron_0111_1_0111=0;
%Let  Ron_0111_1_1000=1;
%Let  Ron_0111_1_1001=1;
%Let  Ron_0111_1_1010=1;
%Let  Ron_0111_1_1011=1;
%Let  Ron_0111_1_1100=1;
%Let  Ron_0111_1_1101=1;
%Let  Ron_0111_1_1110=1;
%Let  Ron_0111_1_1111=1;
%Let  Ron_0111_2_0001=1;
%Let  Ron_0111_2_0010=1;
%Let  Ron_0111_2_0011=1;
%Let  Ron_0111_2_0100=1;
%Let  Ron_0111_2_0101=1;
%Let  Ron_0111_2_0110=1;
%Let  Ron_0111_2_0111=1;
%Let  Ron_0111_2_1000=0;
%Let  Ron_0111_2_1001=1;
%Let  Ron_0111_2_1010=1;
%Let  Ron_0111_2_1011=1;
%Let  Ron_0111_2_1100=1;
%Let  Ron_0111_2_1101=1;
%Let  Ron_0111_2_1110=1;
%Let  Ron_0111_2_1111=1;
%Let  Ron_0111_3_0001=1;
%Let  Ron_0111_3_0010=1;
%Let  Ron_0111_3_0011=1;
%Let  Ron_0111_3_0100=0;
%Let  Ron_0111_3_0101=1;
%Let  Ron_0111_3_0110=1;
%Let  Ron_0111_3_0111=1;
%Let  Ron_0111_3_1000=0;
%Let  Ron_0111_3_1001=1;
%Let  Ron_0111_3_1010=1;
%Let  Ron_0111_3_1011=1;
%Let  Ron_0111_3_1100=0;
%Let  Ron_0111_3_1101=1;
%Let  Ron_0111_3_1110=1;
%Let  Ron_0111_3_1111=1;
%Let  Ron_0111_4_0001=1;
%Let  Ron_0111_4_0010=0;
%Let  Ron_0111_4_0011=1;
%Let  Ron_0111_4_0100=0;
%Let  Ron_0111_4_0101=1;
%Let  Ron_0111_4_0110=0;
%Let  Ron_0111_4_0111=1;
%Let  Ron_0111_4_1000=0;
%Let  Ron_0111_4_1001=1;
%Let  Ron_0111_4_1010=0;
%Let  Ron_0111_4_1011=1;
%Let  Ron_0111_4_1100=0;
%Let  Ron_0111_4_1101=1;
%Let  Ron_0111_4_1110=0;
%Let  Ron_0111_4_1111=1;
%Let  Ron_0011_1_0001=0;
%Let  Ron_0011_1_0010=0;
%Let  Ron_0011_1_0011=0;
%Let  Ron_0011_1_0100=0;
%Let  Ron_0011_1_0101=0;
%Let  Ron_0011_1_0110=0;
%Let  Ron_0011_1_0111=0;
%Let  Ron_0011_1_1000=1;
%Let  Ron_0011_1_1001=1;
%Let  Ron_0011_1_1010=1;
%Let  Ron_0011_1_1011=1;
%Let  Ron_0011_1_1100=1;
%Let  Ron_0011_1_1101=1;
%Let  Ron_0011_1_1110=1;
%Let  Ron_0011_1_1111=1;
%Let  Ron_0011_2_0001=0;
%Let  Ron_0011_2_0010=0;
%Let  Ron_0011_2_0011=0;
%Let  Ron_0011_2_0100=1;
%Let  Ron_0011_2_0101=1;
%Let  Ron_0011_2_0110=1;
%Let  Ron_0011_2_0111=1;
%Let  Ron_0011_2_1000=0;
%Let  Ron_0011_2_1001=1;
%Let  Ron_0011_2_1010=1;
%Let  Ron_0011_2_1011=1;
%Let  Ron_0011_2_1100=1;
%Let  Ron_0011_2_1101=1;
%Let  Ron_0011_2_1110=1;
%Let  Ron_0011_2_1111=1;
%Let  Ron_0011_3_0001=1;
%Let  Ron_0011_3_0010=1;
%Let  Ron_0011_3_0011=1;
%Let  Ron_0011_3_0100=0;
%Let  Ron_0011_3_0101=1;
%Let  Ron_0011_3_0110=1;
%Let  Ron_0011_3_0111=1;
%Let  Ron_0011_3_1000=0;
%Let  Ron_0011_3_1001=1;
%Let  Ron_0011_3_1010=1;
%Let  Ron_0011_3_1011=1;
%Let  Ron_0011_3_1100=0;
%Let  Ron_0011_3_1101=1;
%Let  Ron_0011_3_1110=1;
%Let  Ron_0011_3_1111=1;
%Let  Ron_0011_4_0001=1;
%Let  Ron_0011_4_0010=0;
%Let  Ron_0011_4_0011=1;
%Let  Ron_0011_4_0100=0;
%Let  Ron_0011_4_0101=1;
%Let  Ron_0011_4_0110=0;
%Let  Ron_0011_4_0111=1;
%Let  Ron_0011_4_1000=0;
%Let  Ron_0011_4_1001=1;
%Let  Ron_0011_4_1010=0;
%Let  Ron_0011_4_1011=1;
%Let  Ron_0011_4_1100=0;
%Let  Ron_0011_4_1101=1;
%Let  Ron_0011_4_1110=0;
%Let  Ron_0011_4_1111=1;
%Let  Ron_0001_1_0001=0;
%Let  Ron_0001_1_0010=0;
%Let  Ron_0001_1_0011=0;
%Let  Ron_0001_1_0100=0;
%Let  Ron_0001_1_0101=0;
%Let  Ron_0001_1_0110=0;
%Let  Ron_0001_1_0111=0;
%Let  Ron_0001_1_1000=1;
%Let  Ron_0001_1_1001=1;
%Let  Ron_0001_1_1010=1;
%Let  Ron_0001_1_1011=1;
%Let  Ron_0001_1_1100=1;
%Let  Ron_0001_1_1101=1;
%Let  Ron_0001_1_1110=1;
%Let  Ron_0001_1_1111=1;
%Let  Ron_0001_2_0001=0;
%Let  Ron_0001_2_0010=0;
%Let  Ron_0001_2_0011=0;
%Let  Ron_0001_2_0100=1;
%Let  Ron_0001_2_0101=1;
%Let  Ron_0001_2_0110=1;
%Let  Ron_0001_2_0111=1;
%Let  Ron_0001_2_1000=0;
%Let  Ron_0001_2_1001=1;
%Let  Ron_0001_2_1010=1;
%Let  Ron_0001_2_1011=1;
%Let  Ron_0001_2_1100=1;
%Let  Ron_0001_2_1101=1;
%Let  Ron_0001_2_1110=1;
%Let  Ron_0001_2_1111=1;
%Let  Ron_0001_3_0001=0;
%Let  Ron_0001_3_0010=1;
%Let  Ron_0001_3_0011=1;
%Let  Ron_0001_3_0100=0;
%Let  Ron_0001_3_0101=1;
%Let  Ron_0001_3_0110=1;
%Let  Ron_0001_3_0111=1;
%Let  Ron_0001_3_1000=0;
%Let  Ron_0001_3_1001=1;
%Let  Ron_0001_3_1010=1;
%Let  Ron_0001_3_1011=1;
%Let  Ron_0001_3_1100=0;
%Let  Ron_0001_3_1101=1;
%Let  Ron_0001_3_1110=1;
%Let  Ron_0001_3_1111=1;
%Let  Ron_0001_4_0001=1;
%Let  Ron_0001_4_0010=0;
%Let  Ron_0001_4_0011=1;
%Let  Ron_0001_4_0100=0;
%Let  Ron_0001_4_0101=1;
%Let  Ron_0001_4_0110=0;
%Let  Ron_0001_4_0111=1;
%Let  Ron_0001_4_1000=0;
%Let  Ron_0001_4_1001=1;
%Let  Ron_0001_4_1010=0;
%Let  Ron_0001_4_1011=1;
%Let  Ron_0001_4_1100=0;
%Let  Ron_0001_4_1101=1;
%Let  Ron_0001_4_1110=0;
%Let  Ron_0001_4_1111=1;
