/**************************************************************/
/*                                                            */
/*       SAS RETAIL INTELLIGENCE SOLUTIONS                    */
/*       Demand Forecast For Retail Planning 4.2              */
/*                                                            */
/**************************************************************/
/*                                                            */
/* SAS Retail Group, All rights reserved (c) 2006,2007        */
/*                                                            */
/**************************************************************/

/*****************************************************************************************************/
/*Functionality:Call to program splitting the input history file by distinct KPI and merch/loc level */
/*****************************************************************************************************/
 
%macro splitter1();
 	/*Creating META table for the request*/
	%let input_DS=inp.F_&proj;
	data dp.F_&proj._meta (label='Temp Table to store netadata of project');
		attrib ds_ref_no FORMAT=8. informat=8. label='Dataset Reference Number';
		attrib ds_name FORMAT=$32. informat=$32. label='Dataset Name';
		attrib ds_desc FORMAT=$255. informat=$255. label='Dataset Description';
		attrib M_LVL FORMAT=12. informat=12. label='Merchandise Level';
		attrib L_lvl FORMAT=12. informat=12. label='Location Level';
		attrib M_id FORMAT=12. informat=12. label='Merchandise ID';
		attrib L_id FORMAT=12. informat=12. label='Location ID';
		attrib Ref_Merch_ID FORMAT=12. informat=12. label='Ref_Merch_ID';
		attrib Ref_location_ID FORMAT=12. informat=12. label='Ref_location_ID';
		attrib group_key FORMAT=12. informat=12. label='Group Key';
		attrib p_split FORMAT=12. informat=12. label='Performance Split';
		stop;
	run;
	%RCSet (&syserr);
	 
	/* Splitting logic when Reconciliation is not required */
	%if &recon_single=1 %then %do;
		%if &recon_hier=1 %then %do;
			%let temp=&merch_from_level;
			%let merch_from_level=&merch_to_level;
		%end;
		%else %if &recon_hier=2 %then %do;
			%let temp=&loc_from_level;
			%let loc_from_level=&loc_to_level;
		%end;
	%end;

	/* Initialize dataset reference number (mds_ref_no)(Added on 27/11/2008). */
	%let mds_ref_no=1;
	%let mds_ref_no_1=0;
	
	/* Splitting logic when Reconciliation is required*/
	%do cnt=&&&RECON_DIM._to_level %to &&&RECON_DIM._from_level %by -1;/*START of Recon Dim DO LOOP*/
		%let &&RECON_DIM._level=&cnt;
		%let &&UNRECON_DIM._level=&&&UNRECON_DIM._to_level;
		%Let cnt_plus_1=%eval(&cnt+1);
		%if &cnt=&&&RECON_DIM._to_level %then
			%let drop_var= ;
		%else
			%let drop_var=drop &&RD.ID_&cnt_plus_1;
			 
		data W_&wktid..F_&proj._&mds_ref_no.(label=F_&proj._M&merch_level._L&loc_level._0_0_0_0);
			set &input_DS;
			&drop_var;
		    if &&&RECON_DIM._level=&&&RECON_DIM._to_level and &&&UNRECON_DIM._level=&&&UNRECON_DIM._to_level then ;else group_key=0;
		run;
		%RCSet (&syserr);
	 
		proc sort data=W_&wktid..F_&proj._&mds_ref_no.;
			by  &&UNRD.ID: &&RD.ID: &dfr_time_field.;
		run;
		%RCSet (&syserr);
		%let keep1=;
	 	%if &cnt=&&&RECON_DIM._to_level and (%quote(&dfr_ind_var) ne) %then %do;
			%ind_var_name;
			%let keep1= &ind_var_name_mean;
		%end;
		%if &cnt ne &&&RECON_DIM._to_level %then %do;
		proc summary data=W_&wktid..F_&proj._&mds_ref_no.;
			var &dfr_kpinameset ;
			&var_price;
			&var_Promo;
			&var_Pantry;
			&var_DELAY;
			&var_eve;
			&var_Halo;
			&var_Weather;
			id group_key;
			by  &&UNRD.ID: &&RD.ID: &dfr_time_field. ;
			output out=W_&wktid..F_&proj._&mds_ref_no. (drop= _freq_ _type_ )
			sum(&dfr_kpinameset)=
			%if &price_effect gt 0 %then %do; mean(price_ratio)= %end;
			%if &promo_effect gt 0 %then %do; max(promo:)= %end;
			%if &pantry_effect gt 0 %then %do; max(Pantry:)= %end;
			%if &delay_effect gt 0 %then %do; max(DELAY:)= %end;
			%if &halo_cannib_effect gt 0  and &merch_to_level.-&merch_from_level. >=3  %then %do; mean(TOTAL_:)= %end;
			%if &weather_effect gt 0 %then %do; mean(TEMPERATURE:)= %end;
/*			%if &cnt=&&&RECON_DIM._to_level and (%quote(&dfr_ind_var) ne) %then %do;*/
/*				&ind_var_name_mean*/
/*			%end;*/
			
			;
			
		run;
		%end;
		%RCSet (&syserr);
	 
		%Apply_format(
				proj=&proj.,
				data=W_&wktid..F_&proj._&mds_ref_no.,
				M_to_lev=&merch_level.,
				L_to_lev=&loc_level.,
				M_from_lev=&merch_from_level.,
				L_from_lev=&loc_from_level.
				);
		%RCSet (&syserr);
	 
		%do kpi_cnt=1 %to &dfr_numkpis;/*START of KPIwise splitting loop*/
				%let dummy_ref_no=%eval(&mds_ref_no.-&kpi_cnt.+1);

				%if &dfr_numkpis = 1 %then %do;
					%let drop_kpi_var=%sysfunc(ktrim(&dfr_kpinameset));
					data stage.F_&proj._&mds_ref_no.;
						set W_&wktid..F_&proj._&dummy_ref_no.;
					run;
					%RCSet (&syserr);
				%end;
				%else %do;
					%drop_var(drop1=&kpi_cnt.);
					data stage.F_&proj._&mds_ref_no.;
						set W_&wktid..F_&proj._&dummy_ref_no.;
						drop &withoutdrop;
					run;
					%RCSet (&syserr);
				%end;
				%let datadesc=;
				/*Creating a temporary META table in work for the request*/
				data W_&wktid..F_&proj._meta
					(label='Temp Table to store netadata of project');
					attrib ds_ref_no FORMAT=8. informat=8. label='Dataset Reference Number'; /*Added on 27/11/08*/
					attrib ds_name FORMAT=$32. informat=$32. label='Dataset Name'; /*Modified on 27/11/08*/
					attrib ds_desc FORMAT=$255. informat=$255. label='Dataset Description'; /*Added on 27/11/08*/
					attrib M_LVL  FORMAT=12. informat=12. label='Merchandise Level';
					attrib L_lvl  FORMAT=12. informat=12. label='Location Level';
					attrib M_id  FORMAT=12. informat=12. label='Merchandise ID';
					attrib L_id  FORMAT=12. informat=12. label='Location ID';
					attrib group_key FORMAT=12. informat=12. label='Group Key';
					attrib p_split FORMAT=12. informat=12. label='Performance Split';
					ds_ref_no=&mds_ref_no.;
					ds_name="F_&proj._&mds_ref_no.";
					ds_desc="F_&proj._M&merch_level._L&loc_level._0_0_0_0_&drop_kpi_var"; /*Added on 27/11/2008*/
					ds_lib='stage';
					M_LVL="&merch_level.";
					L_lvl="&loc_level.";
					M_id=0;
					L_id=0;
					group_key=0;
					p_split=0;
					kpi=KUPCASE("&drop_kpi_var");
					call symput('datadesc',ds_desc);
				run;
				
				%RCSet (&syserr);
				/*Updating the META table by the temporary meta table*/
				data dp.F_&proj._meta;
					set dp.F_&proj._meta W_&wktid..F_&proj._meta;
				run;
				%RCSet (&syserr);
			 
				/*Create EST table if not available in dia library*/;
				%if %sysfunc(exist(dia.d_est_F_&proj._&mds_ref_no.))=0 %then %do;
					proc sql;
						create table W_&wktid..a like stage.F_&proj._&mds_ref_no.(keep=mid: lid:);

						create table W_&wktid..b
							(
							_NAME_ char(32) label='Variable Name',
							_SELECT_ char(32) label='Selection List',
							_MODEL_ char(32) label='Model',
							_SCORE_ char(32) label='Score',
							_MODELVAR_ char(32) label='Model Variable',
							_DSVAR_ char(32) label='Data Set Variable',
							_VARTYPE_ char(32) label='Variable Type'
							);

						create table dia.d_est_F_&proj._&mds_ref_no. (label ="&datadesc") like W_&wktid..a,W_&wktid..b;
					quit;
					%rcset(&sqlrc);
				%end;
			    /*Create EST table if not available in for library*/;
				%if %sysfunc(exist(for.f_est_F_&proj._&mds_ref_no.))=0 %then %do;
					proc sql;
						create table W_&wktid..a like stage.F_&proj._&mds_ref_no.(keep=mid: lid:);

						create table W_&wktid..b
						(
						_NAME_ char(32) label='Variable Name',
						_SELECT_ char(32) label='Selection List',
						_MODEL_ char(32) label='Model',
						_SCORE_ char(32) label='Score',
						_MODELVAR_ char(32) label='Model Variable',
						_DSVAR_ char(32) label='Data Set Variable',
						_VARTYPE_ char(32) label='Variable Type'
						);

						create table for.f_est_F_&proj._&mds_ref_no. (label ="&datadesc") like W_&wktid..a,W_&wktid..b;
					quit;
					%rcset(&sqlrc);
				%end;

				/* Increment ds_ref_no value (Added on 27/11/08). */
				%let mds_ref_no=%eval(&mds_ref_no.+1);
				

		%end;/*END of KPIwise splitting loop*/
		%let mds_ref_no_1=%eval(&mds_ref_no.-&dfr_numkpis);
	    %let input_DS=W_&wktid..F_&proj._&mds_ref_no_1.;
		
	%end;/*END of Recon Dim DO LOOP*/
	 
	%if &recon_hier=0 %then %do;
		%let merch_from_level=&temp;
	%end;
	 
	data dp.F_&proj._meta;
		set dp.F_&proj._meta ;
		if Ref_Merch_ID =. then Ref_Merch_ID=0;
		if Ref_location_ID =. then Ref_location_ID=0;
	run;

	/*Code to delete intermediate datasets(WORK)*/
	proc datasets library=W_&wktid. kill nodetails nolist;
	quit;
	%RCSet (&syserr);
 
%mend;
 
 
