/**************************************************************/
/*                                                            */
/*       SAS RETAIL INTELLIGENCE SOLUTIONS                    */
/*       Demand Forecast For Retail Planning 4.2              */
/*                                                            */
/**************************************************************/
/*                                                            */
/* SAS Retail Group, All rights reserved (c) 2006,2007        */
/*                                                            */
/**************************************************************/
 
/*************************************************************************************************/
/*Functionality: This program splits the output of datasets generated by splitter1               */
/*               Splits into unique Merchandise or Location IDs if overrides exist for those ids */
/*************************************************************************************************/
 %macro splitter2;
	 %if &dfr_user_define_gk=1 %then %do;
	 	%splitter2_GK;
	  %end;
	 %else %do;
	 	%splitter2_ML_id;
	 %end;
 %mend;
 %macro splitter2_GK;
	/*Keep only those records where Group key have been overridden*/
	data W_&wktid..jobparam_&proj.;
		set mmdata.jobparam_&proj.;
		if Group_key=0 then delete;
	run;
	%RCSet (&syserr);

	proc sql noprint;
		create table W_&wktid..jobparam_&proj._ as
		select distinct merch_level,merch_id,location_level,location_id,Group_key,upper(fieldkey)as fieldkey
		from W_&wktid..jobparam_&proj.;
	quit;
	%rcset(&sqlrc);

	data W_&wktid..jobparam_&proj. ;
		set W_&wktid..jobparam_&proj._;
	run;
	%RCSet (&syserr);

	%let nobs=0;
	%noobs(ds=W_&wktid..jobparam_&proj.);
	/*If at least one ID has been overwritten then do following,else quit the splitter2*/
	%if &nobs ne 0 %then %do; /*Start of ID overridden*/
		PROC SORT data=W_&wktid..jobparam_&proj. out=W_&wktid..jobparam_&proj. ;
			by Group_key;
		run;
		%RCSet (&syserr);

		data W_&wktid..jobparam_&proj.(keep=merch_level merch_id location_level location_id Group_key fieldkey);
			set W_&wktid..jobparam_&proj.;
			retain key;
			/*Expanding the jobsparam dataset for KPI*/
			IF FIELDKEY='ALL' THEN DO ;
				CNT_kpi_START=1;
				CNT_kpi_end=&dfr_numkpis;
				END; ELSE DO ;
				CNT_kpi_START=1;
				CNT_kpi_end=1;
			END;
			IF FIELDKEY='ALL' then
				key=1;
			else
				key=0;
			&FULL_RECON_DIM._level=&&&RECON_DIM._to_level;
			&FULL_UNRECON_DIM._level=&&&UNRECON_DIM._to_level;
			do j=CNT_kpi_START to CNT_kpi_end;
				if key=1 then FIELDKEY=kscan("&dfr_kpinameset",j);
				output;
			end;
		run;
		%RCSet (&syserr);

		data W_&wktid..jobparam_&proj.;
			set W_&wktid..jobparam_&proj.;
			fieldkey=KUPCASE(fieldkey);
			n=_n_;
		run;
		%RCSet (&syserr);
		 
		PROC SORT data=W_&wktid..jobparam_&proj. out=W_&wktid..jobparam_&proj. nodupkey;
			by merch_level merch_id location_level location_id Group_key fieldkey;
		run;
		%RCSet (&syserr);
		 
		/*Get the records in jobparam back in correct order*/
		PROC SORT data=W_&wktid..jobparam_&proj. out=W_&wktid..jobparam_&proj.(drop = n) nodupkey;
			by n;
		run;
		%RCSet (&syserr);

		data W_&wktid..jobparam_&proj.;
			set  W_&wktid..jobparam_&proj.;
			retain Ref_Location_ID 0 Ref_Merch_ID 0;
			if MERCH_ID ne 0 then Ref_Merch_ID=Ref_Merch_ID+1; else Ref_Merch_ID=0;;
			if LOCATION_ID ne 0 then Ref_Location_ID=Ref_Location_ID+1; else Ref_Location_ID=0;
		run;
		%RCSet (&syserr);
		proc sql noprint;
			select distinct group_key into: dist_gr_key separated by ' ' from W_&wktid..jobparam_&proj.;
		quit;

		%let row_no=0;
		data _null_;
			set W_&wktid..jobparam_&proj. end=L;
			if L then call symput('row_no',_n_);
		run;
		%RCSet (&syserr);
		/*Following code gets executed for every row in jobparam*/
		%do i=1 %to &row_no; /*Start of each row execution*/

			/*Get the merch level,merch_id,location level and location_id from each row of jobparam*/
			%getmetabyrow_split(proj=&proj.,row=&i);
			/*Remove all other group key*/
			%if &&&FULL_RECON_DIM._level=&&&RECON_DIM._to_level and  &&&FULL_UNRECON_DIM._level=&&&UNRECON_DIM._to_level %then %do;
				%get_ds_ref_no(data=dp.F_&proj._meta,desc=F_&proj._m&MERCH_LEVEL._l&LOCATION_LEVEL._0_0_0_0_&kpi);
				%let nobs=0;
				%noobs(ds=stage.F_&proj._&get_ds_ref_no.);
				%if &nobs ne 0 %then %do;
					DATA stage.F_&proj._&get_ds_ref_no.;
						SET stage.F_&proj._&get_ds_ref_no.;
						if group_key not in (&dist_gr_key) then Group_key=0;
					RUN;
				%end;

			%end;

			%get_ds_ref_no(data=dp.F_&proj._meta,desc=F_&proj._m&MERCH_LEVEL._l&LOCATION_LEVEL._0_0_0_0_&FIELDKEY);
			%let nobs=0;
			%noobs(ds=stage.F_&proj._&get_ds_ref_no.);
			/* Following code splits the parent to childs(having IDs). Come out of the loop
				if parent has no records. */
			%if &nobs ne 0 %then %do;/*START of checking number of observations */
				%let last_ds_ref_no=%eval(&last_ds_ref_no+1);
				DATA stage.F_&proj._&last_ds_ref_no. stage.F_&proj._&get_ds_ref_no.;
					SET stage.F_&proj._&get_ds_ref_no.;
					IF Group_key=&Group_key THEN
						OUTPUT stage.F_&proj._&last_ds_ref_no.;
					ELSE
						OUTPUT stage.F_&proj._&get_ds_ref_no.;
				RUN;
				%RCSet (&syserr);

				%let nobs=0;
				%noobs(ds=stage.F_&proj._&last_ds_ref_no.);
				 /*Handles bad input.This will happen if the overridden ID value is not present in the parent*/
				%if &nobs eq 0 %then %do;  /*Start of checking number of observations */
					proc sql;
						drop table stage.F_&proj._&last_ds_ref_no. ;
					quit;
				%end;
				%else %do;
					/* Append observations to Metadata Table*/
					data F_&proj._meta
						(label='Temp Table to store netadata of project');
						attrib ds_ref_no FORMAT=8. informat=8. label='Dataset Reference Number';
						attrib ds_name FORMAT=$32. informat=$32. label='Dataset Name';
						attrib ds_desc FORMAT=$255. informat=$255. label='Dataset Description';
						attrib M_LVL FORMAT=12. informat=12. label='Merchandise Level';
						attrib L_lvl FORMAT=12. informat=12. label='Location Level';
						attrib M_id FORMAT=12. informat=12. label='Merchandise ID';
						attrib L_id FORMAT=12. informat=12. label='Location ID';
						attrib Ref_Merch_ID  FORMAT=12. informat=12. label='Ref_Merch_ID';
						attrib Ref_location_ID FORMAT=12. informat=12. label='Ref_location_ID';
						attrib group_key FORMAT=12. informat=12. label='Group Key';
						attrib p_split FORMAT=12. informat=12. label='Performance Split';
						ds_ref_no=&last_ds_ref_no.;
						ds_name="F_&proj._&last_ds_ref_no.";
						ds_desc="F_&proj._m&MERCH_LEVEL._l&LOCATION_LEVEL._&Ref_Merch_ID._&Ref_location_ID._&Group_key._0_&FIELDKEY";
						ds_lib='stage';
						M_LVL="&merch_level.";
						L_lvl="&LOCATION_LEVEL.";
						M_id="&merch_id";
						L_id="&location_id";
						Ref_Merch_ID="&Ref_Merch_ID";
						Ref_location_ID="&Ref_location_ID";
						group_key="&group_key";
						p_split=0;
						kpi="&FIELDKEY";
					run;
					%RCSet (&syserr);
					data dp.F_&proj._meta;
						set dp.F_&proj._meta F_&proj._meta;
						if Ref_Merch_ID =. then Ref_Merch_ID=0;
						if Ref_location_ID =. then Ref_location_ID=0;
					run;
					%RCSet (&syserr);
					/*Create EST table if not available in dia library*/
					%if %sysfunc(exist(dia.d_est_F_&proj._&last_ds_ref_no.))=0 %then %do;
						proc sql;
							create table dia.d_est_F_&proj._&last_ds_ref_no.  like dia.d_est_F_&proj._&get_ds_ref_no.;
						quit;
						%rcset(&sqlrc);
					%end;
					/*Create EST table if not available in for library*/;
					%if %sysfunc(exist(for.f_est_F_&proj._&last_ds_ref_no.))=0 %then %do;
						proc sql;
							create table for.f_est_F_&proj._&last_ds_ref_no.  like for.f_est_F_&proj._&get_ds_ref_no.;
						quit;
						%rcset(&sqlrc);
					%end;
				%end; /*END of checking number of observations */
				%let nobs=0;
				%noobs(ds=stage.F_&proj._&get_ds_ref_no.);
				/*If the parent gets exhausted( means NO observations remain in it) then delete the parent dataset
				and its corresponding row in metadata table*/
				%if &nobs eq 0 %then %do; /* Start of parent =0 */
					proc sql;
						drop table stage.F_&proj._&get_ds_ref_no., for.f_est_F_&proj._&get_ds_ref_no., dia.d_est_F_&proj._&get_ds_ref_no. ;
					quit;
					 
					data dp.F_&proj._meta;
						set dp.F_&proj._meta ;
						if M_LVL=&merch_level. and L_lvl=&LOCATION_LEVEL. and KUPCASE(kpi)=KUPCASE("&FIELDKEY") and M_id=0 and L_id=0 and group_key=0 then delete;
					run;
					%RCSet (&syserr);
				%end; /* End of parent =0 */

			%end;/*START of checking number of observations */

		%end;/*END of each row execution*/

	%end;/*IF END of ID overridden*/

	/*%else %do;  Else ID Overridden */
		%let row_no=0;
		data _null_;
			set dp.F_&proj._meta end=L;
			if L then call symput('row_no',_n_);
		run;
		%RCSet (&syserr);
		/*Following code gets executed for every row in jobparam*/
		%do i=1 %to &row_no; /*Start of each row execution*/
			/*Get the merch level,merch_id,location level and location_id from each row of jobparam*/
			%getmetabyrow(proj=&proj.,row=&i);
			%if &Group_key=0 and (&&&FULL_RECON_DIM._level= &&&RECON_DIM._to_level) %then %do;
				%get_ds_ref_no(data=dp.F_&proj._meta,desc=F_&proj._m&MERCH_LEVEL._l&LOCATION_LEVEL._0_0_0_0_&kpi);
				%let nobs=0;
				%noobs(ds=stage.F_&proj._&get_ds_ref_no.);
				%if &nobs ne 0 %then %do;
					DATA stage.F_&proj._&get_ds_ref_no.;
						SET stage.F_&proj._&get_ds_ref_no.;
						Group_key=0;
					RUN;
				%end;

			%end;

		%end; /*END of each row execution*/

	/*%end;  ELSE ID overridden */
%mend;
%macro splitter2_ML_id;
	/*Keep only those records where IDs( merch or loc) have been overridden*/
	data W_&wktid..jobparam_&proj.;
		set mmdata.jobparam_&proj.;
		if merch_id=0 and LOCATION_id=0 then delete;
	run;
	%RCSet (&syserr);
	 
	proc sql noprint;
		create table W_&wktid..jobparam_&proj._ as
		select distinct merch_level, merch_id, location_level ,location_id, group_key, upper(fieldkey)as fieldkey
		from W_&wktid..jobparam_&proj.;
	quit;
	%rcset(&sqlrc);
	 
	data W_&wktid..jobparam_&proj. ;
		set W_&wktid..jobparam_&proj._;
	run;
	%RCSet (&syserr);
	 
	%let nobs=0;
	%noobs(ds=W_&wktid..jobparam_&proj.);
	/*If at least one ID has been overwritten then do following,else quit the splitter2*/
	%if &nobs ne 0 %then %do; /*Start of ID overridden*/
		/*Remove invalid combinations*/
		/*Unreconciled dimension will be either equal to 0 or the lowest level */
		/*IDs won't be overwritten if levels are not overridden*/
		/*Reconciled dimension cann't be less than 0 and more than the ***_to_level */
		data W_&wktid..jobparam_&proj.;
			set W_&wktid..jobparam_&proj.;
			if merch_id ne 0 and merch_level=0 then delete;
			if location_id ne 0 and location_level=0 then delete;
			if (
			(&FULL_UNRECON_DIM._level=0 or &FULL_UNRECON_DIM._level=&&&UNRECON_DIM._to_level)
			and (&FULL_RECON_DIM._level >=0 and &FULL_RECON_DIM._level <= &&&RECON_DIM._to_level )
			);
		run;
		%RCSet (&syserr);
		 
		/*Following code performs the precedence action*/
		data W_&wktid..jobparam_&proj.;
			set W_&wktid..jobparam_&proj.;
			if merch_level=0 then
				temp_MERCH_level=0 ;
			else
				temp_MERCH_level=1;
			if merch_id=0 then
				temp_MERCH_id=0 ;
			else
				temp_MERCH_id=1;
			if LOCATION_level=0 then
				temp_LOCATION_level=0 ;
			else
				temp_LOCATION_level=1;
			if LOCATION_id=0 then
				temp_LOCATION_id=0 ;
			else
				temp_LOCATION_id=1;
		run;
		%RCSet (&syserr);
		 
		/* Select distinct combinations */
		PROC SORT data=W_&wktid..jobparam_&proj. out=W_&wktid..jobparam_&proj.(drop =temp_:) ;
			by DESCending temp_&FULL_RECON_DIM._level DESCending temp_&FULL_RECON_DIM._id
			DESCending temp_&FULL_UNRECON_DIM._level DESCending temp_&FULL_UNRECON_DIM._id DESCending FIELDKEY;
		run;
		%RCSet (&syserr);
		 
		/*Expand the jobparam dataset.*/
		data W_&wktid..jobparam_&proj.(keep=merch_level merch_id location_level location_id group_key fieldkey);
			set W_&wktid..jobparam_&proj.;
			retain key;
			/*If unreconciled dimension is 0 then set it to the lowest level*/
			IF &FULL_UNRECON_DIM._level=0 THEN
				&FULL_UNRECON_DIM._level="&&&UNRECON_DIM._to_level";
				/*If reconcile dimension is 0, then it will
				be expanded from  to ****_from_level to ***_to_level
				It will add (***_to_level minus ****_from_level) rows to the jobparam */
			IF &FULL_RECON_DIM._level=0 THEN DO ;
				CNT_START=&&&RECON_DIM._from_level;
				CNT_end=&&&RECON_DIM._to_level;
			END;
			ELSE DO ;
				CNT_START=&FULL_RECON_DIM._level;
				CNT_end=&FULL_RECON_DIM._level;
			END;
			/*Expanding the jobsparam dataset for KPI*/
			IF FIELDKEY='ALL' THEN DO ;
				CNT_kpi_START=1;
				CNT_kpi_end=&dfr_numkpis;
			END;
			ELSE DO ;
				CNT_kpi_START=1;
				CNT_kpi_end=1;
			END;
			IF FIELDKEY='ALL' then
				key=1;
			else
				key=0;
			do i=CNT_START to CNT_end;
				do j=CNT_kpi_START to CNT_kpi_end;
					&FULL_RECON_DIM._level=i;
					if key=1 then
						FIELDKEY=kscan("&dfr_kpinameset",j);
					output;
				end;
			end;
		run;
		%RCSet (&syserr);
		/* Removing duplicate observations(because of expanding to KPIs) from jobparam */
		data W_&wktid..jobparam_&proj.;
			set W_&wktid..jobparam_&proj.;
			fieldkey=KUPCASE(fieldkey);
			n=_n_;
		run;
		%RCSet (&syserr);
		 
		PROC SORT data=W_&wktid..jobparam_&proj. out=W_&wktid..jobparam_&proj. nodupkey;
			by merch_level merch_id location_level location_id group_key fieldkey;
		run;
		%RCSet (&syserr);
		 
		/*Get the records in jobparam back in correct order*/
		PROC SORT data=W_&wktid..jobparam_&proj. out=W_&wktid..jobparam_&proj.(drop = n) nodupkey;
			by n;
		run;
		%RCSet (&syserr);

		data W_&wktid..jobparam_&proj.;
			set  W_&wktid..jobparam_&proj.;
			retain Ref_Location_ID 0 Ref_Merch_ID 0;
			if MERCH_ID ne 0 then Ref_Merch_ID=Ref_Merch_ID+1; else Ref_Merch_ID=0;;
			if LOCATION_ID ne 0 then Ref_Location_ID=Ref_Location_ID+1; else Ref_Location_ID=0;
		run;
		%RCSet (&syserr);

		%let row_no=0;
		data _null_;
			set W_&wktid..jobparam_&proj. end=L;
			if L then call symput('row_no',_n_);
		run;
		%RCSet (&syserr);
		 
		/*Following code gets executed for every row in jobparam*/
		%do i=1 %to &row_no; /*Start of each row execution*/

			/*Get the merch level,merch_id,location level and location_id from each row of jobparam*/
			%getmetabyrow_split(proj=&proj.,row=&i);
			%get_ds_ref_no(data=dp.F_&proj._meta,desc=F_&proj._m&MERCH_LEVEL._l&LOCATION_LEVEL._0_0_0_0_&FIELDKEY);
			%let nobs=0;
			%noobs(ds=stage.F_&proj._&get_ds_ref_no.);
			/* Following code splits the parent to childs(having IDs). Come out of the loop
			if parent has no records. */

			%if &nobs ne 0 %then %do;/*START of checking number of observations */
				 
				/*Condition1: Both Ids have been overridden*/
				%IF &merch_id NE 0 AND &location_id NE 0 %THEN
					%LET IFVAR=MID_&merch_level.=&merch_id. AND LID_&Location_level.=&location_id.;
				 
				/*Condition2: Only merch ID not overridden, location ID  overridden*/
				%IF &merch_id = 0 AND &location_id NE 0 %THEN
					%LET IFVAR= LID_&Location_level.=&location_id.;
				 
				/*Condition3: Only merch ID  overridden ,location ID not overridden,*/
				%IF &merch_id NE 0 AND &location_id =0 %THEN
					%LET IFVAR=MID_&merch_level.=&merch_id.;
				 
				/* Splitting the parent dataset into child using above if conditions*/
				%let last_ds_ref_no=%eval(&last_ds_ref_no+1); 
				DATA stage.F_&proj._&last_ds_ref_no. stage.F_&proj._&get_ds_ref_no.;
					SET stage.F_&proj._&get_ds_ref_no.;
					IF &IFVAR THEN
						OUTPUT stage.F_&proj._&last_ds_ref_no.;
					ELSE
						OUTPUT stage.F_&proj._&get_ds_ref_no.;
				RUN;
				%RCSet (&syserr);
				 
				%let nobs=0;
				%noobs(ds=stage.F_&proj._&last_ds_ref_no.);
				 
				/*Handles bad input.This will happen if the
				overridden ID value is not present in the parent*/
				 
				%if &nobs eq 0 %then %do;  /*Start of checking number of observations */
							proc sql;
								drop table stage.F_&proj._&last_ds_ref_no.;
							quit;
				%end;
				%else %do;
						/* Append observations to Metadata Table*/
							data F_&proj._meta
								(label='Temp Table to store netadata of project');
								attrib ds_ref_no FORMAT=8. informat=8. label='Dataset Reference Number';
						        attrib ds_name FORMAT=$32. informat=$32. label='Dataset Name';
						        attrib ds_desc FORMAT=$255. informat=$255. label='Dataset Description';
								attrib M_LVL  FORMAT=12. informat=12. label='Merchandise Level';
								attrib L_lvl  FORMAT=12. informat=12. label='Location Level';
								attrib M_id  FORMAT=12. informat=12. label='Merchandise ID';
								attrib L_id  FORMAT=12. informat=12. label='Location ID';
								attrib Ref_Merch_ID  FORMAT=12. informat=12. label='Ref_Merch_ID';
								attrib Ref_location_ID FORMAT=12. informat=12. label='Ref_location_ID';
								attrib group_key FORMAT=12. informat=12. label='Group Key';
								attrib p_split FORMAT=12. informat=12. label='Performance Split';
								ds_ref_no=&last_ds_ref_no.;
						        ds_name="F_&proj._&last_ds_ref_no.";
						        ds_desc="F_&proj._m&MERCH_LEVEL._l&LOCATION_LEVEL._&merch_id._&location_id._&Group_key._0_&FIELDKEY";
								ds_lib='stage';
								M_LVL="&merch_level.";
								L_lvl="&LOCATION_LEVEL.";
								M_id="&merch_id";
								L_id="&location_id";
								group_key=0;
								p_split=0;
								Ref_Merch_ID="&Ref_Merch_ID";
								Ref_location_ID="&Ref_location_ID";
								kpi="&FIELDKEY";
							run;
							%RCSet (&syserr);
							 
							data dp.F_&proj._meta;
								set dp.F_&proj._meta F_&proj._meta;
								if Ref_Merch_ID =. then Ref_Merch_ID=0;
								if Ref_location_ID =. then Ref_location_ID=0;
							run;
							%RCSet (&syserr);
					/*Create EST table if not available in dia library*/
					%if %sysfunc(exist(dia.d_est_F_&proj._&last_ds_ref_no.))=0 %then %do;
						proc sql;
							create table dia.d_est_F_&proj._&last_ds_ref_no.  like dia.d_est_F_&proj._&get_ds_ref_no.;
						quit;
						%rcset(&sqlrc);
					%end;
					/*Create EST table if not available in for library*/;
					%if %sysfunc(exist(for.f_est_F_&proj._&last_ds_ref_no.))=0 %then %do;
						proc sql;
							create table for.f_est_F_&proj._&last_ds_ref_no.  like for.f_est_F_&proj._&get_ds_ref_no.;
						quit;
						%rcset(&sqlrc);
					%end;
				%end; /*END of checking number of observations */
						 
					%let nobs=0;
					%noobs(ds=stage.F_&proj._&get_ds_ref_no.);
					/*If the parent gets exhausted( means NO observations remain in it) then delete the parent dataset
					and its corresponding row in metadata table*/
					%if &nobs eq 0 %then %do; /* Start of parent =0 */
						proc sql;
							drop table stage.F_&proj._&get_ds_ref_no., for.f_est_F_&proj._&get_ds_ref_no., dia.d_est_F_&proj._&get_ds_ref_no. ;
						quit;

						data dp.F_&proj._meta;
							set dp.F_&proj._meta ;
							if M_LVL=&merch_level. and L_lvl=&LOCATION_LEVEL. and KUPCASE(kpi)=KUPCASE("&FIELDKEY") and M_id=0 and L_id=0 then delete;
						run;
					%end; /* End of parent =0 */
			%end; /*End of checking number of observations */


		%end; /*End of each row execution*/
	%end; /*End of ID overridden*/
	 
	/*Code to delete intermediate datasets(WORK)*/
	proc datasets library=W_&wktid. kill nodetails nolist;
	quit;
 
%mend;
 
