/**************************************************************/
/*                                                            */
/*       SAS RETAIL INTELLIGENCE SOLUTIONS                    */
/*       Demand Forecast For Retail Planning 4.2              */
/*                                                            */
/**************************************************************/
/*                                                            */
/* SAS Retail Group, All rights reserved (c) 2006,2007        */
/*                                                            */
/**************************************************************/

/********************************************************************************************/
/*	Functionality: Split datasets according to Number of Time Series or Number of Datasets.	*/
/********************************************************************************************/

%macro splitter3();
data dp.F_&proj._meta_copy;
set dp.F_&proj._meta;
run;
%if &dfr_wf_split_type=1 and &dfr_no_ts >0 %then %split_ts;
%if &dfr_wf_split_type=2 %then %split_ds;

/*Clear WORK library*/
proc datasets library=W_&wktid. kill nodetails nolist;
quit;
%mend;

/*	Functionality : split datasets based on number of Time-Series and create new datasets.	*/

%macro split_ts();
data _null_;
	set dp.F_&proj._meta end=L;
	if L then call symputx('row_no',_n_);
run;
%RCSet (&syserr);
%do i=1 %to &row_no;
%let new_p_split=0;
%let end_flag=0;
%getmetabyrow(proj=&proj.,row=&i,In_data=dp.F_&proj._meta_copy);
%do %UNTIL (&end_flag=1);
%get_ds_ref_no(data=dp.F_&proj._meta,desc=F_&proj._m&MERCH_LEVEL._l&LOCATION_LEVEL._&merch_id._&location_id._&Group_Key._&p_split._&kpi.);
%Let new_ref_no=%eval(&last_ds_ref_no. +1);
	data stage.F_&proj._&new_ref_no. stage.F_&proj._&get_ds_ref_no.;
			retain cnt 0;
			set stage.F_&proj._&get_ds_ref_no.;
			by &&UNRD.ID: &&RD.ID: ;
			if first.&&RD.ID_&&&FULL_RECON_DIM._level then cnt=cnt+1;
			if cnt <= &dfr_no_ts then output stage.F_&proj._&new_ref_no.; else output stage.F_&proj._&get_ds_ref_no.;
			drop cnt;
	run;
	%RCSet (&syserr);
	data W_&wktid..F_&proj._meta;
		set dp.F_&proj._meta;
		if &get_ds_ref_no. =ds_ref_no then output;
	run;
	%let new_p_split=%eval(&new_p_split+1);
	data W_&wktid..F_&proj._meta;
		set W_&wktid..F_&proj._meta;
		ds_ref_no=&new_ref_no;
		p_split=&new_p_split;
		ds_name="F_&proj._&new_ref_no."; 
		ds_desc="F_&proj._m&MERCH_LEVEL._l&LOCATION_LEVEL._&merch_id._&location_id._&Group_Key._&new_p_split._&kpi.";
	run;
	%RCSet (&syserr);
	data dp.F_&proj._meta;
		set dp.F_&proj._meta W_&wktid..F_&proj._meta;
	run;
	%RCSet (&syserr);
	/*Create EST table if not available in dia library*/
	%if %sysfunc(exist(dia.d_est_F_&proj._&new_ref_no.))=0 %then %do;
		proc sql;
			create table dia.d_est_F_&proj._&new_ref_no.  like dia.d_est_F_&proj._&get_ds_ref_no.;
		quit;
		%rcset(&sqlrc);
	%end;
	/*Create EST table if not available in for library*/;
	%if %sysfunc(exist(for.f_est_F_&proj._&new_ref_no.))=0 %then %do;
		proc sql;
			create table for.f_est_F_&proj._&new_ref_no.  like for.f_est_F_&proj._&get_ds_ref_no.;
		quit;
		%rcset(&sqlrc);
	%end;

	%let NOBS=0;
	%NOOBS(ds=stage.F_&proj._&get_ds_ref_no.)
	%if &nobs=0 %then %do;
		%let end_flag=1;
		data dp.F_&proj._meta;
			set dp.F_&proj._meta;
			if &get_ds_ref_no. =ds_ref_no then delete;
		run;
		%RCSet (&syserr);
		proc sql noprint;
			drop table stage.F_&proj._&get_ds_ref_no., for.f_est_F_&proj._&get_ds_ref_no., dia.d_est_F_&proj._&get_ds_ref_no. ;
		quit;
	%end;
%end;
	
%end;
%mend;


/*	Functionality : split datasets based on number of datasets and create new datasets.	*/

%macro split_ds();

%let nobs=0;
%noobs(ds=dp.F_&proj._meta_copy);
%if &dfr_no_ds. <= &nobs. %then %do;
	%let job_rc_msg=%sysfunc(sasmsg(catalog.dfrmsg, DFR_DATASET_SPLIT_ERR, noquote)); 
	%put &job_rc_msg.;
	%goto EndMacro;
%end;

%global ts_cnt;
data _null_; 
	set dp.F_&proj._meta end=L;
	if L then call symputx('row_no',_n_);
run;

data W_&wktid..F_&proj._meta_new;
	set dp.F_&proj._meta;
	no_ts=.;
	split_cnt=.;
run;

/* Attach Time Series Count */
%do i=1 %to &row_no;
	%let ts_cnt=0;
	%getmetabyrow(proj=&proj.,row=&i,In_data=dp.F_&proj._meta_copy);
	%get_ds_ref_no(data=dp.F_&proj._meta,desc=F_&proj._m&MERCH_LEVEL._l&LOCATION_LEVEL._&merch_id._&location_id._&Group_Key._&p_split._&kpi.);
	data _null_;
		retain cnt 0;
		set stage.F_&proj._&get_ds_ref_no. end=L;
		by &&UNRD.ID: &&RD.ID: ;
		if first.&&RD.ID_&&&FULL_RECON_DIM._level then cnt=cnt+1;
		if L then call symputx('ts_cnt',cnt);
	run;
	%put &ts_cnt;
	proc sql noprint;
		update W_&wktid..F_&proj._meta_new
		set no_ts=&ts_cnt
		where ds_ref_no=&i.;
	quit;
%end;

/*	calculate split number per dataset for splitting process */
	%calculate_split_number(indata=F_&proj._meta_new,outdata=_b_);

/* Split Datasets based on split number. */
data _null_;
	set W_&wktid.._b_ end=L;
	if L then call symputx('row_no',_n_);
run;
%RCSet (&syserr);

%do row_i=1 %to &row_no;
	%let new_p_split=0;
	%let end_flag=0;
	%getmetabyrow_B(proj=&proj.,row=&row_i,in_data=W_&wktid.._b_);
	%get_ds_ref_no(data=dp.F_&proj._meta,desc=F_&proj._m&MERCH_LEVEL._l&LOCATION_LEVEL._&merch_id._&location_id._&Group_Key._&p_split._&kpi.);
	%Let new_ref_no=%eval(&last_ds_ref_no. +1);
	data 
		%do i=&new_ref_no %to %eval(&number_split+&new_ref_no-1); 
			stage.F_&proj._&i.
		%end;
		;
		retain cnt 0;
		set stage.F_&proj._&get_ds_ref_no.;
		by &&UNRD.ID: &&RD.ID: ;
		if first.&&RD.ID_&&&FULL_RECON_DIM._level then cnt=cnt+1;
		%do i=&new_ref_no %to %eval(&number_split+&new_ref_no-1); 
			if &split_cnt*(&i-1-&last_ds_ref_no.)< cnt and &split_cnt*(&i-&last_ds_ref_no.)>= cnt then output stage.F_&proj._&i.;
		%end;
		drop cnt;
	run;
	%RCSet (&syserr);
	data W_&wktid..F_&proj._meta;
		set dp.F_&proj._meta;
		if &get_ds_ref_no. =ds_ref_no then output;
	run;
	%RCSet (&syserr);
	data W_&wktid..F_&proj._meta;
		set W_&wktid..F_&proj._meta;
		do ds_ref_no=&new_ref_no to &number_split+&new_ref_no-1; 
			p_split=ds_ref_no-&last_ds_ref_no.;
			ds_name="F_&proj._"||strip(ds_ref_no); 
			ds_desc="F_&proj._m&MERCH_LEVEL._l&LOCATION_LEVEL._&merch_id._&location_id._&Group_Key._"||strip(p_split)||"_&kpi.";
			output;
		end;
	run;
	%RCSet (&syserr);
	data dp.F_&proj._meta;
		set dp.F_&proj._meta W_&wktid..F_&proj._meta;
		if &get_ds_ref_no. =ds_ref_no then delete;
	run;
	%RCSet (&syserr);
	%do i=&new_ref_no %to %eval(&number_split+&new_ref_no-1); 
		/*Create EST table if not available in dia library*/
		%if %sysfunc(exist(dia.d_est_F_&proj._&i.))=0 %then %do;
			proc sql;
				create table dia.d_est_F_&proj._&i.  like dia.d_est_F_&proj._&get_ds_ref_no.;
			quit;
			%rcset(&sqlrc);
		%end;
		/*Create EST table if not available in for library*/;
		%if %sysfunc(exist(for.f_est_F_&proj._&i.))=0 %then %do;
			proc sql;
				create table for.f_est_F_&proj._&i.  like for.f_est_F_&proj._&get_ds_ref_no.;
			quit;
			%rcset(&sqlrc);
		%end;
	%end;
		proc sql noprint;
			drop table stage.F_&proj._&get_ds_ref_no., for.f_est_F_&proj._&get_ds_ref_no., dia.d_est_F_&proj._&get_ds_ref_no. ;
		quit;

	%end;
	%EndMacro:
%mend split_ds;



