%macro mm_archive(mm_libs=,mm_files=,mm_archive_backup_root=,mm_archive_ngen=,mm_archive_wsid=);

/*  This macro archives files with name begining with mm_files front strings from the Staging Folder mm_archive_data_dir.
	All files to archive are assumed to be within mm_libs directories.  Files are NOT purged from this direcotry after archive.

	This program also maintains the archive folders. If the number of Folders created exceeds mm_archive_gen 
		then the oldest archive folder would be deleted.
*/

/* Parameters  
		mm_libs = 					Folders from which files needs to be archived
		mm_archive_backup_root = 	Root Folder Name under which the archive folder is created
		mm_archive_ngen = 			Number of Archive Folders to be maintained.
		mm_archive_wsid = 			JobId of the Forecast Run
		mm_files =					Prefix names of files to be deleted.
*/

   /******************************************************************************/
   /* Find files that need to be archived from the mm_archive_data_dir directory */
   /******************************************************************************/

   %put *---* mm_archive: begin data dir=&mm_libs wsid=&mm_archive_wsid;
   /* use proc contents to find all files, then sorts (removes dups) and filters */
   
   /* loop for each library */
   %let nlib=1;
   %let alib=%sysfunc(kscan(&mm_libs.,&nlib.));
   %do %while (%quote(&alib) ne);
   	   /* get file list from library alib */
	   proc contents data=&alib.._all_ noprint out=fileds; run;
	   proc sort data=fileds nodupkey; by memname; run;

	   /* Create file list for the library alib */
	   %let mm_archive_ntab&nlib.=0;
	   data mm_archive_dsnlist&nlib.;
	      set fileds;
		  WHERE 
			%let i=1;
			%let afile = %sysfunc(kscan(&mm_files,&i));
	  		%do %while (%quote(&afile.) ne);
				%if &i>1 %then %do; OR  %end;
				kupcase(MEMNAME) LIKE "&afile.%" 
				%let i = %eval(&i.+1);
				%let afile = %sysfunc(kscan(&mm_files,&i));
			%end;
			;
	      call symput("mm_archive_ntab&nlib.",strip(put(_n_,4.0)));
	   run;
	   %put *---* mm_archive: data dir=&alib wsid=&mm_archive_wsid ntab=&&mm_archive_ntab&nlib.;

       /* IF no files to archive then exit */
	   %if &&mm_archive_ntab&nlib.=0 %then %goto done;

	   %let nlib=%eval(&nlib.+1);
	   %let alib=%sysfunc(kscan(&mm_libs.,&nlib.));
	%end;
	%let nlib=%eval(&nlib.-1);

   /*********************************************************************/
   /*  Find all existing archive directories and put in dirlist dataset */
   /*********************************************************************/
   %macro dirlist(dir=,out=dirlist);
      filename _dirlist "&dir";
      data &out(keep=memname);
        length memname $ 80;
        nmem=0;
        did=dopen('_dirlist');
        put '*---* dirlist(1):' did= " dir=&dir";
        if did=0 then goto done;
        nmem=dnum(did);
        put '*---* dirlist(2):' nmem= " dir=&dir";
        do i=1 to nmem;
          if klength(dread(did,i))>2 then
            do;
              memname=dread(did,i);
              output;
            end;
        end;
        dclose=dclose(did);
        done:
        put '*---* dirlist (3):' nmem= " dir=&dir";
      run;
      proc sort data=&out; by memname; run;
      filename _dirlist clear;
   %mend dirlist;
   %dirlist(dir=&mm_archive_backup_root,out=dirlist);
   
	/********************************************************************************/
	/*  Count how many archive directories currently exist and put them in dirlist2 */
	/********************************************************************************/
	%let mm_archive_ndir=0;
	data dirlist2;
	  set dirlist;
	  where kupcase(memname)=:"ARCHIVE_&mm_archive_wsid.";
	  archive_number=input(kscan(memname,3,'_'), 17.);
	  call symput('mm_archive_ndir',strip(put(_n_,4.0)));
	run;
	%put *---* mm_archive: backup root=&mm_archive_backup_root ndir=&mm_archive_ndir;

	/************************************************/
	/*  Create New Archive Directory and copy files */
	/************************************************/

	/* find existing first and last # archive directories */
	proc sort data=dirlist2; by archive_number; run;
	%let first_archive_number=0;
	%let last_archive_number=9998;
	%let next_archive_number=9999;
	data _null_;
	  set dirlist2 end=lastrec;
	  if _n_=1   then call symput('first_archive_number',strip(put(archive_number,4.0)));
	  if lastrec then 
	     do;
	        call symput('last_archive_number' ,strip(put(archive_number  ,4.0)));
	        call symput('next_archive_number' ,strip(put(archive_number+1,4.0)));
	     end;
	run;
	data _null_;
	  call symput('mm_archive_datestamp',kcompress(put(date(),yymmdd10.),'-'));
	  stop;
	run;
	%if (&next_archive_number.=9999) %then %do; %let next_archive_number=1; %end;
	%put *---* mm_archive: first archive number=&first_archive_number last archive number=&last_archive_number next archive number=&next_archive_number data=&mm_archive_datestamp;

	/* Create next (new) archive */ 
	%let mm_archive_newdir=archive_&mm_archive_wsid._&next_archive_number._&mm_archive_datestamp;
	data _null_;
	  dcreate_rc=dcreate("&mm_archive_newdir","&mm_archive_backup_root");
	  put "*---* mm_archive dir=&mm_archive_newdir:" dcreate_rc=;
	run;

	/* Remove directory if it already exists */
	libname _oa_out "&mm_archive_backup_root./&mm_archive_newdir"; 
	proc datasets library=_oa_out kill; run; quit;

	/* create variables for all filenames to copy for each library */
	%global l;
	%do l=1 %to &nlib.;
		%do i=1 %to &&mm_archive_ntab&l.;
		  %global __t&l._&i.;
		%end;
		data _null_;
		  set mm_archive_dsnlist&l.;
		  call symput('__t'!!symget('l')!!'_'!!strip(put(_n_,4.0)),strip(memname));
		run;
	%end;

	/* copy all files from data directories  to new archive directory */
	%do l=1 %to &nlib.;
		proc copy in=%sysfunc(kscan(&mm_libs,&l.)) out=_oa_out memtype=data;
		  select
		     %do i=1 %to &&mm_archive_ntab&l.;
		        &&__t&l._&i.
		     %end;
		     ;
		run;
	%end;

	/*********************************/
	/* PURGE OLD ARCHIVE DIRECTORIES */
	/*********************************/

	%if &mm_archive_ndir>0 %then %do;

		/* figure out which directories to purge... if number of directory is greater than the number to generate purge it */
		proc sort data=dirlist2; by descending archive_number; run;
		data dirlist_to_delete;
		  set dirlist2;
		  if _n_<&mm_archive_ngen then delete;
		run;

		%if ("&sysscp" eq "WIN") %then %do;
			data _null_;
				set dirlist_to_delete;
				options xsync noxwait;
				call system('rd /s /q "'!!symget('mm_archive_backup_root')!!'/'!!strip(memname)!!'"');
			run;
		%end;
		%else %do; 
	     	data _null_;
		    	set dirlist_to_delete;
				call system('rm -fr '!!symget('mm_archive_backup_root')!!'/'!!strip(memname));
			run;
		%end;

/*		data _null_;*/
/*	      	set dirlist_to_delete;*/
/*			if ("&sysscp" eq "WIN") then do;*/
/*				options xsync noxwait;*/
/*				call system('rmdir /s /q "'!!symget('mm_archive_backup_root')!!'/'!!strip(memname)!!'"');*/
/*				end;*/
/*			else do;*/
/*				call system('rm -r '!!symget('mm_archive_backup_root')!!'/'!!strip(memname));*/
/*				end;*/
/*		run;*/

	%end;

    %done:
    libname _oa_out clear;
%mend mm_archive;
