/**************************************************************/
/*                                                            */
/*       SAS RETAIL INTELLIGENCE SOLUTIONS					  */
/*                                                            */
/**************************************************************/
/*                                                            */
/* SAS Retail Group, All rights reserved (c) 2006,2007        */
/*                                                            */
/**************************************************************/

 /* ************************************************************************* */
 /* MM_EXTRACT Macro                                                          */
 /*                                                                           */
 /* This macro extracts PLAN and HISTORY data for a specific subset of the    */
 /* Retail Merchandise planning or historical fact data.                      */
 /*                                                                           */
 /* DATE      WHO      COMMENT                                                */
 /* 13JUL04   SASGZS   Created macro from extract code from Dale Matheny      */
 /* 15JUL04   SASGZS   Moved LIBNAME statements to Stored Process; added %if  */
 /*                    logic to last SQL statement.			      			  */
 /* 16JUL04   DEM      Added lly and complete conditionals, removed calcs     */
 /* 01Jan05   DEM	   Rearranged params for add-in; fixed bugs               */
 /* Feb0105   DEM      Add-in, formats, to date and OLAP cubes		      	  */
 /* May2505	  DEM	   Added todate and roll logic			      			  */
 /* Sept2005  DEM      Replaced MPLAN/MFINC extract logic with hint-based 4-key */
 /* Dec2005   DEM      Added 61 and plan timeshift logic for v133 release     */
 /* June2006  DEM      Format options, Cluster, new store assignments 	      */
 /* Sept2006  DEM      Got rid of stored procedures and t_cube tables in maxdata */
 /* APR2007   DEM	   Added logic for DFR 4.2 -- split hier code, added loc  */
 /* OCT2007   DEM      Added forecast logic and sort after extract bug fix    */
 /* ***************************************************************************/

%macro hist_async(time=,ds_out=,dsn=,dbtype=,dbpassword=,hfields=,afilter=,facttab=,mfversion=0);
       /* ************************************************** */
       /*         RUN HISTORY EXTRACT IN MULTI_TASK MODE     */
	   /* ************************************************** */
        option sascmd="!sascmd";

        %let Waitlist=;
		%let t=1;
		%let R_rsubmit_cnt=1;
		%let timeid = %sysfunc(kscan(%quote(&timestr.),&t));
		%do %while (%quote(&timeid.) ne);
			%let time_level =%sysfunc(kscan(%quote(&time_levels.),&t));
            %let Waitlist=&WaitList. task&t.;

			signon task&t.;
			%syslput dbtype_remote=&dbtype./REMOTE=task&t.;
            %syslput dsn_remote=&dsn./REMOTE=task&t.;
            %syslput dbpassword_remote=%quote(&dbpassword.)/REMOTE=task&t.;
			%syslput hfields_remote=%quote(&hfields.)/REMOTE=task&t.;
			%syslput t_remote=&t./REMOTE=task&t.;
			%if (%sysfunc(klowcase(&facttab.))=mfact_version) %then %do;
				%syslput facttab_remote=%quote(maxdata.&facttab. subpartition (MFVER_&time_level._&timeid._&mfversion.) A, t_4key_&projid. B )/REMOTE=task&t.;
			%end;
			%else %do;
				%syslput facttab_remote=%quote(maxdata.&facttab. partition (&facttab._&time_level._&timeid.) A, t_4key_&projid. B )/REMOTE=task&t.;
			%end;

			/* Submit remote task for each query...one per time id */

            rsubmit process=task&t. CONNECTWAIT=no inheritlib=(work=r_work);
                proc sql;
                    connect to &dbtype_remote. ( user=maxdata password=&dbpassword_remote. path="&dsn_remote.");
					create table r_work.t_set&t_remote. as select * from connection to &dbtype_remote.(
					 	select merch_id, merch_level, l_realid as location_id, location_level, time_id, time_level, &hfields_remote. 
					 	from &facttab_remote. 
					 	where a.merch_level=b.m_lev and a.merch_id=b.m_id and a.location_level=b.l_lev and a.location_id=b.l_id 
					);
					disconnect from &dbtype_remote.;
                quit;
            endrsubmit;
			/*signoff task&t.;*/

			%let t=%eval(&t.+1);
			%let timeid = %sysfunc(kscan(%quote(&timestr.),&t));
            %let R_rsubmit_cnt=%eval(&R_rsubmit_cnt+1);
            %if &R_rsubmit_cnt >&ma_max_rsubmit. %then %do;
                  waitfor _all_ &Waitlist.;
                  signoff _all_;
                  %let Waitlist=;
                  %Let R_rsubmit_cnt=1;
            %end;
        %end;

      	%if "&Waitlist" ne "" %then %do;
	        waitfor _all_ &Waitlist.;
			signoff _all_;
		%end;

		%do i=1 %to %eval(&t.-1);
			proc append base=&ds_out. data=t_set&i.; run;
			proc sql; drop table t_set&i.; quit;
		%end;
%mend;

%macro mm_extract12(
		out=,
		uid=0,
		trace=0,

		/* Database */
		dsn=,
		dbtype=,
		dbpassword=,

		/* Data Versions */
	    include_hist1=999,		/* =1 = TY */
	    include_hist2=999,		/* =1 = LY */
	    include_hist3=999,		/* =1 = LLY */
		update_time_TY=1,       /* =1 means update LY or LLY to TY time frame for time_id; =0 means don't update */
	    plan_worksheet=2,
		onlysubmitted=0,
		fcast_versions=%quote(),   /* forecast versions */
	    version_groups=%quote(),   /* plan versions */
		version_shifts=%quote(),

		/* KPIS */
		plan_fields=%quote(),
		hist_fields=%quote(),
		fcast_fields=%quote(),

		/* FILTERS */
		filter_hist=%quote(),	/* [mfinc:posit_mvmt>1] */
		filter_plan=%quote(),	/* normal SQL format */
		filter_fcast=%quote(),
		zero_out_nulls=0,

		/* MERCHANDISE PARAMETERS */
		find_merch_ids=1,
		merch_template=,
		merch_from_level=,
		merch_to_level=,
		merch_from_id=,
		merchlevelparent=0,

		/* LOCATION PARAMETERS */
		find_loc_ids=1,
		loc_acts_like=0,
		loc_template=,
		loc_from_level=,
		loc_to_level=,
		loc_from_id=,

		/* TIME PARAMETERS */
		find_time_ids=1,
		time_type=Standard,
		time_template=,
		optimized=0,
		calendar=,
		time_from_level=,
		time_to_level=,
		from_time_id=,		/* use forecast time frame for extract */
		to_time_id=
		);

		%GLOBAL input_cluster_id;
		%let loc_acts_like_field=acts_like;
		%let real_time_to_level=%eval(&time_to_level.+46);

       /* *************************************** */
       /*         GET HIERARCHY MEMBERS           */
	   /* *************************************** */

       /********************************************************************* */
       /* Time parameters
	   /* time_type = 
	   /*       Event	 : use From_time_id date to To_time_id dates
	   /*       Standard : get all from_time_id children at time_to_level; time_template =0
	   /*       ToDateCurrent : get current date and then get all weeks within that year (no input parameters)
	   /*       Roll  	 : to_time_id has # of days to subtract from current for to id, then subtract from_time_id days to get starting date
       /********************************************************************* */
		%if &find_time_ids=1 %then %do;
			/* figure out how many plan time shifts */
			%let numshifts = 0;
			%do %while (%sysfunc(kscan(%quote(&version_shifts),%eval(&numshifts.+1))) ne);
				%let numshifts = %eval(&numshifts.+1);
				%end;

			%if (&optimized.=1) %then %do;
				%get_optimized_time_ids(timetype=&time_type.,fromdate=&from_time_id.,todate=&to_time_id.,time_set=&time_template.,numplanshifts=&numshifts.,timetolevel=&time_to_level.,timefromlevel=&time_from_level.,calendar=&calendar.);
				%end;
			%else %do;
				%get_time_ids(timetype=&time_type.,fromdate=&from_time_id.,todate=&to_time_id.,time_set=&time_template.,numplanshifts=&numshifts.,timetolevel=&time_to_level.,timefromlevel=&time_from_level.,calendar=&calendar.);
				%end;

			/*******************************/
			/* Adjust plan timeshift dates */
			/*******************************/
			%if &numshifts.>0 %then %do;
				proc sql noprint;
					drop table maxdata.time_members_&sysjobid.;
					create table maxdata.time_members_&sysjobid. as select * from time_members;
				quit;

				%let i = 1;
				%let planshift = %sysfunc(kscan(&version_shifts,&i));
				%do %while (%quote(&planshift) ne);
					%if ("&planshift." ne "") %then %do;
						/* find # of days between TY First Period and Plan Time Shift Date */
						proc sql noprint;
						select lv&time_to_level.time_start_date into :start_date from maxapp.lv&time_to_level.time where lv&time_to_level.time_lkup_id = (select min(hist0_id) from time_members);
						%let diffdays = %sysfunc(datdif("&start_date."d, "&planshift."d, 'act/act'));

						/* subtract # of days from each TY Time period to get plan shift time ids */
						connect to &dbtype. ( user=maxdata password=%quote(&dbpassword) path="&dsn.");
						execute(
							update maxdata.time_members_&sysjobid. set plan&i._id = (select lv&time_to_level.time_lkup_id from maxapp.lv&time_to_level.time 
								where (select lv&time_to_level.time_start_date from maxapp.lv&time_to_level.time where lv&time_to_level.time_lkup_id = time_members_&sysjobid..hist0_id)+&diffdays. between lv&time_to_level.time_start_date and lv&time_to_level.time_end_date)
						) by &dbtype.;
						disconnect from &dbtype.;
						quit;
						%end;
					%let i = %eval(&i.+1);
					%let planshift = %sysfunc(kscan(&version_shifts,&i));
					%end;

				proc sql noprint;
					drop table time_members;
					create table time_members as select * from maxdata.time_members_&sysjobid.;
					drop table maxdata.time_members_&sysjobid.;
				quit;
			%end;
		%end;

		/********************************************************************* */
	    /* MERCHANDISE members 													*/
		/********************************************************************* */

		/* IF find_merch_ids=0 then assumption is T_CUBE_MERCH is already provided as working data set */

		%if &find_merch_ids=1 %then %do;
			%get_merch_members(merch_from_level=&merch_from_level.,merch_to_level=&merch_to_level.,merch_template=&merch_template.,merchlevelparent=&merchlevelparent.);
			%end;

		/********************************************************************* */
        /* LOCATION members */
		/********************************************************************* */

		/* IF find_loc_ids=0 then assumption is T_CUBE_LOC is already provided as working data set */
			/* convert from cluster to store levels if worksheet has cluster set */
		%if &loc_from_level.=1001 %then %do;
			%let input_cluster_id = &loc_from_id.;
			%let loc_to_level = 4;
			proc sql noprint;
			select FR_LOC_LVL,FR_LOC_MMBR into :loc_from_level, :loc_from_id from maxdata.clstr_st where CLSTR_ST_ID = &input_cluster_id.;
			quit;
			%let loc_from_level=%sysfunc(ktrim(&loc_from_level.));
			%let loc_from_id=%sysfunc(ktrim(&loc_from_id.));
			%end;
		%else %do;
			%let input_cluster_id = 0;
			%end;

		%if &find_loc_ids=1 %then %do;
			proc sql noprint;
				drop table t_cube_loc;
				CREATE TABLE T_CUBE_LOC (l_lev num, l_id num, l_realid num);

				%let template_count=0;
			 	%if &loc_template. > 0 %then %do;
					select count(*) into :template_count from &template_member. where template_id = &loc_template. and level_number = &loc_to_level.;
					%end;
				insert into T_CUBE_LOC (l_lev, l_id, l_realid) 
			 	%if (&template_count.>0) %then %do;
					select &loc_to_level., member_id, member_id from &template_member. 
						where template_id = &loc_template. and level_number = &loc_to_level.;
					%end;
				%else %do;
					select &loc_to_level.,lv&loc_to_level.loc_id,lv&loc_to_level.loc_id
					from maxdata.lv&loc_to_level.loc where lv&loc_from_level.loc_id = &loc_from_id.;
					%end;

				%if (&loc_acts_like>0) and (&loc_to_level.=4) %then %do;
					create table loc_like as select lv4loc_id as location_id, &loc_acts_like_field. as store_like_id from maxdata.lv&loc_to_level.loc where &loc_acts_like_field. is not null and &loc_acts_like_field.<>0;
					update t_cube_loc set l_id = (select store_like_id from loc_like where l_realid=location_id) where l_realid in (select location_id from loc_like);
					%end;

			quit;
			%end;

		/**********************************************************************/
	    /* BUILD FOUR-KEYS TABLE 											  */
		/**********************************************************************/
		proc sql noprint;
			drop table maxdata.t_4key_&projid.;
			create table maxdata.t_4key_&projid. as 
				select m.m_lev, m.m_id, l.l_lev, l.l_id, l.l_realid from t_cube_merch m, t_cube_loc l;
			connect to &dbtype. ( user=maxdata password=%quote(&dbpassword) path="&dsn.");
/*			execute( drop index ui_4key_&projid. ) by &dbtype.;*/
			execute( create unique index ui_4key_&projid. on maxdata.t_4key_&projid. (m_lev, m_id, l_lev, l_id, l_realid) ) by &dbtype.;
			disconnect from &dbtype.;
		quit;

		/* *************************************** */
		/* *************************************** */

	    %let num_data_versions = 0;

		/* **************************************/
		/* 					  					*/
       	/* GET PLAN DATA         		  		*/
		/* 					  					*/
       	/* **************************************/

		%let i = 1;

		/* Extract from a single plan worksheet */
		%if (&plan_worksheet>2) %then %do;
			proc sql noprint;
	   			%let pfilter = %sysfunc(kscan(%quote(&filter_plan.),&i.,%str(,)));
				%let num_data_versions = %eval(&num_data_versions. + 1);
				create table set&num_data_versions. as 
				%if (&plan_fields. ne) %then %do;
					select merch_id, location_id, time_id,
					%mm_parsefields2(in=%quote(&plan_fields.),frontstr=p&num_data_versions.,tab=&mplan.,dosum=1);
					%end;
				from maxdata.&mplan. 
				where workplan_id = &version_groups. and merch_level = &merch_to_level. and location_level=&loc_to_level. and time_level = &real_time_to_level. 
				%if (%quote(&pfilter.) ne) %then %do;
					and &pfilter.
					%end;
				group by merch_id, location_id, time_id;
				proc sort data=set&num_data_versions.;
					by &sixkeys;
					run;
			quit;
			%end;
		%else %do;

		%let planvers = %sysfunc(kscan(&version_groups,&i));
		%do %while (%quote(&planvers.) ne);

			/* set up worksheets, filter, fields into query tables */
			%let num_data_versions = %eval(&num_data_versions. + 1);
			proc sql noprint;
				%let worksheets=;
				select planworksheet_id into :worksheets separated by ',' from maxdata.planworksheet where whatif=0 
					%if &onlysubmitted=1 %then %do; and submitted=1 %end;
					and planversion_id in (select planversion_id from maxdata.planversion where version_group_id = &planvers.);
			quit;

			%if ("&worksheets." ne "") %then %do;
				/* get fields and filter */
				%let plantab =%sysfunc(kscan(%quote(&plan_fields.),1));
				%let pfields = %mm_parsefields2(in=%quote(&plan_fields.),frontstr=p&num_data_versions.,tab=&plantab.,dosum=1);
	   			%let pfilter =%sysfunc(kscan(%quote(&filter_plan.),&i.,%str(,)));

				/* get time clause */
				%let planshift = %sysfunc(kscan(&version_shifts,&i.));
				proc sql noprint;
		 			select 
						%if ("&planshift." ne "") %then %do; plan&i._id format=10. %end; %else %do; hist0_id format=10. %end;
						into :timestr separated by ',' from work.time_members;
					%let timeclause = ((time_level=&real_time_to_level. and time_id in (&timestr.)));

					/* perform extract */
					connect to &dbtype. ( user=maxdata password=%quote(&dbpassword) path="&dsn.");
						create table set&num_data_versions. as 
						select merch_id, merch_level, l_realid as location_id, location_level, time_id, time_level,	0 as cluster,&pfields. 
							from maxdata.&plantab. A, maxdata.t_4key_&projid. B 
							where a.merch_level=b.m_lev and a.merch_id=b.m_id and a.location_level=b.l_lev and a.location_id=b.l_id and &timeclause. 
							and workplan_id in (%quote(&worksheets.)) 
							%if ("&pfilter." ne "") %then %do;
								and &pfilter.
							%end;
						group by &sixkeyscomma.;
					disconnect from &dbtype.;
					
					/* update time to TY time for plan time shift if there is one */
					%if ("&planshift." ne "") %then %do;
						update set&num_data_versions. set time_id=(select hist0_id from time_members where plan&i._id = time_id);
						%end;
				quit;
			%end;

			proc sort data=set&num_data_versions.;
				by &sixkeys;
				run;

			%let i = %eval(&i+1);
			%let planvers = %sysfunc(kscan(&version_groups,&i.));
			%end;  /* end plan version group loop */
		%end;

	   /*****************************************/
	   /* 					  					*/
       /*     GET HISTORICAL DATA    			*/
	   /* 					  					*/
       /*****************************************/

		%let facttabs = %mm_parsegettables(in=%quote(&hist_fields.));
		%let histversionnum =0;

		%do i = 1 %to 3; /* data versions */
		%let histver = &&include_hist&i.;
		%let fcastvers = %sysfunc(kscan(&fcast_versions,1));

		%if &histver. NE 999 %then %do;
			%let hist_key=%eval(&i.-1);
			%let histversionnum = %eval(&histversionnum.+1);
			%let f = 1;
			%let facttab = %sysfunc(kscan(&facttabs,&f));
			%let fincfilter=;
			%do %while (&facttab ne);	
				%let afilter = %sysfunc(kscan(%quote(&filter_hist.),&i.,%str(,)));
			  	%let pos1 = %sysfunc(kindex(&afilter,%str([&facttab.:)));
				%if &pos1.>0 %then %do;
					%let pos1=%eval(&pos1+%sysfunc(klength(&facttab.))+2);
					%let afilter=%sysfunc(ksubstr(&afilter,&pos1.,%eval(%sysfunc(klength(&afilter.))-&pos1.+1)));
					%let pos2=%sysfunc(kindex(&afilter,%str(])));
					%let afilter = %sysfunc(ksubstr(&afilter,1,%eval(&pos2.-1)));
					%end;
				%else %do;
					%let afilter=null;
					%end;
				%let num_data_versions = %eval(&num_data_versions. + 1);
				%let hfields = %mm_parsefields2(in=%quote(&hist_fields.),frontstr=h&histversionnum.,tab=&facttab,dosum=0);

				/* update time for history */
				proc sql noprint;
		 			select hist&hist_key._id format=10. into :timestr separated by ',' from work.time_members;
		 			select time_level into :time_levels separated by ',' from work.time_members;
					drop table set&num_data_versions.;
				quit;
				%if (%sysfunc(kupcase("&facttab."))="MFINC") %then %do; %let fincfilter=&afilter.; %end;

				%hist_async(time=%quote(&timestr.),ds_out=set&num_data_versions.,dsn=&dsn.,dbtype=&dbtype.,dbpassword=&dbpassword.,
					hfields=%quote(&hfields.),afilter=%quote(&afilter.),facttab=&facttab.,mfversion=&fcastvers.);

				%if (&hist_key.>0) and (&update_time_TY.=1) %then %do;  /* If not TY then update time id */
					proc sql noprint;
						update set&num_data_versions. set 
							time_id=(select hist0_id from time_members where hist&hist_key._id = time_id);
					quit;
					%end;
				proc sort data=set&num_data_versions.;
					by &sixkeys;
					run;

				%let f = %eval(&f+1);
				%let facttab = %sysfunc(kscan(&facttabs,&f));
	          %end; /* fact table loop */

		%end; /* if include */
		%end; /* data version loop */

	   /*****************************************/
	   /* 					  					*/
       /*     GET FORECAST DATA TY     			*/
	   /* 					  					*/
       /*****************************************/

		%let i=1;
		%let fcast_version_num=0;
		%let fcastvers = %sysfunc(kscan(&fcast_versions,&i));
		%do %while ((%quote(&fcastvers.) ne) and (%quote(&fcast_fields.) ne));
			%let fcast_version_num = %eval(&fcast_version_num.+1);
			%let num_data_versions = %eval(&num_data_versions. + 1);
			%let fcasttab =%sysfunc(kscan(%quote(&fcast_fields.),1));
			%let ffields = %mm_parsefields2(in=%quote(&fcast_fields.),frontstr=f&fcast_version_num.,tab=%quote(&fcasttab.),dosum=0);

			proc sql noprint;
	 			select hist0_id format=10. into :timestr separated by ',' from work.time_members;
				%let timeclause = ((time_level=&real_time_to_level. and time_id in (&timestr.)));

				/* Perform mfact_version table query */
				create table set&num_data_versions. as 
				select merch_id, merch_level, l_realid as location_id, location_level, time_id, time_level, 0 as cluster, &ffields. 
					from maxdata.&fcasttab. A, maxdata.t_4key_&projid. B 
					where a.merch_level=b.m_lev and a.merch_id=b.m_id and a.location_level=b.l_lev and a.location_id=b.l_id and &timeclause. 
				%if %sysfunc(kupcase(&fcasttab.))=MFACT_VERSION %then %do;
					and fv_version_id=&fcastvers. 
					%end;
				%if ("&filter_fcast." ne "") %then %do;
					and &filter_fcast. 
					%end;
				order by &sixkeyscomma.;
				quit;

			%let i = %eval(&i+1);
			%let fcastvers=%sysfunc(kscan(&fcast_versions,&i));
            %end;


   /* ********************************************************************/
   /* MERGE DATA VERSIONS												 */
   /* ********************************************************************/
	data &out.;
		merge 
		 %do i = 1 %to &num_data_versions.; set&i. %end;
		;
	    by &sixkeys;
		cluster=0;
	run;

   /* ********************************************************************/
   /* SET CLUSTER ID													 */
   /* ********************************************************************/
	%if &input_cluster_id.>0 %then %do;
		proc sql noprint;
			create table stores as select lvnloc_id as lv4loc_id, FNL_CLSTR_SPC_ID as cluster_id from maxdata.clstr_str where CLSTR_ST_ID = &input_cluster_id.;
			update &out. set cluster = (select cluster_id from stores where location_id = lv4loc_id);
		quit;
		%end;

%mend mm_extract12;

%macro mm_extract12_hier(
	   /* ********************************************************************/
       /* FLATEN THE HIERARCHY ABOVE THE ML TO LEVELS                        */
	   /* GET MERCHANDISE, LOCATION HIER KEY VALUES AND ATTRIBUTES 			 */
       /* ********************************************************************/

		in=,
		out=,
		uid=0,
		libstage=inp,
		keys=reg,
		trace=0,

		attribute_fields=,

		/* MERCHANDISE PARAMETERS */
		merch_from_level=,
		merch_to_level=,

		/* LOCATION PARAMETERS */
		cluster_set_id=0,
		loc_from_level=,
		loc_to_level=,			

		/* TIME PARAMETERS */
		do_time=1,
		calendar=0,
		time_from_level=0,
		time_to_level=0
	);

		proc sql noprint;
		create table t_4key as select distinct merch_id as m_id, location_id as l_id from &in.;

		select max(level_id)-10 into :maxmerch from maxdata.hier_level where hier_id = 11 and level_id < 20;
		select max(level_id) into :maxloc from maxdata.hier_level where hier_id = 1 and level_id < 4;
		%let maxmerch=%sysfunc(ktrim(&maxmerch.));
		%let maxloc=%sysfunc(ktrim(&maxloc.));

		create table mlhier as 
		select m_id as merch_id, l_id as location_id,

		%if (&attribute_fields. ne) %then %do; &attribute_fields., %end;
		%let pos=1;
		%let mlkeys=;
		%let afield=%sysfunc(kscan(&attribute_fields, &pos, %str(,)));
		%do %while(&afield. ne);
			%let aspos = %sysfunc(kindex(&afield,%str( as )));
		    %let mlkeys=&mlkeys. %sysfunc(ksubstr(&afield,%eval(&aspos+4),%eval(%sysfunc(klength(&afield))-&aspos-3))),;
			%let pos=%eval(&pos+1);
			%let afield=%sysfunc(kscan(&attribute_fields,&pos,%str(,)));
			%end;

		/* Location keys */
		%if (&loc_to_level.<1000) and (&cluster_set_id.=0) %then %do;
			%do i=1 %to 4;
				%if (&i.=&loc_to_level.) or ((&i. <= &loc_to_level.) and (&i. <= &maxloc.)) %then %do;
					%if &keys=full %then %do;
						%let mlkeys=&mlkeys. lname_&i.,lid_&i.,;
						ktrim(lv&i.loc.lv&i.loc_userid) || '-' || ktrim(lv&i.mast.name) as lname_&i., lv&i.loc.lv&i.loc_id as lid_&i.,
						%end;
					%else %if &keys=ids %then %do;
						%let mlkeys=&mlkeys. lid_&i.,;
						lv&i.loc.lv&i.loc_id as lid_&i.,
						%end;
					%else %do;
						%let mlkeys=&mlkeys. Loc&i.,;
						ktrim(lv&i.loc.lv&i.loc_userid) || '-' || ktrim(lv&i.mast.name) as Loc&i.,
						%end;
					%end;
				%end;
			%end;
		%else %do; /* Clustered locations */
			%if (&loc_to_level.<1000) %then %do;
				&cluster_set_id. as lid_1, FNL_CLSTR_SPC_ID as lid_2, location_id as lid_4,  /* input dataset at store level so t_4key as stores in it */
				%end;
			%else %do;
				&cluster_set_id. as lid_1, location_id as lid_2,    /* input dataset at cluster level, so t_4key has clusters in it */
				%end;
			%end;

		/* Merchandise keys */
		%if &keys=full %then %do;
				%let mlkeys=&mlkeys. mname_1,mid_1;
				ktrim(lv1cmast_userid) || '-' || ktrim(lv1cmast.name) || '(' || ktrim(kleft(put(lv1cmast.lv1cmast_id,best12.))) ||')' as mname_1, lv1cmast.lv1cmast_id as mid_1
				%end;
			%else %do; %if &keys=ids %then %do;
				%let mlkeys=&mlkeys. mid_1;
				lv1cmast.lv1cmast_id as mid_1
				%end;
			%else %do;
				%let mlkeys=&mlkeys. Merch1;
				ktrim(lv1cmast_userid) || '-' || ktrim(lv1cmast.name) || '(' || ktrim(kleft(put(lv1cmast.lv1cmast_id,best12.))) ||')' as Merch1
				%end;
			%end;

		%do i=2 %to 9;
			%if (&i. <= &merch_to_level.) and (&i. <= &maxmerch.) %then %do;
				%if &keys=full %then %do;
						%let mlkeys=&mlkeys. ,mname_&i.,mid_&i.;
						,ktrim(lv&i.cmast_userid) || '-' || ktrim(lv&i.cmast.name) || '(' || ktrim(kleft(put(lv&merch_to_level.ctree.lv&i.ctree_id,best12.))) ||')' as mname_&i., lv&merch_to_level.ctree.lv&i.ctree_id as mid_&i.
						%end;
					%else %do; %if &keys=ids %then %do;
						%let mlkeys=&mlkeys. ,mid_&i.;
						,lv&merch_to_level.ctree.lv&i.ctree_id as mid_&i.
						%end;
					%else %do;
						%let mlkeys=&mlkeys. ,Merch&i.;
						,ktrim(lv&i.cmast_userid) || '-' || ktrim(lv&i.cmast.name) || '(' || ktrim(kleft(put(lv&merch_to_level.ctree.lv&i.ctree_id,best12.))) ||')' as Merch&i.
						%end;
					%end;
				%end;
	    	%end;
		%if &merch_to_level. = 10 %then %do; 
			%if &keys=full %then %do;
					%let mlkeys=&mlkeys. ,mname_10;
					,lv10mast.lv10mast_id as mid_10, lv10mast.order_code as mname_10
					%end;
				%else %do; %if &keys=ids %then %do;
					%let mlkeys=&mlkeys. ,mid_10;
					,lv10mast.lv10mast_id as mid_10
					%end;
				%else %do;
					%let mlkeys=&mlkeys. ,order_code,merch10;
					,lv10mast.order_code || '(' || ktrim(kleft(put(lv10mast.lv10mast_id,best12.))) || ')' as order_code, lv10mast.name as merch10
					%end;
				%end;
			%end;

    from t_4key, 
		/* merchandise tables */
		%if &merch_to_level. = 10 %then %do; maxdata.lv10mast, maxdata.lv10ctree, %end;
		%do i=2 %to 9;
			%if (&i. <= &merch_to_level.) and (&i. <= &maxmerch.) %then %do; 
				maxdata.lv&i.cmast, maxdata.lv&i.ctree, 
				%end;
	    %end;
		maxdata.lv1cmast 

		%if (&loc_to_level.<1000) and (&cluster_set_id.=0) %then %do;
			/* Location tables */
			%do i=1 %to 4;
				%if (&i.=&loc_to_level.) or ((&i. <= &loc_to_level.) and (&i. <= &maxloc.)) %then %do; ,maxdata.lv&i.loc, maxdata.lv&i.mast %end;
		    	%end;
			%end;
		%else %if (&loc_to_level.<1000) %then %do; /* Clustered locations */
			,maxdata.clstr_str
			%end;

	where 
		/* Merchandise keys */
		/* handle lowest level */
		t_4key.m_id = 
		%if &merch_to_level.=1 %then %do;
			lv1cmast.lv1cmast_id 
			%end;
		%else %do;
			lv&merch_to_level.ctree.lv&merch_to_level.ctree_id 
			%if &merch_to_level. = 10 %then %do; 
				and lv10ctree.lv10ctree_id = lv10mast.lv10mast_id  
				%end;
			%else %do; 
				and lv&merch_to_level.cmast.lv&merch_to_level.cmast_id = lv&merch_to_level.ctree.lv&merch_to_level.cmast_id 
				%end;
			%end;

		/* do all parent levels above lowest level */
		%do i=1 %to 9;
			%if (&i. < &merch_to_level.) and (&i. <= &maxmerch.) %then %do; 
				and lv&i.cmast.lv&i.cmast_id = lv&merch_to_level.ctree.lv&i.cmast_id 
				%if (&i. > 1) %then %do;
					and lv&i.ctree.lv&i.ctree_id = lv&merch_to_level.ctree.lv&i.ctree_id 
				%end;
			%end;
	    %end;

		/* Location keys */
		%if (&loc_to_level.<1000) and (&cluster_set_id.=0) %then %do;
			and t_4key.l_id = lv&loc_to_level.loc.lv&loc_to_level.loc_id 
			and lv&loc_to_level.loc.lv&loc_to_level.mast_id = lv&loc_to_level.mast.lv&loc_to_level.mast_id 
			%do i=1 %to 4;
				%if (&i. < &loc_to_level.) and (&i. <= &maxloc.) %then %do;
					%if &i. ne &loc_to_level. %then %do; and lv&i.loc.lv&i.loc_id = lv&loc_to_level.loc.lv&i.loc_id %end;
					and lv&i.loc.lv&i.mast_id = lv&i.mast.lv&i.mast_id 
					%end;
			    %end;
			%end;
		%else %if (&loc_to_level.<1000) %then %do;  /* translate stores to clusters */
			and CLSTR_ST_ID=&cluster_set_id. and LVNLOC_ID=location_id
			%end;

		order by merch_id, location_id;

   /* ********************************************************************/
   /* FLATEN THE HIERARCHY ABOVE THE TIME FROM LEVELS                     */
   /* GET TIME HIER KEY VALUES AND ATTRIBUTES 			 */
   /* ********************************************************************/

	%if (&do_time.=1) %then %do;
		%get_time_hier(in=&in.,out=merchset3,to_level=&time_to_level.,keys=&keys.);
		%let src=merchset3;
		%end;
	%else %do;
		%let src=&in.;
		%end;

	proc sort data=&src. out=merchset3;
		by merch_id location_id;
	run;

	Data &out.;
		merge mlhier(in=InHier) merchset3(in=InRecs);
	    by merch_id location_id;
		if InRecs and InHier then output &out.;
	run;

/*	%if &input_cluster_id.>0 %then %do;
		proc sql noprint;
		create table stores as select lvnloc_id as lv4loc_id, FNL_CLSTR_SPC_ID as cluster_id from maxdata.clstr_str where CLSTR_ST_ID = &input_cluster_id.;
		update &out. set cluster = (select cluster_id from stores where lid_4 = lv4loc_id);
		quit;
		%end;
*/
   /* ********************************************************************/
	/* Change merch/loc/time key labels to hierarchy level names */
   /* ********************************************************************/
	proc sql noprint;
		create table merchlevs as select level_id, level_name from maxdata.hier_level where hier_id=11 and level_id<=%eval(&merch_to_level.+10) order by level_id;
		create table loclevs as select level_id, level_name from maxdata.hier_level where hier_id=1 and level_id<=&loc_to_level. order by level_id;
		create table timelevs as select level_id, level_name from maxdata.hier_level where hier_id=50 and level_id<>45 and level_id<=&real_time_to_level. order by level_id;
		quit;

	data merchlevs;	set merchlevs; level_name=kcompress(level_name,'-^%$#*()+={}[]|\/<>,.;:!@'); level_name=ktranslate(ktrim(level_name),'_',' '); run;
	data loclevs;	set loclevs; level_name=kcompress(level_name,'-^%$#*()+={}[]|\/<>,.;:!@'); level_name=ktranslate(ktrim(level_name),'_',' '); run;
	data timelevs;	set timelevs; level_name=kcompress(level_name,'-^%$#*()+={}[]|\/<>,.;:!@'); level_name=ktranslate(ktrim(level_name),'_',' '); run;

	proc sql noprint;
		select level_name into :merchlevs separated by ',' from merchlevs;
		select level_name into :loclevs separated by ',' from loclevs;
		select level_name into :timelevs separated by ',' from timelevs;
		quit;
	%put &merchlevs. &loclevs &timelevs;

	/* change labels (first get lib and data set names */
	%if (%sysfunc(kindex(%quote(&out.),%str(.)))>0) %then %do;
		%let inlib=%sysfunc(kupcase(%sysfunc(kscan(%quote(&out.),1)))); 
		%let inset=%sysfunc(kupcase(%sysfunc(kscan(%quote(&out.),2)))); 
		%end;
	%else %do;
		%let inlib=WORK;
		%let inset=%sysfunc(kupcase(&out.));
		%end;

	proc datasets Library=&inlib. nolist;
		modify &inset.; 
		label 
		%let la=1;
		%let item = %sysfunc(kscan(%quote(&merchlevs.),&la.));
		%do %while (%quote(&item.) ne);
			mid_&la.="%sysfunc(kscan(%quote(&merchlevs.),&la.,%str(,)))" 
			%let la = %eval(&la.+1);
			%let item = %sysfunc(kscan(%quote(&merchlevs.),&la.));
			%end;

		%if &loc_to_level.<1000 %then %do;
			%let la=1;
			%let item = %sysfunc(kscan(%quote(&loclevs.),&la.));
			%do %while (%quote(&item.) ne);
				lid_&la.="%sysfunc(kscan(%quote(&loclevs.),&la.,%str(,)))" 
				%let la = %eval(&la.+1);
				%let item = %sysfunc(kscan(%quote(&loclevs.),&la.));
				%end;
		%end;

		%if (&do_time.=1) %then %do;
			%let la=1;
			%let item = %sysfunc(kscan(%quote(&timelevs.),&la.));
			%do %while (%quote(&item.) ne);
				tid_&la.="%sysfunc(kscan(%quote(&timelevs.),&la.,%str(,)))" 
				%let la = %eval(&la.+1);
				%let item = %sysfunc(kscan(%quote(&timelevs.),&la.));
				%end;
		%end;
		;
		run;

   /* ********************************************************************/
   /* SET ALTERNATE TIME HIERARCHY 										 */
   /* ********************************************************************/
	%if (&calendar. gt 50) %then %do;
		proc sql noprint;
		create table calendar_member as select * from maxapp.calendar_member;
		%if (&keys=full) %then %let tname=tname_; %else %let tname=time;

		/* parent becomes month ids (where time_id = week id) */
		/*   ASSUMES WEEK LOWEST TIME LEVEL -- NEEDS UPDATING */
		update &out. set parent = (select parent_id from calendar_member where calendar_member_id = time_id and calendar_id = &calendar. and calendar_level_id=51); 
		/* Set month names */
		%if &keys.=ids %then %do;
			update &out. set tid_4=parent;
			%end;
		%else %do;
			update &out. set &tname.4 = (select ktrim(kleft(put(calendar_cycle_id,best4.))) || '-4-' || ktrim(kleft(put(period_id,z2.))) || ' ' || calendar_mem_name 
				from calendar_member where calendar_member_id = parent and calendar_id = &calendar.);
			%end;

		/* parent becomes quarter ids */
		update &out. set parent = (select parent_id from calendar_member where calendar_member_id = parent and calendar_id = &calendar. and calendar_level_id=304); 
		/* Set quarter names */
		%if &keys.=ids %then %do;
			update &out. set tid_3 =parent;
			%end;
		%else %do;
			update &out. set &tname.3 = (select ktrim(kleft(put(calendar_cycle_id,best4.))) || '-3-' || ktrim(kleft(put(period_id,z2.))) || ' ' || calendar_mem_name 
				from calendar_member where calendar_member_id = parent and calendar_id = &calendar.);
			%end;
			
		/* parent becomes season ids */
		update &out. set parent = (select parent_id from calendar_member where calendar_member_id = parent and calendar_id = &calendar. and calendar_level_id=303); 
		/* Set quarter names */
		%if &keys.=ids %then %do;
			update &out. set tid_2=parent;
			%end;
		%else %do;
			update &out. set &tname.2 = (select ktrim(kleft(put(calendar_cycle_id,best4.))) || '-2-' || ktrim(kleft(put(period_id,z2.))) || ' ' || calendar_mem_name 
				from calendar_member where calendar_member_id = parent and calendar_id = &calendar.);
			%end;
			
		/* parent becomes year ids */
		update &out. set parent = (select parent_id from calendar_member where calendar_member_id = parent and calendar_id = &calendar. and calendar_level_id=302); 
		/* Set quarter names */
		%if &keys.=ids %then %do;
			update &out. set tid_1=parent;
			%end;
		%else %do;
			update &out. set &tname.1 = (select calendar_mem_name from calendar_member where calendar_member_id = parent and calendar_id = &calendar.);
			%end;
		
		quit;
		%end;

%mend;
