/**************************************************************/
/*                                                            */
/*       SAS GLOBAL RETAIL INTELLIGENCE SOLUTIONS             */
/*                                                            */
/**************************************************************/
/*                                                            */
/* SAS Retail Group, All rights reserved (c) 2006,2007        */
/*                                                            */
/**************************************************************/

/***********************************************************/
/*Functionality- MERCHANDISE ANALYTICS UTILITIES           */
/*               Parameter, initialize, and finalize code  */
/***********************************************************/

/***********************************************************/
/* ERROR HANDLING ROUTINE RCSet */
/***********************************************************/
    /* CALLED AFTER EACH DATA, SQL, LIB, or FILE OPERATION */
    /* if job_rc>4 THEN error condition exists */

    /*
    Call the macro as follows after any data/proc step:     %RCSet(&syserr);
    Syserr is the system variable that SAS sets if there is any error in a data step / proc.

    Call the macro as follows after proc sql noprint statement :    %RCSet(&sqlrc); 
    sqlrc is the system variable that SAS sets if there is any error in a proc sql noprint

    Call the macro as follows after libname statement:      %RCSet(&syslibrc); 
    syslibrc is the automatic system variable that SAS sets if there is any error in the libname statement
        
    Call the macro as follows after filename statement:     %RCSet(&sysfilrc); 
    sysfilrc is the automatic system variable that SAS sets if there is any error in the filename statement
    */

%macro RCSet(error1);
    %if (&syserr.>4) and (&syserr.>&job_rc.) %then %do;
        %let job_rc=&syserr.;
        %let job_rc_msg=&sysmsg.;
        %end;
    %if (&error1.>4) and (&error1.>&job_rc.) %then %do;
        %let job_rc=&error1.;
        %end;
    %put RCSET:&job_rc. &job_rc_msg.;
%mend RCSet;


/*Read Error, Message from user define local for I18N*/
%macro InitSASMsg();
	%let dir=%sysfunc(ksubstr(&ic_locale,1,2));
	
	data _null_;
	Length Key $250 Value  $1000;
	infile "&path_pgm.&slash.nls&slash.&dir.&slash.&ic_locale..smd" lrecl=3000 dsd dlm='=' missover ;
	input Key $ value $;
	Key=strip(kcompress(key,"09"x));
	Value=strip(kcompress(Value,"09"x));
	if klength(Key) gt 1 and KSubstr(Key,1,1) ne '#' then do;
	 	call execute('%Global ' || key);
		Call execute ('%let '||Key||'='||Value);
     	output;
	end;
run;
%mend;

/*********************************/
/* Set up alternate error log    */
/*********************************/
%macro startlog;
	%Global logfile templogfile;
    %let logfile=&path_pgm./logs/ma_&projid..log;
	%let templogfile=&path_pgm./logs/temp_&projid..log;
/*	%put &LOCAL;*/
/*    %let ic_locale=&LOCAL;*/
/*    %InitSASMsg;*/
    proc printto log="&logfile" new; run;
%mend;

/*********************************/
/* Scan for errors in the log    */
/*********************************/
%macro scanlog;
    proc printto; run;
	%global MA_JOB_COMPETE_MSG; 
    /* Scan for errors */
   %let job_rc_msg=%sysfunc(sasmsg(catalog.dfrmsg, MA_JOB_COMPETE_MSG, noquote));
   %let MA_JOB_COMPETE_MSG=%sysfunc(sasmsg(catalog.dfrmsg, MA_JOB_COMPETE_MSG, noquote));
   %let a=%sysfunc(sasmsg(catalog.dfrmsg, ERROR_IGNORE, noquote));
	data _null_;
	   			format last $1000. line $1000.;
		        infile "&logfile" truncover end=eof;
		        input line $1-1000;
				if ksubstr(%quote(line),1,6)="%sysfunc(sasmsg(catalog.dfrmsg, LOG_ERROR, noquote))"  then do; 
				if kindex(line,"&a") then 
					call symputx('job_rc_msg',"&ma_job_compete_msg");
				else do;
					call symputx('job_rc_msg',line);
					stop;
				end;
		end;
	run;
	data _null_;
		if "&job_rc_msg" ne "&ma_job_compete_msg" then call symputx('job_rc',200);
	run;
    %put &job_rc_msg.;
%mend;

%macro init_timecheck(inlib=out);
	proc sql noprint;
		drop table &inlib..time_&projid.;
		create table &inlib..time_&projid. (step char(80), datetime char(80), secs num);
	quit;
%mend;

%macro timecheck(inlib=out,step=);
	%let timec=%sysfunc(datetime(),datetime20.);
	%let secs=%sysfunc(datetime());
	%put TIMECHECK &step.: &timec. &secs.;
	proc datasets library=work nolist;

	proc sql noprint;
		insert into &inlib..time_&projid. (step,datetime,secs) 
		values ("&step.","&timec.",&secs.);
	quit;
%mend;

%macro final_timecheck(inlib=out);
	proc sql noprint;
		select max(secs)-min(secs) into :tot_secs 
		from &inlib..time_&projid.;
	quit;
	%put total secs: &tot_secs.;

	data out.time_&projid. (drop=last tot_secs);
		set out.time_&projid.;
		retain last;
		if _n_=1 then last=secs;
		put last secs;
		elapsed=secs-last;
		elapsed_pct=elapsed/&tot_secs.*100;
		last=secs;
	run;
	
	/* Output timings */
	proc sql noprint;
		insert into &inlib..time_&projid. (step,secs) 
		values ('Total',&tot_secs.);
	quit;
	proc print data=&inlib..time_&projid.; run;
%mend;

/**********************************************************************************************************************/
/* Getparams takes jobparam_[projid] dataset parameters and sets global parameters based on dimension level/id inputs */
/**********************************************************************************************************************/

/*%macro getparams(proj=,loadUIparams=0,type='ma',merch_id=0,merch_level=0,location_id=0,location_level=0,time_id=0,time_level=0,fieldkey=ALL);*/
 %macro getparams(proj=,loadUIparams=0,type='ma',merch_id=0,merch_level=0,location_id=0,location_level=0,time_id=0,time_level=0,Group_key=0,fieldkey=ALL);
        /* if no parameter file has been built then get from default parameters */
        %let memname=;
        proc sql noprint;
        	select memname into :memname 
			from sashelp.vtable 
			where kupcase(libname)="MMDATA" and kupcase(memname)="JOBPARAM_&proj.";
        quit;
        %if (&memname eq) %then %do;
            proc sql noprint;
/*	            %put &memname %sysfunc(sasmsg(catalog.dfrmsg, MA_OVERRIDE_FILE_MSG, noquote));*/
	            create table mmdata.&param_file. 
				as 
				select * 
				from mmdata.jobparam 
				where type in (&type);
	            update mmdata.&param_file. set projid=&proj.;
            quit;
		%end;

        /* if connected, load worksheet parameters into project parameters */
        %if (&loadUIparams=1) %then %do;
	        /* create temp planworksheet dataset and translate name with any quotes to underscore */
	        proc sql noprint;
	            create table planworksheet 
				as 
				select * 
				from maxdata.planworksheet 
				where planworksheet_id=&proj.;
	            update planworksheet set name=ktranslate(name,'_',"'");
	        quit;

            proc sql noprint;
            	update mmdata.&param_file. 
				set value="%sysfunc(ktrim(&analytic_id.))" 
				where name="request_id";
            	select srcfield into :srcfields separated by ',' 
				from mmdata.&param_file. 
				where srcfield is not null;
            	select name into :parmnames separated by ',' 
				from mmdata.&param_file. 
				where srcfield is not null;
            	%put &srcfields &parmnames;

            	/* 	loop for each parameter and load from planworksheet 
					into jobparam_[projid] dataset */
	            %let i=1;
	            %let srcfield=%sysfunc(kscan(%quote(&srcfields),&i,%str(,)));
	            %do %while (%quote(&srcfield.) ne);
	                %let parmname=%sysfunc(kscan(%quote(&parmnames),&i.,%str(,)));
	                select &srcfield. into :value
					from planworksheet;
	                %let value=%sysfunc(ktrim(%quote(&value.)));
	                update mmdata.&param_file. 
					set value="%quote(&value.)" 
					where name="&parmname.";
	                %let i=%eval(&i.+1);
	                %let srcfield=%sysfunc(kscan(%quote(&srcfields),&i.,%str(,)));
				%end;
            quit;
            %end;

        /* filter for KEY records only */
        /* fieldkey = ALL is represented as a value of 1 so it is sorted first */
        %let fieldkey=%sysfunc(kupcase(&fieldkey.));
        data params;
            set mmdata.&param_file.(where=(kupcase(fieldkey) in ("&fieldkey.",'ALL') 
                and merch_level in (&merch_level.,0) 
                and location_level in (&location_level.,0) 
                and merch_id in (&merch_id.,0)
                and location_id in (&location_id.,0)
				and Group_key in (&Group_key.,0) /*By Kalyan*/
                ));
            select;
                when (fieldkey = 'ALL') do; x=1; end;
                otherwise do; x=0; end;
            end;
        run;

        /* sort by priority of lowest level first default is always 0*/
        proc sort data=params;
            by name x descending Group_key descending merch_level descending location_level descending merch_id descending location_id;
        run;/*By kalyan*/

        /* load parameters into global variables */
        data _null_;
            set params;
            by name;
            if (first.name) then do;
                call symputx(name,value,'g');
                put name value;
			end;
        run;
    %mend;

    %macro mm_step_finish(proj=,standalone=0,step=);
        /* set error codes into worksheet for end of a workflow step */
        %if (&standalone.=0) %then %do;
            proc sql noprint;
            	update maxdata.planworksheet 
				set analytic_err_id=&job_rc. 
				where planworksheet_id = &proj.;
            quit;
		%end;

        /* save parameters where param_type > 0 and only save to default job level (don't save to overrides) */
        data mmdata.&param_file.;
            set mmdata.&param_file.;
            if merch_level=0 and location_level=0 and time_level=0 and param_type>0 then 
                value=symget(name);
            /*put name value;*/
        run;
    %mend;


/********************************************************************************************/
/* INITIALSETUP                                                     */
/********************************************************************************************/

    %macro initialsetup(proj=,type=,standalone=,paramfile=,doreport=1);
        %global template template_level template_member mplan real_time_to_level slash rptout logfile projid param_file six_keys sixkeys sixkeyscomma 
            orig_merch_from_level orig_merch_to_level orig_loc_from_level orig_loc_to_level orig_loc_from_id;

        %let projid=&proj;

		%init_timecheck();
		%timecheck(step=%quote(init));

        %if ("&paramfile." eq "") 
            %then %let param_file=jobparam_&proj.;
            %else %let param_file=&paramfile.;

        %let loaduiparams=0;
        %if &standalone.=0 %then %do; %let loaduiparams=1; %end;

        %startlog;
        %if &standalone<2 %then %getparams(proj=&proj., type=&type.,loadUIparams=&loaduiparams.);
        %let job_rc=0;
        %let job_rc_msg=;

        /********************************************************************************************/
        /* SETUP GENERIC PARAMTERS                                                                  */
        /********************************************************************************************/
        data _null_;
			length jobname $ 100.;
			jobname=kcompress("&jobname.",'-^%$#*()+={}[]|\/<>,.;:!@'); 
            call symput('jobname',jobname); 
         run;
        %RCSet(&syserr.);

        /* Setup levels */
        %let real_time_to_level = %eval(&time_to_level.+46);
        %let orig_merch_from_level=&merch_from_level;
        %let orig_merch_to_level=&merch_to_level;
        %let orig_loc_from_level=&loc_from_level;
        %let orig_loc_to_level=&loc_to_level;
        %let orig_loc_from_id=&loc_from_id;
        
        /* set up six_key constant strings */
    	%let six_keys=ktrim(kleft(put(merch_id,best10.))) || ktrim(kleft(put(merch_level,best10.))) ;
		%let six_keys=&six_keys. || ktrim(kleft(put(location_id,best10.))) || ktrim(kleft(put(location_level,best10.))) ;
		%let six_keys=&six_keys. || ktrim(kleft(put(time_id,best10.))) || ktrim(kleft(put(time_level,best10.)));
        %let sixkeys=merch_level merch_id location_level location_id time_level time_id;
        %let sixkeyscomma=merch_level, merch_id, location_level, location_id, time_level, time_id;

        /* Set template names based on version of application */
        %if &mmax_version.=6 %then %do;
            %let template=maxdata.dimset_template;
            %let template_level=maxdata.dimset_template_lev;
            %let template_member=maxdata.dimset_template_mem;
            %let mplan=mplan_submit;
/*          %let mplan=mplan_working;*/
		%end;
        %else %do;
            %let template=maxapp.template;
            %let template_level=maxapp.template_level;
            %let template_member=maxapp.template_member;
            %let mplan=mplan;
		%end;

        /********************************************************************************************/
        /* SETUP REPORT                                                                             */
        /********************************************************************************************/
        /*
    	%if &doreport.=1 %then %do;
	        %put &path_report.;
	        ods listing close;
	        %let rptout = %quote(&path_report.\fr_%sysfunc(ktrim(&jobname.))_&projid..html);
	        &output_format_type. body= "&rptout."
	        gpath="&path_plots.";
	        footnote "%sysfunc(sasmsg(catalog.dfrmsg, MA_SETUP_RPT_FNOTE, noquote))";
	        title "&jobname. &ma_setup_rpt_summary";
	        ods graphics on;
	        goptions device=gif;
	        ods trace off;
        %end;
        */
        
    %mend;

/***************/
/* END PROCESS */
/***************/

    %macro mm_endprocess(standalone=0,doreport=1);
    
    /********************************************************************************************/
    /* Close Report */
    /********************************************************************************************/
    /*
    %if &doreport.=1 %then %do;
        ods listing close;
        ods graphics off;
        &output_format_type. close;
        ods listing;
    %end;
    */
    /********************************************************************************************/
    /* SET ERROR MESSAGE                                                                        */
    /********************************************************************************************/
    %scanlog;
    
    %if &job_rc=0 %then %do; 
        %let job_rc_msg=&ma_job_compete_msg.;
    %end;
    %put %sysfunc(sasmsg(catalog.dfrmsg, MA_RETURN_CODE_MSG, noquote)) &job_rc &job_rc_msg;

    %if (&standalone.=0) %then %do;
        proc sql noprint;
	        %let curr = %sysfunc(datetime(),datetime20.);
	        update maxdata.planworksheet 
			set WKSHT_URL = "file:&rptout.", 
				analytic_err_id =&job_rc., 
				bat_error_msg="&job_rc_msg.", 
				LAST_UPDATE_DATE="&curr."dt 
	            where planworksheet_id = &projid.;
		quit;
	    %RCSet(&sqlrc.);
	%end;

	%timecheck(step=%quote(FINAL));
	%final_timecheck();
    %mend;


/********************************************************************************************/
/* GETFIELDSTR                                                      */
/********************************************************************************************/

    %macro mm_getfieldstr(in=, field=);

        data Time_Flat;
        	set &in.(keep=&field.) end=Endobs;

	        format time_ids $75.;

	        retain time_ids ;
	        if (_n_ eq 1) then time_ids = ' ';

	        time_ids = kcompress(time_ids) ||&field.;

	        if (Endobs) then
	            do;
		            time_ids = kcompress(time_ids);
		            output;
	            end;
	        else
	            do;
	            	time_ids = kcompress(time_ids) ||',';
	            end;
        run;

    %mend mm_getfieldstr;


/*******************************************************************************/
/* MM_parsefields2                                                          */
/*                                                                              */
/* This macro parses a list of fields and extracts a unique table list      */
/* (if frontstr = NULL)                             */
/*  OR creates the same list of fields with FRONTSTR string in front of each field */
/*                                                                          */
/*******************************************************************************/
    %macro mm_parsegettables2(in=);
       %let num=1;
       %let tabs = ;
       %let tabfield=%sysfunc(kscan(&in,&num,%str(,)));           /* get a field from the list */
       %do %while(&tabfield ne);                    /* while fields are in the list do */
          %let table = %sysfunc(klowcase(%sysfunc(kscan(&tabfield,1,%str(.))))).%sysfunc(klowcase(%sysfunc(kscan(&tabfield,2,%str(.)))));
          %if %sysfunc(kindex(%quote(&tabs.),%quote(&table.)))=0 %then %do;         /*add table if not in tabs list already */
              %let tabs=%str(&tabs) &table,;
              %end;
          %let num=%eval(&num+1);
          %let tabfield=%sysfunc(kscan(&in,&num,%str(,)));
          %end;
       &tabs
    %mend;

  %macro mm_parsefields3(in=, frontstr=, tab=, dosum=2, docomma=0);
       %let num=1;
       %let numadded=0;
       %let tabfield=%sysfunc(kscan(&in,&num,%str(,)));           /* get a field from the list */
       %if (&frontstr. ne) %then %do; %let frontstr=%str(&frontstr.)_; %end;

       %do %while(&tabfield ne);                        /* while fields are in the list do */

          %let table = %sysfunc(klowcase(%sysfunc(kscan(&tabfield,1,%str(.)))));
          %put &table &tab &tabfield;
          %if &table = %sysfunc(klowcase(&tab)) %then %do;  /* only output field if it is in the input table */
              %let numadded=%eval(&numadded+1);
              %if (&numadded. > 1) and (&docomma.=1) %then %do; , %end;
              %sysfunc(kscan(&tabfield,3,%str(.)))
          %end;

          %let num=%eval(&num+1);
          %let tabfield=%sysfunc(kscan(&in,&num,%str(,)));
          %end;
    %mend;

    %macro mm_parsefields2(in=, frontstr=, tab=, dosum=);
       	%let num=1;
       	%let numadded=0;
       	%let tabfield=%sysfunc(kscan(&in,&num,%str(,)));           /* get a field from the list */
       	%if (&frontstr. ne) %then %do; 
			%let frontstr=%str(&frontstr.)_; 
		%end;

       	%do %while(&tabfield ne);                        /* while fields are in the list do */
        	%let table = %sysfunc(kscan(&tabfield,1,%str(.)));
          	%if (%sysfunc(kupcase(&table.)) eq %sysfunc(kupcase(&tab.))) %then %do;  /* only output field if it is in the input table */
              	%let numadded=%eval(&numadded+1);
              	%if &numadded. > 1 %then %do;
			, 
			%end;
              	%let afield = %sysfunc(kscan(&tabfield,2,%str(.)));
	        %let aspos = %sysfunc(kindex(&tabfield,%str( as )));
              	%if &dosum=1 %then %do; 
					sum(&afield) as 
				%end;
                %else %do; 
					&afield as 
				%end;
              	%if &aspos=0 %then %do; 
					&frontstr.&afield 
				%end;                            /* no as clause */ 
                %else %do; 
					&frontstr.%sysfunc(ksubstr(&tabfield,%eval(&aspos+4),%eval(%sysfunc(klength(&tabfield))-&aspos-3))) 
				%end; /* as caluse exists already */
          	%end;

	          %let num=%eval(&num+1);
	          %let tabfield=%sysfunc(kscan(&in,&num,%str(,)));
          %end;
    %mend;


/****************************************************************************************/
/* mm_parsegettables                                                */
/*--------------------------------------------------------------------------------------*/
/* from in parameter parses out table names from SQL table.fields           */
/****************************************************************************************/

    %macro mm_parsegettables(in=);
       %let num=1;
       %let tabs = ;
       %let tabfield=%sysfunc(kscan(&in,&num,%str(,)));           /* get a field from the list */
       %do %while(&tabfield ne);                    /* while fields are in the list do */
          	%let table = %sysfunc(klowcase(%sysfunc(kscan(&tabfield,1,%str(.)))));     
          	%if %sysfunc(kindex(&tabs,&table))=0 %then %do;         /*add table if not in tabs list already */
            	%let tabs = %str(&tabs) &table;
			%end;
          	%let num=%eval(&num+1);
          	%let tabfield=%sysfunc(kscan(&in,&num,%str(,)));
    	%end;
       &tabs
    %mend;

/*------------------------------------------------------------------------------
    Demand Forecasting for Retail
    Description: This macro prints return codes from the PASS-THROUGH Facility 
                 of the SAS/Access engin
    Developer: Paul Dettorre, X8265
    Created: 11/2008
    Modification Log:

------------------------------------------------------------------------------*/
%Macro RDBMS_Report(macro_call=,code_area=,switch=);
    %If (%sysfunc(kupcase(&Switch.)) eq ON) %Then
        %do;
            %PUT ;
            %PUT ==============================================;
            %PUT Relation Database Report;
            %PUT macro_call:&macro_call. code_area:&code_area. ;
            %PUT Database Message           : &SYSDBMSG.;
            %PUT Database Return Code       : &SYSDBRC.;
            %IF (%symexist(SQLXMSG)) %Then
                %Do;
                    %PUT Pass Through Message       : &SQLXMSG.;
                    %PUT Pass Through Return Code   : &SQLXRC.;
                %End;
            %PUT ;
            %PUT ==============================================;
            %PUT ;
        %End;
%Mend;

/****************************************************************************************/
/* subarg replaces a substr sstr within astr with rstr                                  */
/*        the resultant string is %put output                                           */
/*        ex: %subarg(astr=abc align=end def,sstr=align=,rstr=BEGINNING);               */
/****************************************************************************************/
%macro subarg(astr=,sstr=,rstr=);
    /* ss is remaining string after sstr is found */
    %let aspos = %sysfunc(kindex(&astr.,%str(&sstr.)));
    %if &aspos>0 %then %do;
        %let ss=%sysfunc(ksubstr(&astr.,%eval(&aspos+%sysfunc(klength(&sstr.))),%eval(%sysfunc(klength(&astr.))-&aspos-%sysfunc(klength(&sstr.))+1)));
        %let epos = %sysfunc(kindex(&ss.,%str( )));
        %if &epos>0 %then %do;  
            %if &aspos=1 %then %do;  /* at beginning of string */
                &sstr.&rstr.%sysfunc(ksubstr(&ss.,&epos.,%eval(%sysfunc(klength(&ss.))-&epos.+1)))
			%end;
            %else %do;  /* in the middle of string */
                %sysfunc(ksubstr(&astr.,1,&aspos-1))&sstr.&rstr.%sysfunc(ksubstr(&ss.,&epos.,%eval(%sysfunc(klength(&ss.))-&epos.+1)))
			%end;
		%end;
        %else %do;  /* at end of string */
            %sysfunc(ksubstr(&astr.,1,&aspos-1))&sstr.&rstr. 
        %end;
	%end;
%mend;

/********************************************************************/
/* Get all Merch ids from plan worksheet -- used in extract of data */
/*    returns work.t_cube_merch dataset                             */
/********************************************************************/
%macro get_merch_members(merch_from_level=0,merch_to_level=0,merch_template=0,merchlevelparent=0);

    /* create merch member table */
    proc sql noprint;
        drop table t_cube_merch;
        CREATE TABLE T_CUBE_MERCH (m_lev num, m_id num);
	quit;

    %if &merch_to_level. = 1 %then %do; 
		%let merchtotab = cmast; 
	%end;
    %else %do; 
		%let merchtotab = ctree; 
	%end;
    %if &merch_from_level. = 1 %then %do; 
		%let merchfromtab = cmast; 
	%end;
    %else %do; 
		%let merchfromtab = ctree; 
	%end;

    /* Add merch members from either template or static hierarchy */
    proc sql noprint;
        %let temcount=0;
        %if (&merch_template. > 0) %then %do;
            select count(*) into :temcount 
			from &template_member. 
			where template_id = &merch_template. and level_number = %eval(&merch_to_level.+10);
    	%end;
        %if (&merch_template. > 0)  and (&temcount. > 0) %then %do;
            /* if merchelevelparent given then pull ALL static members from parent merch ids in template as defined by the merchlevelparent level within the template */
            %if (&merchlevelparent > 0) and (&merch_to_level > &merchlevelparent) %then %do;  
                /* get all merchlevelparent members from template THEN all to level children for these parents from the hierarchy */
                create table tempmerch 
				as 
				select member_id 
				from &template_member.
                where template_id = &merch_template. and level_number = %eval(&merch_to_level.+10);
                create table tempmerch2 
				as 
				select distinct lv&merchlevelparent.&merchtotab._id as member_id 
				from maxdata.lv&merch_to_level.&merchtotab. 
                where lv&merch_to_level.&merchfromtab._id in (select member_id from tempmerch);
                insert into T_CUBE_MERCH (m_lev, m_id)
                select &merch_to_level., lv&merch_to_level.&merchtotab._id 
				from maxdata.lv&merch_to_level.&merchtotab. 
				where lv&merchlevelparent.&merchfromtab._id in (select member_id from tempmerch2);
			%end;
            %else %do;  /* add from template */
                insert into T_CUBE_MERCH (m_lev, m_id)
                select &merch_to_level., member_id 
				from &template_member. 
                where template_id = &merch_template. and level_number = %eval(&merch_to_level.+10);
			%end;
		%end;
        %else %do;  /* add members from static hierarchy */
            insert into T_CUBE_MERCH (m_lev, m_id)
			select &merch_to_level., lv&merch_to_level.&merchtotab._id 
            from maxdata.lv&merch_to_level.&merchtotab. 
			where lv&merch_from_level.&merchfromtab._id = &merch_from_id.;
		%end;
    quit;
%mend;

/********************************************************************/
/* Get all TY time periods using static time hierarchy or time sets */
/*  returns work.time_members                                       */
/********************************************************************/

%macro get_time_ids(timetype=,fromdate=,todate=,time_set=0,numplanshifts=0,timetolevel=,timefromlevel=,calendar=50);

    %if (&time_set.>0) and (&timetype.=Standard) %then %do; 
		%let timetype=Event; 
	%end;

    %if &timetype.=Standard %then %do;  /* get all time children of one time parent */
        proc sql noprint;
    	    create table time_members1 
			as 
            select lv&timetolevel.time_lkup_id as src_time_id 
			from maxapp.lv&timetolevel.time 
            where lv&timefromlevel.time_lkup_id = &fromdate.;   /*  and loaded_flag=1 */
        quit;
	%end;
    %else %do;          /* get all time ids between from and to time dates */
        %let set_type=0;
        proc sql noprint;
	        %if (&time_set.>0) %then %do;
	            select set_type,start_date, end_date into :set_type, :temp_sdate, :temp_tdate 
				from &template. 
				where template_id=&time_set.;
	            %let set_type=%sysfunc(ktrim(&set_type.));
	            %if (&set_type. ne 9) %then %do; 
					%let fromdate=&temp_sdate; 
					%let todate=&temp_tdate; 
				%end;
			%end;

	        create table time_members1 as 
            	select lv&timetolevel.time_lkup_id as src_time_id 
				from maxapp.lv&timetolevel.time 
	            where 	(lv&timetolevel.time_start_date > "&fromdate."d and 
						lv&timetolevel.time_end_date < "&todate."d) or 
						(lv&timetolevel.time_start_date = "&fromdate."d) or 
						(lv&timetolevel.time_end_date = "&todate."d);
						/* and loaded_flag=1; */
        quit;
	%end;

    /**********************************************/
    /* Get time members with TY, LY, LLY versions */
    /**********************************************/
    proc sql noprint;
    	create table time_members (hist0_id num, hist1_id num, hist2_id num, 
        /* add plan time shifts into time_members table */
        %do i=1 %to &numplanshifts.; plan&i._id num, %end;
        time_level num, period_id num, cycle_id num);
	quit;

    /* fill in LY, LLY -- use prior year if no values as determined by loaded_flag or to find 53-week LY value */
	%if &calendar. = 50 %then %do;
	    proc sql noprint;
        	insert into time_members (hist0_id, time_level, period_id, cycle_id) 
	            select src_time_id, &real_time_to_level., lv&timetolevel.time_id, cycle_id 
	            	from time_members1, maxapp.lv&timetolevel.time 
	            	where lv&timetolevel.time_lkup_id = src_time_id;

			update time_members 
			set hist1_id = (select lv&timetolevel.time_lkup_id 
							from maxapp.lv&timetolevel.time 
							where cycle_id = time_members.cycle_id-1 and 
							lv&timetolevel.time_id = time_members.period_id and loaded_flag=1);
	        update time_members 
			set hist1_id = (select lv&timetolevel.time_lkup_id 
							from maxapp.lv&timetolevel.time 
							where cycle_id = time_members.cycle_id-2 and 
							lv&timetolevel.time_id = 1) 
			where hist1_id is null;
	        update time_members 
			set hist2_id = (select lv&timetolevel.time_lkup_id 
							from maxapp.lv&timetolevel.time 
							where cycle_id = time_members.cycle_id-2 and 
							lv&timetolevel.time_id = time_members.period_id and loaded_flag=1);
	        update time_members 
			set hist2_id = (select lv&timetolevel.time_lkup_id 
							from maxapp.lv&timetolevel.time 
							where cycle_id = time_members.cycle_id-3 and 
							lv&timetolevel.time_id = 1) 
			where hist2_id is null;
		quit;
	%end;
	%else %do;
		proc sql noprint;
			insert into time_members (hist0_id, time_level, period_id, cycle_id) 
				select src_time_id, &real_time_to_level., period_id, calendar_cycle_id 
					from time_members1, maxapp.calendar_member 
					where calendar_id=&calendar. and calendar_level_id=&real_time_to_level. and calendar_member_id = src_time_id;

        	update time_members 
			set hist1_id = (select CALENDAR_MEMBER_ID 
							from maxapp.calendar_member 
							where calendar_id=&calendar. and 
							calendar_level_id=&real_time_to_level. and 
							CALENDAR_CYCLE_ID = time_members.cycle_id-1 and 
							calendar_member.period_id = time_members.period_id);
        	update time_members 
			set hist1_id = (select CALENDAR_MEMBER_ID 
							from maxapp.calendar_member 
							where calendar_id=&calendar. and 
							calendar_level_id=&real_time_to_level. and 
							CALENDAR_CYCLE_ID = time_members.cycle_id-2 and 
							calendar_member.period_id = 1) 
			where hist1_id is null;
	        update time_members 
			set hist2_id = (select CALENDAR_MEMBER_ID 
							from maxapp.calendar_member 
							where calendar_id=&calendar. and 
							calendar_level_id=&real_time_to_level. and 
							CALENDAR_CYCLE_ID = time_members.cycle_id-2 and 
							calendar_member.period_id = time_members.period_id);
        	update time_members 
			set hist2_id = (select CALENDAR_MEMBER_ID 
							from maxapp.calendar_member 
							where calendar_id=&calendar. and 
							calendar_level_id=&real_time_to_level. and 
							CALENDAR_CYCLE_ID = time_members.cycle_id-3 and 
							calendar_member.period_id = 1)
			where hist2_id is null;
		quit;
	%end;
	data W_&wktid..time_members;
		set time_members;
	run;
%mend;

%macro add_parents();

	proc sql noprint;

	/* Merchandise Parents */
    	%if &merch_from_level.<&merch_to_level. %then %do;
	        create table mkeys 
			as 
			select * 
			from stage.mkeys_&projid.;
	        %do m=&merch_from_level. %to %eval(&merch_to_level.-1);
	            %if &m<=&dfr_maxmerch or &m=&merch_to_level. %then %do;
	                %if &m. = 1 %then %do; 
						%let merchtab = cmast; 
					%end;
	                %else %do; 
						%let merchtab = ctree; 
					%end;
	                insert into stage.mkeys_&projid. (merch_id, merch_level) 
					select distinct(lv&m.&merchtab._id), &m. 
					from maxdata.lv&merch_to_level.ctree
					where lv&merch_to_level.ctree_id in (select merch_id from mkeys);
				%end;
			%end;
        %end;

	/* Static Location parents */
    	%if (&orig_loc_from_level.<1000) and (&loc_from_level.<&l_lev.) %then %do; /* static location hierarchy */
        	create table lkeys 
			as 
			select * 
			from stage.lkeys_&projid.;
        	%do l=&loc_from_level. %to %eval(&l_lev.-1);
            	insert into stage.lkeys_&projid. (location_id, location_level) 
            	select distinct (lv&l.loc_id), &l. 
				from maxdata.lv&loc_to_level.loc 
                where lv&loc_to_level.loc_id in (select location_id from lkeys);
            %end;
        %end;

		/* Time parents */
    	%if &time_from_level.<&time_to_level. %then %do;
		/* >> need to account for alternate time hierarchy parents << */
        	create table tkeys 
			as 
			select * 
			from stage.tkeys_&projid.;
        	%do t=&time_from_level. %to %eval(&time_to_level.-1);
	            insert into stage.tkeys_&projid. (time_id, time_level)
	            select distinct(lv&t.time_lkup_id),%eval(&t.+46) 
				from maxapp.lv&time_to_level.time 
	            where lv&time_to_level.time_lkup_id in (select time_id from tkeys);
			%end;
		%end;

	    %if (&time_template.>0) %then %do; 
	        insert into stage.tkeys_&projid. (time_id, time_level) values (&from_time_id., 45);
	    %end;
	quit;
%mend;

%macro import_fact_table(indata=,finalname=,outtable=,import_version=0,delete_off_indata=0,use_hist=0);

	%if (&import_version.=0) and (%sysfunc(kupcase("&outtable."))="MAXDATA.MFACT_VERSION") %then %do;
		%let job_rc=102;
		%let job_rc_msg=%sysfunc(sasmsg(catalog.dfrmsg, MA_IMP_MFACT_VERSION_ERR, noquote));
	%end;
	%else %do;

	    /***********************************************/
	    /* SAVE EXISTING DATA FOR VERSIONED DATA       */
	    /***********************************************/
	    %if (&import_version.>0) %then %do;  /* IF VERSIONED DATA */
	        proc sql noprint;
	        	create table existing 
				as 
				select * 
				from &outtable. 
	            where &six_keys in (select &six_keys from &indata.) 
				%if &import_version.>0 %then %do; 
					and FV_VERSION_ID=&import_version. 
				%end;
	            order by &sixkeyscomma.;
	        quit;

	        /***********************************************/
	        /* MERGE EXISTING AND NEW                      */
	        /***********************************************/
	        proc sort data=&indata.;
	            by &sixkeys.;
			run;
	        data &indata.;
	            merge existing &indata.;
	            by &sixkeys.;
			run;
		%end;

    /***********************************************/
    /* REMOVE EXISTING DATA */
    /***********************************************/

	    /**************************************************************************/
	    /* TRUNCATE PARTITION INSTEAD OF DELETE IF dfr_import_truncate_version=1  */
	    /**************************************************************************
	    
/* DEM 03/08/09 Removed as new import performs this operation on mfact_version */
/*	    %if (&import_version.>0) and (&dfr_import_truncate_version.=1) and (&dbtype.=oracle) %then %do;
			proc sql noprint;
				connect to &dbtype. ( user=maxdata password=%quote(&dbpassword) path="&dsn.");
				execute( alter table maxdata.mfact_version truncate partition P_FV_VER_&import_version.
				) by &dbtype.;
				disconnect from &dbtype.;
			quit;
		%end;
*/

	    /***********************************************/
	    /* DELETE TARGET RECORDS ONLY                  */
	    /***********************************************/
/*	    %else */
		%if &delete_off_indata.=1 %then %do;
			/* Delete target records based only off of keys in import data */
	        proc sql noprint;
				delete from &outtable. 
				where &six_keys in (select &six_keys from &indata.) 
				%if (&import_version.>0) %then %do; 
					and FV_VERSION_ID=&import_version. 
				%end;
			    ;
			quit;
		%end;

	    /***********************************************/
	    /* DELETE BASED ON FULL CUBE KEYS              */
	    /***********************************************/
	    %else %do;
	        /* Save time ids for worksheet boundary */
			%if &use_hist.=0 %then %do;
	    	    %get_time_ids(timetype=Event,fromdate=&from_time.,todate=&to_time.,time_set=&time_template.,timetolevel=&time_to_level.,timefromlevel=&time_from_level.);
			%end;
			%else %do;
		        %get_time_ids(timetype=Event,fromdate=&hist_from_time.,todate=&hist_to_time.,time_set=&time_template.,timetolevel=&time_to_level.,timefromlevel=&time_from_level.);
			%end;
	        proc sql noprint;
		        create table stage.tkeys_&projid. 
				as 
				select hist0_id as time_id, time_level 
				from time_members;
	        quit;

			/* Get full cube of parents for delete */
	        %add_parents;

			/* ensure all keys in input dataset are also deleted */
			proc sql noprint;
				create table newtime 
				as 
				select distinct time_id, time_level 
				from &indata. 
				order by time_id, time_level;
				create table newmerch 
				as 
				select distinct merch_id, merch_level 
				from &indata. 
				order by merch_id, merch_level;
				create table newloc 
				as 
				select distinct location_id, location_level 
				from &indata. 
				order by location_id, location_level;
			quit;

			proc sort data=stage.tkeys_&projid. out=tkeys; 
				by time_id time_level; 
			run;
			proc sort data=stage.lkeys_&projid. out=lkeys; 
				by location_id location_level; 
			run;
			proc sort data=stage.mkeys_&projid. out=mkeys; 
				by merch_id merch_level; 
			run;
			data tkeys; 
				merge tkeys newtime; 
				by time_id time_level;
			run;
			data lkeys; 
				merge lkeys newloc; 
				by location_id location_level;
			run;
			data mkeys; 
				merge mkeys newmerch; 
				by merch_id merch_level;
			run;

			/* Delete target records */
		    proc sql noprint;
		        delete from &outtable. 
				where &six_keys in (select &six_keys 
									from tkeys,lkeys,mkeys) 
				%if (&import_version.>0) %then %do; 
					and FV_VERSION_ID=&import_version. 
				%end;
			    ;
		    quit;
	%end;

    /***********************************************/
    /* Insert new forecast */
    /***********************************************/
	%put %sysfunc(sasmsg(catalog.dfrmsg, MA_IMP_TO_TABLE_MSG, noquote)) &outtable.;

	    proc sql noprint;
	        /* get output field names */
	        %if %sysfunc(kindex(%quote(&indata.),%str(.)))>0 %then %do;
	            %let inlib=%sysfunc(kupcase(%sysfunc(kscan(%quote(&indata.),1)))); 
	            %let inset=%sysfunc(kupcase(%sysfunc(kscan(%quote(&indata.),2)))); 
	  		%end;
	        %else %do;
	            %let inlib=WORK;
	            %let inset=%sysfunc(kupcase(&indata.));
			%end;
	        select name into :outfields separated by ',' 
			from sashelp.vcolumn
	        where libname = "&inlib." and memname ="&inset.";

	        /* import to fact table */
	        insert into &outtable. (&outfields.) 
			select &outfields. 
			from &indata.;

	        /* create same data to disk data set */
	        create table &finalname. 
			as 
			select * 
			from &indata.;
		quit;
	%end;
%mend;

%macro get_time_hier(in=,out=,to_level=,keys=ids);
	proc sql noprint;
		create table t_time 
		as 
		select distinct time_id 
		from &in.;

		%let is1time=1; %let is2time=0; %let is3time=0; %let is4time=0; %let is5time=0;

		select 1 into :is2time 
		from maxdata.hier_level 
		where hier_id = 50 and level_id=48;
		select 1 into :is3time 
		from maxdata.hier_level 
		where hier_id = 50 and level_id=49;
		select 1 into :is4time 
		from maxdata.hier_level 
		where hier_id = 50 and level_id=50;
		select 1 into :is5time 
		from maxdata.hier_level 
		where hier_id = 50 and level_id=51;

		create table inp.t_&projid. 
		as select 
	/* Time keys */
		%do i=2 %to 5;
			%if (&i. <= &to_level.) and (&&is&i.time.>0) %then %do; 
				%if &keys.=full or &keys.=ids %then %do;
					lv&i.time.lv&i.time_start_date as tname_&i.,
					lv&i.time.lv&i.time_lkup_id as tid_&i.,
				%end;
				%else %do;
					%sysfunc(ktrim(%sysfunc(kleft(put(lv&i.time.cycle_id,best4.))))) || '-' || %sysfunc(ktrim(%sysfunc(kleft(put(&i.,best1.))))) || '-' || %sysfunc(ktrim(%sysfunc(kleft(put(lv&i.time.lv&i.time_id,z2.))))) || ' ' || lv&i.time.time_name as Time&i., 
				%end;
			%end;
		%end;

		%if &keys.=full or &keys.=ids %then %do;
			lv1time.lv1time_start_date as tname_1, lv1time.lv1time_lkup_id as tid_1, t.time_id,
		%end;
		%else %do;
			lv1time.time_name as Time1, t.time_id,
		%end;
		1000000000 as parent	/*, 0 as cluster*/

		from t_time t, 
		%do i=2 %to 5;
			%if (&i. <= &to_level.) and (&&is&i.time.>0) %then %do; 
				maxapp.lv&i.time, 
			%end;
	    %end;
		maxapp.lv1time
		where 
	/* Time keys */
		t.time_id = lv&to_level.time.lv&to_level.time_lkup_id
		%do i=1 %to 4;
			%if (&i. < &to_level.) and (&&is&i.time.>0) %then %do; 
				and lv&i.time.lv&i.time_lkup_id = lv&to_level.time.lv&i.time_lkup_id  
			%end;
		%end;
		order by time_id;
	quit;

   /* ******************************************************/
   /* MERGE DATAVERSIONS AND HIERARCHY 				 */
   /* ************************************************/
	proc sort data=&in. out=merchset2;
		by time_id;
	run;

	data &out.;
		merge inp.t_&projid.(in=InHier) merchset2(in=InRecs);
	    by time_id;
		if InRecs and InHier then output;
	run;
%mend;

%macro ma_optimize_time_members(vStartDate=,vEndDate=,ds_out=time_members);
    proc sql;
        create table extract_time_members as
        select lv5time_lkup_id
             , lv5time_id
             , cycle_id
             , lv5time_start_date
             , lv5time_end_date
             , 0 as status_flag
             , lv4time_lkup_id
             , lv3time_lkup_id
             , lv2time_lkup_id
             , lv1time_lkup_id
             , 0 as time_level
             , 0 as time_id
          from maxapp.lv5time
         where lv5time_start_date >= "&vStartDate."d
           and lv5time_end_date <= "&vEndDate."d
        ;
    quit;

    proc sort data=maxdata.hier_level(where=(hier_id = 50 and level_id between 47 and 51)) 
               out=hier_level
               nodupkey;
        by level_id;
    run;

    data _null_;
        set hier_level end=eof;
        if (eof eq 1) then call symputx('Hier_Level_Rec_Count',_n_,'g');
    run;

    %let Loop_Flag=1;
    %Do %While (&Loop_Flag. ne 0);

        data _null_;
            set extract_time_members(where=(status_flag=0));
            if (_n_ eq 1) then
                do;
                    call symputx('cycle_id',cycle_id,'g');
                    call symputx('lv1time_lkup_id',lv1time_lkup_id,'g');
                    call symputx('lv2time_lkup_id',lv2time_lkup_id,'g');
                    call symputx('lv3time_lkup_id',lv3time_lkup_id,'g');
                    call symputx('lv4time_lkup_id',lv4time_lkup_id,'g');
                    call symputx('lv5time_lkup_id',lv5time_lkup_id,'g');
                end;
        run;

        /*Get a count of hier_level records*/
        %Do hier_loop_ctr=1 %to &Hier_Level_Rec_Count.;

            data _null_;
                set hier_level;
                if (_n_ eq &hier_loop_ctr.) then
                    do;
                        call symputx('level_id',level_id,'g');
                        call symputx('iTimeId',&&lv&hier_loop_ctr.time_lkup_id.,'g');
                    end;
            run;


            proc sql noprint;
                Select min(lv5time_lkup_id), max(lv5time_lkup_id) into :iMinWeekId, :iMaxWeekId
                  from maxapp.lv5time
                 where cycle_id = &cycle_id.
                   and lv&hier_loop_ctr.time_lkup_id = &iTimeID.
                ;
            quit;

            proc sql noprint;
		      Select count(*) into :ictr 
                from extract_time_members 
               where (lv5time_lkup_id  = &iMinWeekId. or lv5time_lkup_id = &iMaxWeekId.)  and status_flag = 0
                ;
            quit;

            %If (&ictr eq 2 or &iMinWeekId. eq &iMaxWeekId.) %then
                %do;
                    proc sql noprint;
                        Update extract_time_members
                           Set status_flag = 1
                             , time_level = &level_id.
                             , time_id = &iTimeID.
                         where lv&hier_loop_ctr.time_lkup_id = &iTimeID.
                        ;
                    quit;
                    %let hier_loop_ctr=&Hier_Level_Rec_Count.;
                %end;
        %end;

        /* Exit loop if all record status have been set*/
        proc sql noprint;
            select count(*) into :Loop_Flag
              from extract_time_members
             where status_flag = 0
            ;
        quit;

    %end;

	/* extract distinct time ids and levels */
	proc sql noprint;
		create table &ds_out. as select distinct time_id, time_level from extract_time_members;
	quit;
%mend;

%macro get_optimized_time_ids(timetype=,fromdate=,todate=,time_set=0,numplanshifts=0,timetolevel=,timefromlevel=,calendar=50);
    %if (&time_set.>0) and (&timetype.=Standard) %then %do; 
		%let timetype=Event; 
	%end;

    %if &timetype.=Standard %then %do;  /* get all time children of one time parent */
        proc sql noprint;
            select min(lv&timetolevel.time_start_date), max(lv&timetolevel.time_end_date) into :fromdate, :todate 
			from maxapp.lv&timetolevel.time 
            where lv&timefromlevel.time_lkup_id = &fromdate.;
        quit;
	%end;
    %else %do;          /* get all time ids between from and to time dates */
        %let set_type=0;
        proc sql noprint;
	        %if (&time_set.>0) %then %do;
	            select set_type,start_date, end_date into :set_type, :temp_sdate, :temp_tdate 
					from &template. 
					where template_id=&time_set.;
	            %let set_type=%sysfunc(ktrim(&set_type.));
	            %if (&set_type. ne 9) %then %do; 
					%let fromdate=&temp_sdate; 
					%let todate=&temp_tdate; 
				%end;
			%end;
        quit;
	%end;

    %ma_optimize_time_members(vStartDate=&fromdate.,vEndDate=&todate.,ds_out=time_members1);
   
    Proc sql noprint;
    	create table time_members (hist0_id num, hist1_id num, hist2_id num, 
        /* add plan time shifts into time_members table */
        %do i=1 %to &numplanshifts.; plan&i._id num, %end;
        time_level num, period_id num, cycle_id num);
	quit;

    /* fill in LY, LLY -- use prior year if no values as determined by loaded_flag or to find 53-week LY value */
	%do t_level=1 %to 5;
		%let real_t_level=%eval(&t_level.+46);
	    proc sql noprint;
	        insert into time_members (hist0_id, time_level, period_id, cycle_id) 
	        select time_id, time_level, lv&t_level.time_id, cycle_id 
		        from time_members1, maxapp.lv&t_level.time 
		        where lv&t_level.time_lkup_id = time_id and time_level=&real_t_level.;

			update time_members 
			set hist1_id = (select lv&t_level.time_lkup_id 
							from maxapp.lv&t_level.time 
							where cycle_id = time_members.cycle_id-1 and 
							lv&t_level.time_id = time_members.period_id and loaded_flag=1)
			where time_level=&real_t_level.;

	        update time_members 
			set hist1_id = (select lv&t_level.time_lkup_id 
							from maxapp.lv&t_level.time 
							where cycle_id = time_members.cycle_id-2 and 
							lv&t_level.time_id = 1) 
			where hist1_id is null and time_level=&real_t_level.;

	        update time_members 
			set hist2_id = (select lv&t_level.time_lkup_id 
							from maxapp.lv&t_level.time 
							where cycle_id = time_members.cycle_id-2 and 
							lv&t_level.time_id = time_members.period_id and loaded_flag=1) 
	        where time_level=&real_t_level.;

			update time_members 
			set hist2_id = (select lv&t_level.time_lkup_id 
							from maxapp.lv&t_level.time 
							where cycle_id = time_members.cycle_id-3 and 
							lv&t_level.time_id = 1) 
			where hist2_id is null and time_level=&real_t_level.;
		quit;
	%end;
%mend;
