
/**************************************************************/
/*                                                            */
/*        SAS RETAIL INTELLIGENCE  -- Process Server          */
/*                                                            */
/**************************************************************/
/*                                                            */
/* SAS Global Retail Group, All rights reserved (c) 2006      */
/*                                                            */
/*      Date:6/12/2006                                        */
/**************************************************************/
%include "params.sas";
/*options mprint mlogic symbolgen;*/

%macro do_analytic();
%global jobname analytic_lkup_id wktid;

/* Add workflow */
/* analtyic_id > 0 if request, when done set to 0 and set error code */

/* Loop forever */
%do %until (1=0);

	proc sql;
	%let countwkt=0;
	select count(*) into :countwkt from maxdata.planworksheet where analytic_lkup_id>0 
		and planworksheet_id not in (select WORKSHEET_ID from maxdata.mmax_locks);
	%put &countwkt.;
	%if &countwkt.=0 %then %do;
		data _null_; time_slept=sleep(&delaysec.,1); run;
		%end;

	%else %do;
		/* Find worksheet needing processing */
		proc sql;
		select name, analytic_lkup_id, planworksheet_id
			into :jobname, :analytic_lkup_id, :wktid
			from maxdata.planworksheet where analytic_lkup_id>0 
			and planworksheet_id not in (select WORKSHEET_ID from maxdata.mmax_locks);
		data _null_; jobname=compress("&jobname.",,"ps"); call symput('jobname',jobname); run;

		/* LOCK WORKSHEET */
		proc sql noprint;
		%let currdate = %sysfunc(date(),date9.);
		Insert into maxdata.mmax_locks ( LOCK_ID, LOCK_LEVEL, LOCK_TYPE, LOCK_DATE, NAME, USER_NAME, SERVER, SESSION_ID, WORKSHEET_ID, PLAN_SIZE, PLAN_PHYSICAL_MEM, PLAN_VIRTUAL_MEM, HASOPENED, STATUS, CURR_STATUS, CURR_TIME, PATHWAY_TYPE) 
			Values ( &wktid., 94, 'WK_OPEN', "&currdate."d, "&jobname.", 'Admin', '0.0.0.0', 'Update from AP', &wktid., 0, 0 , 0, 'Y', 'E', 'O', 23344, 'ap');

		/* Sets in-process flag for user */
		update maxdata.planworksheet set bat_error_msg='In-process' where planworksheet_id = &wktid.;

		/* Calls main analytic macro */
		select macroname into :macroname from maxapp.batch_analytic_lkup where analytic_lkup_id=&analytic_lkup_id.;
		quit;

	    	%&macroname.(wktid=&wktid., dsn=&dsn., dbtype=&dbtype.);

		/* Set worksheet status to finished */
		proc sql noprint;
		%let curr = %sysfunc(datetime(),datetime20.);
		update maxdata.planworksheet set analytic_lkup_id=0,LAST_UPDATE_DATE="&curr."dt where planworksheet_id = &wktid.;

		/* Unlock Worksheet */
		delete from maxdata.mmax_locks where lock_id = &wktid.;
		quit;

		proc datasets library=work memtype=data kill; 
		run;
		%end;

	%end;   /* DO LOOP */

%mend;

%do_analytic();
